﻿#include "mecom.h"

/////////////////////////////////////////////////
// Global variables defined just in the same file
double *Per, // Permutation array
       Sav;  // Buffer

long MODIFICATOR PieceLinearFunction (double *y,double *x,long n,double *a,double *b)
//
// The function for building piece-linear model.
// Designations:
//   y - function array by length n,
//   x - argument array by length n,
//   n - number of pairs of function-argument values,
//   a - calculated vector of coefficients by length n - 1,
//   b - calculated vector of coefficients by length n - 1,
// Return code:
//   0 - when normal result of computation,
//  -2 - when loss of significance.
// References:
//  Ryan T.P. Modern regression methods. - New York, NY: John Wiley & Sons, 1997.
//  Forster E., Ronz B. Methoden der Korrelations und Regressionsanalyse (in Deutsch). - Berlin: Verlag Die Wirtschaft, 1979.
//  Dougherty C. Introduction to econometrics. - New York, NT: Oxford University Press, 1992.
//  Pollard J.H. A handbook of numerical anad statistical techniques. - Cambridge: Cambridge University Press, 1981.
//
{
 long i; // Counter

 for (i = 0; i < n - 1; i++)
   if (x[i + 1] == x[i])
     return -2;
   else
     {
      a[i] = (y[i] * x[i + 1] - y[i + 1] * x[i]) / (x[i + 1] - x[i]);
      b[i] = (y[i + 1] - y[i]) / (x[i + 1] - x[i]);
     }

 return 0;
}

long MODIFICATOR PolynomialFunction (double *y,double *x,long u,long m,long n,double *c)
//
// The function for building of polynomial function.
// Designations:
//   y - function array by length n,
//   x - argument array by length n,
//   u - minimal degree of polymon,
//   m - degree of polymon,
//   n - number of pairs of function-argument values,
//   c - calculated vector of coefficients by length m - u + 1.
// Return code:
//   0 - when normal result of computation,
//  -2 - when loss of significance.
// References:
//  Ryan T.P. Modern regression methods. - New York, NY: John Wiley & Sons, 1997.
//  Forster E., Ronz B. Methoden der Korrelations und Regressionsanalyse (in Deutsch). - Berlin: Verlag Die Wirtschaft, 1979.
//  Dougherty C. Introduction to econometrics. - New York, NT: Oxford University Press, 1992.
//  Pollard J.H. A handbook of numerical anad statistical techniques. - Cambridge: Cambridge University Press, 1981.
//
{
 long i,j,k,l = 0,     // Counters
       Code,           // Error code
       mu = m - u + 1; // Work variable

 long double *z,       // Left parts array
             *b;       // Right parts array

 z = new long double[mu * mu];
 b = new long double[mu];

 for (i = 0; i < mu; i++)
   {
    b[i] = 0;
    for (j = 0; j < mu; j++)
      z[l++] = 0;
   }

 for (k = u; k <= m; k++)
   for (i = 0; i < n; i++)
     {
      for (j = u, l = k - u; j <= m; j++, l += mu)
        z[l] += Pow (x[i],j + k);
      b[k - u] += y[i] * Pow (x[i],k);
     }

 Code = GaussL (z,b,mu,1,0.000000001);

 for (i = 0; i < mu; i++)
   c[i] = b[i];

 delete [] z; delete [] b;

 return Code;
}

long MODIFICATOR ExponentFunction (double *y,double *x,long n,double *a)
//
// The function of approximation by exponential model.
// Designations:
//   y - values of empirical function,
//   x - values of argument,
//   n - number of pairs of function-argument values,
//  *a - coefficients of function y = a * exp (b * x).
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
// References:
//  Ryan T.P. Modern regression methods. - New York, NY: John Wiley & Sons, 1997.
//  Forster E., Ronz B. Methoden der Korrelations und Regressionsanalyse (in Deutsch). - Berlin: Verlag Die Wirtschaft, 1979.
//  Dougherty C. Introduction to econometrics. - New York, NT: Oxford University Press, 1992.
//  Pollard J.H. A handbook of numerical anad statistical techniques. - Cambridge: Cambridge University Press, 1981.
//
{
 long i;            // Cycle counter

 double m,m1,s1,s2; // Work variables

 for (i = 0, m1 = 0; i < n; i++)
   if (y[i] <= 0)
     return -2;
   else
     m1 += log (y[i]);

 m1 /= n;

 if ((m = VectorSum (x,n)) == 0)
   return -2;

 for (i = 0, s1 = 0; i < n; i++)
   s1 += x[i] * log (y[i]);
 s1 /= m;
 s1 -= m1;

 for (i = 0, s2 = 0; i < n; i++)
   s2 += x[i] * x[i];
 s2 /= m;
 m /= n;
 s2 -= m;
 if (s2 == 0)
   return -2;

 s1 /= s2;

 a[0] = exp (m1 - s1 * m);
 a[1] = s1;

 return 0;
}

long MODIFICATOR PowerFunction (double *y,double *x,long n,double *r)
//
// The function of approximation by power model.
// Designations:
//   y - values of empirical function,
//   x - values of argument,
//   n - number of pairs of function-argument values,
//  *r - coefficients of function y = a * x ** b.
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
// References:
//  Ryan T.P. Modern regression methods. - New York, NY: John Wiley & Sons, 1997.
//  Forster E., Ronz B. Methoden der Korrelations und Regressionsanalyse (in Deutsch). - Berlin: Verlag Die Wirtschaft, 1979.
//  Dougherty C. Introduction to econometrics. - New York, NT: Oxford University Press, 1992.
//  Pollard J.H. A handbook of numerical anad statistical techniques. - Cambridge: Cambridge University Press, 1981.
//
{
 long i;      // Cycle counter

 double t[4]; // Matrix of equations system

 t[0] = n;

 for (i = 0,t[1] = 0,t[3] = 0; i < n; i++)
   if (x[i] <= 0)
     return -2;
   else
     {
      t[1] += log (x[i]);
      t[3] += log (x[i]) * log (x[i]);
     }

 t[2] = t[1];

 for (i = 0,r[0] = 0,r[1] = 0; i < n; i++)
   if (y[i] <= 0)
     return -2;
   else
     {
      r[0] += log (y[i]);
      r[1] += log (x[i]) * log (y[i]);
     }

 if (Gauss (t,r,2,1,0.000001) != 0)
   return -2;

 r[0] =  exp (r[0]);

 return 0;
}

long MODIFICATOR HyperFunction (double *y,double *x,long n,double *r)
//
// The function of approximation by hyperbolic model.
// Designations:
//   y - values of empirical function,
//   x - values of argument,
//   n - number of pairs of function-argument values,
//  *r - coefficients of function y = a / x + b.
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
// References:
//  Ryan T.P. Modern regression methods. - New York, NY: John Wiley & Sons, 1997.
//  Forster E., Ronz B. Methoden der Korrelations und Regressionsanalyse (in Deutsch). - Berlin: Verlag Die Wirtschaft, 1979.
//  Dougherty C. Introduction to econometrics. - New York, NT: Oxford University Press, 1992.
//  Pollard J.H. A handbook of numerical anad statistical techniques. - Cambridge: Cambridge University Press, 1981.
//
{
 long i;      // Cycle counter

 double t[4]; // Matrix of equations system

 for (i = 0,t[0] = 0,t[1] = 0,r[0] = 0,r[1] = 0; i < n; i++)
   if (x[i] == 0)
     return -2;
   else
     {
      t[0] += 1 / x[i];
      t[1] += 1 / x[i] / x[i];
      r[0] += y[i];
      r[1] += y[i] / x[i];
     }

 t[2] = n;
 t[3] = t[0];

 if (Gauss (t,r,2,1,0.000001) != 0)
   return -2;

 return 0;
}

long MODIFICATOR ExpPowerFunction (double *y,double *x,long n,double *r)
//
// The function of approximation by exponential-power model.
// Designations:
//   y - values of empirical function,
//   x - values of argument,
//   n - number of pairs of function-argument values,
//  *r - coefficients of function y = exp (a * x) * x ** b
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
// References:
//  Ryan T.P. Modern regression methods. - New York, NY: John Wiley & Sons, 1997.
//  Forster E., Ronz B. Methoden der Korrelations und Regressionsanalyse (in Deutsch). - Berlin: Verlag Die Wirtschaft, 1979.
//  Dougherty C. Introduction to econometrics. - New York, NT: Oxford University Press, 1992.
//  Pollard J.H. A handbook of numerical anad statistical techniques. - Cambridge: Cambridge University Press, 1981.
//
{
 long i;       // Cycle counter

 double t[4],  // Matrix of equations system
        bx,by; // Work variables

 for (i = 0,t[0] = 0,t[1] = 0,t[3] = 0,r[0] = 0,r[1] = 0;
      i < n; i++)
   {
    if (x[i] < 0 || y[i] < 0)
      return -2;

    bx = log (x[i]);
    by = log (y[i]);

    r[0] += x[i] * by;
    r[1] += bx * by;
    t[0] += x[i] * x[i];
    t[1] += x[i] * bx;
    t[3] += bx * bx;
   }

 t[2] = t[1];

 if (Gauss (t,r,2,1,0.000001) != 0)
   return -2;

 return 0;
}

long MODIFICATOR LogarithmFunction (double *y,double *x,long n,double *r)
//
// The function of approximation by logarithmic model.
// Designations:
//   y - values of empirical function,
//   x - values of argument,
//   n - number of pairs of function-argument values,
//  *r - coefficients of function y = a + b * x + d * ln (x)
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
// References:
//  Ryan T.P. Modern regression methods. - New York, NY: John Wiley & Sons, 1997.
//  Forster E., Ronz B. Methoden der Korrelations und Regressionsanalyse (in Deutsch). - Berlin: Verlag Die Wirtschaft, 1979.
//  Dougherty C. Introduction to econometrics. - New York, NT: Oxford University Press, 1992.
//  Pollard J.H. A handbook of numerical anad statistical techniques. - Cambridge: Cambridge University Press, 1981.
//
{
 long i;      // Cycle counter

 double t[9], // Matrix of equations system
        buf;  // Work variable

 t[0] = 1;

 for (i = 0,t[1] = 0,t[2] = 0,t[4] = 0,t[5] = 0,t[8] = 0,
            r[0] = 0,r[1] = 0,r[2] = 0; i < n; i++)
   if (x[i] <= 0)
     return -2;
   else
     {
      buf = log (x[i]);
      t[1] += x[i];
      t[2] += buf;
      t[4] += x[i] * x[i];
      t[5] += x[i] * buf;
      t[8] += buf * buf;
      r[0] += y[i];
      r[1] += x[i] * y[i];
      r[2] += y[i] * buf;
     }
 t[3] = t[1];
 t[6] = t[2];
 t[7] = t[5];

 if (Gauss (t,r,3,1,0.000001) != 0)
   return -2;

 return 0;
}

long MODIFICATOR LogisticFunction (double *y,double *x,long n,double *c)
//
// The function of approximation by logistic model.
// Designations:
//   y - values of empirical function,
//   x - values of argument,
//   n - number of pairs of function-argument values,
//  *c - coefficients of function y = c / (1 + exp (a + b * x))
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
// References:
//  Ryan T.P. Modern regression methods. - New York, NY: John Wiley & Sons, 1997.
//  Forster E., Ronz B. Methoden der Korrelations und Regressionsanalyse (in Deutsch). - Berlin: Verlag Die Wirtschaft, 1979.
//  Dougherty C. Introduction to econometrics. - New York, NT: Oxford University Press, 1992.
//  Pollard J.H. A handbook of numerical anad statistical techniques. - Cambridge: Cambridge University Press, 1981.
//
{
 long i;      // Cycle counter

 double t[4], // Matrix of equations system
        r[2], // Vector of free members
        buf;  // Work variable

 c[0] =  MaximumOf (y,n);

 for (i = 0,t[0] = 0,t[1] = 0,t[3] = 0,r[0] = 0,r[1] = 0; i < n; i++)
   {
    if (y[i] == 0)
      return -2;

    buf = MaximumOf (y,n) / y[i] - 1;

    if (x[i] != 0 && buf > 0)
      {
       buf = log (buf);
       r[0] += buf;
       r[1] += x[i] * buf;
       t[0] += 1;
       t[1] += x[i];
       t[3] += x[i] * x[i];
      }
   }

 t[2] = t[1];

 if (Gauss (t,r,2,1,0.000001) != 0)
   return -2;

 c[1] =  r[0];
 c[2] =  r[1];

 return 0;
}

double Pow (double x,long n)
//
// The function calculates the power of value.
// Designations:
//  x - value,
//  n - power.
// Return code:
//  value in power.
// References:
//  Bloom D.M Linear algebra and geometry. - Cambridge, NY: Cambridge University Press, 1979.
//
{
 long i;         // Counter

 double s = 1.0; // Return value

 for (i = 0; i < n; i++)
   s *= x;

 return s;
}

void MODIFICATOR ScheirerRayHare (double *a,double *b,long k,long p,double *h,long *df)
//
// The function computes Scheirer-Ray-Hare test.
// Designations:
//   a - 1st multidimensional sample (p k-dimensions rows after each other),
//   b - 2nd multidimensional sample (p k-dimensions rows after each other),
//   k - each sample size,
//   p - size of each sample,
//   h - statistics for a (column effect), b (row effect), ab (effect interactions),
//  df - degrees of freedom for a, b, ab.
// Return code:
//  none.
// References:
//  Sokal R.R., Rohlf F.J. Biometry: The principles and practice of statistics in biological research. - New York, NY: WH Freeman and Company, 1995.
//  Scheirer C.J., Ray W.S., Hare N. The analysis of ranked data derived from completely randomized factorial designs // Biometrics, June 1976, vol. 32, no. 2, pp. 429-434.
//
{
 long i,j,           // Cycle counter
      m,             // Counter of members arrays
      n = 2 * k * p, // Common size
      s = k * p,     // Each sample size
      *ix;           // Information array

 double *x,          // Common array
        *y,          // Common array of ranks
        *ay,         // Array of ranks of sample a
        *by,         // Array of ranks of sample b
        *ta,         // Table sums of ranks
        *tb,         // Table sums of ranks
        *sc,         // Sums columns of table
        sr[2],       // Sums rows of table
        c,           // Work variable
        total,       // Work variable
        treat = 0;   // Work variable

 x = new double[n];
 y = new double[n];
 ay = new double[s];
 by = new double[s];
 ix = new long[n];
 ta = new double[p];
 tb = new double[p];
 sc = new double[p];

 for (i = 0,m = 0; i < s; i++)
   x[m++] = a[i];
 for (i = 0; i < s; i++)
   x[m++] = b[i];

 //////////////////
 // Sort array down
 SortArrayDown (x,n,ix);

 //////////
 // Ranking
 Rank (x,y,n);

 //////////////////
 // Restoring ranks
 UnSortArray (y,x,n,ix);
 for (i = 0,m = 0; i < s; i++)
   ay[i] = x[m++];
 for (i = 0; i < s; i++)
   by[i] = x[m++];

 //////////////////////////////////
 // Building of table sums of ranks
 for (i = 0, m = 0; i < p; i++)
   for(j = 0,ta[i] = 0; j < k; j++)
     ta[i] += ay[m++];
 for (i = 0, m = 0; i < p; i++)
   for(j = 0,tb[i] = 0; j < k; j++)
     tb[i] += by[m++];

 ///////////////////////////////
 // Computing of test statistics
 c = 0.25 * n * (n + 1) * (n + 1);
 total = 0.25 * n * (n + 1) / 3;
 for (i = 0,sr[0] = 0,sr[1] = 0; i < p; i++)
   {
    treat += ta[i] * ta[i];
    treat += tb[i] * tb[i];
    sc[i] = ta[i] + tb[i];
    sr[0] += ta[i];
    sr[1] += tb[i];
   }
 treat = treat / k - c;
 for (i = 0,h[0] = 0; i < p; i++)
   h[0] += sc[i] * sc[i];
 h[0] = h[0] / k / 2 - c;
 h[1] = (sr[0] * sr[0] + sr[1] * sr[1]) / s - c;
 h[2] = treat - h[0] - h[1];
 for (i = 0; i < 3; i++)
   h[i] /= total;
 df[0] = p - 1;
 df[1] = 1;
 df[2] = df[0];

 delete [] x; delete [] y; delete [] ay; delete [] by; delete [] ix;
 delete [] ta; delete [] tb; delete [] sc;
}

double MODIFICATOR Kullback (double *y1,double *y2,long n1,long n2,long p)
//
// The function computes Kullback test.
// Designations:
//  y1 - 1st multidimensional sample (rows after each other),
//  y2 - 2nd multidimensional sample (rows after each other),
//  n1 - size of each row of 1st sample,
//  n2 - size of each row of 2nd sample,
//   p - size of each sample.
// Return code:
//  test statistic.
// References:
//  Kullback S. Information theory and statistics. - Gloucester, MA: Peter Smith, 1978.
//
{
 long i,j,k,   // Cycle counter
      m,m1,m2; // Array counter

 double *x1,   // Work array
        *x2,   // Work array
        *s,    // Covariation matrix of 1st sample
        *t,    // Covariation matrix of 2nd sample
        *r,    // Common covariation matrix
        Ret;   // Return code

 x1 = new double[MAX(n1,n2)];
 x2 = new double[MAX(n1,n2)];
 s = new double[p * p];
 t = new double[p * p];
 r = new double[p * p];

 /////////////////////
 // Covariation matrix
 for (i = 0, m = 0, m1 = 0; i < p; i++)
   {
    for (j = 0; j < n1; j++)
      x1[j] = y1[m1++];
    for (j = 0, m2 = 0; j < p; j++)
      {
       for (k = 0; k < n1; k++)
         x2[k] = y1[m2++];
       s[m++] = Cov(x1,x2,n1);
      }
   }
 for (i = 0, m = 0, m1 = 0; i < p; i++)
   {
    for (j = 0; j < n2; j++)
      x1[j] = y2[m1++];
    for (j = 0, m2 = 0; j < p; j++)
      {
       for (k = 0; k < n2; k++)
         x2[k] = y2[m2++];
       t[m++] = Cov(x1,x2,n2);
      }
   }
 for (i = 0, m = 0; i < p; i++)
   for (j = 0; j < p; j++, m++)
     r[m] = (s[m] * (n1 - 1) + t[m] * (n2 - 1)) / (n1 + n2 - 2);

 /////////////////////////////
 // Test statistic computation
 Ret = Krauth(r,p,0.000000001);
 Ret = log(Ret / Krauth(s,p,0.000000001)) * (n1 - 1) + log(Ret /
                 Krauth(t,p,0.000000001)) * (n1 - 1);

 delete [] x1; delete [] x2; delete [] s; delete [] t; delete [] r;

 return Ret;
}

double MODIFICATOR JamesSue (double *y1,double *y2,long k1,long k2,long p)
//
// The function computes James-Sue test.
// Designations:
//  y1 - 1st multidimensional sample (rows after each other),
//  y2 - 2nd multidimensional sample (rows after each other),
//  k1 - size of each row of 1st sample,
//  k2 - size of each row of 2nd sample,
//   p - size of each sample.
// Return code:
//  test statistic.
// References:
//  Kullback S. Information theory and statistics. - Gloucester, MA: Peter Smith, 1978.
//
{
 long i,j,        // Cycle counter
      m,          // Array counter
      p2 = p * p; // Work variable

 double *ys1,     // Vector medium of 1st sample
                  // and array difference of medium
        *ys2,     // Vector medium of 2nd sample
        *s,       // Covariation matrix and work array
        *t,       // Work array
        *r,       // Work array
        Ret;      // Return code

 ys1 = new double[p];
 ys2 = new double[p];
 s = new double[p2];
 t = new double[p2];
 r = new double[p2];

 /////////////////////////////
 // Parameters of 1st sample
 for (i = 0,m = 0; i < p; ys1[i++] /= k1)
   for (j = 0,ys1[i] = 0; j < k1; j++)
     ys1[i] += y1[m++];
 MatrixTMultiplyMatrix (y1,y1,s,p,k1,p);
 MatrixMultiplyMatrixT (ys1,ys1,r,p,1,p);
 MatrixMultiplyNumber (r,p2,-k1);
 MatrixPlusMatrix (s,r,p2);

 //////////////////////////////
 // Parameters of 2nd sample
 for (i = 0,m = 0; i < p; ys2[i++] /= k2)
   for (j = 0,ys2[i] = 0; j < k2; j++)
     ys2[i] += y2[m++];
 MatrixTMultiplyMatrix (y2,y2,t,p,k2,p);
 MatrixMultiplyMatrixT (ys2,ys2,r,p,1,p);
 MatrixMultiplyNumber (r,p2,-k2);
 MatrixPlusMatrix (t,r,p2);

 /////////////////////
 // Covariation matrix
 MatrixMultiplyNumber (s,p2,1.0 / k1);
 MatrixMultiplyNumber (t,p2,1.0 / k2);
 MatrixPlusMatrix (s,t,p2);

 ////////////////////////////////////
 // Matrix inverse covatiation matrix
 InverseMatrix (s,t,p);

 /////////////////////////////
 // Array of difference medium
 for (i = 0; i < p; i++)
   ys1[i] -= ys2[i];

 /////////////////////////////
 // Test statistic computation
 MatrixTMultiplyMatrix (ys1,t,s,1,p,p);
 MatrixMultiplyMatrix (s,ys1,t,1,p,1);
 Ret = fabs (t[0] * k1 / (k1 + k2) * k1);

 delete [] ys1; delete [] ys2; delete [] s; delete [] t; delete [] r;

 return Ret;
}

double MODIFICATOR Hotelling (double *y1,double *y2,long k1,long k2,long p)
//
// The function computes Hotelling T-square test.
// Designations:
//  y1 - 1st multidimensional sample (rows after each other),
//  y2 - 2nd multidimensional sample (rows after each other),
//  k1 - size of each row of 1st sample,
//  k2 - size of each row of 2nd sample,
//   p - size of each sample.
// Return code:
//  test statistic.
// References:
//  Afifi F.F., Azen S.P. Statistical analysis. A computer oriented approach. - New York, NY: Academic Press, 1979.
//  Maindonald J.H. Statistical computation. - New York, NY: John Wiley & Sons, 1984.
//
{
 long i,j,        // Cycle counter
      m,          // Array counter
      p2 = p * p; // Work variable

 double *ys1,     // Vector medium of 1st sample
                  // and array of difference medium
        *ys2,     // Vector medium of 2nd sample
        *s,       // Covariation matrix
                  // and work array
        *t,       // Work array
        *r,       // Work array
        Ret;      // Return code

 ys1 = new double[p];
 ys2 = new double[p];
 s = new double[p2];
 t = new double[p2];
 r = new double[p2];

 ///////////////////////////
 // Parameters of 1st sample
 for (i = 0,m = 0; i < p; ys1[i++] /= k1)
   for (j = 0,ys1[i] = 0; j < k1; j++)
     ys1[i] += y1[m++];
 MatrixTMultiplyMatrix (y1,y1,s,p,k1,p);
 MatrixMultiplyMatrixT (ys1,ys1,r,p,1,p);
 MatrixMultiplyNumber (r,p2,-k1);
 MatrixPlusMatrix (s,r,p2);

 ///////////////////////////
 // Parameters of 2nd sample
 for (i = 0,m = 0; i < p; ys2[i++] /= k2)
   for (j = 0,ys2[i] = 0; j < k2; j++)
     ys2[i] += y2[m++];
 MatrixTMultiplyMatrix (y2,y2,t,p,k2,p);
 MatrixMultiplyMatrixT (ys2,ys2,r,p,1,p);
 MatrixMultiplyNumber (r,p2,-k2);
 MatrixPlusMatrix (t,r,p2);

 /////////////////////
 // Covariation matrix
 MatrixPlusMatrix (s,t,p2);
 MatrixMultiplyNumber (s,p2,1.0 / (k1 + k2 - 2));

 /////////////////////////////
 // Inverse covariation matrix
 InverseMatrix (s,t,p);

 /////////////////////////////
 // Array of difference medium
 for (i = 0; i < p; i++)
   ys1[i] -= ys2[i];

 /////////////////////////////
 // Test statistic computation
 MatrixTMultiplyMatrix (ys1,t,s,1,p,p);
 MatrixMultiplyMatrix (s,ys1,t,1,p,1);
 Ret = fabs (t[0] * k1 / (k1 + k2) * k1);

 delete [] ys1; delete [] ys2; delete [] s; delete [] t; delete [] r;

 return Ret;
}

double MODIFICATOR PuriSenTamura (double *y1,double *y2,long k1,long k2,long p)
//
// The function computes Puri-Sen-Tamura test.
// Designations:
//  y1 - 1st multidimensional sample (rows after each other),
//  y2 - 2nd multidimensional sample (rows after each other),
//  k1 - size of each row of 1st sample,
//  k2 - size of each row of 2nd sample,
//   p - size of each sample.
// Return code:
//  test statistic.
// References:
//  Tamura R. Multivariate nonparametric several-sample tests // The Annals of Mathematical Statistics, June 1966, vol. 37, no. 3, pp. 611-618.
//  Pontes A.C.F. Analise de variancia multivariada com a utilizacao de testes nao-parametricos e componentes principais baseados em matrizes de postos. - Sao Paulo: Paracicaba, 2005.
//
{
 long i,j,l,       // Cycle counter
      m,m1,m2,     // Array counter
      p2 = p * p,  // Work variable
      k = k1 + k2, // Work variable
      *ix;         // Information array

 double *ys1,      // Vector medium of ranks of 1st sample and work array
        *ys2,      // Vector medium of ranks of 2nd sample and work array
        *ys,       // Work array and common vector medium of ranks
        *t,        // Covariation matrix and work array
        *s,        // Inverse covariation matrix
        *x,        // Work array
        *r,        // Common array of ranks
        *r1,       // Array of ranks of 1st sample
        *r2,       // Array of ranks of 2nd sample
        Ret;       // Return code

 ix = new long[k];
 ys1 = new double[MAX(k,p)];
 ys2 = new double[MAX(k,p)];
 ys = new double[MAX(k,p)];
 t = new double[p2];
 s = new double[p2];
 x = new double[MAX(k,p)];
 r = new double[k * p];
 r1 = new double[k1 * p];
 r2 = new double[k2 * p];

 //////////////////////////////
 // Obtaining of array of ranks
 for (i = 0, m = 0; i < p; i++)
   {
    ////////////////////////
    // Unity copy of columns
    for (j = 0, m1 = i * k1; j < k1; j++)
      ys[j] = y1[m1++];
    for (j = k1, m1 = i * k2; j < k; j++)
      ys[j] = y2[m1++];

    //////////
    // Sort up
    SortArrayUp (ys,k,ix);

    //////////
    // Ranking
    Rank (ys,x,k);

    //////////////////
    // Restoring ranks
    UnSortArray (x,ys,k,ix);

    //////////////////////////////////////////
    // Completion of array of ranks by columns
    for (j = 0, m1 = i * k1; j < k1; j++)
      {
       r1[m1++] = ys[j];
       r[m++] = ys[j];
      }
    for (j = k1, m1 = i * k2; j < k; j++)
      {
       r2[m1++] = ys[j];
       r[m++] = ys[j];
      }
   }

 ///////////////////////////////
 // Parameters of array of ranks
 for (i = 0, m = 0, m1 = 0; i < p; i++)
   {
    for (j = 0; j < k; j++)
      ys1[j] = r[m1++];
    for (j = 0, m2 = 0; j < p; j++)
      {
       for (l = 0; l < k; l++)
         ys2[l] = r[m2++];
       t[m++] = Cov(ys1,ys2,k);
      }
   }
 InverseMatrix (t,s,p);
 for (i = 0, m = 0, m1 = 0, m2 = 0; i < p; i++)
   {
    for (j = 0,ys[i] = 0; j < k; j++)
      ys[i] += r[m++];
    ys[i] /= k;
    for (j = 0,ys1[i] = 0; j < k1; j++)
      ys1[i] += r1[m1++];
    ys1[i] = ys1[i] / k1 - ys[i];
    for (j = 0,ys2[i] = 0; j < k2; j++)
      ys2[i] += r2[m2++];
    ys2[i] = ys2[i] / k2 - ys[i];
   }

 /////////////////////////////
 // Test statistic computation
 MatrixTMultiplyMatrix (ys1,s,t,1,p,p);
 MatrixMultiplyMatrix (t,ys1,x,1,p,1);
 Ret = x[0];
 MatrixTMultiplyMatrix (ys2,s,t,1,p,p);
 MatrixMultiplyMatrix (t,ys2,x,1,p,1);
 Ret += x[0];

 delete [] ys1; delete [] ys2; delete [] s; delete [] t; delete [] r;
 delete [] ys; delete [] ix; delete [] r1; delete [] r2; delete [] x;

 return Ret;
}

double MODIFICATOR PuriSen (double *y1,double *y2,long k1,long k2,long p)
//
// The function computes Puri-Sen test.
// Designations:
//  y1 - 1st multidimensional sample (rows after each other),
//  y2 - 2nd multidimensional sample (rows after each other),
//  k1 - size of each row of 1st sample,
//  k2 - size of each row of 2nd sample,
//   p - size of each sample.
// Return code:
//  test statistic.
// References:
//  Puri M.L., Sen P.K. Nonparametric methods in multivariate analysis. - New York, NY: John Wiley & Sons, 1971.
//  Pontes A.C.F. Analise de variancia multivariada com a utilizacao de testes nao-parametricos e componentes principais baseados em matrizes de postos. - Sao Paulo: Paracicaba, 2005.
//
{
 long i,j,l,       // Cycle counter
      m,m1,m2,     // Array counter
      p2 = p * p,  // Work variable
      k = k1 + k2, // Work variable
      *ix;         // Information array

 double *ys1,      // Vector medium of ranks of 1st sample and work array
        *ys2,      // Vector medium of ranks of 2nd sample and work array
        *ys,       // Work array and common vector medium of ranks
        *t,        // Covariation matrix and work array
        *s,        // Matrix inverse of covariation matrix
        *x,        // Work array
        *r,        // Common array of ranks of labels
        *r1,       // Array of ranks of labels of 1st sample
        *r2,       // Array of ranks of labels of 2nd sample
        Ret;       // Return code

 ix = new long[k];
 ys1 = new double[MAX(k,p)];
 ys2 = new double[MAX(k,p)];
 ys = new double[MAX(k,p)];
 t = new double[p2];
 s = new double[p2];
 x = new double[MAX(k,p)];
 r = new double[k * p];
 r1 = new double[k1 * p];
 r2 = new double[k2 * p];

 //////////////////////////////
 // Obtaining of array of ranks
 for (i = 0, m = 0; i < p; i++)
   {
    ////////////////////////
    // Unity copy of columns
    for (j = 0, m1 = i * k1; j < k1; j++)
      ys[j] = y1[m1++];
    for (j = k1, m1 = i * k2; j < k; j++)
      ys[j] = y2[m1++];

    //////////
    // Sort up
    SortArrayUp (ys,k,ix);

    //////////
    // Ranking
    Rank (ys,x,k);

    //////////////////////////////////
    // Restoring ranks of given arrays
    UnSortArray (x,ys,k,ix);

    ////////////////////////////////////////////////////
    // Completion of array of ranks of labels by columns
    for (j = 0, m1 = i * k1; j < k1; j++)
      {
       r1[m1++] = ys[j];
       r[m++] = ys[j];
      }
    for (j = k1, m1 = i * k2; j < k; j++)
      {
       ys[j] = ys[j] / (k + 1) - 0.5;
       ys[j] *= ys[j];
       r2[m1++] = ys[j];
       r[m++] = ys[j];
      }
   }

 ///////////////////////////////
 // Parameters of array of ranks
 for (i = 0, m = 0, m1 = 0; i < p; i++)
   {
    for (j = 0; j < k; j++)
      ys1[j] = r[m1++];
    for (j = 0, m2 = 0; j < p; j++)
      {
       for (l = 0; l < k; l++)
         ys2[l] = r[m2++];
       t[m++] = Cov(ys1,ys2,k);
      }
   }
 InverseMatrix (t,s,p);
 for (i = 0, m = 0, m1 = 0, m2 = 0; i < p; i++)
   {
    for (j = 0,ys[i] = 0; j < k; j++)
      ys[i] += r[m++];
    ys[i] /= k;
    for (j = 0,ys1[i] = 0; j < k1; j++)
      ys1[i] += r1[m1++];
    ys1[i] = ys1[i] / k1 - ys[i];
    for (j = 0,ys2[i] = 0; j < k2; j++)
      ys2[i] += r2[m2++];
    ys2[i] = ys2[i] / k2 - ys[i];
   }

 /////////////////////////////
 // Test statistic computation
 MatrixTMultiplyMatrix (ys1,s,t,1,p,p);
 MatrixMultiplyMatrix (t,ys1,x,1,p,1);
 Ret = x[0];
 MatrixTMultiplyMatrix (ys2,s,t,1,p,p);
 MatrixMultiplyMatrix (t,ys2,x,1,p,1);
 Ret += x[0];

 delete [] ys1; delete [] ys2; delete [] s; delete [] t; delete [] r;
 delete [] ys; delete [] ix; delete [] r1; delete [] r2; delete [] x;

 return Ret;
}

double MODIFICATOR RepeatedANOVA(double *x,long r,long c)
//
// The function builds repeated ANOVA.
// Designations:
//  x - array of data vectors (of vector one by one),
//  r - number of rows,
//  c - number of columns.
// Return code:
//  test statistic.
// References:
//  Scheffe H. The Analysis of Variance. - New York, NY: John Wiley & Sons, 1999.
//  Brandt S. Data analysis. Statistical and computational methods for scientists and engineers. - New York, NY: Springer-Verlag, 2003.
//
{
 long i,j,m;     // Counter

 double *ti,     // Mediums of rows
        *tj,     // Mediums of columns
        tt = 0,  // Common medium
        sst = 0,
        ssr = 0;

 ti = new double[r];
 tj = new double[c];

 ///////////////
 // Sums of rows
 for (i = 0; i < r; i++)
   for (j = 0, m = i, ti[i] = 0; j < c; j++, m += r)
     {
      ti[i] += x[m];
      tt += x[m];
     }

 //////////////////
 // Sums of columns
 for (i = 0, m = 0; i < c; i++)
   for (j = 0, tj[i] = 0; j < r; j++)
     tj[i] += x[m++];

 //////////
 // Mediums
 tt /= (r * c);
 for (i = 0; i < c; i++)
   tj[i] /= r;
 for (i = 0; i < r; i++)
   ti[i] /= c;

 /////////////
 // Variations
 for (i = 0; i < c; i++)
   sst += (tj[i] - tt) * (tj[i] - tt);
 sst *= r;
 for (i = 0, m = 0; i < c; i++)
   for (j = 0; j < r; j++, m++)
     ssr += (x[m] - ti[j] - tj[i] + tt) * (x[m] - ti[j] - tj[i] + tt);

 ssr /= (r - 1);

 delete [] ti; delete [] tj;

 return sst / ssr;
}

long MODIFICATOR ANCOVA (double *d,long *n,long k,double *yz,double *zz,double *gc,double *zc,double *szc,double *sm,double *sg,double *sb,double *bb,double *bm,double *bt,double *yzz, double *zzz)
//
// The function builds covariation analysis.
// Designations:
//     d - array k groups (of vector one by one),
//     n - array size k groups,
//   *yz - vector group medium predictors,
//   *zz - vector group medium of dependent values,
//   *gc - vector group of coefficients,
//   *zc - corrected mediums of dependent values,
//  *szc - errors corrected medium of dependent values,
//   *sm - statistic Sm,
//   *sg - statistic Sg,
//   *sb - statistic Sb,
//   *bb - estimation of coefficient of regression,
//   *bm - estimation of coefficient of regression medium values,
//   *bt - estimation of full coefficient of regression,
//  *yzz - common medium predictors,
//  *zzz - common medium of dependent variables.
// Return code:
//   0 - when normal end of computation,
//  -1 - when lack of memory.
// References:
//  Wildt A.R. Analysis of covariance / A.R. Wildt, O. Ahtola. - Newbury Park, CA: Sage Publications, 1978.
//  Milliken G.A., Johnson D.E. Analysis of messy data. Volume III: Analysis of covariance. - Boca Raton, FL: Chapman & Hall / CRC, 2002.
//  Sokal R.R., Rohlf F.J. Biometry: The principles and practice of statistics in biological research. - New York, NY: WH Freeman and Company, 1995.
//  Hsu J.C. Multiple comparisons: Theory and methods. - London: Chapman & Hall, 1996.
//  Edwards L.K. Applied analysis of variance in behavioral science / Ed. by L.K. Edwards. - New York, NY: Marcel Dekker, 1993.
//  Westfall P.H., Young S.S. Resampling-based multiple testing: Examples and methods for p-value adjustment. - New York, NY: Wiley & Sons, 1993.
//  Sahai H., Ageel M.I. The analysis of variance: fixed, random, and mixed models. - Boston, MA: Birkhauser, 2000.
//  Afifi F.F., Azen S.P. Statistical analysis. A computer oriented approach. - New York, NY: Academic Press, 1979.
//
{
 long i,j,                  // Cycle counter
      s = 0,                // Counter of members arrays
      nn = VectorSum (n,k); // Common size pairs

 double *exyi,
        *exxi,
        mxx = 0,            // Sum of squares deviations predictors inside groups
        mxy = 0,            // Mixed composition inside groups
        myy = 0,            // Sum of squares deviations of dependent variables inside groups
        exx = 0,            // Sum of squares deviations predictors between groups
        exy = 0,            // Mixed composition between groups
        eyy = 0,            // Sum of compositions deviations of dependent variables between groups
        txx = 0,            // Full sum of squares deviations predictors
        txy = 0,            // Full mixed composition
        tyy = 0,            // Full sum of compositions deviations of dependent variables
        msm,mse,msz,        // Work variables
        msb = 0,msr;        // Work variables

 if ((exyi = new double[k]) == 0)
   return -1;
 if ((exxi = new double[k]) == 0)
   {
    delete [] exyi;
    return -1;
   }

 for (i = 0, *yzz = 0, *zzz = 0; i < k; s += n[i++])
   {
    yz[i] = 0;
    zz[i] = 0;
    for (j = 0; j < n[i]; j++, s++)
      {
       yz[i] += d[s];
       zz[i] += d[s + n[i]];
      }
    *yzz += yz[i];
    *zzz += zz[i];
    yz[i] /= n[i];
    zz[i] /= n[i];
   }
 *yzz /= nn;
 *zzz /= nn;

 for (i = 0,s = 0; i < k; s += n[i++])
   {
    for (j = 0,exxi[i] = 0,exyi[i] = 0; j < n[i]; j++, s++)
      {
       exx += (d[s] - yz[i]) * (d[s] - yz[i]);
       exy += (d[s] - yz[i]) * (d[s + n[i]] - zz[i]);
       eyy += (d[s + n[i]] - zz[i]) * (d[s + n[i]] - zz[i]);
       txx += (d[s] - *yzz) * (d[s] - *yzz);
       txy += (d[s] - *yzz) * (d[s + n[i]] - *zzz);
       tyy += (d[s + n[i]] - *zzz) * (d[s + n[i]] - *zzz);
       exxi[i] += (d[s] - yz[i]) * (d[s] - yz[i]);
       exyi[i] += (d[s] - yz[i]) * (d[s + n[i]] - zz[i]);
      }
    gc[i] = exyi[i] / exxi[i];

    mxx += (yz[i] - *yzz) * (yz[i] - *yzz) * n[i];
    mxy += (yz[i] - *yzz) * (zz[i] - *zzz) * n[i];
    myy += (zz[i] - *zzz) * (zz[i] - *zzz) * n[i];
   }

 *bb = exy / exx;
 msm = (myy - txy * txy / txx +  *bb * exy) / (k - 1);
 msz = *bb * exy;
 mse = (eyy - *bb * exy) / (nn - k - 1);
 for (i = 0; i < k; i++)
   msb += exyi[i] * exyi[i] / exxi[i];
 msr = (eyy - msb) / (nn - k * 2);
 msb = (msb - exy * *bb) / (k - 1);
 for (i = 0; i < k; i++)
   {
    zc[i] = zz[i] - *bb * (yz[i] - *yzz);
    szc[i] = sqrt(mse * (1.0 / n[i] +  (yz[i] - *yzz) * (yz[i] - *yzz) / exx));
   }
 *bm = mxy / mxx;
 *bt = txy / txx;

 *sm = msm / mse;
 *sg = msz / mse;
 *sb = msb / msr;

 delete [] exyi; delete [] exxi;

 return 0;
}

double MODIFICATOR ANOVA (double *dat,long *n,long k)
//
// The function builds ANOVA.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - array of size k vectors.
// Return code:
//  test statistic.
// References:
//  Scheffe H. The Analysis of Variance. - New York, NY: John Wiley & Sons, 1999.
//  Brandt S. Data analysis. Statistical and computational methods for scientists and engineers. - New York, NY: Springer-Verlag, 2003.
//  Van der Waerden B.L. Mathematische Statistik (in Deutsch). - Berlin: Springer, 1971.
//
{
 long i,j,m = 0,             // Counter
       nn = VectorSum (n,k); // Common number of samle members

 double *x,*t,               // Work array
        s = 0,               // Full variance
        s1 = 0,              // Sum sample variances
        tt = 0;              // Work variable

 t = new double[k];
 x = new double[MaximumOf (n,k)];

 for (i = 0; i < k; i++)
   {
    for (j = 0,t[i] = 0; j < n[i]; j++)
      {
       x[j] = dat[m++];
       t[i] += x[j];
       s += x[j] * x[j];
      }
    s1 += t[i] * t[i] / n[i];
    tt += t[i];
   }

 tt *= tt;
 s -= tt / nn;
 s1 -= tt / nn;

 delete [] t; delete [] x;

 return s1 / (k - 1) / ((s - s1) / (nn - k));
}

double MODIFICATOR KruskalWallis (double *dat,long *n,long k,long t)
//
// The function for computation Kruskal-Wallis test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - array of size k vectors,
//    t - flag of bundle adjustment (0 - no adjustment).
// Return code:
//  test statistic.
// References:
//  Hettmansperger T.P. Statistical inference based on ranks. - New York, NY: John Wiley & Sons, 1984.
//  Johnson N.L., Leone F.C. Statistics and experimental design in engineering and the physical sciences. Vol. I. - New York, NY: John Wiley & Sons, 1977.
//
{
 long i,j,m,                 // Counter
       kk = VectorSum (n,k), // Common number of samle members
       *ix;                  // Information about given arrays

 double s = 0,s1,            // Work variable
        *y,                  // Array of ranks sorted arrays
        *x,                  // Array of ranks of one of vector
        *r,                  // Copy of given arrays and array of ranks
        c;                   // Bundle adjustment of ranks

 ix = new long[kk];
 y = new double[kk];
 x = new double[MaximumOf (n,k)];
 r = new double[kk];

 //////////////////////////
 // Copy of given arrays
 for (i = 0; i < kk; i++)
   r[i] = dat[i];

 //////////
 // Sort up
 SortArrayUp (r,kk,ix);

 //////////
 // Ranking
 c = RankKruskal (r,y,kk);
 if (!t)
   c = 1;

 /////////////////////////////////////
 // Restoring ranks of given arrays
 UnSortArray (y,r,kk,ix);

 for (i = 0, m = 0; i < k; i++)
   {
    for (j = 0, s1 = 0; j < n[i]; j++)
      {
       x[j] = r[m++];
       s1 += x[j];
      }
    s += s1 * s1 / n[i];
   }

 delete [] ix; delete [] y; delete [] r; delete [] x;

 return (s * 12 / kk / (kk + 1) - 3 * (kk + 1)) / c;
}

double MODIFICATOR JonckheereTerpstra (double *dat,long *n,long k)
//
// The function for computation of Jonckheere-Terpstra test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - array of size k vectors.
// Return code:
//  normalized test statistic.
// references:
//  Cuzick J. A Wilcoxon-type test for trend // Statistics in Medicine, January/March 1985, vol. 4, no. 1, pp. 87-90.
//  Hollander M., Wolfe D.A., Chiken E. Nonparametric statistical methods. - New York, NY: John Wiley & Sons, 2014.
//
{
 long i,j,l,m,m1 = 0,m2,    // Counter
      nn = VectorSum (n,k); // Common number of samle members

 double *x,*y,              // Work array
        Rs = 0,             // Test statistic
        MI = 0,DI = 0;      // Parameters of normal distribution

 x = new double[MaximumOf (n,k)];
 y = new double[MaximumOf (n,k)];

 for (i = 0; i < k - 1; i++)
   {
    for (j = 0; j < n[i]; j++)
      x[j] = dat[m1++];
    for (m = i + 1, m2 = m1; m < k; m++)
      {
       for (l = 0; l < n[m]; l++)
         y[l] = dat[m2++];
       Rs += MannWhitneyTest (x,y,n[i],n[m]);
      }
   }

 delete [] x; delete [] y;

 for (j = 0; j < k; j++)
   {
    MI -= n[j] * n[j];
    DI -= n[j] * n[j] * (2 * n[j] + 3);
   }

 return (Rs - (MI + nn * nn) / 4) / sqrt ((DI + nn * nn * (2 * nn + 3)) / 72);
}

double MODIFICATOR Bartlett (double *dat,long *n,long k)
//
// The function for computation of Bartlett test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - array of size k vectors.
// Return code:
//  test statistic.
// References:
//  Brownlee K.A. Statistical theory and metodology in sciences and engineering. - New York, NY: John Wiley & Sons, 1965.
//
{
 long i,j,m = 0,             // Counter
      nn = VectorSum (n,k);  // Common number of samle members

 double *x,                  // Work array
        s,                   // Sample variance
        c = 0,s1 = 0,s2 = 0; // Work variables

 x = new double[MaximumOf (n,k)];

 for (i = 0; i < k; i++)
   {
    for (j = 0; j < n[i]; j++)
      x[j] = dat[m++];
    c += 1.0 / (n[i] - 1);
    s = Variance (x,n[i]);
    if (s < 0.000001)
      {
       delete [] x;
       return 0;
      }
    s1 += log (s) * (n[i] - 1);
    s2 += s * (n[i] - 1);
   }

 delete [] x;

 return ((nn - k) * log (s2 /(nn - k)) - s1) /
        ((c - 1.0 / (nn - k)) / 3 / (k - 1) + 1);
}

double MODIFICATOR Friedman (double *dat,long n,long k,long t)
//
// The function for computation of Friedman test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - size of each of vector,
//    t - flag of bundle adjustment (0 - no adjustment).
// Return code:
//  test statistic.
// References:
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//  Brownlee K.A. Statistical theory and metodology in sciences and engineering. - New York, NY: John Wiley & Sons, 1965.
//
{
 long i,j,m,       // Counter
      *ix;         // Information about given arrays

 double s = 0,buf, // Work variable
        ss = 0,    // Work variable
        *y,        // Work array
        *x,        // Array of ranks of one of vector
        *r;        // Array of ranks of data matrix

 ix = new long[k];
 y = new double[k];
 x = new double[k];
 r = new double[n * k];

 for (i = 0; i < n; i++)
   {
    ///////////////
    // Copy of rows
    for (j = 0, m = i; j < k; j++, m += n)
      y[j] = dat[m];

    //////////
    // Sort up
    SortArrayUp (y,k,ix);

    //////////
    // Ranking
    ss += (RankFriedman (y,x,k) - k);

    /////////////////////////////////////
    // Restoring ranks of given arrays
    UnSortArray (x,y,k,ix);

    /////////////////////////////
    // Completion arrays of ranks
    for (j = 0, m = i; j < k; j++, m += n)
      r[m] = y[j];
   }
 if (!t)
   ss = 0;

 for (i = 0, m = 0; i < k; i++)
    {
     for (j = 0, buf = 0; j < n; j++)
       buf += r[m++];
     buf = buf - n * (k + 1) / 2;
     s += buf * buf;
    }

 delete [] ix; delete [] x; delete [] y; delete [] r;

 return s * 12 / (1.0 * n * k * (k + 1) - ss / (k - 1));
}

double MODIFICATOR Quade (double *dat,long n,long k)
//
// The function for computation of Quade test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - size of each of vector.
// Return code:
//  test statistic.
//
{
 long i,j,m,    // Counter
      *ix;      // Information about given arrays

 double s,      // Work variable
        tn = 0, // Work variable
        *y,     // Work array
        *x,     // Array of ranks of one of vector
        *q,     // Array of ranges
        *t,     // Work array
        *r;     // Array of ranks of data matrix

 ix = new long[MAX(k,n)];
 y = new double[k];
 x = new double[k];
 q = new double[n];
 t = new double[k * n];
 r = new double[n * k];

 for (i = 0; i < n; i++)
   {
    ///////////////
    // Copy of rows
    for (j = 0, m = i; j < k; j++, m += n)
      y[j] = dat[m];

    //////////
    // Sort up
    SortArrayUp (y,k,ix);

    ///////////////////////////////
    // Computation of range of rows
    q[i] =  y[k - 1] - y[0];

    //////////
    // Ranking
    Rank (y,x,k);

    //////////////////////////////////
    // Restoring ranks of given arrays
    UnSortArray (x,y,k,ix);

    /////////////////////////////
    // Completion arrays of ranks
    for (j = 0, m = i; j < k; j++, m += n)
      r[m] = y[j];
   }

 //////////
 // Sort up
 SortArrayUp (q,n,ix);

 //////////
 // Ranking
 Rank (q,t,n);

 //////////////////
 // Restoring ranks
 UnSortArray (t,q,n,ix);

 //////////////////
 // Sums of columns
 for (i = 0, m = 0; i < k; i++)
   {
    for (j = 0, s = 0; j < n; j++)
      {
       t[m] = q[j] * (r[m] - (k + 1) / 2);
       s += t[m++];
      }
    tn += s * s;
   }
 tn /= n;

 for (i = 0, m = 0, s = 0; i < k; i++)
   {
    for (j = 0; j < n; j++, m++)
      s += t[m] * t[m];
   }

 delete [] ix; delete [] x; delete [] y; delete [] r; delete [] q; delete [] t;

 return tn * (n - 1) / (s - tn);
}

double MODIFICATOR Page (double *dat,long n,long k, double *Rs)
//
// The function for computation of Page test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - size of each of vector,
//  *Rs - test statistic.
// Return code:
//  normalized test statistic.
//
{
 long i,j,m1,m2 = 0, // Counter
      *ix;

 double *y,          // Work array, later Array sums of ranks
        *x,          // Array of ranks of one of vector
        *r;          // Array of ranks of data matrix

 ix = new long[k];
 y = new double[k];
 x = new double[k];
 r = new double[n * k];

 for (i = 0; i < n; i++)
   {
    ///////////////
    // Copy of rows
    for (j = 0, m1 = m2; j < k; j++, m1 += n)
      y[j] = dat[m1];

    SortArrayUp (y,k,ix);
    Rank (y,x,k);
    UnSortArray (x,y,k,ix);

    /////////////////////////////
    // Completion arrays of ranks
    for (j = 0, m1 = m2++; j < k; j++, m1 += n)
      r[m1] = y[j];
   }

 ////////////////////////////
 // Sums computation of ranks
 for (i = 0, m1 = 0; i < k; i++)
   for (j = 0, y[i] = 0; j < n; j++)
     y[i] += r[m1++];

 delete [] ix; delete [] r; delete [] x;

 SortArrayUp0 (y,k);

 /////////////////////////////
 // Test statistic computation
 for (i = 0, *Rs = 0; i < k; i++)
   *Rs += y[i] * (i + 1);

 delete [] y;

 return (*Rs - 1.0 * n * k * (k + 1) * (k + 1) / 4.0) /
         sqrt (1.0 * n * (k * k * k - k) * (k * k * k - k) / 144.0 / (k - 1));
}

double MODIFICATOR CochraneG (double *dat,long n,long k)
//
// The function for computation of Cochrane G-test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - size of each of vector.
// Return code:
//  test statistic.
// References:
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//  Siegel S., Castellan N.J. Nonparametric statistics for the behavioral sciences. - New York, NY: McGraw Hill, 1988.
//
{
 long i,j,m = 0; // Counters

 double *x,      // Work array
        *s,      // Array of sample variances
        ss = 0;  // Sum of variances

 x = new double[n];
 s = new double[k];

 for (i = 0; i < k; i++)
   {
    for (j = 0; j < n; j++)
      x[j] = dat[m++];
    s[i] = Variance (x,n);
    ss += s[i];
   }

 ss = MaximumOf (s,k) / ss;

 delete [] x; delete [] s;

 return ss;
}

double MODIFICATOR CochraneQ (double *dat,long n,long k)
//
// The function for computation of Cochrane Q-test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - size of each of vector.
// Return code:
//  test statistic.
// References:
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//  Brownlee K.A. Statistical theory and metodology in sciences and engineering. - New York, NY: John Wiley & Sons, 1965.
//  Siegel S., Castellan N.J. Nonparametric statistics for the behavioral sciences. - New York, NY: McGraw Hill, 1988.
//
{
 long i,j,m = 0; // Counter

 double *x,*t,   // Work array
        s1 = 0,  // Sum of summary values by columns
        s2 = 0,  // Sum of squares s1 by columns
        s3 = 0,  // Sum of summary values
        s4 = 0;  // Sum of squares s3

 x = new double[MAX(n,k)];
 t = new double[MAX(n,k)];

 for (i = 0; i < k; i++)
   {
    for (j = 0, t[i] = 0; j < n; j++)
      {
       x[j] = dat[m++];
       t[i] += x[j];
      }
    s1 += t[i];
    s2 += t[i] * t[i];
   }

 for (i = 0; i < n; i++)
   {
    for (j = 0, m = i, t[i] = 0; j < k; j++)
      {
       x[j] = dat[m];
       m += n;
       t[i] += x[j];
      }
    s3 += t[i];
    s4 += t[i] * t[i];
   }

 delete [] x; delete [] t;

 return fabs ((s2 * k  - s1 * s1) * (k - 1) / (s3 * k - s4));
}

double MODIFICATOR Levene (double *dat,long *n,long k)
//
// The function for computation Levene test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - array of size k vectors.
// Return code:
//  test statistic.
//
{
 long i,j,l; // Counter

 double *x,  // Work array
        *m,  // Vector medium
        *z,  // Matrix of absolute deviations
        ret; // Return code

 x = new double[MaximumOf (n,k)];
 z = new double[VectorSum (n,k)];
 m = new double[k];

 /////////////////////
 // Computation medium
 for (i = 0, l = 0; i < k; i++)
   {
    for (j = 0; j < n[i]; j++)
      x[j] = dat[l++];
    m[i] = VectorSum (x,n[i]) / n[i];
   }

 ///////////////////////////////////////////////
 // Computation of matrix of absolute deviations
 for (i = 0, l = 0; i < k; i++)
   for (j = 0; j < n[i]; j++, l++)
     z[l] = fabs(dat[l] - m[i]);

 ////////
 // ANOVA
 ret = ANOVA (z,n,k);

 delete [] x; delete [] z; delete [] m;

 return ret;
}

double MODIFICATOR Brown(double *dat,long *n,long k)
//
// The function for computation of Brown-Forsythe test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - array of size k vectors.
// Return code:
//  test statistic.
//
{
 long i,j,l;   // Counter

 double *x,    // Work array
        *m,    // Vector medium
        *z,    // Matrix of absolute deviations
        ret,   // Return code
        lo,up; // Dummy

 x = new double[MaximumOf (n,k)];
 z = new double[VectorSum (n,k)];
 m = new double[k];

 /////////////////////
 // Computation medium
 for (i = 0, l = 0; i < k; i++)
   {
    for (j = 0; j < n[i]; j++)
      x[j] = dat[l++];
    Median(x,n[i],0.95,&ret,&lo,&up);
    m[i] = ret;
   }

 ///////////////////////////////////////////////
 // Computation of matrix of absolute deviations
 for (i = 0, l = 0; i < k; i++)
   for (j = 0; j < n[i]; j++, l++)
     z[l] = fabs(dat[l] - m[i]);

 ////////
 // ANOVA
 ret = ANOVA (z,n,k);

 delete [] x; delete [] z; delete [] m;

 return ret;
}

long MODIFICATOR Scheffe(double dat[],long n[],long k,double *Rs)
//
// The function for Scheffe analysis of independent samples.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - array of size k vectors,
//  *Rs - array of test statistics by length (k * k - k) / 2.
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
// References:
//  Scheffe H. The Analysis of Variance. - New York, NY: John Wiley & Sons, 1999.
//  Brownlee K.A. Statistical theory and metodology in sciences and engineering. - New York, NY: John Wiley & Sons, 1965.
//
{
 long i,j,m = 0,            // Counter
      nn = VectorSum (n,k); // Common number of samle members

 double *x,*t,              // Work array
        s = 0;              // Average quadratic remainder

 x = new double[MaximumOf (n,k)];
 t = new double[k];

 ///////////////
 // Array medium
 for (i = 0; i < k; i++)
   {
    for (j = 0, t[i] = 0; j < n[i]; j++)
      {
       x[j] = dat[m++];
       t[i] += x[j];
      }
    t[i] /= n[i];
   }

 //////////////////////////////////
 // The average quadratic remainder
 for (i = 0, m = 0; i < k; i++)
   for (j = 0; j < n[i]; j++, m++)
      s += (dat[m] - t[i]) * (dat[m] - t[i]);
 if (s < 0.000001)
   {
    delete [] t; delete [] x;
    return -2;
   }
 s /= (nn - k);

 ////////////////////////////////
 // Array of pairwise comparisons
 for (i = 0, m = 0; i < k - 1; i++)
   for (j = i + 1; j < k; j++)
     Rs[m++] = fabs (t[i] - t[j]) / sqrt (s * (1.0 / n[i] + 1.0 / n[j]));

 delete [] t; delete [] x;

 return 0;
}

long MODIFICATOR Dunn(double dat[],long n[],long k,double *Rg,double *Er,double *Rs)
//
// The function for computation Dunn test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - array of size k vectors,
//  *Rg - array of ranks difference by length (k * k - k) / 2,
//  *Er - array of standard errors by length (k * k - k) / 2,
//  *Rs - array of criterial statistics by length (k * k - k) / 2.
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//  Daniel W.W. Applied nonparametric statistics. - Florence, KY: Wadsworth Publishing, 1990.
//  Siegel S., Castellan N.J. Nonparametric statistics for the behavioral sciences. - New York, NY: McGraw Hill, 1988.
//  Hollander M., Wolfe D.A., Chiken E. Nonparametric statistical methods. - New York, NY: Wley, 2014.
//
{
 long i,j,m = 0,            // Counter
      nn = VectorSum (n,k), // Common number of samle members
      *ix;                  // Information array

 double *x,*t;              // Work array

 x = new double[nn];
 t = new double[nn];
 ix = new long[nn];

 ///////////////////////
 // Copy of given arrays
 for (i = 0; i < nn; i++)
   x[i] = dat[i];

 //////////
 // Sort up
 SortArrayUp (x,nn,ix);

 //////////
 // Ranking
 Rank (x,t,nn);

 //////////////////////////////////
 // Restoring ranks of given arrays
 UnSortArray (t,x,nn,ix);

 ////////////////////////
 // Array medium of ranks
 for (i = 0; i < k; i++)
   {
    for (j = 0, t[i] = 0; j < n[i]; j++)
      t[i] += x[m++];
    t[i] /= n[i];
   }

 ////////////////////////////////
 // Array of pairwise comparisons
 for (i = 0, m = 0; i < k - 1; i++)
   for (j = i + 1; j < k; j++,m++)
     {
      Rg[m] = fabs (t[i] - t[j]);
      Er[m] = sqrt ((1.0 / n[i] + 1.0 / n[j]) * nn * (nn + 1) / 12);
      Rs[m] = Rg[m] / Er[m];
     }

 delete [] t; delete [] x; delete [] ix;

 return 0;
}

long MODIFICATOR Conover (double dat[],long n[],long k,double *Rg,double *Rs,long ti)
//
// The function for computation of the Conover post hoc test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - array of size k vectors,
//  *Rg - array of ranks differences by length (k * k - k) / 2.
//  *Rs - array of criterial statistics by length (k * k - k) / 2,
//   ti - flag of bundle adjustment (0 - no adjustment).
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
// References:
//  Conover W.J. Practical nonparametric statistics. - New York, NY: John Wiley & Sons, 1999.
//  Sprent P., Smeeton N.C. Applied nonparametric statistical methods. - London, UK: Chapman & Hall / CRC, 2005.
//  Bortz J., Lienert G.A., Boehnke K. Verteilungsfreie methoden in der biostatistik (in Deutsch). - Heidelberg: Springer Medizin Verlag, 2008.
//
{
 long i,j,m = 0,                        // Counter
      nn = VectorSum (n,k),             // Common number of samle members
      *ix;                              // Information array

 double *x,*t,                          // Work array
        h = KruskalWallis (dat,n,k,ti); // Kruskal-Wallis statistic

 x = new double[nn];
 t = new double[nn];
 ix = new long[nn];

 ///////////////////////
 // Copy of given arrays
 for (i = 0; i < nn; i++)
   x[i] = dat[i];

 //////////
 // Sort up
 SortArrayUp (x,nn,ix);

 //////////
 // Ranking
 Rank (x,t,nn);

 /////////////////////////////////
 // Restoring ranks of given array
 UnSortArray (t,x,nn,ix);

 ////////////////////////
 // Array of ranks medium
 for (i = 0; i < k; i++)
   {
    for (j = 0, t[i] = 0; j < n[i]; j++)
      t[i] += x[m++];
    t[i] /= n[i];
   }

 ////////////////////////////////
 // Array of pairwise comparisons
 for (i = 0, m = 0; i < k - 1; i++)
   for (j = i + 1; j < k; j++,m++)
     {
      Rg[m] = ABS (t[i] - t[j]);
      Rs[m] = Rg[m] / sqrt ((1.0 / n[i] + 1.0 / n[j]) * nn * (nn + 1) / 12 * (nn - 1 - h) / (nn - k));
     }

 delete [] t; delete [] x; delete [] ix;

 return 0;
}

long MODIFICATOR Tukey (double dat[],long n,long k,double *Rs)
//
// The function for computation Tukey test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - size of each of vector,
//  *Rs - array of criterial statistic by length (k * k - k) / 2.
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//  Johnson N.L., Leone F.C. Statistics and experimental design in engineering and the physical sciences. Vol. I. - New York, NY: John Wiley & Sons, 1977.
//
{
 long i,j,m = 0,        // Counter
      nn = k * (n - 1); // Work variable

 double *x,*t,          // Work array
        s = 0,          // Full variance
        s1 = 0,         // Variance between samples, remains variance
        tt = 0;         // Work variable

 x = new double[n];
 t = new double[k];

 for (i = 0; i < k; i++)
   {
    for (j = 0, t[i] = 0; j < n; j++)
      {
       x[j] = dat[m++];
       t[i] += x[j];
       s += x[j] * x[j];
      }
    s1 += t[i] * t[i] / n;
    tt += t[i];
   }

 tt *= tt;
 s -= tt / nn;
 s1 -= tt / nn;

 ///////////////////
 // Remains variance
 s -= s1;

 if (s < 0.000001)
   {
    delete [] t; delete [] x;
    return -2;
   }

 ///////////////
 // Array medium
 for (i = 0; i < k; i++)
   t[i] /= n;

 ////////////////////////////////
 // Array of pairwise comparisons
 for (i = 0, m = 0; i < k - 1; i++)
   for (j = i + 1; j < k; j++)
     Rs[m++] = fabs (t[i] - t[j]) / sqrt (s / n);

 delete [] t; delete [] x;

 return 0;
}

long MODIFICATOR Duncan (double dat[],long n[],long k,double *Rs,long *r)
//
// The function for computation Duncan test.
// Designations:
//  dat - array k vectors of data (of vector one by one),
//    n - array of size k vectors,
//  *Rs - array of critrial statiscs by length (k * k - k) / 2,
//   *r - array work parameters by length (k * k - k) / 2.
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
// References:
//  Owen D.B. Handbook of statistical tables. - Reading, MA: Addison-Wesley, 1962.
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//
{
 long i,j,m = 0,            // Counter
      nn = VectorSum (n,k), // Common number of samle members
      *ix;                  // Information array

 double *x,*t,              // Work array
        s = 0,              // Full variance
        s1 = 0,             // Variance between samples, remains variance
        tt = 0;             // Work variable

 x = new double[MaximumOf (n,k)];
 t = new double[k];
 ix = new long[k];

 for (i = 0; i < k; i++)
   {
    for (j = 0, t[i] = 0; j < n[i]; j++)
      {
       x[j] = dat[m++];
       t[i] += x[j];
       s += x[j] * x[j];
      }
    s1 += t[i] * t[i] / n[i];
    tt += t[i];
   }

 tt *= tt;
 s -= tt / nn;
 s1 -= tt / nn;

 ///////////////////
 // Remains variance
 s -= s1;

 if (s < 0.000001)
   {
    delete [] t; delete [] x; delete [] ix;
    return -2;
   }

 s = s / (nn - k) / 2;

 ///////////////
 // Array medium
 for (i = 0; i < k; i++)
   t[i] /= n[i];

 ////////////////////////////////
 // Array of pairwise comparisons
 for (i = 0, m = 0; i < k - 1; i++)
   for (j = i + 1; j < k; j++)
     Rs[m++] = fabs (t[i] - t[j]) / sqrt (s * (1.0 / n[i] + 1.0 / n[j]));

 ///////////////////////////
 // Array of work parameters
 SortArrayUp (t,k,ix);
 for (i = 0, m = 0; i < k - 1; i++)
   for (j = i + 1; j < k; j++)
     r[m++] = ABS(ix[i] - ix[j]);

 delete [] t; delete [] x; delete [] ix;

 return 0;
}

long MODIFICATOR ScheffePair (double dat[],long n,long k,double *Row,double *Col)
//
// The function for Scheffe analysis of related samples.
// Designations:
//   dat - array k vectors of data (of vector one by one),
//     n - size of each of vector,
//  *Row - effect rows by length (n * n - n) / 2,
//  *Col - effect columns by length (k * k - k) / 2.
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
// References:
//  Scheffe H. The Analysis of Variance. - New York, NY: John Wiley & Sons, 1999.
//  Brownlee K.A. Statistical theory and metodology in sciences and engineering. - New York, NY: John Wiley & Sons, 1965.
//
{
 long i,j,m,      // Counter
      nn = n * k; // Common number of samle members

 double *x,       // Work array
        *t,       // Work array, later mediums
        *t1,      // Work array, later mediums by columns
        s = 0,    // Full variance
        sr = 0,   // Variance between of rows
        sc = 0,   // Variance between of columns
        tt = 0;   // Work variable

 x = new double[MAX(n,k)];
 t = new double[k];
 t1 = new double[n];

 //////////////////
 // Effects of rows
 for (i = 0; i < k; i++)
   t[i] = 0;
 for (i = 0, m = 0; i < k; i++)
   {
    for (j = 0; j < n; j++)
      {
       x[j] = dat[m++];
       t[i] += x[j];
       s += x[j] * x[j];
      }
    sr += t[i] * t[i] / n;
    tt += t[i];
   }

 tt *= tt;
 s -= tt / nn;
 sr -= tt / nn;

 /////////////////////
 // Effects of columns
 for (i = 0; i < n; i++)
   t1[i] = 0;
 for (i = 0, tt = 0; i < n; i++)
   {
    for (j = 0, m = i, t1[i] = 0; j < k; j++, m += n)
      {
       x[j] = dat[m];
       t1[i] += x[j];
      }
    sc += t1[i] * t1[i] / k;
    tt += t1[i];
   }

 tt *= tt;
 sc -= tt / nn;

 ///////////////////
 // Remains variance
 s = (s - sr - sc) / (k - 1) / (n - 1);
 if (s < 0.000001)
   {
    delete [] t; delete [] t1; delete [] x;
    return -2;
   }

 for (i = 0; i < k; i++)
   t[i] /= n;
 for (i = 0; i < n; i++)
   t1[i] /= k;

 for (i = 0, m = 0; i < n - 1; i++)
   for (j = i + 1; j < n; j++)
     Row[m++] = (t1[i] - t1[j]) * (t1[i] - t1[j]) * k / (n - 1) / s / 2;

 for (i = 0, m = 0; i < k - 1; i++)
   for (j = i + 1; j < k; j++)
     Col[m++] = (t[i] - t[j]) * (t[i] - t[j]) * n / (k - 1) / s / 2;

 delete [] t; delete [] t1; delete [] x;

 return 0;
}

long MODIFICATOR MANOVA (double *x,long r, long c,double *srow,double *scol,double *sr,double *sc,double *s)
//
// The function for MANOVA.
// Designations:
//      x - array c vectors of data (of vector one by one),
//      r - size of each of vector (number of rows),
//      c - number vectors of data (number of columns),
//  *srow - effect of rows,
//  *scol - effect of columns,
//    *sr - middle square of rows,
//    *sc - middle square of columns,
//     *s - error rate.
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
// References:
//  Scheffe H. The Analysis of Variance. - New York, NY: John Wiley & Sons, 1999.
//  Brandt S. Data analysis. Statistical and computational methods for scientists and engineers. - New York, NY: Springer-Verlag, 2003.
//  Maindonald J.H. Statistical computation. - New York, NY: John Wiley & Sons, 1984.
//
{
 long i,j,m,       // Counter
      rc  = r * c; // Common number of samle members

 double *ti,       // Sums of rows
        *tj,       // Sums of columns
        tt = 0;    // Work variable

 ////////////////////////////////////////
 // Common sum and residual middle square
 for (i = 0, m = 0,*s = 0; i < r; i++)
   for (j = 0; j < c; j++, m++)
     {
      tt += x[m];
      *s += x[m] * x[m];
     }
 tt = tt * tt / rc;
 *s -= tt;
 if (*s == 0)
   return -2;

 ti = new double[r];
 tj = new double[c];

 ///////////////
 // Sums of rows
 for (i = 0; i < r; i++)
   for (j = 0, m = i, ti[i] = 0; j < c; j++, m += r)
     ti[i] += x[m];

 //////////////////
 // Sums of columns
 for (i = 0, m = 0; i < c; i++)
   for (j = 0, tj[i] = 0; j < r; j++)
     tj[i] += x[m++];

 ///////////////////////////
 // Variance between of rows
 for (i = 0,*sr = 0; i < r; i++)
   *sr += ti[i] * ti[i];
 *sr /= c;
 *sr -= tt;

 //////////////////////////////
 // Variance between of columns
 for (i = 0,*sc = 0; i < c; i++)
   *sc += tj[i] * tj[i];
 *sc /= r;
 *sc -= tt;

 ///////////////////
 // Remains variance
 *s = *s - *sc - *sr;

 *sr /= (r - 1);
 *sc /= (c - 1);
 *s /= (r - 1) * (c - 1);

 *srow = *sr / (*s);
 *scol = *sc / (*s);

 delete [] ti; delete [] tj;

 return 0;
}

double MannWhitneyTest(double *x,double *y,long nx,long ny)
//
// The function for computation Mann-Whitney U-test.
// Designations:
//    x - 1st sample,
//   nx - size of 1st sample,
//    y - 2nd sample,
//   ny - size of 2nd sample.
// Return code:
//  value of test.
// References:
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//
{
 long i,               // Counter
      *ix,             // Permutation info
      n = nx + ny;     // Common size

 double *xx,*xy,       // Work array
        xn = 0,yn = 0, // Sums of ranks
        t;             // Value of test

 ix = new long[n];
 xy = new double[n];
 xx = new double[n];

 ////////////////////////
 // Given arrays junction
 for (i = 0; i < nx; i++)
   xy[i] = x[i];
 for (i = 0; i < ny; i++)
   xy[i + nx] = y[i];

 //////////
 // Sort up
 SortArrayUp (xy,n,ix);

 //////////
 // Ranking
 Rank (xy,xx,n);

 //////////////////////
 // Restore ranks order
 UnSortArray (xx,xy,n,ix);

 delete [] ix; delete [] xx;

 ////////////////////////////
 // Sums computation of ranks
 for (i = 0; i < nx; i++)
   xn += xy[i];
 xn = 0.5 * nx * (nx + 1) - xn + nx * ny;
 for (i = 0; i < ny; i++)
   yn += xy[i + nx];
 yn = 0.5 * ny * (ny + 1) - yn + nx * ny;

 delete [] xy;

 /////////////////////////////
 // Test statistic computation
 t = xn >= yn ? xn : yn;

 return t;
}

double MODIFICATOR Bhapkar (double *x,long *n,long k)
//
// The function for computation of Bhapkar test.
// Designations:
//  x - given of sample (k samples one by one),
//  n - array size k samples,
//  k - number of samples.
// Return code:
//  value of test.
// References:
//  Bhapkar V.P. A note on the equivalence of two test criteria for hypotheses in categorical data // Journal of the American Statistical Association, 1966, vol. 61, pp. 228-235.
//  Bhapkar V.P., Gore A.P. A distribution-free test for symmetry in hierarchical data // Journal of Multivariate Analysis, 1973, vol. 3, pp. 483-489.
//
{
 long i,l,m,      // Counters of cycles
      *j,         // Array of counters samples
      ss = n[0],  // Common size of generated arrays
      p[1] = {0}, // Counter for use in recursion
      r = 0,      // Array counter
      fl,         // Flag
      *s;         // Array of "begins" in common arrays of data

 double *f,       // Work array of combinations
        *ff,      // Full Array of combinations
        *v,       // Numbers of less samples
        s1 = 0,   // Return code
        s2 = 0;   // Work variable

 j = new long [k];
 s = new long [k];
 f = new double [k];

 for (i = 1; i < k; i++)
   {
    s[i] = s[i - 1] + n[i - 1];
    ss *= n[i];
   }
 ff = new double[ss * k];
 v = new double [k];
 for (i = 0; i < k; i++)
   v[i] = 0;

 ////////////////////////////
 // Generation arrays samples
 Generate (j,k,n,f,x,s,0,ff,p);

 ///////////////////
 // Work computation
 for (i = 0; i < ss; i++)
   {
    for (l = 0; l < k; l++)
      f[l] = ff[r++];

    for (m = 0; m < k; m++)
      {
       fl = 0;
       for (l = 0; l < k; l++)
         if (f[m] > f[l])
           {
            fl = 1;
            break;
           }
        if (!fl)
          v[m]++;
       }
   }

 for (m = 0; m < k; m++)
   {
    v[m] /= ss;
    s1 += (v[m] - 1.0 / k) * (v[m] - 1.0 / k) * n[m] / ss;
    s1 += (v[m] - 1.0 / k) * n[m] / ss;
   }

 delete [] j; delete [] s; delete [] f; delete [] ff; delete [] v;

 return (s1 - s2 * s2) * ss * (2 * k - 1);
}

double MODIFICATOR Deshpande (double *x,long *n,long k)
//
// The function for Deshpande test.
// Designations:
//  x - given of sample (k samples one by one),
//  n - array size k samples,
//  k - number of samples.
// Return code:
//  value of test.
// References:
//  Deshpande, J.V. A class of tests for exponentiality against increasing failure rate average alternatives // Biometrika, 1983, vol. 70, pp. 514-518.
//
{
 long i,l,m,      // Counters of cycles
      *j,         // Array of counters samples
      ss = n[0],  // Common size of generated array
      p[1] = {0}, // Counter for use in recursion
      r = 0,      // Array counter
      fl,         // Flag
      *s;         // Array of "begins" in common arrays of data

 double *f,       // Work array of combinations
        *ff,      // Full Array of combinations
        *v,       // Numbers of less samples
        *u,       // Numbers of large samples
        s1 = 0,   // Return code
        s2 = 0,   // Work variable
        s3;       // Work variable

 j = new long [k];
 s = new long [k];
 f = new double [k];

 s[0] = 0;
 for (i = 1; i < k; i++)
   {
    s[i] = s[i - 1] + n[i - 1];
    ss *= n[i];
   }
 ff = new double[ss * k];
 v = new double [k];
 u = new double [k];
 for (i = 0; i < k; i++)
   {
    v[i] = 0;
    u[i] = 0;
   }

 ////////////////////////////
 // Generation arrays samples
 Generate (j,k,n,f,x,s,0,ff,p);

 //////////////////////
 // Work computation
 for (i = 0; i < ss; i++)
   {
    for (l = 0; l < k; l++)
      f[l] = ff[r++];

    for (m = 0; m < k; m++)
      {
       fl = 0;
       for (l = 0; l < k; l++)
         if (f[m] > f[l])
           {
            fl = 1;
            break;
           }
        if (!fl)
          v[m]++;
       }

    for (m = 0; m < k; m++)
      {
       fl = 0;
       for (l = 0; l < k; l++)
         if (f[m] < f[l])
           {
            fl = 1;
            break;
           }
        if (!fl)
          u[m]++;
       }
   }

 for (m = 0; m < k; m++)
   {
    v[m] = (v[m] + u[m]) / ss;
    s1 += v[m] * v[m] * n[m] / ss;
    s2 += v[m] * n[m] / ss;
   }

 s3 = exp(LogFactorial(2 * (k - 1)) - LogFactorial(k - 1) - LogFactorial(k - 1));

 delete [] j; delete [] s; delete [] f; delete [] ff; delete [] v; delete [] u;

 return (s1 - s2 * s2) * ss * (2 * k - 1) / 2 * (k - 1) * (k - 1) / (s3 * (k * k + 4 * k + 2) + k * k) * s3;
}

double MODIFICATOR BhapkarDeshpande (double *x,long *n,long k)
//
// The function for computation of Bhapkar-Deshpande L-test.
// Designations:
//  x - given of sample (k samples one by one),
//  n - array size k samples,
//  k - number of samples.
// Return code:
//  value of test.
// References:
//  Bhapkar V. P. A nonparametric test for the several sample location problem // University of North Carolina, Institute of Statistics, Mimeo series, No. 411, 1964.
//
{
 long i,l,m,      // Counters of cycles
      *j,         // Array of counters samples
      ss = n[0],  // Common size of generated array
      p[1] = {0}, // Counter for use in recursion
      r = 0,      // Array counter
      fl,         // Flag
      *s;         // Array of "begins" in common arrays of data

 double *f,       // Work array of combinations
        *ff,      // Full Array of combinations
        *v,       // Numbers of less samples
        *u,       // Numbers of large samples
        s1 = 0,   // Return code
        s2 = 0,   // Work variable
        s3;       // Work variable

 j = new long [k];
 s = new long [k];
 f = new double [k];
 s[0] = 0;
 for (i = 1; i < k; i++)
   {
    s[i] = s[i - 1] + n[i - 1];
    ss *= n[i];
   }
 ff = new double[ss * k];
 v = new double [k];
 u = new double [k];
 for (i = 0; i < k; i++)
   {
    v[i] = 0;
    u[i] = 0;
   }

 ////////////////////////////
 // Generation arrays samples
 Generate (j,k,n,f,x,s,0,ff,p);

 ///////////////////
 // Work computation
 for (i = 0; i < ss; i++)
   {
    for (l = 0; l < k; l++)
      f[l] = ff[r++];

    for (m = 0; m < k; m++)
      {
       fl = 0;
       for (l = 0; l < k; l++)
         if (f[m] > f[l])
           {
            fl = 1;
            break;
           }
        if (!fl)
          v[m]++;
       }

    for (m = 0; m < k; m++)
      {
       fl = 0;
       for (l = 0; l < k; l++)
         if (f[m] < f[l])
           {
            fl = 1;
            break;
           }
        if (!fl)
          u[m]++;
       }
   }

 for (m = 0; m < k; m++)
   {
    v[m] = (-v[m] + u[m]) / ss;
    s1 += v[m] * v[m] * n[m] / ss;
    s2 += v[m] * n[m] / ss;
   }

 s3 = exp(LogFactorial(2 * (k - 1)) - LogFactorial(k - 1) - LogFactorial(k - 1));

 delete [] j; delete [] s; delete [] f; delete [] ff; delete [] v; delete [] u;

 return (s1 - s2 * s2) * ss * (2 * k - 1) / 2 * (k - 1) * (k - 1) / (s3 - 1) * s3 / k / k;
}

void MODIFICATOR Generate (long *j,long k,long *n,double *f,double *x,long *s,long z,double *ff,long *p)
//
// The function of generation arrays samples for Bhapkar-Deshpande test.
// Designations:
//   j - array of counters samples,
//   k - number of samples,
//   n - array size k samples,
//   f - work array of combinations,
//   x - given of sample (k samples one by one),
//   s - array of "begins" of samples in common arrays of data,
//   z - indicator of end of recursion,
//  ff - full Array of combinations,
//   p - counter.
// Return code:
//  none.
//
{
 long i;   // Counter

 double a; // Indicator of recursion end

 for (j[z] = 0; j[z] < n[z]; j[z]++)
   {
    f[z] = x[j[z] + s[z]];
    if ((a = z + 1) < k)
      Generate (j,k,n,f,x,s,a,ff,p);
    else
      for (i = 0; i < k; i++)
        ff[(p[0])++] = f[i];
   }
}

double MODIFICATOR Variance (double *x,long n)
//
// The function for computation unbiased estimation of variance.
// Designations:
//   x - array,
//   n - size arrays,
// Return code:
//  value of unbiased estimation of variance.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//
{
 long i;       // Counter

 double m = 0, // Mathematical expectation
        s = 0; // Variance

 for (i = 0; i < n; i++)
   {
    m += x[i];
    s += x[i] * x[i];
   }
 m /= n;

 return (s - n * m * m) / (n - 1);
}

long MODIFICATOR McQueen (double data[],long n,long k,long m,long Mode,long inf[],double c[])
//
// The function builds cluster analysis by k-medium McQueen method.
// Designations:
//  data - matrix of given data by size n * k (k n-dimensional vectors
//         one by one),
//     n - number of objecs for classification,
//     k - number of parameters,
//     m - number of clusters,
//  Mode - likeness mode:
//       =  0 - Euclid distance,
//       =  1 - Manhattan distance,
//       =  2 - Mahalanobis distance,
//       =  3 - supremum-norma,
//       =  4 - distance Pearson,
//       =  5 - distance Spearman,
//       =  6 - distance Kendall,
//       =  7 - distance Jaccard,
//       =  8 - distance Russel-Rao,
//       =  9 - distance Bravais,
//       = 10 - distance Yule,
//       = 11 - partial order relation.
//   inf - on exit - numbers array by length n with information
//         about the belonging of objects to a particular cluster,
//     c - array of centers of gravity of clusters of size m * k
//         (k m-dimansional vectors one by one),
//  *sum - the function of classification quality.
// Return code:
//   0 - when normal end of computation,
//  -1 - when lack of memory,
//  -2 - when loss of significance.
// References:
//  Fukunaga K. Intruduction to statistical pattern recognition. - New York, NY: Academic Press, 1972.
//  Aldenderfer M.S., Blashfield R.K. Cluster analysis. - Beverly Hills, CA: SAGE Publications, 1985.
//
{
 long i,j,l,       // Counter
       m1,m2,      // Array counter
       Count,      // Counter of cluster members
       Code;       // Error code

 double *z,        // Extended of likeness matrix
        *dat,      // Extended of data matrix
        *t,        // Inverse dispersion-covariation matrix
        Crit1 = 0, // The quality function
        Crit;      // The quality function in the previous step

 ///////////////////////////////////////////////////
 // Building inverse dispersion-cavariance of matrix
 // for computation Mahalanobis distance
 if (Mode == 2)
   {
    /////////////////////////////////////
    // For memory economy array t defined
    // in Mode = 2, in other case - dummy array
    if ((t = new double[k * k]) == 0)
      return -1;

    /////////////////////////////////////
    // Array z temporary used for keeping
    // dispersion-covariation matrix
    if ((z = new double[k * k]) == 0)
      {
       delete [] t;
       return -1;
      }

    /////////////////////////////////////////
    // Building dispersion-covariation matrix
    Code = CovMatrix (data,z,n,k);

    if (Code != 0)
      {
       delete [] t; delete [] z;
       return Code;
      }

    for (i = 0; i < k * k; i++)
      z[i] = fabs (z[i]);

    ////////////////////////////////////////
    // Inverse dispersion-covariation matrix
    Code = InverseMatrix (z,t,k);

    delete [] z;

    if (Code != 0)
      {
       delete [] t;
       return Code;
      }
   }
 else
   if ((t = new double[1]) == 0)
     return -1;

 if ((z = new double[(n + m) * (n + m)]) == 0)
   {
    delete [] t;
    return -1;
   }
 if ((dat = new double[(n + m) * k]) == 0)
   {
    delete [] t; delete [] z;
    return -1;
   }

 /////////////////////////////////////////
 // The 1st n rows of extended data matrix
 // match the given data matrix
 for (i = 0, m2 = 0; i < k; i++)
   for (j = 0, m1 = i * (n + m); j < n; j++)
     dat[m1++] = data[m2++];

 /////////////////////////////////////////////////
 // The others m rows - cluster centers of gravity
 // Let's take them coinciding with the 1st m objects
 for (i = 0; i < k; i++)
   for (j = 0, m1 = i * (n + m) + n, m2 = i * n; j < m; j++)
     dat[m1++] = data[m2++];

 do
   {
    Crit = Crit1;

    ////////////////////////////////////
    // Building extended likeness matrix
    if ((Code = LikenessMatrix (dat,z,t,n + m,k,Mode)) != 0)
      {
       delete [] t; delete [] z; delete [] dat;
       return Code;
      }

    ////////////////////////////////////////////////////////////////
    // Analysis of the likeness matrix for the separation of objects
    // by clusters. It uses the fact that the last m of rows
    // (or m of columns) of the extended likeness matrix is
    // the distances between objects and the centers of gravity of clusters
    for (i = 0, m1 = n * (m + n); i < n; i++, m1++)
      for (j = 1, m2 = m1 + m + n, inf[i] = 0; j < m; j++, m2 += m + n)
    if (z[m2] < z[m1])
      inf[i] = j;

    ////////////////////////////////////////////////////
    // Computation of the centers of gravity of clusters
    for (i = 0; i < k; i++)
      for (j = 0, m1 = i * (n + m) + n; j < m; j++, m1++)
        {
         //////////////////////////////////////////////
         // Selecting objects included in the cluster j
         for (l = 0, Count = 0, dat[m1] = 0, m2 = i * n; l < n; l++, m2++)
           if (j == inf[l])
             {
              dat[m1] += data[m2];
              Count++;
             }
         /////////////////////////
         // If cluster is no empty
         if (Count != 0)
           dat[m1] /= Count;
        }

    /////////////////////////////////////////////////
    // Computation quality function of classification
    for (i = 0, Crit1 = 0; i < k; i++)
      for (j = 0, m1 = i * (n + m) + n; j < m; j++,m1++)
        for (l = 0, m2 = i * n; l < n; l++, m2++)
          if (j == inf[l])
            Crit1 += fabs (dat[m1] - data[m2]);

   } while (Crit1 != Crit);

 /////////////////////////////////
 // Centers of gravity of clusters
 for (i = 0, m1 = 0; i < k; i++)
   for (j = 0, m2 = i * (n + m) + n; j < m; j++)
     c[m1++] = dat[m2++];

 delete [] z; delete [] dat; delete [] t;

 return Code;
}

long MODIFICATOR LikenessMatrix (double data[],double z[],double s[],long n,long k,long Mode)
//
// The function build likeness matrix for cluster analysis.
// Designations:
//  data - array of given data about objects (k vectors one by one),
//  z    - calculated likeness array by size n x n,
//  s    - before calculated inverse dispersion-covariation
//         matrix (only for computation of Mahalanobis distance),
//  n    - number of classifed objects,
//  k    - number of parameters, described each object,
//  Mode - likeness mode:
//       =  0 - Euclid distance,
//       =  1 - Manhattan distance,
//       =  2 - Mahalanobis distance,
//       =  3 - supremum-norma,
//       =  4 - distance Pearson,
//       =  5 - distance Spearman,
//       =  6 - distance Kendall,
//       =  7 - distance Jaccard,
//       =  8 - distance Russel-Rao,
//       =  9 - distance Bravais,
//       = 10 - distance Yule,
//       = 11 - partial order relation.
// Return code:
//   0 - when normal end of computation,
//  -1 - when lack of memory.
// References:
//  Aldenderfer M.S., Blashfield R.K Cluster analysis. - Beverly Hills, CA: SAGE Publications, 1985.
//
{
 long i,j,l,m,m1,m2; // Counters

 double *dat1,*dat2, // Copy vectors of data
        temp;        // Work variable

 if ((dat1 = new double[k]) == 0)
   return -1;
 if ((dat2 = new double[k]) == 0)
   {
    delete [] dat1;
    return -1;
   }

 //////////////////////////////
 // Building of likeness matrix
 for (i = 0, m2 = 0; i < n; i++)
   for (j = 0; j < n; j++)
     {
      /////////////////////////////////////////////
      // Copy builds i in current cycle for restore
      // after destruction dat1 by Mahalanobis function
      for (l = 0, m = i; l < k; l++, m += n)
        dat1[l] = data[m];

      for (l = 0, m1 = j; l < k; l++, m1 += n)
        dat2[l] = data[m1];

      switch (Mode)
        {
         case 0:
           z[m2++] = Euclid (dat1,dat2,k);
           break;
         case 1:
           z[m2++] = CityBlock (dat1,dat2,k);
           break;
         case 2:
           z[m2++] = Mahalanobis (dat1,dat2,s,k);
           break;
         case 3 :
           z[m2++] = Sup (dat1,dat2,k);
           break;
           case 4:
           z[m2++] = 1 - Pearson (dat1,dat2,n);
           break;
         case 5:
           if (Spearman (dat1,dat2,n,&temp) == -1)
             {
              delete [] dat1; delete [] dat2;
              return -1;
             }
           z[m2++] = 1 - temp;
           break;
         case 6:
            if (Kendall (dat1,dat2,n,&temp) == -1)
              {
               delete [] dat1; delete [] dat2;
               return -1;
              }
           z[m2++] = 1 - temp;
           break;
         case 7:
           z[m2++] = 1 - Jaccard (dat1,dat2,k);
           break;
         case 8:
           z[m2++] = 1 - Rao (dat1,dat2,k);
           break;
         case 9:
           z[m2++] = 1 - Bravais (dat1,dat2,k);
           break;
         case 10:
           z[m2++] = 1 - Yule (dat1,dat2,k);
           break;
         default:
           z[m2++] = Likeness (dat1,dat2,k);
           break;
        }
     }

 delete [] dat1; delete [] dat2;

 return 0;
}

double MODIFICATOR Mahalanobis (double dat1[],double dat2[],double s[],long k)
//
// The function computes Mahalanobis distance.
// Designations:
//  dat1 - the 1st of the compared vectors of data,
//  dat2 - the 2nd of the compared vectors of data,
//  s    - before calculated inverse dispersion-covariation matrix,
//     k - size of each of vector of data and order of matrix s.
// Notice: arrays dat1 and dat2 destructed.
// Return code:
//   Mahalanobis distance value.
// References:
//  Aldenderfer M.S., Blashfield R.K Cluster analysis. - Beverly Hills, CA: SAGE Publications, 1985.
//
{
 long i,j, // Cycle counters
      m;   // Counter

 double r = 0; // Work variable

 for (i = 0; i < k; i++)
   dat1[i] -= dat2[i];

 for (i = 0, m = 0; i < k; i++)
   {
    dat2[i] = 0;
    for (j = 0; j < k; j++)
      dat2[i] += dat1[j] * s[m++];
   }

 for (i = 0; i < k; i++)
   r += dat2[i] * dat1[i];

 return sqrt (r);
}

double MODIFICATOR Sup (double x[],double y[],long n)
//
// The function computes of supremum-norma.
// Designations:
//  x - 1st array,
//  y - 2nd array,
//  n - size of each arrays.
// Return code:
//  value of metric.
// References:
//  Aldenderfer M.S., Blashfield R.K Cluster analysis. - Beverly Hills, CA: SAGE Publications, 1985.
//
{
 double s = 0,buf; // Work variable

 for (long i = 0; i < n; i++)
   {
    buf = fabs (x[i] - y[i]);
    if (buf > s)
      s = buf;
   }

 return s;
}

double MODIFICATOR Jaccard (double x[],double y[],long n)
//
// The function computes Jaccard similarity indicator.
// Designations:
//  x - 1st array,
//  y - 2nd array,
//  n - size of each arrays.
// Return code:
//  value of metric.
// References:
//  Aldenderfer M.S., Blashfield R.K Cluster analysis. - Beverly Hills, CA: SAGE Publications, 1985.
//
{
 double s,               // Work variable
        p11 = 0,p10 = 0, // Parameters
        p01 = 0,p00 = 0;

 for (long i = 0; i < n; i++)
   if (x[i] == 1 && y[i]  == 1)
     p11 += 1;
   else if (x[i] == 1 && y[i]  == 0)
     p10 += 1;
   else if (x[i] == 0 && y[i]  == 1)
     p01 += 1;
   else
     p00 += 1;

 s = p11 + p10 + p01;

 if (s == 0)
   return 0;
 else
   return p11 / s;
}

double Rao (double x[],double y[],long n)
//
// The function computes Russel-Rao similarity indicator.
// Designations:
//  x - 1st array,
//  y - 2nd array,
//  n - size of each arrays.
// Return code:
//  value of metric.
// References:
//  Aldenderfer M.S., Blashfield R.K Cluster analysis. - Beverly Hills, CA: SAGE Publications, 1985.
//
{
 double s,               // Work variable
        p11 = 0,p10 = 0, // Parameters
        p01 = 0,p00 = 0;

 for (long i = 0; i < n; i++)
   if (x[i] == 1 && y[i]  == 1)
     p11 += 1;
   else if (x[i] == 1 && y[i]  == 0)
     p10 += 1;
   else if (x[i] == 0 && y[i]  == 1)
     p01 += 1;
   else
     p00 += 1;

 s = p11 + p10 + p01 + p00;

 if (s == 0)
   return 0;
 else
   return p11 / s;
}

double MODIFICATOR Yule (double x[],double y[],long n)
//
// The function computes Yule similarity indicator.
// Designations:
//  x - 1st array,
//  y - 2nd array,
//  n - size of each arrays.
// Return code:
//  value of metric.
// References:
//  Aldenderfer M.S., Blashfield R.K Cluster analysis. - Beverly Hills, CA: SAGE Publications, 1985.
//
{
 double s,               // Work variable
        p11 = 0,p10 = 0, // Parameters
        p01 = 0,p00 = 0;

 for (long i = 0; i < n; i++)
   if (x[i] == 1 && y[i]  == 1)
     p11 += 1;
   else if (x[i] == 1 && y[i]  == 0)
     p10 += 1;
   else if (x[i] == 0 && y[i]  == 1)
     p01 += 1;
   else
     p00 += 1;

 s = p11 * p00 + p10 * p01;

 if (s == 0)
   return 0;
 else
   return (p11 * p00 - p10 * p01) / s;
}

double MODIFICATOR Bravais (double x[],double y[],long n)
//
// The function computes Bravais coefficient contingency.
// Designations:
//  x - 1st compared sample,
//  y - 2nd compared sample,
//  n - each sample size.
// Return code:
//  value of metric.
// References:
//  Aldenderfer M.S., Blashfield R.K. Cluster analysis. - Beverly Hills, CA: SAGE Publications, 1985.
//
{
 double s,               // Work variable
        p11 = 0,p10 = 0, // Parameters
        p01 = 0,p00 = 0;

 for (long i = 0; i < n; i++)
   if (x[i] == 1 && y[i]  == 1)
     p00 += 1;
   else if (x[i] == 0 && y[i]  == 1)
     p10 += 1;
   else if (x[i] == 1 && y[i]  == 0)
     p01 += 1;
   else
     p11 += 1;

 s = (p11 + p10) * (p11 + p01) * (p10 + p00) * (p01 + p00);

 if (s == 0)
   return 0;
 else
   return (p11 * p00 - p10 * p01) / sqrt (s);
}

double MODIFICATOR Tetra (double x[],double y[],long n)
//
// The function computes coefficient of tetrachoric correlation.
// Designations:
//  x - 1st compared sample,
//  y - 2nd compared sample,
//  n - each sample size.
// Return code:
//  value of metric.
// References:
//  Digby P.G.N. Approximating the tetrachoric correlation coefficient // Biometrics, 1983, vol. 39, pp. 753-757.
//  Edwards J.H., Edwards A.W.F. Approximating the tetrachoric correlation coefficient // Biometrics, 1984, vol. 40, p. 563.
//
{
 double a,               // Work variable
        p11 = 0,p10 = 0, // Parameters
        p01 = 0,p00 = 0;

 for (long i = 0; i < n; i++)
   if (x[i] == 1 && y[i]  == 1)
     p00 += 1;
   else if (x[i] == 0 && y[i]  == 1)
     p10 += 1;
   else if (x[i] == 1 && y[i]  == 0)
     p01 += 1;
   else
     p11 += 1;

 if (p01 == 0 || p10 == 0)
   return 1;

 if (p00 == 0 || p11 == 0)
   return -1;

 a = exp (M_PI * log (p00 / p01 * p11 / p10) / 4);

 return (a - 1) / (a + 1);
}

double MODIFICATOR TetraVariance (double x[],double y[],long n)
//
// The function computes variance of the coefficient of tetrachoric correlation.
// Designations:
//  x - 1st compared sample,
//  y - 2nd compared sample,
//  n - each sample size.
// Return code:
//  value of SE.
// References:
//  Bonett D.G., Price R.M. Inferential methods for the tetrachoric correlation coefficient // Journal of the educational and behavioral statistics, 2005, vol. 30, no.2, pp. 213-225.
//  Digby P.G.N. Approximating the tetrachoric correlation coefficient // Biometrics, 1983, vol. 39, pp. 753-757.
//  Edwards J.H., Edwards A.W.F. Approximating the tetrachoric correlation coefficient // Biometrics, 1984, vol. 40, p. 563.
//
{
 double a,               // Work variable
        p11 = 0,p10 = 0, // Parameters
        p01 = 0,p00 = 0;

 for (long i = 0; i < n; i++)
   if (x[i] == 1 && y[i]  == 1)
     p00 += 1;
   else if (x[i] == 0 && y[i]  == 1)
     p10 += 1;
   else if (x[i] == 1 && y[i]  == 0)
     p01 += 1;
   else
     p11 += 1;

 if (p01 == 0 || p10 == 0 || p00 == 0 || p11 == 0)
   return 0;

 a = exp (M_PI * log (p00 / p01 * p11 / p10) / 4);

 a = M_PI / 2 * a / (a + 1) / (a + 1);

 return a * a * (1.0 / p00 + 1.0 / p10 + 1.0 / p01 + 1.0 / p11);
}

long MODIFICATOR King (double data[],long n,long k,long Mode,long ph[],long pl[],double s[])
//
// The function performs cluster analysis by the King mean coupling method.
// Designations:
//   data - matrix of given data by size n * k
//          (k vectors by length n one by one),
//      n - number of objects for classification,
//      k - number of parameters,
//   Mode - likiness mode,
//  ph,pl - on exit - arrays numbers each by length (n - 1) with
//          information about grouping of objects,
//      s - the level of pairing [ph,pl].
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory,
//  -2 loss of significance.
// References:
//  Aldenderfer M.S., Blashfield R.K Cluster analysis. - Beverly Hills, CA: SAGE Publications, 1985.
//
{
 long i,j,l,     // Counter
      m,m1,      // Array counter
      r,         // Number of objects on current step
      Code = 0;  // Error code

 double *z,*dat, // Work matrix
        *t;      // Inverse dispersion-covariation matrix

 if ((t = new double[k * k]) == 0)
   return -1;

 /////////////////////////////////////////
 // Building dispersion-covariation matrix
 if (Mode == 2)
   {
    /////////////////////////////////////////
    // Here z is temporarily used for storing
    // dispersion-covariation matrix
    if ((z = new double[k * k]) == 0)
      {
       delete [] t;
       return -1;
      }

    /////////////////////////////////////////
    // Building dispersion-covariation matrix
    Code = CovMatrix (data,z,n,k);

    if (Code != 0)
      {
       delete [] t; delete [] z;
       return Code;
      }

    ////////////////////////////////////////
    // Inverse dispersion-covariation matrix
    Code = InverseMatrix (z,t,k);

    delete [] z;

    if (Code != 0)
      {
       delete [] t;
       return Code;
      }
   }

 if ((dat = new double[n * k]) == 0)
   {
    delete [] t;
    return -1;
   }

 ///////////////////////////////
 // Copy of given arrays of data
 ArrayToArray  (data,dat,n * k);

 /////////////
 // Main cycle
 for (l = 0, r = n; l < n - 1; l++, r--)
   {
    if ((z = new double[r * r]) == 0)
      {
       delete [] t; delete [] dat;
       return -1;
      }

    //////////////////////////////
    // Building of likeness matrix
    if ((Code = LikenessMatrix (dat,z,t,r,k,Mode)) != 0)
      {
       delete [] t; delete [] z; delete [] dat;
       return Code;
      }

    /////////////////////////////////////////////
    // Search for the two most similar objects by
    // analyzing the lower triangle of the likeness matrix
    for (i = 0; i < r; i++)
      for (j = i + 1, m = i * r + i + 1; j < r; j++, m++)
        if (m == 1 || z[m] < s[l])
          {
           ph[l] = i;   // Number of the 1st object in pair
           pl[l] = j;   // Number of the 2nd object in pair
           s[l] = z[m]; // Distance between objects of cluster
          }

    //////////////////////////////////////
    // Now Array z is used to store arrays
    // of data obtained in the previous step
    delete [] z;
    if ((z = new double[r * k]) == 0)
      {
       delete [] t; delete [] dat;
       return -1;
      }

    /////////////////////////////////
    // Copy of given arrays of data
    ArrayToArray (dat,z,r * k);

    /////////////////////////////////////
    // Reduced by 1 of rows array of data
    delete [] dat;
    dat = new double[(r - 1) * k];

    /////////////////////////////////////////////////////////////
    // The center of gravity of the new cluster is in place ph[l]
    for (j = 0, m1 = ph[l], m = pl[l]; j < k;
         j++, m1 += r, m += r)
      z[m1] = (z[m1] + z[m]) / 2;

    ///////////////////////////////////////////
    // Move of rows to the place pl[l] and next
    for (i = pl[l]; i < r - 1; i++)
      for (j = 0, m = i; j < k; j++, m += r)
        z[m] = z[m + 1];

    ////////////////////////
    // Copy of reduced array
    for (i = 0; i < r - 1; i++)
      for (j = 0, m = i, m1 = i; j < k;
           j++, m += r - 1, m1 += r)
        dat[m] = z[m1];

    ///////////////////////////////////////////
    // Now array z is free and will be used for
    // storing of likeness matrix
    delete [] z;
   }

 delete [] t; delete [] dat;

 return Code;
}

long MODIFICATOR Ward (double data[],long n,long k,long ph[],long pl[],double s[])
//
// The function performs cluster analysis by the Ward method.
// Designations:
//   data - matrix of given data by size n * k
//          (k vectors by length n one by one),
//      n - number of objects for classification,
//      k - number of parameters,
//  ph,pl - on exit - arrays numbers each by length (n - 1) with
//          information about grouping of objects,
//      s - the level of pairing [ph,pl].
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory,
//  -2 when loss of significance.
// References:
//  Aldenderfer M.S., Blashfield R.K Cluster analysis. - Beverly Hills, CA: SAGE Publications, 1985.
//
{
 long i,j,l1,        // Cycle counter
      l,             // Counter of clusters
      m,m1,m2,       // Array counter
      r,             // Number of objects on current step
      Code = 0,      // Error code
      *inf;          // Number of members in clusters

 double *z,*dat,     // Work Matrix
        *dat1,*dat2; // Copy vectors of data

 if ((dat = new double[n * k]) == 0)
   return -1;
 if ((inf = new long[n]) == 0)
   {
    delete [] dat;
    return -1;
   }
 if ((dat1 = new double[n]) == 0)
   {
    delete [] dat; delete [] inf;
    return -1;
   }
 if ((dat2 = new double[n]) == 0)
   {
    delete [] dat1; delete [] inf; delete [] dat1;
    return -1;
   }

 ////////////////////////////////////////
 // On 1st step - by 1 element in cluster
 FillUp (inf,n,1);

 ///////////////////////////////
 // Copy of given arrays of data
 ArrayToArray (data,dat,n * k);

 ////////////////
 // Main cycle
 for (l = 0, r = n; l < n - 1; l++, r--)
   {
    if ((z = new double[r * r]) == 0)
      {
       delete [] dat; delete [] inf; delete [] dat1;
       delete [] dat2;
       return -1;
      }

    ///////////////////////////////////////
    // Building insidegroup sums of squares
    for (i = 0, m2 = 0; i < r; i++)
      {
       for (l1 = 0, m = i; l1 < k; l1++, m += r)
         dat1[l1] = dat[m];
       for (j = 0; j < r; j++)
         {
          for (l1 = 0, m1 = j; l1 < k; l1++, m1 += r)
            dat2[l1] = dat[m1];
          z[m2++] = Euclid (dat1,dat2,k) *
                    inf[i] * inf[j] / (inf[i] + inf[j]);
         }
      }

    /////////////////////////////////////////////
    // Search for the two most similar objects by
    // analyzing the lower triangle of the likeness matrix
    for (i = 0; i < r; i++)
      for (j = i + 1, m = i * r + i + 1; j < r; j++, m++)
        if (m == 1 || z[m] < s[l])
          {
           ph[l] = i;   // Number of the 1st object in pair
           pl[l] = j;   // Number of the 2nd object in pair
           s[l] = z[m]; // Distance between objects of cluster
          }

    //////////////////////////////////////
    // Now Array z is used to store arrays
    // of data obtained in the previous step
    delete [] z;
    if ((z = new double[r * k]) == 0)
      {
       delete [] dat; delete [] inf; delete [] dat1;
       delete [] dat2;
       return -1;
      }

    ///////////////////////////////
    // Copy of given arrays of data
    ArrayToArray (dat,z,r * k);

    /////////////////////////////////////
    // Reduced by 1 of rows array of data
    delete [] dat;
    dat = new double[(r - 1) * k];

    /////////////////////////////////////////////////////////////
    // The center of gravity of the new cluster is in place ph[l]
    for (j = 0, m1 = ph[l], m = pl[l]; j < k;
         j++, m1 += r, m += r)
      z[m1] = (z[m1] + z[m]) / 2;

    /////////////////////////////////////////////////
    // An element has been added to the ph[l] cluster
    inf[ph[l]] += 1;

    ///////////////////////////////////////////
    // Move of rows to the place pl[l] and next
    for (i = pl[l]; i < r - 1; i++)
      {
       inf[i] = inf[i + 1];
       for (j = 0, m = i; j < k; j++, m += r)
         z[m] = z[m + 1];
      }

    ////////////////////////
    // Copy of reduced array
    for (i = 0; i < r - 1; i++)
      for (j = 0, m = i, m1 = i; j < k;
           j++, m += r - 1, m1 += r)
        dat[m] = z[m1];

    ///////////////////////////////////////////
    // Now array z is free and will be used for
    // storing of likeness matrix
    delete [] z;
   }

 delete [] dat; delete [] inf; delete [] dat1; delete [] dat2;

 return Code;
}

void MODIFICATOR ClusterPresentKW(long Y1[],long Y2[],long C[],long Q[],long N,long M)
//
// The function for presentation of the results of cluster analysis by the average
// King connection method or the Ward method.
// Designations:
//  Y1 - vector added numbers of objects or clusters (numbering from zero)
//  Y2 - vector added numbers of objects or clusters (numbering from zero)
//  N  - number of objects
//  M  - number of clusters
//  C  - array numbers of objects distributed across clusters
//  Q  - array of clusters size
// Return code:
//  none.
// References:
//  Aldenderfer M.S., Blashfield R.K Cluster analysis. - Beverly Hills, CA: SAGE Publications, 1985.
//
{
 long I,J,K,L = 1,P; // Counter

 ///////////////////////////////
 // Assignment of initial values
 for (I = 1; I <= N; I++, L = L + N)
   {
    C[L - 1] = I;
    Q[I - 1] = 1;
   }

 //////////////////////////////
 // Compute clusters parameters
 for (I = 1; I <= N - M; I++)
   {
    ////////////////////////
    // Add numbers in column
    K = Y1[I - 1] * N + Q[Y1[I - 1]];
    L = Y2[I - 1] * N;
    for (J = 1; J <= Q[Y2[I - 1]]; J++)
      {
       C[++K - 1] = C[++L - 1];
       Q[Y1[I - 1]] += 1;
      }

    ///////////////////
    // Shift of columns
    L = Y2[I - 1] * N + 1;
    P = Y2[I - 1] + 1;
    for (J = P; J <= N - I; J++)
      {
       for (K = 1; K <= N; K++)
         {
          C[L - 1] = C[L + N - 1];
          L += 1;
         }
       Q[P - 1] = Q[P];
       P += 1;
      }
   }
}

void MODIFICATOR ClusterPresentMQ(long Y[],long C[],long Q[],long N)
//
// The function for presentation the results of cluster analysis by the k-medium McQueen method.
// Designations:
//  Y - vector belonging of objects to clusters (numbering from zero)
//  N - number of objects
//  C - array of objects numbers,d istributed across clusters
//  Q - array size of clusters
// Return code:
//  none.
// References:
//  Aldenderfer M.S., Blashfield R.K Cluster analysis. - Beverly Hills, CA: SAGE Publications, 1985.
//
{
 long I; // Counter

 ///////////////////////////////
 // Assignment of initial values
 for (I = 0; I < N; I++)
   Q[I] = 0;

    ////////////////////////
    // Add numbers in column
 for (I = 1; I <= N; I++)
   {
    C[N * Y[I - 1] + Q[Y[I - 1]]] = I;
    Q[Y[I - 1]] += 1;
   }
}

long MODIFICATOR CanonicalCorrelationAnalysis (double y1[],double y2[],long p,long q,long n,long Mode,double l[],double t[],double r[],double xi[],double u[])
//
// The function builds the canonical correlation analysis between
// two groups samples.
// Designations:
//  y1 - array - 1 group - size n x p,
//  y2 - array - 2 group - size n x q,
//   p - number of series in 1st group,
//   q - number of series in 2nd group,
//   n - number of samle members in each series of any group,
//  Mode - mode of correlation coefficient:
//       =  0 - coefficient of Pearson,
//       =  1 - coefficient of Fechner,
//       =  2 - indicator of Spearman,
//       =  3 - coefficient of Kendall,
//       =  4 - similarity indicator of Rao,
//       =  5 - coefficient contingency of Bravais,
//   l - calculated vector of canonical correlation by length q,
//   t - calculated array of right side coefficients by length q x q,
//   r - calculated array of left side coefficients by length q x p,
//  xi - array values of chi-square test by length q,
//   u - array of degree of freedom for chi-square test by length q.
// Return code.
//   0 when normal end of computation,
//  -1 when lack of memory for work array,
//  -2 when computation error.
// References:
//  Eaton M.L. Multivariate statistics: A vector space approach (Lecture notes - Monograph series, vol. 53). - Beachwood, OH: Institute of Mathematical Statistics, 2007.
//  System/360 scientific subroutine package (360A-CM-03X) Version III. Programmer-s manual. New York, NY: Technical Publication Department IBM, 1974.
//
{
 long i,j,i1,             // Cycle counter
      m1 = p > q ? p : q, // Work variables
      m2,m3;

 double *dat1,*dat2,      // Copy of current samples
        *a,*b,            // Work arrays
        save;             // Buffer

 if ((dat1 = new double[n]) == 0)
    return -1;
 if ((dat2 = new double[n]) == 0)
   {
    delete [] dat1;
    return -1;
   }
 if ((a = new double[m1 * m1]) == 0)
   {
    delete [] dat1; delete [] dat2;
    return -1;
   }
 if ((b = new double[m1 * m1]) == 0)
   {
    delete [] a; delete [] dat1; delete [] dat2;
    return -1;
   }

 ///////////////////////////////////////////////
 // Matrix of mutual correlations between groups
 for (i = 0, m1 = 0; i < p; i++)
   {
    for (i1 = 0; i1 < n; i1++)
      dat1[i1] = y1[m1++];

    for (j = 0, m2 = 0, m3 = i; j < q; j++, m3 += p)
      {
       for (i1 = 0; i1 < n; i1++)
         dat2[i1] = y2[m2++];

       switch (Mode)
         {
          case 0:
            r[m3] = Pearson (dat1,dat2,n);
            break;
          case 1:
            r[m3] = Fechner (dat1,dat2,n);
            break;
          case 2:
           if (Spearman (dat1,dat2,n,&save) == -1)
             {
               delete [] dat1; delete [] dat2; delete [] b; delete [] a;
              return -1;
             }
            r[m3] = save;
            break;
          case 3:
            if (Kendall (dat1,dat2,n,&save) == -1)
              {
               delete [] dat1; delete [] dat2; delete [] b; delete [] a;
               return -1;
              }
              r[m3] = save;
            break;
          case 4:
              r[m3] = Rao (dat1,n,dat2,n);
            break;
          default:
              r[m3] = Bravais (dat1,n,dat2,n);
            break;
         }
      }
   }

 //////////////////////////////////////////
 // Matrix mutual correlations in 1st group
 for (i = 0, m3 = 0, m2 = 0; i < p; i++)
   {
    for (i1 = 0; i1 < n; i1++)
      dat1[i1] = y1[m3++];

    for (j = 0, m1 = 0; j < p; j++)
      {
       for (i1 = 0; i1 < n; i1++)
         dat2[i1] = y1[m1++];

       switch (Mode)
         {
          case 0:
            b[m2++] = Pearson (dat1,dat2,n);
            break;
          case 1:
            b[m2++] = Fechner (dat1,dat2,n);
            break;
          case 2:
           if (Spearman (dat1,dat2,n,&save) == -1)
             {
               delete [] dat1; delete [] dat2; delete [] b; delete [] a;
              return -1;
             }
            b[m2++] = save;
            break;
          case 3:
            if (Kendall (dat1,dat2,n,&save) == -1)
              {
               delete [] dat1; delete [] dat2; delete [] b; delete [] a;
               return -1;
              }
            b[m2++] = save;
            break;
          case 4:
            b[m2++] = Rao (dat1,n,dat2,n);
            break;
          default:
            b[m2++] = Bravais (dat1,n,dat2,n);
            break;
         }
      }
   }
 if ((m2 = InverseMatrix (b,a,p)) != 0)
   {
    delete [] dat1; delete [] dat2; delete [] b; delete [] a;
    return m2;
   }

 /////////////////////////////////////////////
 // Matrix of mutual correlations in 2nd group
 for (i = 0, m3 = 0, m2 = 0; i < q; i++)
   {
    for (i1 = 0; i1 < n; i1++)
      dat1[i1] = y2[m3++];

    for (j = 0, m1 = 0; j < q; j++)
      {
       for (i1 = 0; i1 < n; i1++)
         dat2[i1] = y2[m1++];

       switch (Mode)
         {
          case 0:
            t[m2++] = Pearson (dat1,dat2,n);
            break;
          case 1:
            t[m2++] = Fechner (dat1,dat2,n);
            break;
          case 2:
           if (Spearman (dat1,dat2,n,&save) == -1)
             {
               delete [] dat1; delete [] dat2; delete [] b; delete [] a;
              return -1;
             }
            t[m2++] = save;
            break;
          case 3:
            if (Kendall (dat1,dat2,n,&save) == -1)
              {
               delete [] dat1; delete [] dat2; delete [] b; delete [] a;
               return -1;
              }
            t[m2++] = save;
            break;
          case 4:
            t[m2++] = Rao (dat1,n,dat2,n);
            break;
          default:
            t[m2++] = Bravais (dat1,n,dat2,n);
            break;
         }
      }
   }

 MatrixMultiplyMatrix (a,r,b,p,p,q);
 MatrixTMultiplyMatrix (r,b,a,q,p,q);

 if ((m2 = Eigen (a,t,q,0.000001,&m1)) != 0)
   {
    delete [] dat1; delete [] dat2; delete [] b; delete [] a;
    return m2;
   }

 /////////////////////////////////////
 // Ordering of own values and vectors
 for (i = 0; i < q; i++)
   {
    l[i] = a[i * (q + 1)];
    if (l[i] <= 0 && l[i] >= 1)
      {
       delete [] dat1; delete [] dat2; delete [] b; delete [] a;
       return -2;
      }
    else
      l[i] = sqrt (l[i]);
   }

 for (i = 0; i < q; i++)
   {
    m2 = i;
    save = l[i];
    for (j = i + 1; j < q; j++)
      if (save < l[j])
        {
         m2 = j;
         save = l[j];
        }

    ColInterchange (l,1,i,m2);
    ColInterchange (t,q,i,m2);
   }

 ////////////////////////////////////
 // Computation of quality parameters
 for (i = 0; i < q; i++)
   {
    for (j = i, save = 1; j < q; j++)
      save *= (1.0 - l[j]);
    xi[i] = -(n - (p + q + 1) / 2) * log (save);
    u[i] = (p - i) * (q - i);
   }

 ////////////////////////////////////////
 // Computation of left side coefficients
 MatrixMultiplyMatrix (b,t,r,p,q,q);

 for (i = 0; i < p; i++)
   for (j = 0, m1 = i; j < q; j++, m1 += p)
     r[m1] /= l[j];

 delete [] dat1; delete [] dat2; delete [] b; delete [] a;

 return 0;
}

long MODIFICATOR Spearman (double x[],double y[],long n,double *coef)
//
// The function computes indicator Spearman of rank correlation.
// Designations:
//      x - 1st array,
//      y - 2nd array,
//      n - size of each array,
//  *coef - calculated value indicator.
// Return code:
//    0 when normal result of computation,
//   -1 when lack of memory.
// References:
//  Van der Waerden B.L. Mathematische Statistik (in Deutsch). - Berlin: Springer, 1971.
//
{
 long i,           // Counter
      *ix,*iy;     // Work variables

 double *spx,*spy, // Copy of given array
        *xx,*yy,   // Arrays of ranks
        bx,by,     // corrections to the merger of ranks
        s = 0;

 if ((spx = new double[n]) == 0)
   return -1;
 if ((spy = new double[n]) == 0)
   {
    delete [] spx;
    return -1;
   }
 if ((xx = new double[n]) == 0)
   {
    delete [] spx; delete [] spy;
    return -1;
   }
 if ((yy = new double[n]) == 0)
   {
    delete [] spx; delete [] spy; delete [] xx;
    return -1;
   }
 if ((ix = new long[n]) == 0)
    {
    delete [] spx; delete [] spy; delete [] xx; delete [] yy;
    return -1;
   }
 if ((iy = new long[n]) == 0)
   {
    delete [] spx; delete [] spy; delete [] xx; delete [] yy;
    delete [] ix;
    return -1;
   }

 //////////////////////
 // Copy of given array
 ArrayToArray (x,spx,n);
 ArrayToArray (y,spy,n);

 //////////
 // Sort up
 SortArrayUp (spx,n,ix);
 SortArrayUp (spy,n,iy);

 //////////
 // Ranking
 bx = Rank (spx,xx,n);
 by = Rank (spy,yy,n);

 ///////////////////////
 // Ranks of given array
 UnSortArray (xx,spx,n,ix);
 UnSortArray (yy,spy,n,iy);

 delete [] xx; delete [] yy; delete [] ix; delete [] iy;

 //////////////////////////////////////////////////////
 // Sum of squares of the pairwise differences of ranks
 for (i = 0; i < n; i++)
   s += (spx[i] - spy[i]) * (spx[i] - spy[i]);

 delete [] spx; delete [] spy;

 *coef = 1.0 - 6.0 * (s + bx + by) / n / (n * n - 1);

 return 0;
}

long MODIFICATOR Kendall (double x[],double y[],long n,double *coef)
//
// The function computes value of Kendall coefficient of rank.
// Designations:
//      x - 1st array,
//      y - 2nd array,
//      n - size of each arrays,
//  *coef - calculated coefficient value,
// Return code:
//    0 when normal result of computation,
//   -1 when lack of memory.
//
{
 long i,j,         // Counters
      *ix,*iy;     // Work variables

 double *spx,*spy, // Copy of given array
        *xx,*yy,   // Arrays of ranks
        bx,by,     // corrections to the merger of ranks
        s = 0,
        rs;

 if ((spx = new double[n]) == 0)
   return -1;
 if ((spy = new double[n]) == 0)
   {
    delete [] spx;
    return -1;
   }
 if ((xx = new double[n]) == 0)
   {
    delete [] spx; delete [] spy;
    return -1;
   }
 if ((yy = new double[n]) == 0)
   {
    delete [] spx; delete [] spy; delete [] xx;
    return -1;
   }
 if ((ix = new long[n]) == 0)
   {
    delete [] spx; delete [] spy; delete [] xx; delete [] yy;
    return -1;
   }
 if ((iy = new long[n]) == 0)
   {
    delete [] spx; delete [] spy; delete [] xx; delete [] yy;
    delete [] ix;
    return -1;
   }

 //////////////////////////
 // Copy of orirginal array
 ArrayToArray (x,spx,n);
 ArrayToArray (y,spy,n);

 ////////////
 // Sort down
 SortArrayDown (spx,n,ix);
 SortArrayDown (spy,n,iy);

 //////////////////
 // Arrays of ranks
 bx = RankKendall (spx,xx,n);
 by = RankKendall (spy,yy,n);

 ///////////////////////
 // Ranks of given array
 UnSortArray (xx,spx,n,ix);
 UnSortArray (yy,spy,n,iy);

 //////////////////////////////
 // Common sort up of 1st array
 SortArrayCommon (spx,spy,n);

 delete [] ix; delete [] spx; delete [] iy; delete [] yy; delete [] xx;

 for (i = 0; i < n - 1; i++)
   for (j = i + 1; j < n; j++)
     if (spy[i] < spy[j])
       s++;
     else if (spy[i] > spy[j])
       s--;

 delete [] spy;

 rs = 0.5 * n * (n - 1);

 *coef = s / sqrt ((rs - bx) * (rs - by));

 return 0;
}

long MODIFICATOR KendallCoeff (double x[],double y[],double spx[],double spy[],long n,double *coef)
//
// The function computes value of Kendall coefficient of rank correlation.
// Designations:
//      x - 1st array,
//      y - 2nd array,
//    spx - 1st array of ranks,
//    spy - 2nd array of ranks,
//      n - size of each arrays,
//  *coef - calculated coefficient value,
// Return code:
//   0 when normal result of computation,
//  -1 when lack of memory.
//
{
 long i,j,       // Counters
      *ix,*iy;   // Work variables

 double *xx,*yy, // Arrays of ranks
        bx,by,   // corrections to the merger of ranks
        s = 0,
        rs;

 if ((xx = new double[n]) == 0)
   return -1;
 if ((yy = new double[n]) == 0)
   {
    delete [] xx;
    return -1;
   }
 if ((ix = new long[n]) == 0)
   {
    delete [] xx; delete [] yy;
    return -1;
   }
 if ((iy = new long[n]) == 0)
   {
    delete [] xx; delete [] yy; delete [] ix;
    return -1;
   }

 //////////////////////
 // Copy of given array
 ArrayToArray (x,spx,n);
 ArrayToArray (y,spy,n);

 ////////////
 // Sort down
 SortArrayDown (spx,n,ix);
 SortArrayDown (spy,n,iy);

 //////////////////
 // Arrays of ranks
 bx = RankKendall (spx,xx,n);
 by = RankKendall (spy,yy,n);

 ///////////////////////
 // Ranks of given array
 UnSortArray (xx,spx,n,ix);
 UnSortArray (yy,spy,n,iy);

 //////////////////////////////
 // Common sort up of 1st array
 SortArrayCommon (spx,spy,n);

 delete [] ix; delete [] iy; delete [] yy; delete [] xx;

 for (i = 0; i < n - 1; i++)
   for (j = i + 1; j < n; j++)
     if (spy[i] < spy[j])
       s++;
     else if (spy[i] > spy[j])
       s--;

 rs = 0.5 * n * (n - 1);

 *coef = s / sqrt ((rs - bx) * (rs - by));

 return 0;
}

double Rao (double x[],long n1,double y[],long n2)
//
// The function computes similarity Russel-Rao indicator.
// Designations:
//   x - 1st compared sample,
//  n1 - size of 1st sample,
//   y - 2nd compared sample,
//  n2 - size of 2nd sample.
// Return code:
//  value of metric.
//
{
 long i;             // Counter

 double a = 0,b = 0, // Parameters
        c = 0,d = 0;

 for (i = 0; i < n1; i++)
   if (x[i] == 0)
     a++;
   else
     c++;
 for (i = 0; i < n2; i++)
   if (y[i] == 0)
     b++;
   else
     d++;

 return a / (n1 + n2);
}

double Bravais (double x[],long n1,double y[],long n2)
//
// The function computes Bravais contingency coefficient.
// Designations:
//   x - 1st compared sample,
//  n1 - size of 1st sample,
//   y - 2nd compared sample,
//  n2 - size of 2nd sample.
// Return code:
//  value of metric.
//
{
 long i;             // Counter

 double a = 0,b = 0, // Parameters
        c = 0,d = 0;

 for (i = 0; i < n1; i++)
   if (x[i] == 0)
     a++;
   else
     c++;
 for (i = 0; i < n2; i++)
   if (y[i] == 0)
     b++;
   else
     d++;

 return (a * d - b * c - 2 * (n1 + n2)) / sqrt ((a + b) * (c + d) * (a + c) * (b + d));
}

double MODIFICATOR Fechner (double a[],double b[],long n)
//
// The function computes Fechner correlation coefficient.
// Designations:
//  a - 1st compared sample,
//  b - 2nd compared sample,
//  n - each sample size.
// Return code:
//  value of coefficient.
//
{
 long i;                          // Cycle counter

 double xa = VectorSum (a,n) / n, // Medium of 1st series
        xb = VectorSum (b,n) / n, // Medium of 2nd series
        c = 0;                    // Number sign matches

 for (i = 0; i < n; i++)
   if (SIGN(a[i] - xa) == SIGN(b[i] - xb))
     c++;

 return 2.0 * c / n - 1;
}

long MODIFICATOR Gower (double x[],long k,long n,long inf[],double a[])
//
// The function computes the likeness matrix of mixed features with Gower coefficient for factor analysis.
// Designations:
//    x - array of given data (k columns of n of members one by one),
//    k - number of objects,
//    n - number signs,
//  inf - array of information about scale of each sign:
//      = 0 for quantitative,
//      = 1 for ordinal,
//      = 2 for dichotomous,
//    a - calculated likeness matrix of objects by size k x k.
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory for work array,
//  -2 when computation error,
//  -3 when error in data.
//
{
 long i,j,l,     // Cycle counter
      mr = 0,my, // Counters 1st compared arrays
      mz,        // Counter 2nd compared array
      ma,mt;     // Counters upper and lower triangles

 double *r,      // Vector of range
        s,w;     // Work variables, later numerator and
                 // denominator of the element of likeness matrix

 if ((r = new double[n]) == 0)
    return -1;

 /////////////////////////
 // Making of range vector
 for (i = 0; i < n; i++)
   {
    for (j = 0, ma = i, w = x[i], s = x[i]; j < k; j++, ma += n)
      {
       if (x[ma] > w)
         w = x[ma];
       if (x[ma] < s)
         s = x[ma];
      }
    r[i] = w - s;
   }

 ////////////////////////////
 // Making of likeness matrix
 for (j = 0; j < k - 1; j++, mr += n)
   {
    ma = j * k + j + 1;
    mt = j * k + j + k;
    mz = j * n + n;

    for (l = j + 1, s = 0, w = 0; l < k; l++, mt += k)
      {
       for (i = 0, my = mr; i < n; i++, mz++, my++)

         /////////////////////////
         // For quantitative signs
         if (inf[i] == 0)
           {
            s += 1.0 - fabs (x[mz] - x[my]) / r[i];
            w++;
           }
         ////////////////////
         // For ordinal signs
         else if (inf[i] == 1)
           {
            if (x[mz] == x[my])
              {
               s++;
               w++;
              }
           }
         ////////////////////////
         // For dichotomous signs
         else if (inf[i] == 2)
           {
            if (x[mz] == 1 && x[my] == 1)
              {
               s++;
               w++;
              }
            else if ((x[mz] == 1 && x[my] == 0) || (x[mz] == 0 && x[my] == 1))
              w++;
           }
         ////////////////
         // Error in data
         else
           {
            delete [] r;
            return -3;
           }
       if (w == 0)
         {
          delete [] r;
          return -2;
         }
       a[ma] = s / w;
       a[mt] = a[ma++];
      }
   }

 //////////////////////////////
 // Diagonal of likeness matrix
 for (i = 0; i < k; i++)
   a[i * (k + 1)] = 1;

 delete [] r;

 return 0;
}

long MODIFICATOR LikenessMatrixMixed (double x[],long k,long n,long inf[],double a[])
//
//  The function computes the likeness matrix of mixed features for cluster analysis.
// Designations:
//    x - array of given data (k columns of n of members one by one),
//    k - number of objects,
//    n - number signs,
//  inf - array of information about scale of each sign:
//      = 0 for quantitative,
//      = 1 for ordinal,
//      = 2 for dichotomous,
//    a - calculated likeness matrix of objects by size k x k.
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory for work array,
//  -2 when computation error,
//  -3 when error in data.
//
{
 long i,j,l,      // Counter
       i1 = 0,j1,
       Code;      // Error code

 double *xx,*yy,  // Work array
        t;        // Work variable

 if ((xx = new double[n]) == 0)
   return -1;

 if ((yy = new double[n]) == 0)
   {
    delete [] xx;
    return -1;
   }

 ///////////////////////////////////////////////////////////////
 // Formation of the lower and upper triangle of likeness matrix
 for (i = 0; i < k - 1; i++)
   {
    for (l = 0; l < n; l++)
      xx[l] = x[i1++];
    for (j = i + 1, j1 = i * n + n; j < k; j++)
      {
       for (l = 0; l < n; l++)
         yy[l] = x[j1++];

       ////////////////////////////////////
       // Correlation of quantitative signs
       if (inf[i] == 0 && inf[j] == 0)
         t = Pearson (xx,yy,n);

       ///////////////////////////////
       // Correlation of ordinal signs
       else if ((inf[i] == 1 && inf[j] != 2) || (inf[i] != 2 && inf[j] == 1))
         {
          if ((Code = Kendall (xx,yy,n,&t)) != 0)
            {
             delete [] xx; delete [] yy;
             return Code;
            }
          }
       /////////////////////////////
       // Point-biserial correlation
       else if (inf[i] == 2 && inf[j] != 2)
         {
          if ((Code = Point (xx,yy,n,&t)) != 0)
            {
             delete [] xx; delete [] yy;
             return Code;
            }
         }
       else if (inf[i] != 2 && inf[j] == 2)
         {
          if ((Code = Point (yy,xx,n,&t)) != 0)
            {
             delete [] xx; delete [] yy;
             return Code;
            }
         }
       ///////////////////////////////////
       // Correlation of dichotomous signs
       else if (inf[i] == 2 && inf[j] == 2)
         t = Bravais (xx,n,yy,n);

       //////////////////
       // Error in of data
       else
         {
          delete [] xx; delete [] yy;
          return -3;
         }

       a[i * k + j] = t;
       a[i + j * k] = t;
      }
   }

 //////////////////////////////
 // Diagonal of likeness matrix
 for (i = 0; i < k; i++)
   a[i * (k + 1)] = 1;

 delete [] xx; delete [] yy;

 return 0;
}

long MODIFICATOR Point (double a[],double x[],long n,double *t)
//
// The function computes point-biserial correlation.
// Designations:
//   a - dichotomous array,
//   x - quantitative array,
//   n - each array size,
//  *t - point-biserial correlation.
// Return code:
//   0 when normal end of computation,
//  -2 when computation error.
//
{
 long i;        // Cycle counter

 double n1 = 0, // Number of events in upper group
        n0 = 0, // Number of events in low group
        m1 = 0, // Medium of quantitative sample (level 1)
        m0 = 0, // Medium of quantitative sample (level 0)
        m = 0,  // Medium of quantitative sample
        sn = 0; // Standard deviation

 ///////////////////////
 // Parameters of sample
 for (i = 0; i < n; i++)
   {
    m += x[i];
    if (a[i] == 1)
      {
       n1++;
       m1 += x[i];
      }
    else
      {
       n0++;
       m0 += x[i];
      }
   }
 m /= n;
 if (n1 == 0)
   return -2;
 m1 /= n1;
 if (n0 == 0)
   return -2;
 m0 /= n0;
 for (i = 0; i < n; i++)
   sn += (x[i] - m) * (x[i] - m);
 sn = sqrt(sn / n);
 if (sn == 0)
   return -2;

 //////////////////////////////////////
 // Value of point-biserial correlation
 *t = (m1 - m0) / sn * sqrt (n1 * n0) / n;

 return 0;
}

double MODIFICATOR Tanh(double x)
//
// The function computes hyperbolic tangent.
// Designations:
//  x - parameter.
// Return code:
//  value of hyperbolic tangent.
//
{
 return (exp(x) - exp(-x)) / (exp(x) + exp(-x));
}

double MODIFICATOR Atanh(double x)
//
// The function computes hyperbolic arctangent.
// Designations:
//  x - parameter.
// Return code:
//  value of hyperbolic arctangent.
//
{
 return log((1.0 + x) / (1.0 - x)) / 2;
}

void MODIFICATOR IntervalKendall(double X[],double Y[],long N,double Z,double P,double *Low,double *High,long *Meth)
//
// The function computes confidence interval of Kendall coefficient.
// Designations:
//      X - independent variable,
//      Y - dependent variable,
//      N - size of each samples,
//      Z - Kendall coefficient,
//      p - confidence level,
//   *Low - low bound,
//  *High - upper bound,
//  *Meth - used computation method.
// Return code:
//  none.
//
{
 long I,J;       // Counter

 double B = 0,C, // Work variable
        S = 0;   // Variance, Standard, interval

 for (I = 0; I < N; I++)
   {
    for (J = 0, C = 0; J < N; J++)
      if (I != J)
        C += KendallCI(X[I], X[J], Y[I], Y[J]);
    B += C * C;
    S += C;
   }
 S = (B * 4 - S * 2 - S * 2 / N * S * (2 * N - 3) / (N - 1));
 if (S >= 0)
   {
    S = sqrt(S);
    S *= 2 * InverseNormalDistribution(P / 2 + 0.5) / N / (N - 1);
    *Meth = 1;
   }
 else
   {
    S = sqrt(2 * (2 * N + 5) / 9.0 / N / (N - 1));
    S *= InverseTDistribution(N - 1, (1 + P) / 2.0) / sqrt(N);
    *Meth = 2;
   }
 *Low = Z - S;
 *High = Z + S;
}

double MODIFICATOR KendallCI(double A,double B,double C,double D)
//
// The service function of computation confidence interval of Kendall coefficient.
// Designations:
//  A - parameter,
//  B - parameter,
//  C - parameter,
//  D - parameter.
// Return code:
//  value of function.
//
{
 return (A - B) * (C - D) > 0 ? 1 : 0;
}

void MODIFICATOR IntervalCorrelation(double Z,long N,double P,double *Low,double *High)
//
// The function computes confidence interval of correlation coefficient.
// Designations:
//     Z - Fisher transformation,
//     N - size of each samples,
//     P - confidence level,
//   Low - low bound,
//  High - upper bound.
// Return code:
//  none.
//
{
 double B = InverseNormalDistribution((1 + P) / 2) / sqrt(N - 3); // Work variable

 *Low = Tanh(Z - B);
 *High = Tanh(Z + B);
}

double MODIFICATOR TietjenMooreDistribution(long n, long k)
//
// The function computes critical value of Tietjen-Moore test.
// Designations:
//  N - sample size,
//  K - number of blowout.
// Return code:
//  critical value for significance level 0,05.
//
{
 double y1[] = {0.003, 0.051, 0.125, 0.203, 0.273, 0.326, 0.372, 0.418,
                0.454, 0.489, 0.517, 0.54, 0.556, 0.575, 0.594, 0.608, 0.624},
        y2[] = {0.001, 0.018, 0.055, 0.106, 0.146, 0.194, 0.233, 0.27,
                0.305, 0.337, 0.363, 0.387, 0.41, 0.427, 0.447, 0.462},
        y3[] = {0.01, 0.032, 0.064, 0.099, 0.129, 0.162, 0.196, 0.224,
                0.25, 0.276, 0.3, 0.322, 0.337, 0.354},
        y4[] = {0.022, 0.045, 0.07, 0.098, 0.125, 0.15, 0.174, 0.197,
                0.219, 0.24, 0.259, 0.277},
        y5[] = {0.034, 0.054, 0.076, 0.098, 0.122, 0.14, 0.159, 0.181,
                0.2, 0.209},
        y6[] = {0.042, 0.06, 0.079, 0.097, 0.115, 0.136, 0.154, 0.168},
        y7[] = {0.05, 0.066, 0.082, 0.1, 0.116, 0.13},
        y8[] = {0.055, 0.072, 0.086, 0.099},
        y9[] = {0.062, 0.074},
        a,b;

 if (n < 20)
   /////////////////////
   // Exact table values
   {
    if (k == 1)
      return y1[n - 3];
    else if (k == 2)
      return y2[n - 4];
    else if (k == 3)
      return y3[n - 6];
    else if (k == 4)
      return y4[n - 8];
    else if (k == 5)
      return y5[n - 10];
    else if (k == 6)
      return y6[n - 12];
    else if (k == 7)
      return y7[n - 14];
    else if (k == 8)
      return y8[n - 16];
    else
      return y9[n - 18];
    }
 else
   ////////////////////////////////////////
   // Approximation by logarithmic function
   {
    if (k == 1)
      {
       a = 0.19418112;
       b = -0.06575383;
      }
    else if (k == 2)
      {
       a = 0.25812829;
       b = 0.28278038;
      }
    else if (k == 3)
      {
       a = 0.29275863;
       b = 0.49380456;
      }
    else if (k == 4)
      {
       a = 0.31458311;
       b = 0.63943566;
      }
    else if (k == 5)
      {
       a = 0.3245631;
       b = 0.73176359;
      }
    else if (k == 6)
      {
       a = 0.33029662;
       b = 0.79975254;
      }
    else if (k == 7)
      {
       a = 0.32887508;
       b = 0.83561828;
      }
    else if (k == 8)
      {
       a = 0.32971362;
       b = 0.87499117;
      }
    else if (k == 9)
      {
       a = 0.32553272;
       b = 0.89145385;
      }
    else
      {
       a = 0.31999176;
       b = 0.89985535;
      }
   }

 return a * log(n) - b;
}

double MODIFICATOR InverseSMMDistribution (double k,double n,double p)
//
// The function for computation of probabilities of the studentized maximum of modules.
// Designations:
//  k - parameter,
//  n - number of degrees of freedom,
//  p - parameter.
// Return code:
//  value of inverse function of distribution of the studentized maximum of modules.
//
{
 double c = 1,  // Left bound of root localization
        d = 50, // Right bound of root localization
        y,      // The middle of interval localization
        cx;     // Values of function on end of interval

 //////////////////////////////////////////////////////////////
 // Refinement of the root method of dividing a segment in half
 do
   {
    cx = p - SMMDistribution (k,n,c);
    y = (c + d) / 2;
    cx *= p - SMMDistribution (k,n,y);
    if (!cx)
      return y;
    else if (cx > 0)
      c = y;
    else
      d = y;
   } while (fabs (c - d) > 1E-6);

 return y;
}

double MODIFICATOR SMMDistribution (double k,double nu,double q)
//
// The function for computation probabilities of the studentized maximum of modules.
// Designations:
//   k - parameter,
//  nu - number degree of freedom,
//   q - parameter.
// Return code:
//  Value of probability of the studentized maximum of modules.
//
{
 long i,                                // Counter
      n = 10000;                        // Number of intervals integration

 double a = 0.000000000000001,          // Limits integration
        b = 100,
        s1 = 0,s2 = 0,                  // Work variable
        x1,x2,                          // Current value of variable
        h,                              // Double step integration
        cons = 0.5 * nu * log(nu) -     // The constant part of the density formula
               LogGamma(0.5 * nu) -
               (0.5 * nu - 1) * log(2);

 ////////////////////////////////////////////
 // Computation of integral by Simpson method
 h = 2 * (b - a) / n;
 x1 = a + h;
 x2 = a + h / 2;

 for (i = 1; i < n - 1; i += 2, x1 += h, x2 += h)
   {
    s1 += SMMDensity (x1,q,k,nu,cons);
    s2 += SMMDensity (x2,q,k,nu,cons);
   }

 return h * (SMMDensity (a,q,k,nu,cons) + SMMDensity (b,q,k,nu,cons) +
        2 * s1 + (s2 + SMMDensity (x2,q,k,nu,cons)) * 4) / 6;
}

double MODIFICATOR SMMDensity (double x,double q,double k,double n,double cons)
//
// The function for computation probability densities of the studentized maximum of modules.
// Designations:
//     x - variable integration,
//     q - parameter,
//     k - parameter,
//     n - number degree of freedom,
//  cons - constant part of formula.
// Return code:
//  value of probability densities of the studentized maximum of modules
//
{
 double corr = 2.0 * NormalDistribution (q * x);

 if (corr <= 1)
   corr = 1.000000000000001;

 return exp (cons + (n - 1) * log(x) - 0.5 * n * x * x + log(corr - 1) * k);
}

double MODIFICATOR GammaIncomlete(double a,double x)
//
// The function for computation of incomplete gamma function.
// Designations:
//  a - parameter,
//  x - parameter.
// Return code:
//  value of incomplete gamma function.
// References:
//  Brandt S. Data Analysis. Statistical and computational methods for scientists and engineers. - New York, NY: Springer-Verlag, 2003.
//
{
 double i,                   // Counter
        f,a0,a1,b0,b1,anum,  // Work variables
        s,a2j,a2j1,b2j,b2j1,
        fn,cf,cfnew = 0,h;

 if(x <= a + 1)
   {
    f = 1 / a;
    s = f;
    anum = a;
    i = 0;
    while (i < 100)
      {
       f = x * f / (++anum);
       s += f;
       if (f < 0.000001)
         break;
       i++;
      }
    if (x < 0.000001)
      return 0;
    else
      {
       h = a * log(x) - x - LogGamma(a);
       if (fabs(h) >= 500)
         return 0;
       else
         return s * exp(h);
      }
   }
 else
   {
    a0 = 0;
    b0 = 1;
    a1 = 1;
    b1 = x;
    cf = 1;
    fn = 1;
    i = 1;
    while (i <= 100)
      {
       a2j = i - a;
       a2j1 = i;
       b2j = 1;
       b2j1 = x;
       a0 = (b2j * a1 + a2j * a0) * fn;
       b0 = (b2j * b1 + a2j * b0) * fn;
       a1 = b2j1 * a0 + a2j1 * a1 * fn;
       b1 = b2j1 * b0 + a2j1 * b1 * fn;
       if (b1)
         {
          fn = 1 / b1;
          cfnew = a1 * fn;
          if (fabs(cf - cfnew) / cf < 0.000001)
            break;
          cf = cfnew;
         }
       i++;
      }
    h = a * log(x) - x - LogGamma(a);
    if (fabs(h) >= 500)
      return 1.0;
    else
      return 1.0 - exp(h) * cfnew;
   }
}

double MODIFICATOR BDistribution (double aa,double bb,double xx)
//
// The function for computation regularized incomplete function --distribution.
// Designations:
//  aa - parameter,
//  bb - parameter,
//  xx - parameter.
// Return code:
//  value of regularized incomplete function B-distribution.
// References:
//  Brandt S. Data Analysis. Statistical and computational methods for scientists and engineers. - New York: Springer-Verlag, 2003.
//
{
 long flag;          // Flag of selection parameters of computation

 double m,           // Counter of infinite loop
        a,b,x,       // Parameters distribution
        d2m,d2m1,    // coefficients  of approximation
        a1,a2,b1,b2, // Parameters computation of the chain fraction
        ap,          // Work Parameter
        cf,          // Value of function distribution
        cfn,         // Value of function distribution on the previous step
        fn;          // The normalizing multiplier

 ////////////////////////////////////////////////
 // Selection parameters for ensuring convergence
 if (xx < (aa + 1) / (aa + bb + 1))
   {
    flag = 0;
    a = aa;
    b = bb;
    x = xx;
   }
 else
   {
    flag = 1;
    a = bb;
    b = aa;
    x = 1 - xx;
   }
 //////////////////////////////////
 // Outside the area of definitions
 if (x <= 0)
   cf = 0;
 else
   {
    /////////////////
    // Chain fraction
    a1 = 1;
    b1 = 1;
    a2 = 1;
    b2 = 1 - (a + b) * x / (a + 1);
    fn = 1 / b2;
    cf = a2 * fn;
    m = 1;
    while (m <= 100)
      {
       ap = a + 2 * m;
       d2m = m * (b - m) * x / (ap - 1) / ap;
       d2m1 = -(a + m) * (a + b + m) * x / ap / (ap + 1);
       a1 = (a2 + d2m * a1) * fn;
       b1 = (b2 + d2m * b1) * fn;
       a2 = a1 + d2m1 * a2 * fn;

       if ((b2 = b1 + d2m1 * b2 * fn) != 0)
         {
          //////////////////
          // Renormalization
          fn = 1 / b2;
          cfn = a2 * fn;

          /////////////////////////////////////////////////////
          // Checking the achievement of the specified accuracy
          if (fabs(cf - cfn) < 1E-12)
            break;
          cf = cfn;
         }
        m++;
      }
    if (fabs(Beta(a,b)) > 1.0E-36)
      cf *= PowerOf(x,a) / a * PowerOf(1 - x,b) / Beta(a,b);
    else
      cf *= exp((a + b - 0.5) * log(a + b) -(a - 0.5) * log(a) -(b - 0.5) * log(b) +
            a * log(x) + b * log(1 - x) - log(a) -log(2.0 * M_PI) / 2);
   }
 if (flag != 0)
   return 1 - cf;
 else
   return cf;
}

double Gamma (double a, double x)
//
// The function for computation of incomplete Gamma-function.
// Designations:
//  a - parameter,
//  x - parameter.
// Return code:
//  value of incomplete Gamma-function.
// References:
//  Brandt S. Data Analysis. Statistical and computational methods for scientists and engineers. - New York: Springer-Verlag, 2003.
//
{
 double i,             // Counter of infinite loop
        cf,            // Value of function
        cfn,           // Value of function on the previous step
        f,s,an,h,      // Working parameters of decomposition
        a0,a1,b0,b1,   // Parameters of computation of the chain fraction
        a2,a21,b2,b21, // Coefficients of approximation
        fn;            // The normalizing multiplier

 /////////////////////////////////////
 // Selection formula of approximation
 if (x <= a + 1)
   {
    /////////////////////
    // Expansion in a row
    f = 1 / a;
    s = f;
    an = a;
    for (i = 1;; i++)
      {
       f *= x / ++an;
       s += f;
       if (f < 1E-12)
         break;
      }
    if (x < 1E-12)
      return 0;
    else
      {
       h = a * log (x) - x - LogGamma(a);
       if (fabs (h) >= 500)
         return 0;
       else
         return s * exp(h);
      }  
   }
 else
   {
    /////////////////////
    // The chain fraction
    a0 = 0;
    b0 = 1;
    a1 = 1;
    b1 = x;
    cf = 1;
    fn = 1;
    for(i = 1;;i++)
      {
       a2 = i - a;
       a21 = i;
       b2 = 1;
       b21 = x;
       a0 = (b2 * a1 + a2 * a0) * fn;
       b0 = (b2 * b1 + a2 * b0) * fn;
       a1 = b21 * a0 + a21 * a1 * fn;
       
       if ((b1 = b21 * b0 + a21 * b1 * fn) != 0)
         {
          //////////////////
          // Renormalization
          fn = 1 / b1;
          cfn = a1 * fn;
          
          ////////////////////////////////////////
          // -------- --------- -------- --------
          if (fabs(cf - cfn) < 1E-12)
            break;
          cf = cfn;  
         }
      }
    h = a * log(x) - x - LogGamma(a);
    if (fabs(h) >= 500)
      return 1;
    else
      return 1 - exp(h) * cfn;  
   } 
}

double MODIFICATOR FDistribution (double n1,double n2,double f)
//
// The function for computation of function F-distribution.
// Designations:
//  n1 - parameter,
//  n2 - parameter,
//   f - parameter.
// Return code:
//  value of function F-distribution.
// References:
//  Bolshev L.N., Smirnov N.V. Tables of mathematical statistics (in Russian). - Moscow: Nauka, 1983.
//
{
 return BDistribution (0.5 * n2,0.5 * n1,n2 / (n2 + n1 * f));
}

double MODIFICATOR InverseFDistribution (double n1,double n2,double p)
//
// The function for computation inverse of function F-distribution.
// Designations:
//  n1 - parameter,
//  n2 - parameter,
//   p - parameter.
// Return code:
//  value inverse of function F-distribution.
// References:
//  Bolshev L.N., Smirnov N.V. Tables of mathematical statistics (in Russian). - Moscow: Nauka, 1983.
//
{
 double c = 0,     // Left bound of root localization
        d = 10000, // Right bound of root localization
        y,         // The middle of interval localization
        cx;        // Values of function on end of interval

 //////////////////////////////////////////////////////////////
 // Refinement of the root method of dividing a segment in half
 do
   {
    cx = p - FDistribution (n1,n2,c);
    y = (c + d) / 2;
    cx *= p - FDistribution (n1,n2,y);
    if (!cx)
      return y;
    else if (cx > 0)
      c = y;
    else
      d = y;
   } while (fabs (c - d) > 1E-12);

 return y;
}

double MODIFICATOR ChiSquareDistribution (double n,double f)
//
// The function for computation of function chi-square distribution.
// Designations:
//  n - parameter,
//  f - parameter.
// Return code:
//  value of function chi-square distribution.
//
{
 return 1.0 - Gamma (0.5 * n,0.5 * f);
}

double MODIFICATOR InverseChiSquareDistributionNC (double n,double a,double f)
//
// The function for computation inverse of function non-central chi-square distribution.
// Designations:
//  n - parameter,
//  a - parameter non-centrality.
//  f - parameter.
// Return code:
//  value inverse of function non-central chi-square distribution.
//
{
 double n2a = 2 * a + n; // Work variable
 double n3a = n2a + a;   // Work variable
 double s = n2a / n3a;   // Work variable

 return InverseChiSquareDistribution (n2a * s * s,f) * n3a / n2a - a * a / n3a;
}

double MODIFICATOR ChiSquareDistributionNC (double n,double a,double x)
//
// The function for computation of the function non-central chi-square distribution.
// Designations:
//  n - parameter,
//  a - parameter non-centrality,
//  x - value inverse of function distribution.
// Return code:
//  value of the function non-central chi-square distribution.
//
{
 double c = 0, // Left bound of root localization
        d = 1, // Right bound of root localization
        y,     // The middle of interval localization
        cx;    // Values of function on end of interval

 //////////////////////////////////////////////////////////////
 // Refinement of the root method of dividing a segment in half
 do
   {
    cx = x - InverseChiSquareDistributionNC (n,a,c);
    y = (c + d) / 2;
    cx *= x - InverseChiSquareDistributionNC (n,a,y);
    if (cx == 0)
      return y;
    else if (cx > 0)
      c = y;
    else
      d = y;
   } while (fabs (c - d) > 0.000001);

 return y;
}

double MODIFICATOR TDistribution (double n,double t)
//
// The function for computation of function t-distribution.
// Designations:
//  n - parameter,
//  t - parameter.
// Return code:
//  value of function t-distribution.
//
{
 if (t >= 0)
   return 1 - BDistribution(0.5 * n,0.5,n / (n + t * t)) / 2;
 else
   return BDistribution(0.5 * n,0.5,n / (n + t * t)) / 2;
}

double MODIFICATOR InverseTDistribution (double n,double p)
//
// The function for computation inverse of function t-distribution.
// Designations:
//  n - non-integer parameter,
//  p - parameter.
// Return code:
//  value inverse of function t-distribution.
//
{
 double c = 0,     // Left bound of root localization
        d = 10000, // Right bound of root localization
        y,         // The middle of interval localization
        cx;        // Values of function on end of interval

 //////////////////////////////////////////////////////////////
 // Refinement of the root method of dividing a segment in half
 do
   {
    cx = p - TDistribution (n,c);
    y = (c + d) / 2;
    cx *= p - TDistribution (n,y);
    if (!cx)
      return y;
    else if (cx > 0)
      c = y;
    else
      d = y;
   } while (fabs (c - d) > 0.000001);

 return y;
}

double MODIFICATOR InverseChiSquareDistribution (double n,double p)
//
// The function for computation inverse of function chi-square distribution.
// Designations:
//  n - non-integer parameter,
//  p - parameter.
// Return code:
//  value inverse of function chi-square distribution.
//
{
 double c = 0,      // Left bound of root localization
        d = 100000, // Right bound of root localization
        y,          // The middle of interval localization
        cx;         // Values of function on end of interval

 //////////////////////////////////////////////////////////////
 // Refinement of the root method of dividing a segment in half
 do
   {
    cx = p - ChiSquareDistribution (n,c);
    y = (c + d) / 2;
    cx *= p - ChiSquareDistribution (n,y);
    if (!cx)
      return y;
    else if (cx > 0)
      c = y;
    else
      d = y;
   } while (fabs (c - d) > 0.000001);

 return y;
}

double MODIFICATOR Beta (double a,double b)
//
// The function for computation of B-function of valid argument.
// Designations:
//  a - argument,
//  b - argument.
// Return code:
//  value of B-function.
//
{
 return exp (LogGamma(a) + LogGamma(b) - LogGamma(a + b));
}

double MODIFICATOR LogGamma (double x)
//
// The function for computation of logarithm Gamma-function of valid argument.
// Designations:
//  x - argument.
// Return code:
//  value of logarithm of Gamma-function.
//
{
 long i;                                     // Counter

 double s = 1,                               // Sum of the row
        c[] = {76.18009173,-86.50532033,     // Row coefficients
               24.01409822,-1.231739516,
               0.120858003E-2,-0.536382E-5};

 for (i = 0; i <= 5; i++)
   s += c[i] / (x + i);

 return log (2.50662827463100050241 * s) - x - 4.5 + (x - 0.5) * log(x + 4.5);
}

double MODIFICATOR PowerOf (double x, double y)
//
// The function for computation of power of number.
// Designations:
//  x - number,
//  y - degree.
// Return code:
//  power of number.
//
{
 if (y == 0)
   return 1.0;
 else if (x == 0)
   return 0;
 else
   return exp (y * log (x));
}

double MODIFICATOR NormalDistribution (double x)
//
// The function for computation of standard of normal distribution.
// Designations:
//  x - standardized random variable.
// Return code:
//  value of function of standard normal distribution.
// References:
//  Bolshev L.N., Smirnov N.V. Tables of mathematical statistics (in Russian). - Moscow: Nauka, 1983.
//
{
 if (x >= 0)
   return 0.5 + 0.5 * Gamma(0.5,0.5 * x * x);
 else
   return 0.5 - 0.5 * Gamma(0.5,0.5 * x * x);
}

double MODIFICATOR LambdaDistribution (double y)
//
// The function for computation Kolmogorov distribution for simple hypotheses.
// Designations:
//  y - The function distribution.
// Return code:
//  significance level.
// References:
//  Conover W.J. Practical nonparametric statistics. - New York, NY: John Wiley & Sons, 1999.
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//
{
 double a = 0; // Return code

 if (y <= 0)
   return 0;

 for (long k = -15; k <= 15; k++)
   if (Parity(k))
     a += exp (-2.0 * k * k * y * y);
   else
     a -= exp (-2.0 * k * k * y * y);

 return a;
}

long MODIFICATOR Parity (long n)
//
// The function integer parity checks.
// Designations:
//  n - number.
// Return code:
//  0 - if number is even,
//  1 - if number is odd.
//
{
 if (n / 2 * 2 == n)
   return 1;
 else
   return 0;
}

double MODIFICATOR InverseNormalDistribution (double p)
//
// The function for computation inverse of function of normal distribution.
// Designations:
//  p - parameter.
// Return code:
//  value inverse of function of normal distribution.
// References:
//  Bolshev L.N., Smirnov N.V. Tables of mathematical statistics (in Russian). - Moscow: Nauka, 1983.
//
{
 double c = -9, // Left bound of root localization
        d = 9,  // Right bound of root localization
        y,      // The middle of interval localization
        cx;     // Values of function on end of interval

 ////////////////////////////////////////////////////////////////
 // Refinement of the root method of dividing a segment in a half
 do
   {
    cx = p - NormalDistribution (c);
    y = (c + d) / 2;
    cx *= p - NormalDistribution (y);
    if (!cx)
      return y;
    else if (cx > 0)
      c = y;
    else
      d = y;
   } while (fabs (c - d) > 0.000001);

 return y;
}

double MODIFICATOR InverseSerialDistribution (long m,long n,long u)
//
// The function for computation the significance level of the series test.
// Designations:
//  m - size of members 1st group,
//  n - size of members 2nd group,
//  u - series test value.
// Return code:
//  significance level.
//
{
 long k,       // Parameter of summations
      j;       // Counter

 double s = 0; // Accumulated value

 for (j = 2; j <= u; j++)
   {
    if (Parity (j))
      {
       k = j / 2;
       s += 2 * Combination (m - 1,k - 1) *
                Combination (n - 1,k - 1);
      }
    else
      {
       k = (j + 1) / 2;
       s += Combination (m - 1,k) * Combination (n - 1,k - 1) +
            Combination (m - 1,k - 1) * Combination (n - 1,k);
      }
   }

 return s / Combination (m + n,m);
}

double MODIFICATOR NormalIntegral (double x)
//
// The function for computation of normal integral.
// Designations:
//  x - standardized random variable.
// Return code:
//  value of the functionof of normal integral .
//
{
  return (NormalDistribution(x) - 0.5) * 2;
}

double MODIFICATOR SmirnovGrubbsDistribution (long n,double a)
//
// The function for computation of critical value Smirnov-Grubbs test.
// Designations:
//  n - number of observations,
//  a - significance level.
// Return code:
//  critical one-sided value.
// References:
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//
{
 double t = InverseTDistribution (n - 2,a / n); // Work Parameter

 t = t * t;

 return sqrt (t / (t + n - 2)) * (n - 1) / sqrt (n);
}

double MODIFICATOR ThompsonDistribution (long n, double a)
//
// The function for computation of critical value of Thompson rule.
// Designations:
//  n - number of observations,
//  a - significance level.
// Return code:
//  critical value.
// References:
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//
{
 double t = InverseTDistribution (n - 2,1 - 0.5 * a); // Work Parameter

 t *= t;

 return sqrt (t * (n - 1) / (t + n - 2));
}

double MODIFICATOR GammaDistribution (double x,double a,double b,double c)
//
// The function for computation of generalized Gamma-distribution.
// Designations:
//  x - parameter,
//  a - parameter,
//  b - parameter,
//  c - parameter.
// Return code:
//  value of function of generalized Gamma-distribution.
{
 return Gamma (a, (x - c) / b);
}

double MODIFICATOR Arsh (double a)
//
// The function for computation of hyperbolic arcsinus.
// Designations:
//  a - parameter.
// Return code:
//  value of hyperbolic arcsinus.
{
 return log (a + sqrt(a * a + 1));
}

double MODIFICATOR RangeStudentDistribution (double q,long n,long f)
//
// The function for computation probabilities of a studentized range
// of sample from a normal population.
// Designations:
//  q - studentized scope,
//  n - sample size of definitions of range,
//  f - sample size of definitions variations.
// Return code:
//  Probability that the range of the sample will not exceed q.
// References:
//  Owen D.B. Handbook of statistical tables. - Reading, MA: Addison-Wesley, 1962.
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//  Harter H.L. Tables for the use of range and studentized range in tests of hypotheses / H.L. Harter, N. Balakrishnan. - Boca Raton, FL: CRC Press LLC, 1998.
//  Baum J.J. Percentage points of the studentized range test for dispersion of normal means / J.J. Baum, H.J. Chen, M. Xiong // Journal of Statistical Computation and Simulation. - 1993. - Vol. 44. - No. 3. - P. 149-163.
//  Copenhaver M.D. Computation of the distribution of the maximum studentized range statistic with application to multiple significance testing of simple effects / M.D. Copenhaver, B.S. Holland // Journal of Statistical Computation and Simulation. - 1988. - Vol. 30. - No. 1. - P. 1-15.
//  Currie I.D. On the distribution of the studentized range in a single normal sample // Scandinavian Journal of Statistics. - 1980. - No. 7. - P. 150-154.
//  Gleason J.R. An accurate, non-iterative approximation for studentized range quantiles // Computational statistics & data analysis. - 1999. - Vol. 31. - No. 2. - P. 147-158.
//
{
 long i,                        // Counter
      n1 = 500;                 // Number of intervals integration

 double a = 0.000000000000001,  // Limits integration
        b = 10.000000000000001,
        s1 = 0,s2 = 0,          // Work variable
        x1,x2,                  // Current value of variable
        h,                      // Double step integration
        c[] =                   // Threshold values
              {8.929,13.44,16.36,18.49,20.15,21.51,22.64,23.62,24.48,
               25.24,25.92,26.54,27.10,27.62,28.10,28.54,28.96,29.35,29.71},
        d[] =                   // coefficients of regression
              {0.89326883,1.34139749,1.63270350,1.84506031,2.01134877,
               2.14668101,2.25920652,2.35710800,2.44305420,2.51938913,
               2.58709453,2.64917496,2.70595180,2.75694765,2.80477208,
               2.84883799,2.89057797,2.92956265,2.96534101},
        Res;                    // Return code

 ////////////////////////////////////////////
 // Approximation for large values q by f = 1
 if (f == 1)
   {
    if (n < 20)
      {
       /////////////////////////////
       // Approximation by hyperbola
       if (q > c[n - 2])
         return 1.0 - d[n - 2] / q;
      }
    else
      {
       ////////////////////////////////////////
       // Approximation by complicated function
       if (q > 39.91)
         return 1.0 - (2.3673883676 * log (log10 (n)) + 2.3285514134) / q;
      }
   }

 ////////////////////////////////////////////
 // Computation on integral by Simpson method
 h = 2 * (b - a) / n1;
 x1 = a + h;
 x2 = a + h / 2;
 for (i = 1; i < n1 - 1; i += 2, x1 += h, x2 += h)
   {
    s1 += RangeStudentDensity (x1,q,n,f);
    s2 += RangeStudentDensity (x2,q,n,f);
   }

 Res = (RangeStudentDensity (a,q,n,f) + RangeStudentDensity (b,q,n,f) +
        2 * s1 + 4 * (s2 + RangeStudentDensity (x2,q,n,f))) / 6 * h *
        exp(LOG_2 + 0.5 * f * (log (f) - LOG_2) - LogGamma (0.5 * f));

 if (Res > 1)
   Res = 0.99999;

 return Res;
}

double MODIFICATOR RangeStudentDensity (double x,double q,long n,long f)
//
// The function for computation probability densities of a studentized
// range of sample from a normal population.
// Designations:
//  x - variable integration,
//  q - studentized range,
//  n - sample size of definitions of range,
//  f - sample size of definitions Variations.
// Return code:
//  value of probability densities.
// References:
//  Owen D.B. Handbook of statistical tables. - Reading, MA: Addison-Wesley, 1962.
//  Currie I.D. On the distribution of the studentized range in a single normal sample // Scandinavian Journal of Statistics. - 1980. - No. 7. - P. 150-154.
//
{
 return exp ((f - 1) * log (x) + log (RangeDistribution (q * x,n)) - 0.5 * f * (x * x));
}

double MODIFICATOR RangeDistribution (double w,long n1)
//
// The function for computation of probability of range of sample from a normal population.
// Designations:
//   w - range,
//  n1 - sample size.
// Return code:
//  Probability that the range of sample will not exceed w.
// References:
//  Owen D.B. Handbook of statistical tables. - Reading, MA: Addison-Wesley, 1962.
//
{
 long i,                // Counter
      n = 100;          // Number of intervals integration

 double a = -10,b = 10, // Limits integration
        s1 = 0,s2 = 0,  // Work variable
        x1,x2,          // Current value of variable
        h;              // Double step integration

 ////////////////////////////////////////////
 // Computation on integral by Simpson method
 h = 2 * (b - a) / n;
 x1 = a + h;
 x2 = a + h / 2;
 for (i = 1; i < n - 1; i += 2, x1 += h, x2 += h)
   {
    s1 += RangeDensity (x1,w,n1);
    s2 += RangeDensity (x2,w,n1);
   }

 return ((RangeDensity (a,w,n1) + RangeDensity (b,w,n1) +
        s1 * 2 + (s2 + RangeDensity (x2,w,n1)) * 4) / 6 * h) * n1 / 2.5066282;
}

double MODIFICATOR RangeDensity (double x,double w,long n)
//
// The function for computation of probability densities of range of sample from a normal population.
// Designations:
//  x - variable integration,
//  w - range,
//  n - sample size.
// Return code:
//  value of probability densities.
// References:
//  Owen D.B. Handbook of statistical tables. - Reading, MA: Addison-Wesley, 1962.
//
{
 return pow (NormalDistribution (x + w) - NormalDistribution (x),n - 1) / exp (x * x / 2);
}

double MODIFICATOR GammaInteger (double n)
//
// The function computes Gamma-function whole or half-whole argument.
// Designations:
//  n - argument.
// Return code:
//  value of Gamma-function.
//
{
 double a,    // Work variable, later step
        seed; // Return code

 ////////////////////////////////////////
 // Selection between whole or half-whole
 if (!modf (n,&a))
   {
    seed = 1;
    a = 2;
   }
 else
   {
    seed = 1.7724538;
    a = 1.5;
   }

 while (a <= n)
   {
    seed *= (a - 1);
    a++;
   }

 return seed;
}

double MODIFICATOR DunnDistribution (double x, long k)
//
// The function for computation distribution of Dunn test.
// Designations:
//  x - parameter.
// Return code:
//  value of function distribution.
//
{
 double c = 0.000001, // Left bound of root localization
        d = 0.999999, // Right bound of root localization
        y,            // The middle of interval localization
        cx;           // Values of function on end of interval

 //////////////////////////////////////////////////////////////
 // Refinement of the root method of dividing a segment in half
 do
   {
    cx = x - fabs(InverseNormalDistribution (c / k / (k - 1)));
    y = (c + d) / 2;
    cx *= x - fabs(InverseNormalDistribution (y / k / (k - 1)));
    if (!cx)
      return y;
    else if (cx > 0)
      c = y;
    else
      d = y;
   } while (fabs (c - d) > 0.000000001);

 return y;
}

double MODIFICATOR CochranGDistribution (double n1,double n2,double x, long k)
//
// The function for computation distribution Cochran G-test.
// Designations:
//  x - parameter.
// Return code:
//  value of function distribution.
//
{
 double c = 0.000001, // Left bound of root localization
        d = 0.999999, // Right bound of root localization
        y,            // The middle of interval localization
        cx;           // Values of function on end of interval

 ////////////////////////////////////////////////////////////////
 // Refinement of the root method of dividing a segment in a half
 do
   {
    cx = x - InverseFDistribution (n1,n2,c / k);
    y = (c + d) / 2;
    cx *= x - InverseFDistribution (n1,n2,y / k);
    if (!cx)
      return y;
    else if (cx > 0)
      c = y;
    else
      d = y;
   } while (fabs (c - d) > 0.000000001);

 return y;
}

double MODIFICATOR KuiperDistribution (double y)
//
// The function for computation Kuiper distribution for simple hypotheses.
// Designations:
//  y - The function distribution.
// Return code:
//  significance level of hypothesis.
//
{
 long k;       // Counter

 double a = 0, // Return code
        s;     // Work variable

 if (y <= 0)
   return 0;

 for (k = 1; k <= 15; k++)
   {
    s = 2.0 * k * k * y * y;
    a += (exp (-s) * (s * 2 - 1));
   }

 return 1.0 - a;
}

double MODIFICATOR McNemarDistribution (double b,double c)
//
// The function for computation of function distribution of McNemar test.
// Designations:
//  b - parameter,
//  c - parameter.
// Return code:
//  value of function distribution.
//
{
 return exp ((b + c) * log (0.5) - LogFactorialD(b) + LogFactorialD(b + c) - LogFactorialD(c));
}


double MODIFICATOR BinomialDensity(long k, long n,double p)
//
// The function for computation a probabilities of binomial distribution.
// Designations:
//  k - parameter,
//  n - parameter,
//  p - parameter.
// Return code:
//  value of probabilities of binomial distribution.
// References:
//  Brandt S. Data Analysis. Statistical and computational methods for scientists and engineers. - New York: Springer-Verlag, 2003.
//
{
 double ak, an, albinc, alfact, absarg, arg; // Work parameters

 ak = 1.0 * k;
 an = 1.0 * n;

 if (n < 1 || k < 0 || k > n)
   return 0;
 else
   {
    albinc = LogEuler(an + 1) - LogEuler(ak + 1) - LogEuler(an - ak + 1);
    if (p <= 0)
      {
       if (k == 0)
         return 1;
       else
         return 0;
      }
    else if (p >= 1)
      {
       if (k == n)
         return 1;
       else
         return 0;
      }
    else
      {
       alfact = log(p) * ak + log(1.0 - p) * (an - ak);
       arg = alfact + albinc;
       absarg = fabs(arg);
       if (absarg < 1e-4)
         return 1;
       else if (absarg > 100.0)
         return 0;
       else
         return exp(arg);
      }
   }
}

double MODIFICATOR BinomialDistribution(long k, long n, double p)
//
// The function for computation of function of binomial distribution.
// Designations:
//   k - parameter,
//   n - parameter,
//  p - parameter.
// Return code:
//  value of function of binomial distribution.
// References:
//  Brandt S. Data Analysis. Statistical and computational methods for scientists and engineers. - New York: Springer-Verlag, 2003.
//
{
 long i,j;     // Counters

 double s = 0; // Accumulated value

 if (k > 0)
   {
    i = k - 1;
    for(j = 0; j <= i; ++j)
      s += BinomialDensity(j, n, p);
   }

 return (1.0 - s);
}

double MODIFICATOR LogEuler(double x)
//
// The function for computation of natural logarithm of Euler Gamma-function.
// Designations:
//   x - parameter.
// Return code:
//  value of function of natural logarithm of Euler Gamma-function.
// References:
//  Brandt S. Data Analysis. Statistical and computational methods for scientists and engineers. - New York: Springer-Verlag, 2003.
//
{
 long i,                                                // Counter
      Flag;                                             // Flag

 double anum, g, s, xh, xx, xgh,                        // Work Parameters
        c[6] = {76.18009173,-86.50532033,24.01409822,   // Constants of row
                -1.231739516,.00120858003,-5.36382e-6};

 if (x >= 1)
   {
    Flag = 0;
    xx = x - 1;
   }
 else
   {
    Flag = 1;
    xx = 1 - x;
   }
 xh = xx + 0.5;
 xgh = xx + 5.5;
 s = 1;
 anum = xx;
 for(i = 1; i <= 6; ++i)
   s += c[i - 1] / ++anum;
 s *= 2.506628275;
 g = xh * log(xgh) + log(s) - xgh;
 if (Flag == 1)
   return log(xx * M_PI) - g - log(sin(xx * M_PI));
 else
   return g;
}

double MODIFICATOR LogFactorialD (double n)
//
// The function computes logarithm of factorial.
// Designations:
//   n - number.
// Return code:
//  value of logarithm of factorial.
//
{
 double i,       // Counter
        Ret = 0; // Return code

 i = 1;
 while (i <= n)
   {
    Ret += log (i);
    i++;
   }

 return Ret;
}

double MODIFICATOR LogNormalDistribution (double x,double a,double b)
//
// The function for computation of function of lognormal distribution.
// Designations:
//  x - parameter,
//  a - parameter,
//  b - parameter.
// Return code:
//  value of function of lognormal distribution.
{
 return NormalDistribution((log (x) - a) / b);
}

double MODIFICATOR SuJohnsonDistribution (double x,double a,double b,double c,double d)
//
// The function for computation of function Johnson distribution Su.
// Designations:
//  x - parameter,
//  a - parameter,
//  b - parameter,
//  c - parameter,
//  d - parameter.
// Return code:
//  value of function Johnson distribution Su.
// References:
//  Hahn G.J., Shapiro S.S. Statistical models in engineerting. - New York, NY: John Wiley & Sons, 1967.
//
{
 return 1.0 - NormalDistribution(a + b * Arsh ((x - c) / d));
}

double MODIFICATOR GammaIncomplete(double a,double x)
//
// The function for computation of incomplete Gamma-function.
// Designations:
//  a - parameter,
//  x - parameter.
// Return code:
//  value of incomplete Gamma-function.
// References:
//  Brandt S. Data Analysis. Statistical and computational methods for scientists and engineers. - New York: Springer-Verlag, 2003.
//
{
 double i,                   // Counter
        f,a0,a1,b0,b1,anum,  // Work variables
        s,a2j,a2j1,b2j,b2j1,
        fn,cf,cfnew = 0,h;

 if(x <= a + 1)
   {
    f = 1.0 / a;
    s = f;
    anum = a;
    i = 0;
    while (i < 100)
      {
       f = x * f / (++anum);
       s += f;
       if (f < 0.000001) break;
       i++;
      }
    if (x < 0.000001)
      return 0;
    else
      {
       h = a * log(x) - x - LogGamma(a);
       if (fabs(h) >= 500)
         return 0;
       else
         return s * exp(h);
      }
   }
 else
   {
    a0 = 0;
    b0 = 1;
    a1 = 1;
    b1 = x;
    cf = 1;
    fn = 1;
    i = 1;
    while (i <= 100)
      {
       a2j = i - a;
       a2j1 = i;
       b2j = 1;
       b2j1 = x;
       a0 = (b2j * a1 + a2j * a0) * fn;
       b0 = (b2j * b1 + a2j * b0) * fn;
       a1 = b2j1 * a0 + a2j1 * a1 * fn;
       b1 = b2j1 * b0 + a2j1 * b1 * fn;
       if (b1)
         {
          fn = 1 / b1;
          cfnew = a1 * fn;
          if (fabs(cf - cfnew) / cf < 0.000001)
            break;
          cf = cfnew;
         }
       i++;
      }
    h = a * log(x) - x - LogGamma(a);
    if (fabs(h) >= 500)
      return 1.0;
    else
      return 1.0 - exp(h) * cfnew;
   }
}

double MODIFICATOR AbbeDistribution(double m, long n)
//
// The function for computation of Abbe test distribution.
// Designations:
//  m - Abbe statistic
//  n - size
// Return code:
//  value of function of Abbe test distribution.
// References:
//  Bolshev L.N., Smirnov N.V. Tables of mathematical statistics (in Russian). - Moscow: Nauka, 1983.
//
{
 double e1 = 1.0 - m / 2,
        ebar, t, p,
        v = n + 1;

 ebar = e1 * sqrt((n * n - 1) / (n * n - 4));
 t = sqrt(1.0 + n) * ebar / sqrt(1.0 - e1 * e1);
 t = fabs(t);
 p = 1.0 - (0.447428865 + 0.570294294 * t + 0.043600206 / v - 0.19977566 * t * t +
     0.318439107 / v / v - 0.25529723 * t / v + 0.023582022 * t * t * t -
     0.15260372 / v / v / v - 0.21264039 * t / v / v + 0.083785374 * t * t / v);

 if (p < 0)
   return 0;

 return p;
}

double MODIFICATOR A1 (double x)
//
// The function for computation of function distribution A1 type of omega-square.
// Designations:
//  z - argument.
// Return code:
//  value of function distribution A1.
// References:
//  Bolshev L.N., Smirnov N.V. Tables of mathematical statistics (in Russian). - Moscow: Nauka, 1983.
//
{
 long j;                   // Counter

 double a = 0,             // Return code
        g = LogGamma(0.5); // Constant

 if (x < 0.01)
   return 0;

 for (j = 0; j < 20; j++)
   a += exp (LogGamma(0.5 + j) - g - LogGamma(1.0 + j)) * sqrt(4.0 * j + 1) *
        exp (-(4.0 * j + 1) * (4.0 * j + 1) / 16 / x) *
        (Bessel(-0.25,(4.0 * j + 1) * (4.0 * j + 1) / 16 / x) -
         Bessel(0.25,(4.0 * j + 1) * (4.0 * j + 1) / 16 / x));

 return a / sqrt (x * 2);
}

double MODIFICATOR Bessel (double n,double z)
//
// The function for computation of modified Bessel's function of the 1st kind of valid argument.
// Designations:
//  n - argument,
//  z - argument.
// Return code:
//  value of modified Bessel's function.
//
{
 long k;       // Counter

 double b = 0;

 for (k = 0; k < 50; k++)
   b += exp ((n + 2 * k) * log(z / 2) - LogFactorial(k) - LogGamma(n + k + 1));

 return b;
}

long MODIFICATOR Sturgess(long n)
//
// The function for computation number of classes by Sturgess rule.
// Designations:
//  n - size
// Return code:
//  number of classes.
// References:
//  Scott D.W. Sturges- rule // WIREs Computational Statistics, 2009, vol. 3, pp. 303-306.
//
{
 double buf, // Work variable
        s;   // Return code

 buf = 1.44 * log(n) + 1;
 s = int(buf);
 if (buf - s >= 0.5)
   s += 1;
 if (s < 1)
   s = 1;

 return s;
}

double MODIFICATOR Gini(double x[], long n)
//
// The function for computation average of difference Gini.
// Designations:
//  x - sample
//  n - size
// Return code:
//  value of average of difference.
//
{
 long i,j;     // Counter

 double g = 0; // Return code

 for (i = 0; i < n; i++)
   for (j = 0; j < n; j++)
     g += fabs(x[i] - x[j]);

 return g / n / (n - 1);
}

double MODIFICATOR Proportion(double x[], long n)
//
// The function for computation proportion.
// Designations:
//  x - sample,
//  n - size of sample.
// Return code:
//  value of proportion.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//
{
 long i;       // Counter

 double p = 0; // Returned value

 for (i = 0; i < n; i++)
   if (x[i] == 1)
     p++;

 return p;
}

double MODIFICATOR Abbe(double x[], long n)
//
// The function for computation of Abbe test.
// Designations:
//  x - sample
//  n - size
// Return code:
//  value of Abbe test.
// References:
//  Bolshev L.N., Smirnov N.V. Tables of mathematical statistics (in Russian). - Moscow: Nauka, 1983.
//  Brownlee K.A. Statistical theory and metodology in sciences and engineering. - New York, NY: John Wiley & Sons, 1965.
//
{
 long i;                // Counter

 double xm = Mean(x,n), // Medium
        q = 0,          // Numerator
        s = 0;          // Denominator

 for (i = 0; i < n - 1; i++)
   q += (x[i + 1] - x[i]) * (x[i + 1] - x[i]);
 for (i = 0; i < n; i++)
   s += (x[i] - xm) * (x[i] - xm);

 return q / s;
}

double MODIFICATOR Moment(double x[], long n, long k)
//
// The function for computation of central moment.
// Designations:
//  x - sample
//  n - size
//  k - order of moment
// Return code:
//  value of central moment.
// References:
//  Pollard J.H. A handbook of numerical anad statistical techniques. - Cambridge: Cambridge University Press, 1981.
//
{
 long i,j;              // Counter

 double xm = Mean(x,n), // Medium
        buf,buf1,       // Work variable
        m = 0;          // Returned value

 for (i = 0; i < n; i++)
   {
    buf = x[i] - xm;
    buf1 = 1;
    for (j = 0; j < k; j++)
      buf1 *= buf;
    m += buf1;
   }

 return m / n;
}

double MODIFICATOR Aberration(double x[], long n)
//
// The function for computation of average aberration.
// Designations:
//  x - sample
//  n - size
// Return code:
//  value of average aberration.
//
{
 long i;                // Counter

 double xm = Mean(x,n), // Medium
        a = 0;          // Returned value

 for (i = 0; i < n; i++)
   a += fabs(x[i] - xm);

 return a / n;
}

double MODIFICATOR Mean(double x[], long n)
//
// The function for computation of average qualitative value.
// Designations:
//  x - sample
//  n - size
// Return code:
//  value of average value.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//
{
 long i;        // Counter

 double xm = 0; // Returned value

 for (i = 0; i < n; i++)
   xm += x[i];

 return xm / n;
}

double MODIFICATOR Rate(double x[], long n)
//
// The function for computation of average quantitative value.
// Designations:
//  x - sample
//  n - size
// Return code:
//  value of average value.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//
{
 long i;        // Counter

 double xm = 0; // Returned value

 for (i = 0; i < n; i++)
   if (x[i] == 1)
     xm++;

 return xm / n;
}

double MODIFICATOR Skewness(double x[], long n)
//
// The function for computation of skewness.
// Designations:
//  x - sample
//  n - size
// Return code:
//  value of skewness.
// References:
//  Pollard J.H. A handbook of numerical anad statistical techniques. - Cambridge: Cambridge University Press, 1981.
//
{
 long i;                      // Counter

 double xm = Mean(x,n),       // Medium
        s = sqrt(Cov(x,x,n)), // Standard
        buf,                  // Work variable
        sk = 0;               // Returned value

 for (i = 0; i < n; i++)
   {
    buf = (x[i] - xm) / s;
    buf = buf * buf * buf;
    sk += buf;
   }

 return sk * n / (n - 1) / (n - 2);
}

double MODIFICATOR Kurtosis(double x[], long n)
//
// The function for computation of kurtosis.
// Designations:
//  x - sample
//  n - size
// Return code:
//  value of kurtosis.
// References:
//  Pollard J.H. A handbook of numerical anad statistical techniques. - Cambridge: Cambridge University Press, 1981.
//
{
 long i;                      // Counter

 double xm = Mean(x,n),       // Medium
        s = sqrt(Cov(x,x,n)), // Standard
        buf,                  // Work variable
        k = 0;                // Returned value

 for (i = 0; i < n; i++)
   {
    buf = (x[i] - xm) / s;
    buf = buf * buf * buf * buf;
    k += buf;
   }

 return k * n / (n - 1) * (n + 1) / (n - 2) / (n - 3) - 3.0 * (n - 1) / (n - 2) * (n - 1) / (n - 3);
}

void MODIFICATOR PseudoMedian(double x[],long n,double p,double *med,double *low,double *upp)
//
// The function for computation of estimation of pseudomedian.
// Designations:
//    x - sample,
//    n - sample size,
//    p - confidence level,
//  med - pseudomedian (Hodges-Lehmann estimator),
//  low - low bound of interval,
//  upp - upper bound of interval.
// Return code:
//  none.
//
{
 long i,j,k = 0,           // Counter
      m = n * (n + 1) / 2, // Work variable
      c;                   // Work Parameter

 double *w;                // Work array

 w = new double[m];

 ///////////////////
 // Work computation
 for (j = 0; j < n; j++)
   for (i = 0; i <= j; i++)
     w[k++] = ((x[i] + x[j]) / 2);
 SortArrayUp0 (w,m);
 c = floor(0.25 * n * (n + 1)) - floor(InverseNormalDistribution ((p + 1) / 2) *
     sqrt(0.041666666667 * n * (n + 1) * (2 * n + 1)));
 if (c < 1)
   c = 1;

 ///////////////////////////////////////////////////
 // Computation of pseudomedian confidence intervals
 *low = w[c];
 *upp = w[m - 1 - c];

 //////////////////////////////
 // Computation of pseudomedian
 if (!Parity (m))
   *med = w[m / 2];
 else
   *med = (w[m / 2 - 1] + w[m / 2]) / 2;

 delete [] w;
}

void MODIFICATOR Median(double x[],long n,double p,double *med,double *low,double *upp)
//
// The function for computation of median estimator.
// Designations:
//    x - sample,
//    n - sample size,
//    p - confidence level,
//  med - median,
//  low - low bound of interval,
//  upp - upper bound of interval.
// Return code:
//  none.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//  Pollard J.H. A handbook of numerical anad statistical techniques. - Cambridge: Cambridge University Press, 1981.
//
{
 long i,    // Counter
      c;    // Work parameter

 double *w; // Work array

 w = new double[n];

 ///////////////////
 // Work computation
 for (i = 0; i < n; i++)
   w[i] = x[i];
 SortArrayUp0 (w,n);
 c = floor(0.5 * n - 0.5 * InverseNormalDistribution((p + 1) / 2) * sqrt(n));
 if (c < 1)
   c = 1;

 /////////////////////////////////////////////
 // Computation of median confidence intervals
 *low = c;
 *upp = n - 1 - c;

 ////////////////////////
 // Computation of median
 if (!Parity (n))
   *med = w[n / 2];
 else
   *med = (w[n / 2 - 1] + w[n / 2]) / 2;

 delete [] w;
}

void MODIFICATOR MedianVar(double x[],long n,double *med,double *low,double *upp)
//
// The function for computation of median variants.
// Designations:
//    x - sample,
//    n - sample size,
//  med - median,
//  low - Left median,
//  upp - Right median.
// Return code:
//  none.
//
{
 long i,j;        // Counter

 double *w,       // Work array
        buf,      // Work Parameter
        save = 0;

 w = new double[n];

 ////////////////////////////////
 // Computation of median variant
 *med = x[0];
 for (i = 0; i < n; i++)
   save += fabs(x[i] - x[0]);
 for (i = 1; i < n; i++)
   {
    for (j = 0, buf = 0; j < n; j++)
      buf += fabs(x[j] - x[i]);
    if (buf < save)
      {
       save = buf;
       *med = x[i];
      }
   }

 /////////////////////////////////////
 // Computation left and right medians
 for (i = 0; i < n; i++)
   w[i] = x[i];
 SortArrayUp0 (w,n);
 if (!Parity (n))
   {
    *low = w[n / 2];
    *upp = w[n / 2];
   }
 else
   {
    *low = w[n / 2 - 1];
    *upp = w[n / 2];
   }

 delete [] w;
}

void MODIFICATOR Inter(double x[],long n,double *i25,double *i75)
//
// The function for computation of median estimation.
// Designations:
//    x - sample,
//    n - sample size,
//  i25 - low quartile,
//  i75 - upper quartile.
// Return code:
//  none.
//
{
 long i,          // Counter
       m = n / 2; // Work variable

 double *y,       // Work array
        *z,       // Empirical variation series
        MeZ,      // Median
        MeY,      // Median
        Low,      // Dummy
        Upp;      // Dummy

 y = new double[m];
 z = new double[n];

 ///////////////////////////
 // Copy of empirical sample
 for (i = 0; i < n; i++)
   z[i] = x[i];

 ///////////////////////////////
 // computation of the indicator
 SortArrayUp0 (z,n);
 if (!Parity (n))
   for (i = 0; i < m; i++)
     y[i] = z[i + m - 1];
 else
   for (i = 0; i < m; i++)
     y[i] = z[i + m];
 Median(y, m, 0.95, &MeY, &Low, &Upp);
 Median(z, m, 0.95, &MeZ, &Low, &Upp);

 delete [] y; delete [] z;

 *i25 = MeZ;
 *i75 = MeY;
}

double MODIFICATOR InterRange(double x[], long n)
//
// The function for computation of interquartile range.
// Designations:
//  x - empirical sample,
//  n - sample size.
// Return code:
//  value of interquartile range.
//
{
 long i,          // Counter
       m = n / 2; // Work variable

 double *y,       // Work array
        *z,       // Empirical variation series
        MeZ,      // Median
        MeY,      // Median
        Low,      // Dummy
        Upp;      // Dummy

 y = new double[m];
 z = new double[n];

 //////////////////////////////
 // Copy of empirical of sample
 for (i = 0; i < n; i++)
   z[i] = x[i];

 ///////////////////////////////
 // computation of the indicator
 SortArrayUp0(z,n);
 if (!Parity (n))
   for (i = 0; i < m; i++)
     y[i] = z[i + m - 1];
 else
   for (i = 0; i < m; i++)
     y[i] = z[i + m];
 Median(y, m, 0.95, &MeY, &Low, &Upp);
 Median(z, m, 0.95, &MeZ, &Low, &Upp);

 delete [] y; delete [] z;

 return MeY - MeZ;
}

long MODIFICATOR ClassesOptimal(double x[], long n)
//
// The function for computation of optimal classes number.
// Designations:
//  x - empirical sample,
//  n - sample size.
// Return code:
//  optimal classes number.
//
{
 long i,j,l,  // Counter
       s,     // Array counter
       k = n; // Return code

 double *y,   // Variation series
        *m,   // Middles of classes
        *z,   // Roughened row
        p;    // Work variable, P-value

 y = new double[n];
 m = new double[n];
 z = new double[n];

 for (i = 2; i < n; i++)
   {
    Pocket(x,n,y,m,&p,i);

    for (j = 0, s = 0; j < i; j++)
      for (l = 0; l < y[j]; l++)
        z[s++] = m[j];

    KuiperExactTest (x,n,z,n,&p);
    if (p > 0.05)
      {
       k = i;
       break;
      }
   }

 delete [] y; delete [] m; delete [] z;

 return k;
}

long MODIFICATOR ShimazakiShinomoto(double x[], long n, double c[], double d[])
//
// The function for computation of optimal classes number by Shimazaki-Shinomoto method.
// Designations:
//  x - empirical sample,
//  n - sample size,
//  c - the cost function (array by length n - 2 must be reserved),
//  d - class width (array by length n - 2 must be reserved).
// Return code:
//  optimal number of classes.
// References:
//  Shimazaki H., Shinomoto S. A method for selecting the bin size of a time histogram // Neural Computation, 2007, vol. 19, no. 6, pp. 1503-1527.
//  Shimazaki H., Shinomoto S. A recipe for optimizing a time-histogram // Neural Information Processing Systems, 2007, vol. 19, pp. 1289-1296.
//
{
 long i,j,l = 0; // Counter

 double *y,      // Variation series
        *m,      // Middles of classes
        p,       // Work variable, P-value
        k,       // Medium
        s;       // Variance

 y = new double[n];
 m = new double[n];

 for (i = 2; i < n; i++, l++)
   {
    Pocket(x,n,y,m,&p,i);

    for (j = 0, k = 0; j < i; j++)
      k += y[j];
    k = k / i;
    for (j = 0, s = 0; j < i; j++)
      s += (y[j] - k) * (y[j] - k);
    s = s / i;
    d[l] = m[1] - m[0];
    c[l] = (k * 2 - s) / d[l] / d[l];
   }

 delete [] y; delete [] m;

 return MinimumOf(c,n - 2) + 2;
}

void MODIFICATOR Pocket(double *x, long n, double *w, double *m, double *Step, long k)
//
// The function builds a histogramm.
// Designations:
//      x - empirical sample,
//      n - sample size,
//      w - variation series,
//      m - middles of classes,
//  *Step - the classes interval,
//      k - number of classes.
// Return code:
//  none.
// References:
//  Gravetter F.J., Wallnau L.B. Essentials of statistics for the behavioral sciences. - Belmont, CA: Wadsworth Cengage Learning, 2011.
//
{
 long i,j;  // Counter

 double *c; // Bounds of classes

 /////////////////////////////////
 // Forming  the classes intervals
 c = new double[k + 1];
 c[0] = Minimum(x, n);
 c[k] = Maximum(x, n);
 *Step = (c[k] - c[0]) / k;
 for (j = 1; j < k; j++)
   c[j]= c[j - 1] + *Step;

 ////////////////////////////////////////
 // Separation of the variants by classes
 for (j = 0; j < k; j++)
   {
    w[j] = 0;
    for (i = 0; i < n; i++)
      if (j != k - 1)
        {
         if (x[i] >= c[j] && x[i] < c[j + 1])
           w[j] += 1;
        }
      else
        {
         if (x[i] >= c[j] && x[i] <= c[j + 1])
           w[j] += 1;
        }
   }

 /////////////////////////////////
 // computation middles of classes
 for (i = 0; i < k; i++)
   m[i] = c[i] + *Step / 2;

 delete [] c;
}

double MODIFICATOR Minimum(double *x, long n)
//
// The function for computation of minimal value of sample.
// Designations:
//  X - empirical sample,
//  N - sample size.
// Return code:
//  minimal value of sample.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//
{
 long i;            // Counter

 double Ret = x[0]; // Return code

 for (i = 1; i < n; i++)
  if (x[i] < Ret)
    Ret = x[i];

 return Ret;
}

double MODIFICATOR Maximum(double *x, long n)
//
// The function for computation of maximal values of sample.
// Designations:
//  X - empirical sample,
//  N - sample size.
// Return code:
//  maximal value of sample.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//
{
 long i;            // Counter

 double Ret = x[0]; // Return code

 for (i = 1; i < n; i++)
  if (x[i] > Ret)
    Ret = x[i];

 return Ret;
}

double MODIFICATOR WilcoxonExact (double *x,long nx,double *y,long ny,double *p)
//
// The function of exact computation of Wilcoxon W-test.
//    x - 1st sample,
//   nx - size of 1st sample,
//    y - 2nd sample,
//   ny - size of 2nd sample,
//   *p - P-value of the two-sided hypothesis.
// Return code:
//  value of test.
// References:
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//
{
 long i,l = 0,     // Counter
      *ix,         // Permutation info
      n = nx + ny; // Common size

 double *xx,*xy,   // Work array
        *xr,*yr,   // Arrays of ranks
        criterion, // Value of test
        p1;        // P-value

 ix = new long[n];
 xy = new double[n];
 xx = new double[n];
 xr = new double[nx];
 yr = new double[ny];

 ////////////////////////
 // Given arrays junction
 for (i = 0; i < nx; i++)
   xy[l++] = x[i];
 for (i = 0; i < ny; i++)
   xy[l++] = y[i];

 //////////
 // Sort up
 SortArrayUp (xy,n,ix);

 //////////
 // Ranking
 Rank (xy,xx,n);

 //////////////////////
 // Restore ranks order
 UnSortArray (xx,xy,n,ix);
 l = 0;
 for (i = 0; i < nx; i++)
   xr[i] = xy[l++];
 for (i = 0; i < ny; i++)
   yr[i] = xy[l++];

 delete [] ix; delete [] xx;  delete [] xy;

 //////////////////////
 // Computation of test
 criterion = RandomITest (xr,nx,yr,ny,&p1);
 *p = p1;

 delete [] xr; delete [] yr;

 return criterion;
}

double MODIFICATOR WilcoxonSignedExact (double *x,long n,double *y,double *p)
//
// The function of exact computation Wilcoxon T-test for related samples.
// Designations:
//   x - 1st sample,
//   n - each sample size,
//   y - 2nd sample,
//  *p - P-value of the two-sided hypothesis;
// Return code:
//  value of test.
// References:
//  Van der Waerden B.L. Mathematische Statistik (in Deutsch). - Berlin: Springer, 1971.
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//
{
 long i,           // Counter
      *ix,         // Permutation info
      n2 = 2 * n;  // Common size

 double *xx,*xy,   // Work array
        *xr,*yr,   // arrays of ranks
        criterion, // Value of test
        p1;        // P-value

 ix = new long[n2];
 xy = new double[n2];
 xx = new double[n2];
 xr = new double[n];
 yr = new double[n];

 ////////////////////////
 // Given arrays junction
 for (i = 0; i < n; i++)
   xy[i] = x[i];
 for (i = 0; i < n; i++)
   xy[i + n] = y[i];

 //////////
 // Sort up
 SortArrayUp (xy,n2,ix);

 //////////
 // Ranking
 Rank (xy,xx,n2);

 //////////////////////
 // Restore ranks order
 UnSortArray (xx,xy,n2,ix);
 for (i = 0; i < n; i++)
   xr[i] = xy[i];
 for (i = 0; i < n; i++)
   yr[i] = xy[i + n];

 delete [] ix; delete [] xx;  delete [] xy;

 //////////////////////
 // Computation of test
 criterion = RandomTest (xr,n,yr,&p1);
 *p = p1;

 delete [] xr; delete [] yr;

 return criterion;
}

double MODIFICATOR McNemarExact (double b,double c,double *p)
//
// The function of exact computation exact McNemar's test for nominal signs.
// Designations:
//   b - cell of table 2 - 2,
//   c - cell of table 2 - 2,
//  *p - P-value of the one-sided hypothesis,
//   s - flag of correction for continuity.
// Return code:
//  value of test.
// References:
//  Bennett B.M., Underwood R.E. On McNemar-s test for the 2 x 2 table and its power // Biometrics, June 1970, vol. 26, no. 2, pp. 339-343.
//  McNemar Q. Psychological statistics. - New York, NY: John Wiley & Sons, 1966.
//
{
 double i,                              // Counter
        f,                              // Test statistic
        bc = b + c,                     // Work variable
        pf = McNemarDistribution (b,c), // Probability of given configuration
        p1 = 0,p2 = 0,pc;               // Work variable

 f = fabs(b - c);

 /////////////////////////////////////////
 // Test statistic computation and P-value
 if (bc == 0)
   {
    f = 0;
    *p = 0.5;
   }
 else
   {
    f = f * f / bc;
    i = 0;
    while (i <= bc)
      {
       pc = McNemarDistribution (i,bc - i);
       if (pc >= pf)
         p1 += pc;
       if (pc <= pf)
         p2 += pc;
       *p = MIN(p1,p2) / 2;
       i++;
      }
   }

 return f;
}

double MODIFICATOR SignTest(double *x,long n,double *y,double *p)
//
// The function for computation of sign test.
// Designations:
//   x - 1st sample,
//   n - each sample size,
//   y - 2nd sample,
//  *p - P-value of the one-sided hypothesis.
// Return code:
//  value of the test.
// References:
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//  Conover W.J. Practical nonparametric statistics. - New York, NY: John Wiley & Sons, 1999.
//  Van der Waerden B.L. Mathematische Statistik (in Deutsch). - Berlin: Springer, 1971.
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//  Brownlee K.A. Statistical theory and metodology in sciences and engineering. - New York, NY: John Wiley & Sons, 1965.
//  Johnson N.L., Leone F.C. Statistics and experimental design in engineering and the physical sciences. Vol. I. - New York, NY: John Wiley & Sons, 1977.
//
{
 long i,     // Counter
      n1,    // Corrected size
      t = 0; // Value of test

 n1 = n;

 /////////////////////////////////////////
 // Test statistic computation and P-value
 for(i = 0; i < n; i++)
   {
    if (x[i] < y[i])
      t++;
    if (x[i] == y[i])
      n1--;
   }
 *p = BinomialDistribution (t,n1,0.5);

 return t;
}

double MODIFICATOR RandomITest(double *x,long nx,double *y,long ny,double *p)
//
// The function for computation of randomization test for related samples.
// Designations:
//    x - 1st sample,
//   nx - size of 1st sample,
//    y - 2nd sample,
//   ny - size of 2nd sample,
//   *p - P-value of the one-sided hypothesis.
// Return code:
//  value of test.
//
{
 long j,              // Cycle counter
      n = nx + ny,    // Size Merged arrays
      k = 0,          // Counter
      m;              // Size arrays of combinations

 double nSUCCESS = 0, // Number of favorable outcomes - test value
        *dat,         // Merged array
        sum,          // Sum members
        sumx = 0,     // Sum members of array x
        sumy = 0;     // Sum members of array y

 dat = new double[n];

 ////////////////////////////////////////
 // Selection array with a smaller amount
 for(j = 0; j < nx; j++)
   sumx += x[j];
 for(j = 0; j < ny; j++)
   sumy += y[j];
 if (sumx >= sumy)
   {
    m = ny;
    sum = sumy;
   }
 else
   {
    m = nx;
    sum = sumx;
   }

 /////////////////////////
 // Building Merged arrays
 for(j = 0; j < nx; j++)
   dat[k++] = x[j];
 for(j = 0; j < ny; j++)
   dat[k++] = y[j];

 /////////////////////////////
 // Test statistic computation
 Combination (dat,n,m,sum,&nSUCCESS);

 delete [] dat;

 //////////////////////
 // P-value computation
 *p = nSUCCESS / exp(LogFactorial (n) - LogFactorial (nx) - LogFactorial (ny));

 sumx = Mean(x,nx);
 sumy = Mean(y,ny);

 /////////////////////////////////////
 // Test vakue and P-value computation
 if (sumx > sumy)
   *p = MAX(*p,1.0 - *p);
 else if (sumx < sumy)
   *p = MIN(*p,1.0 - *p);
 else
   *p = 0.5;

 return nSUCCESS;
}

double MODIFICATOR RandomTest(double *y,long n,double *x,double *p)
//
// The function for computation of randomization test for related samples.
// Designations:
//   y - 1st sample,
//   n - each sample size,
//   x - 2nd sample,
//  *p - P-value of the one-sided hypothesis.
// Return code:
//  value of test.
// References:
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//  Montgomery D.C. Design and analysis of experiments. - New York: John Wiley & Sons, 2017.
//
{
 long i,                 // Cycle counter
      *step,             // Vector of alternation step
      *a;                // Row of planning matrix

 long j,                 // Cycle counter
      n2 = 2 << (n - 1); // Number of possible outcomes

 double nSUCCESS = 0,    // Number of favorable outcomes - test valu
        *dat,            // Arrayof differences  modules
        sum = 0,         // Threshold value
        sum1;            // Current sum

 dat = new double[n];
 step = new long[n];
 a = new long[n];

 for(i = 0; i < n; i++)
   {
    dat[i] = x[i] - y[i];
    sum += dat[i];
    step[i] = TwoDegree (i);
    a[i] = -1;
   }

 ///////////////////////////////////
 // Enumeration of possible outcomes
 for(j = 0; j < n2; j++)
   {
    for(i = 0, sum1 = 0; i < n; i++)
      {
       if (j / step[i] * step[i] == j)
         a[i] *= -1;
       sum1 += dat[i] * a[i];
      }

    ////////////////////
    // Favorable outcome
    if (sum1 >= sum)
      nSUCCESS++;
   }

 delete [] dat; delete [] step; delete [] a;

 //////////////////////
 // P-value computation
 if (sum == 0)
   *p = 0.5;
 else
   *p = nSUCCESS / n2;

 return nSUCCESS;
}

double MODIFICATOR FisherTest(double a2,double b2,double c2,double d2,double *p1,double *p2)
//
// The function for computation of exact Fisher test.
// Designations:
//   a2 - number of observations with effect A in 1st sample,
//   b2 - number of observations without effect A in 1st sample,
//   c2 - number of observations with effect A in 2nd sample,
//   d2 - number of observations without effect A in 2nd sample,
//  *p1 - P-value of the one-sided hypothesis,
//  *p2 - P-value of the one-sided hypothesis.
// Return code:
//  statistic Wald.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//
{
 long a = floor(a2),                          // Number of observations with effect A in 1st sample
      b = floor(b2),                          // Number of observations without effect A in 1st sample
      c = floor(c2),                          // Number of observations with effect A in 2nd sample
      d = floor(d2),                          // Number of observations without effect A in 2nd sample
      s;                                      // Work variable

 long a1 = a,                                 // Copy of table
      b1 = b,
      c1 = c,
      d1 = d,
      k = MAX(a + c,b + d) + 1;               // Size of array

 double r = LogFactorialD (a2 + b2) +         // Constant part of formula
            LogFactorialD (c2 + d2) +         // for probability
            LogFactorialD (a2 + c2) +
            LogFactorialD (b2 + d2) -
            LogFactorialD (a2 + b2 + c2 + d2),
        *lf,                                  // Array of logarithm factorials
        p,                                    // Probability of given of table
        f;                                    // Work variable

 //////////////////////////
 // Preliminary computation
 lf = new double[k];
 for (s = 0; s < k; s++)
   lf[s] = LogFactorial(s);

 p = FisherProbability(r,a,b,c,d,lf);
 if (a2 * d2 - b2 * c2 < 0)
   {
    *p1 = p;
    *p2 = 0;
   }
 else
   {
    *p1 = 0;
    *p2 = p;
   }

 //////////////////////////////////////////
 // Enumeration of filling options of table
 s = MIN(b,c);
 while (--s >= 0)
   {
    f = FisherProbability(r,++a,--b,--c,++d,lf);
    if (f < p)
      {
       if (a * d < b * c)
         *p1 += f;
       else
         *p2 += f;
     }
   }
 s = MIN(a1,d1);
 while (--s >= 0)
   {
    f = FisherProbability(r,--a1,++b1,++c1,--d1,lf);
    if (f < p)
      {
       if (a1 * d1 < b1 * c1)
         *p1 += f;
       else
         *p2 += f;
      }
   }

 delete [] lf;

 return Wald(a2,b2,c2,d2);
}

double MODIFICATOR ChiSquareFisherTest (double *x,double *p,double *y,long k,double *z)
//
// The function for computation of chi-square Fisher test.
// Designations:
//   x - empirical frequences,
//   p - theoretical frequences,
//   y - midldles of class intervals,
//   k - number of rows,
//  *z - P-value of the one-sided hypothesis.
// Return code:
//  test statistic.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//

//
{
 long i;            // ------- -----

 double r = 0,       // ---------- -------
        n = 0,       // ----------- -------
        d,           // -------- ---------- ---------
        m = 0,s = 0; // --------- ------------
                                 
 //////////////////////////////////
 // ------ ---------- ------------
 for (i = 0; i < k; i++)
   n += x[i];
 for (i = 0; i < k; i++)
   {
    m += x[i] * y[i];
    s += x[i] * y[i] * y[i];
   } 
 s = sqrt((s - m * m / n) / n);
 m /= n;
 if (s == 0)
   return 0;

 /////////////////////////////////
 // ---------- ---------- -------   
 for (i = 0; i < k; i++)
   {
    p[i] = (y[i] - m) / s;
    if (i == 0)
      d = y[1] - y[0];
    else if (i == k - 1)
      d = y[k - 1] - y[k - 2];
    else
      d = (y[i + 1] - y[i - 1]) / 2;
    p[i] = d * n * 0.398942280401 / s / exp(p[i] * p[i] / 2);
    r = r + (x[i] / p[i] - 1) * (x[i] - p[i]);
   }

 ////////////////////////
 // ---------- P--------
 *z = ChiSquareDistribution (k - 3, r);

 return r;
}

double MODIFICATOR FisherProbability(double r,long a,long b,long c,long d,double *lf)
//
// The function for computation of probability of Fisher exact test.
// Designations:
//   r - constant part of formula,
//   a - number of observations with effect A in 1st sample,
//   b - number of observations without effect A in 1st sample,
//   c - number of observations with effect A in 2nd sample,
//   d - number of observations without effect A in 2nd sample,
//  lf - array of logarithm factorials.
// Return code:
//  probability of Fisher exact test.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//
{
 return exp (r - lf[a]- lf[b] - lf[c] - lf[d]);
}

double MODIFICATOR BarnardTest(double a1,double b1,double c1,double d1,double *p,double *pi)
//
// The function for computation of Barnard test.
// Designations:
//   a1 - number of observations with effect A in 1st sample,
//   b1 - number of observations without effect A in 1st sample,
//   c1 - number of observations with effect A in 2nd sample,
//   d1 - number of observations without effect A in 2nd sample,
//   *p - P-value of the two-sided hypothesis,
//  *pi - parameter distribution.
// Return code:
//  Wald statistic.
// References:
//  Barnard G.A. Significatce tests for 2 x 2 tables // Biometrika, 1947, vol. 34, no. 1/2, pp. 123-138.
//
{
 long j,                       // Counter
      a = floor(a1),           // Number of observations with effect A in 1st sample
      b = floor(b1),           // Number of observations without effect A in 1st sample
      c = floor(c1),           // Number of observations with effect A in 2nd sample
      d = floor(d1);           // Number of observations without effect A in 2nd sample

 long ac = a + c,              // Marginal frequency
      bd = b + d;              // Marginal frequency

 long k = MAX(ac,bd) + 1;      // Size arrays

 double i,                     // Counter
        w = Wald(a1,b1,c1,d1), // Wald statistic
        delta = 0.0001,        // Rocking step
        h = 0.01,              // Step for global search
        eps = 0.001,           // Absolute sharpness of computation of parameter distribution
        l,r,                   // Bounds of interval of root localization
        *lf,                   // Array of logarithm factorials
        f;                     // Work variable

 //////////////////////////
 // Preliminary computation
 lf = new double[k];
 for (j = 0; j < k; j++)
   lf[j] = LogFactorial(j);

 ////////////////
 // Global search
 i = 0.01;
 *pi = 0.01;
 *p = 0;
 while (i <= 0.5)
   {
    if ((f = BarnardTestProb (ac,bd,w,i,lf)) > *p)
      {
       *pi = i;
       *p = f;
      }
    i += h;
   }
 l = *pi - h;
 r = *pi + h;

 //////////////////////////////////////////////
 // Maximum search method of dichotomous search
 do
   {
    *pi = (l + r) / 2;

    if (BarnardTestProb (ac,bd,w,*pi - delta,lf) > BarnardTestProb (ac,bd,w,*pi + delta,lf))
      r = *pi;
    else
      l = *pi;

   } while (r - l > eps);

 *pi = (l + r) / 2;
 *p = BarnardTestProb (ac,bd,w,*pi,lf);

 delete [] lf;

 return w;
}

double MODIFICATOR BarnardTestProb (long ac,long bd,double w,double i,double *lf)
//
// The function for computation of probability of Barnard test.
// Designations:
//  ac - number of observations 1st sample,
//  bd - number of observations in 2nd sample,
//   w - statistic Wald of given of table,
//   i - parameter distribution,
//  lf - array of logarithm factorials.
// Return code:
//  probability of Barnard test.
// References:
//  Barnard G.A. Significatce tests for 2 x 2 tables // Biometrika, 1947, vol. 34, no. 1/2, pp. 123-138.
//
{
 long a,b,c,d;  // Parameters of current table

 double pp = 0; // Return code

 //////////////////////////////////////////
 // Enumeration of filling options of table
 for(a = 0; a <= ac; a++)
   for(b = 0; b <= bd; b++)
     {
      c = ac - a;
      d = bd - b;
      if (Wald(a,b,c,d) > w)
        pp += BarnardProbability(a,b,c,d,i,lf);
     }

 return pp;
}

double MODIFICATOR BarnardProbability(long a,long b,long c,long d,double p,double *lf)
//
// The function for computation of probability Barnard test.
// Designations:
//   a - number of observations with effect A in 1st sample,
//   b - number of observations without effect A in 1st sample,
//   c - number of observations with effect A in 2nd sample,
//   d - number of observations without effect A in 2nd sample,
//   p - parameter,
//  lf - array of logarithm factorials.
// Return code:
//  Probability of Barnard test.
//
{
 return exp((a + b) * log(p) + (c + d) * log(1 - p) + lf[a + c] + lf[b + d] - lf[a] - lf[b] - lf[c] - lf[d]);
}

double MODIFICATOR WaldWolfowitzExact(double *x,long n1,double *y,long n2,double *p)
//
// The function for computation of Wald-Wolfowitz series test (exact version).
//   x - 1st sample,
//  n1 - size of 1st sample,
//   y - 2nd sample,
//  n2 - size of 2nd sample,
//  *p - P-value of the hypothesis.
// Return code:
//  value of test.
// References:
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//  Brownlee K.A. Statistical theory and metodology in sciences and engineering. - New York, NY: John Wiley & Sons, 1965.
//
{
 long i,k = 0,     // Counter
      *b,          // Array of conditions
      *ix,         // Information about given arrays
      n = n1 + n2; // Common size

 double *xx,       // Merged array
        t = 1.0;   // Cumulative parameter value

 b = new long[n];
 ix = new long[n];
 xx = new double[n];

 ////////////////////
 // Merging of arrays
 for(i = 0; i < n1; i++)
   xx[k++] = x[i];
 for(i = 0; i < n2; i++)
   xx[k++] = y[i];

 //////////
 // Sort up
 SortArrayUp (xx,n,ix);

 /////////////////////////////////////
 // Value 1 - 1st array, 2 - 2nd array
 for(i = 0; i < n; i++)
   b[i] = ix[i] < n1 ? 1 : 2;

 ///////////////////////////////
 // computation of series number
 for(i = 1; i < n; i++)
   if (b[i - 1] == b[i])
     t += 1.0;

 delete [] xx; delete [] ix; delete [] b;

 *p = InverseSerialDistribution (n1,n2,t);

 return t;
}

long MODIFICATOR Combination (double dat[],long n,long m,double sum,double *nSUCCESS)
//
// The function for computation of numbers favorable outcomes.
// Designations:
//        dat - array variables by length n,
//          m - size arrays of combinations,
//        sum - given critical value,
//  *nSUCCESS - number of favorable outcomes.
// Return code:
//   0 - when normal end of computation,
//  -1 - when lack of memory for work arrays.
//
{
 long j,            // Cycle counter
      *i;           // Array of counters

 double *ext,       // Array of combinations
        s[1] = {0}; // Work variable (to simplify in array mode)

 if ((ext = new double[m]) == 0)
   return -1;
 if ((i = new long[m]) == 0)
   {
    delete [] ext;
    return -1;
   }

 if (m == 1)
   {
    *nSUCCESS = 0;
    for(j = 0; j < n; j++)
      if (dat[j] <= sum)
    s[0] += 1;
   }
 else
   CombinationAdd (dat,n,m,ext,sum,i,s,0);

 *nSUCCESS = s[0];

 delete [] ext; delete [] i;

 return 0;
}

void MODIFICATOR CombinationAdd (double dat[],long n,long m,double ext[],double sum,long i[],double s[],long k)
//
// Additional function for computation of numbers of favorable outcomes.
// Designations:
//   dat - array variables by length n,
//     m - size arrays of combinations,
//   ext - array of combinations,
//   sum - given critical value,
//     i - array of counters,
//  s[0] - numbers of favorable outcomes,
//     k - parameter for recursion control.
// Return code:
//  none.
//
{
 long count = !k ? - 1 : i[k - 1]; // Work variable

 for(i[k] = count + 1; i[k] < n - m + k + 1; i[k] += 1)
   {
    /////////////////////////
    // Arrays of combinations
    ext[k] = dat[i[k]];

    if (k == m - 1)
      {
       /////////////////////
       // Favorable outcomes
       if (VectorSum (ext,m) <= sum)
         s[0] += 1;
      }
    else
      ////////////////////////////////////////////////////////////////////
      // Continuation of the recruitment process of arrays of combinations
      CombinationAdd (dat,n,m,ext,sum,i,s,k + 1);
   }
}

long MODIFICATOR PrincipalFactor (double z[],long k,double r[],double z2[],long *m)
//
// The function builds factor analysis by method of principal factors.
// Designations:
//   z - correlation matrix,
//   k - order of correlation matrix,
//   r - factor pattern matrix by size at least k * k,
//  z2 - eigen values,
//  *m - number of factors.
// Return code:
//   0 when normal result of computation,
//  -1 when lack of memory for work of array,
//  -2 when computation error.
// References:
//  Harman H.H. Modern factor analysis. - Chicago, IL: University of Chicago Press, 1976.
//  Afifi F.F., Azen S.P. Statistical analysis. A computer oriented approach. - New York, NY: Academic Press, 1979.
//  Kim J.-O., Mueller C.W. Factor analysis: Statistical methods and practical issues. - Beverly Hills, CA: SAGE Publications, 1986.

//
{
 long i,j,     // Counters
      m1,Code; // Work variable

 double *z1,   // Working copy of correlation matrix
        save;  // Buffer

 *m = 0;

 if ((z1 = new double[k * k]) == 0)
   return -1;

 /////////////////////////////////////
 // Working copy of correlation matrix
 ArrayToArray (z,z1,k * k);

 /////////////////////////////////
 // Solving the problem of factors
 if ((Code = Jacobi (z1,k,0.000001,&m1,r)) != 0)
   {
    delete [] z1;
    return Code;
   }

 /////////////////////////////////////
 // Ordering of own values and vectors
 for (i = 0; i < k; i++)
   z2[i] = z1[i * (k + 1)];  // In 1st column are eigen values

 delete [] z1;

 for (i = 0; i < k; i++)
   {
    Code = i;
    save = z2[i];
    for (j = i + 1; j < k; j++)
      if (save < z2[j])
        {
         Code = j;
         save = z2[j];
        }

    ColInterchange (z2,1,i,Code);
    ColInterchange (r,k,i,Code);
   }

 ///////////////////////
 // Selection own values
 for (i = 0, m1 = 0; i < k; i++)
   if (z2[i] < 0)
     break;
   else
     {
      (*m)++;

      ///////////////////////////////////////
      // Computation of factor pattern matrix
      save = sqrt (z2[i]);
      for (j = 0; j < k; j++, m1++)
        r[m1] *= save;
     }

 return 0;
}

void MODIFICATOR PercentOfVariance (double z[],long k,double r[],double s[],long ms)
//
// The function computes percent of variations highlighted by factors.
// Designations:
//   z - correlation matrix,
//   k - size of correlation matrix,
//   r - factor pattern matrix,
//   s - vector of percent variations,
//  ms - number of factors.
// Notice:
//  The function used after successful end of factor analysis
//  because error type of zero sum of variation not controlled.
// Return code:
//  none.
//
{
 long i,j,      // Counters
      m;        // Array counter

 double sd = 0; // Sum of variance

 for (i = 0, m = 0; i < k; i++)
   {
    sd += z[m];
    m += k + 1;
   }

 for (i = 0, m = 0; i < ms; i++)
   {
    for (j = 0, s[i] = 0; j < k; j++, m++)
      s[i] += r[m] * r[m];

    s[i] /= sd / 100;
   }
}

long MODIFICATOR Reduction (double z[], double r[],long k,long Mode)
//
// The function builds reduction- of correlation matrix.
// Designations:
//     z - correlation Matrix,
//     r - reduced correlation Matrix,
//     k - size of correlation matrix,
//  Mode - method of computation communities:
//       = 0 - principal components analysis,
//       = 1 - method of the greatest correlation,
//       = 2 - coefficient of multiple correlation,
//       = 3 - mediums coefficients of correlation,
//       = 4 - method of triads.
// Return code:
//   0 when normal end of computation.
//  -1 when lack of memory,
//  -2 when error in parameters, for example k >= 3 for method of triads.
//
{
 long i,j;        // Counters

 long m,m1,k1,k2; // Counters of one-dimensional arrays

 double *t,       // Work array
        s,s1,s2;  // Work variables

 if ((t = new double[k * k]) == 0)
   return -1;

 /////////////////////////////
 // Copy of correlation matrix
 ArrayToArray (z,r,k * k);

 //////////////////////////////
 // Definition of commonalities
 switch (Mode)
   {
    case 0:
      ///////////////////////////////////////////////
      // No reduction - principal components analysis
      break;
    case 1:
      /////////////////////////////////////
      // Method of the greatest correlation
      for (i = 0, m = 0, m1 = 0; i < k; i++)
        {
         for (j = 0; j < k; j++, m++)
           t[j] = i == j ? 0 : fabs (z[m]);
         r[m1] = MaximumOf (t,k);
         m1 += k + 1;
        }
      break;
    case 2:
      //////////////////////////////////////
      // Coefficient of multiple correlation
      if (InverseMatrix (r,t,k) == 0)
        for (i = 0, m = 0; i < k; i++,m += k + 1)
          if (t[m] != 0 || t[m] != 1)
            r[m] = 1 - 1 / t[m];
      break;
    case 3:
      //////////////////////////////////////
      // Mediums coefficients of correlation
      for (i = 0, m = 0, m1 = 0; i < k; i++,m1 += k + 1)
        {
         for (j = 0, r[m1] = 0; j < k; j++, m++)
           r[m1] += i == j ? 0 : fabs (z[m]);
         r[m1] /= k - 1;
        }
      break;
    default:
      ///////////////////
      // Method of triads
      if (k < 3)
        {
         delete [] t;
         return -2;
        }

      for (i = 0, m = 0, m1 = 0; i < k; i++,m1 += k + 1)
        {
         /////////////////////////////////
         // Finding two maxima in a column
         for (j = 0; j < k; j++)
           t[j] = fabs (z[m++]);
         t[i] = 0;
         s1 = MaximumOf (t,k,&k1);
         t[k1] = 0;
         s2 = MaximumOf (t,k,&k2);

         ////////////////////////////////////////////
         // Correlation between maximal of parameters
         s = fabs (z[k1 + k2 * k]);

         if (s != 0)
           r[m1] = s1 * s2 / s;
        }
   }

 delete [] t;

 return 0;
}

long MODIFICATOR MaximumLikelihood (double z[],long k,double r[],double b[],long mz,long *ms)
//
// The function builds factor analysis by maximum likelihood method.
// Designations:
//    z - non-reduction correlation Matrix,
//    k - size of correlation matrix,
//    r - matrix of factor representation size nevertheless k * k,
//    b - calculated vector communities,
//   mz - given number of factors,
//  *ms - calculated number of factors.
// Notice:
//  if given number of factors is to large, it corrected.
// Return code:
//   0 - when normal result of computation,
//  -1 - when lack of memory for work of array,
//  -2 - when computation error.
// References:
//  Harman H.H. Modern factor analysis. - Chicago, IL: University of Chicago Press, 1976.
//  Lawley D.N., Maxwell A.E. Factor analysis as a statistical method. - New York, NY: Elsevier Science, 1971.
//
{
 long i,        // Cycle counter
      m,        // Number of factors
      Code,     // Error code
      m1;       // Array counter

 double *a,*am, // Factor matrix
        *d,*dm, // Characteristics matrix and inverse to it
        *t,     // Work matrix
        buf;    // Sum for convergence checks

 ////////////////////////////////////////
 // Computation of m principal components
 if ((Code = PrincipalFactor (z,k,r,b,&m)) != 0) // Use b as dummy
   return Code;

 if ((a = new double[k * k]) == 0)
   return -1;
 if ((am = new double[k * k]) == 0)
   {
    delete [] a;
    return -1;
   }
 if ((d = new double[k * k]) == 0)
   {
    delete [] a; delete [] am;
    return -1;
   }
 if ((dm = new double[k * k]) == 0)
   {
    delete [] a; delete [] d; delete [] am;
    return -1;
   }
 if ((t = new double[k * k]) == 0)
   {
    delete [] a; delete [] d; delete [] dm; delete [] am;
    return -1;
   }

 //////////////////////////////////
 // Correction of number of factors
 if (mz < m && mz > 0)
   {
    //////////////////////////
    // Given number of factors
    *ms = mz;
    m = mz;
   }
 else
   ///////////////////////////////
   // Calculated number of factors
   *ms = m;

 FillUp (b,k,1);

 ////////////////////////////////////////////////////
 // Being of iterations - matrix principal components
 ArrayToArray (r,am,k * m);

 do
   {
    ///////////////////////////////////////
    // Matrix A for next step of iterations
    // Here a - matrix A(i - 1/2), as an alternative,
    // you can apply the function ArrayToArray
    for (i = 0; i < k * m; i++)
      a[i] = am[i];

    //////////////////////////////////////
    // Computation of commonalities matrix
    MatrixMultiplyMatrixT (a,a,d,k,m,k);

    for (i = 0, m1 = 0; i < k; i++, m1 += k + 1)
      {
       //////////////////////////////////
       // Vector commonalities for output
       b[i] = d[m1];

       ///////////////////////////
       // Matrix characteristics D
       d[m1] = 1 - d[m1];
      }

    ///////////////////////////////////////////////////
    // computation of inverse matrix characteristics DM
    if ((Code = InverseMatrix (d,dm,k)) != 0)
      {
       delete [] d; delete [] t; delete [] dm; delete [] am; delete [] a;
       return Code;
      }

    ///////////////////////////////////
    // Computation of matrix J(i - 1/2)

    // Here d - matrix J(i - 1/2)
    MatrixTMultiplyMatrix (a,dm,t,m,k,k);
    MatrixMultiplyMatrix (t,a,d,m,k,m);

    /////////////////////////////////////////////////////////////
    // Rotation of matrix J(i - 1/2) - computation of matrix J(i)
    // Here t - rotation matrix Q
    //      d - matrix J(i)
    if ((Code = Jacobi (d,m,0.000001,&m1,t)) != 0)
      {
       delete [] d; delete [] t; delete [] dm; delete [] am; delete [] a;
       return Code;
      }

    ////////////////////////////////////
    // Computation of factor matrix A(i)
    MatrixMultiplyMatrix (a,t,r,k,m,m);

    ////////////////////////////////////////
    // Computation of matrix inverse to J(i)
    // Here t - Inverse Matrix J(i)
    if ((Code = InverseMatrix (d,t,m)) != 0)
      {
       delete [] d; delete [] t; delete [] dm; delete [] am; delete [] a;
       return Code;
      }

    ///////////////////////////////////
    // computation of matrix A(i + 1/2)
    // Here am - matrix A(i + 1/2)
    MatrixMultiplyMatrix (z,dm,am,k,k,k);
    for (i = 0, m1 = 0; i < k; i++, m1 += k + 1)
      am[m1] -= 1;
    MatrixMultiplyMatrix (am,r,d,k,k,m);
    MatrixMultiplyMatrix (d,t,am,k,m,m);

    /////////////////////
    // End of computation
    for (i = 0, buf = 0; i < k * m; i++)
      buf += fabs (am[i] - a[i]);

   } while (buf < 0.001);

 delete [] d; delete [] t; delete [] dm; delete [] am; delete [] a;

 return 0;
}

double MODIFICATOR Wilks(double *P,long K,long M,long N)
//
// The function checks the significance of the number of common factors.
// Designations:
//  P - reduced correlation Matrix,
//  K - order of correlation matrix,
//  M - number of factors, M < K,
//  N - number of observation.
// Return code:
//  significance of Wllks statistic.
//
{
 long I,L = 0; // Counter

 double *R,    // Correlaton matrix
        S;     // Work variable

 R = new double[K * K];

 for (I = 0; I < K * K; I++)
   R[I] = P[I];
 for (I = 0; I < K; I++, L = L + K + 1)
   R[L] = 1;

 //////////////////
 // Wilks statistic
 if ((S = Krauth(R,K,0.000000001)) > 0 && K > M)
   S = ChiSquareDistribution(0.5 * ((K - M) * (K - M) + K - M), log(S /
                             Krauth(P,K,0.000000001)) * N);
 else
   S = 0;

 delete [] R;

 return S;
}

double MODIFICATOR Entropy(double *f,long n,double o)
//
// The function for computation uncertainty (entropy) of the system
// for discrete processes according to Shannon.
// Designations:
//  f - frequency range distribution,
//  n - number of classes,
//  o - log(the base of the logarithm).
// Return code:
//  calculated value entropy (in bit).
// References:
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//
{
 long i;       // Cycle counter

 double h = 0; // Entropy in nit

 for (i = 0; i < n; i++)
   if (f[i] > 0)
     h += f[i] * log(f[i]);

 return -h / o;
}

double MODIFICATOR DispersionEntropy(double *f,long n,long nn,double o)
//
// The function for computation of variations of system entropy for discrete processes.
// Designations:
//  f - frequency range distribution,
//  n - number of classes,
// nn - number of samle members,
//  o - log(the base of the logarithm).
// Return code:
//  calculated value of variations entropy.
//
{
 long i;        // Cycle counter

 double h = 0,  // Entropy
        dh = 0, // Work variable
        d1;     // Work variable

 for (i = 0; i < n; i++)
   if (f[i] > 0)
     {
      d1 = log(f[i]) / o;
      h += f[i] * d1;
      dh += f[i] * d1 * d1;
     }

 return (dh - h * h) / nn + 0.5 * (n - 1) / nn / nn;
}

void MODIFICATOR Hutcheson(double x[],long n,long nn,double y[],long m,long mm,double o,double *t,double *p)
//
// The function for computation of Hutcheson test.
// Designations:
//   x - frequency range distribution,
//   n - number of classes,
//  nn - number of sample members,
//   y - frequency range distribution,
//   m - number of classes,
//  mm - number of sample members,
//   o - log(the base of the logarithm),
//  *t - test value,
//  *p - P-value.
// Return code:
//  none.
//
{
 long i;        // Cycle counter

 double h1 = 0, // Entropy
        h2 = 0, // Entropy
        d1,     // Work variable
        d2,     // Work variable
        dh = 0; // Work variable

  for (i = 0; i < n; i++)
    if (x[i] > 0)
      {
       d1 = log(x[i]) / o;
       h1 -= x[i] * d1;
       dh += x[i] * d1 * d1;
      }
  d1 = (dh - h1 * h1) / nn + (n - 1) / 2 / nn / nn;

  dh = 0;
  for (i = 0; i < m; i++)
    if (y[i] > 0)
      {
       d2 = log(y[i]) / o;
       h2 -= y[i] * d2;
       dh += y[i] * d2 * d2;
      }
  d2 = (dh - h2 * h2) / mm + (m - 1) / 2 / mm / mm;

  *t = fabs(h1 - h2);
  *p = TDistribution((d1 + d2) * (d1 + d2) / (d1 * d1 / nn + d2 * d2 / mm),(*t));
}

double MODIFICATOR MaxEntropy(long n,double o)
//
// The function for computation of maximum uncertainty (entropy) of the system
// for discrete processes according to Shannon.
// Designations:
//  n - number digits of code positions,
//  o - log(the base of the logarithm).
// Return code:
//  value of entropy (in bit).
//
{
 return log(n) / o;
}

double MODIFICATOR RelEntropy(double hmax,double h)
//
// The function for computation of relative entropy of the system.
// Designations:
//  hmax - maximal entropy,
//     h - current entropy.
// Return code:
//  value of relative entropy.
//
{
 return h / hmax;
}

double MODIFICATOR Surplus(double hmax,double h)
//
// The function for computation of the system .
// Designations:
//  hmax - maximal entropy,
//     h - current entropy.
// Return code:
//  value of redundancy (in percent).
//
{
 return (1.0 - h / hmax) * 100;
}

double MODIFICATOR Organization(double hmax,double h)
//
// The function for computation absolute organization of the system.
// Designations:
//  hmax - maximal entropy,
//     h - current entropy.
// Return code:
//  value of absolute organization (in bit).
//
{
 return hmax - h;
}

long MODIFICATOR MeanRank (double x[],long n,long k,double p[])
//
// The function computes mediums arithmetic ranks.
// Designations:
//   x - matrix survey,
//   n - number of objects,
//   k - number of experts,
//   p - medium.
// Return code:
//  -1 - when less memory for work of array,
//   0 - when the normal end of the computation.
//
{
 long i,j,           // Cycle counter
       l = 0,l1 = 0, // Counter elements
       *ix;          // Information array

 double *z,          // Ranked survey matrix
        *c;          // Work array

 if ((ix = new long[n]) == 0)
   return -1;
 if ((z = new double[n * n]) == 0)
   {
    delete [] ix;
    return -1;
   }
 if ((c = new double[n]) == 0)
   {
    delete [] ix; delete [] z;
    return -1;
   }

 ///////////////////////////
 // Ranking of matrix survey
 for (i = 0; i < k; i++)
   {
    for (j = 0; j < n; j++)
      p[j] = x[l++];
    SortArrayUp (p,n,ix);
    Rank (p,c,n);
    UnSortArray (c,p,n,ix);
    for (j = 0; j < n; j++)
      z[l1++] = p[j];
  }

 //////////////////////////////
 // Computation medium of ranks
 for (i = 0; i < n; i++)
   {
    p[i] = 0;
    for (j = 0, l = i; j < k; j++, l += n)
      p[i] += z[l];
  }

 SortArrayUp (p,n,ix);
 Rank (p,c,n);
 UnSortArray (c,p,n,ix);

 delete [] ix; delete [] z; delete [] c;

 return 0;
}

double MODIFICATOR Concord (double x[],long n,long k)
//
// The function computes value of Kendall's coefficient of concordance.
// Designations:
//  x - matrix survey,
//  n - number of objects,
//  k - number of experts.
// Return code:
//  value of coefficient concordation.
// References:
//  Kendall M.G., Gibbons J.D. Rank correlation methods. - New York, NY: Oxford University Press,  1990.
//
{
 long i,j,                   // Cycle counters
      l = 0,l1 = 0,          // Counter arrays
      *ix;                   // Information array

 double r = k * (n + 1) / 2, // Medium value
        s,                   // Sum by columns
        Ret = 0,             // Return code
        *p,                  // Vector of answers of one expert
        *pp,                 // Work array
        *z,                  // Ranked matrix survey
        b = 0;               // Bundle adjustment of ranks

 p = new double[n];
 pp = new double[n];
 ix = new long[n];
 z = new double[n * n];

 ///////////////////////////
 // Ranking of matrix survey
 for (i = 0; i < k; i++)
   {
    for (j = 0; j < n; j++)
      p[j] = x[l++];
    SortArrayUp (p,n,ix);
    b += Rank (p,pp,n);
    UnSortArray (pp,p,n,ix);
    for (j = 0; j < n; j++)
      z[l1++] = p[j];
  }
 delete [] p; delete [] pp; delete [] ix;

 //////////////////////////////////////////
 // Computation of coefficient concordation
 for (i = 0; i < n; i++)
   {
    for (j = 0,s = 0,l = i; j < k; j++, l += n)
      s += z[l];
    Ret += (s - r) * (s - r);
   }
 delete [] z;

 Ret /= (1.0 / 12 * k * k * n * (n * n - 1) - k * b);

 return Ret;
}

long MODIFICATOR KemenyMedian (double z[],long n,long k,double p[])
//
// The function computes Kemeny median.
// Designations:
//   z - matrix survey,
//   n - number of objects,
//   k - number of experts,
//   p - median.
// Return code:
//     -1 - when less memory for work of array,
//  other - the ordinal number of the average in survey matrix.
// References:
//  Bury H. Kemeny-s median algorithm: Application for determining group judgement // 16th JISR-IIASA Workshop on Methodologies and Tools for Complex System Modeling and Integrated Policy Assessment, July 15-17, 2002, IIASA, Laxenburg, Austria.
//
{
 long i,j,j1,m,   // Cycle counter
       l = 0,l1;  // Elements counter

 double Ret = 1,  // The ordinal number of the average
        *b,*c,    // Work arrays
        buf,      // Work variable
        save = 0; // Work variable

 if ((b = new double[n]) == 0)
   return -1;
 if ((c = new double[n]) == 0)
   {
    delete [] b;
    return -1;
   }

 ////////////////////////
 // Computation of median
 for (i = 0; i < k; i++)
   {
    for (j = 0; j < n; j++)
      b[j] = z[l++];
    for (m = 0,l1 = 0,buf = 0; m < k; m++)
      {
       for (j1 = 0; j1 < n; j1++)
         c[j1] = z[l1++];
       buf += Likeness(b,c,n);
      }
    if (i == 0)
      {
       save = buf;
       for (j = 0; j < n; j++)
         p[j] = b[j];
      }
    else
      {
       if (buf < save)
         {
          Ret = i + 1;
          save = buf;
          for (j = 0; j < n; j++)
            p[j] = b[j];
         }
      }
   }

 delete [] b; delete [] c;

 return Ret;
}

long MODIFICATOR KemenyMean (double z[],long n,long k,double p[])
//
// The function computes the Kemeny mean.
// Designations:
//   z - matrix survey,
//   n - number of objects,
//   k - number of experts,
//   p - medium.
// Return code:
//    0 - when the normal end of the computation,
//   -1 - when less memory for work of array.
// References:
//  Bury H. Kemeny-s median algorithm: Application for determining group judgement // 16th JISR-IIASA Workshop on Methodologies and Tools for Complex System Modeling and Integrated Policy Assessment, July 15-17, 2002, IIASA, Laxenburg, Austria.
//
{
 long i,j,   // Cycle counter
      l = 0; // Counter elements

 double *c;  // Work arrays

 if ((Per = new double[n]) == 0)
    return -1;
 if ((c = new double[n]) == 0)
   {
    delete [] Per;
    return -1;
   }

 //////////////////////
 // Computation of mean
 for (j = 1; j <= n; j++)
   Per[j - 1] = j;
 for (i = 0, Sav = 0; i < k; i++)
   {
    for (j = 0; j < n; j++)
      c[j] = z[l++];
    Sav += Likeness(Per,c,n);
   }

 Antilex(n,n,z,k,c,p);

 delete [] Per; delete [] c;

 return 0;
}

void MODIFICATOR Antilex(long n,long m,double *z,long k,double *c,double *p)
//
// The function of generating permutations in an antilexicographic order.
// Designations:
//  n - number of objects,
//  m - number of first elements for generating permutations,
//  z - matrix survey,
//  k - number of experts,
//  with - work array,
//  p - medium.
// Return code:
//  none.
//
{
 long i,j,l,      // Counter
      m1 = m - 1, // Work variable
      m2 = m - 2; // Work variable

 double Save,Buf; // Buffer

 if (m > 1)
   {
    for (i = 0; i < m; i++)
      {
       Antilex(n,m1,z,k,p,c);
       if (i < m1)
         {
          Save = Per[i];
          Per[i] = Per[m1];
          Per[m1] = Save;
          Reverse(m2);
         }
      }
   }
 else
   {
    for (i = 0,l = 0,Buf = 0; i < k; i++)
      {
       for (j = 0; j < n; j++)
         c[j] = z[l++];
       Buf += Likeness(Per,c,n);
      }
    if (Buf < Sav)
      {
       Sav = Buf;
       for (j = 0; j < n; j++)
         p[j] = Per[j];
      }
   }
}

void MODIFICATOR Reverse (long m)
//
// The function for order inversion of members arrays.
// Designations:
//  m - number of elements for inversion.
// Return code:
//  none.
//
{
 long i = 0;  // Counter

 double Save; // Buffer

 while (i < m)
   {
    Save = Per[i];
    Per[i++] = Per[m];
    Per[m--] = Save;
   }
}

double Likeness (double x[],double y[],long n)
//
// The function computes a measure of likeness for rankings.
// Designations:
//  x - 1st array,
//  y - 2nd array,
//  n - size of each arrays.
// Return code:
//  value of measure.
//
{
 long i,j,       // Cycle counters
      l;         // Counter of members arrays

 double Ret = 0, // Return code
        *ax,*bx; // Relationship matrices

 ax = new double[n * n];
 bx = new double[n * n];

 ///////////////////////////////////////
 // Computation of relationship matrices
 for (i = 0, l = 0; i < n; i++)
   for (j = 0; j < n; j++, l++)
     if (x[i] > x[j])
       ax[l] = 1;
     else if (x[i] < x[j])
       ax[l] = -1;
     else
       ax[i] = 0;
 for (i = 0, l = 0; i < n; i++)
   for (j = 0; j < n; j++, l++)
     if (y[i] > y[j])
       bx[l] = 1;
     else if (y[i] < y[j])
       bx[l] = -1;
     else
       bx[i] = 0;

 //////////////////////////
 // Computation of likeness
 for (i = 0, l = 0; i < n; i++)
   for (j = 0; j < n; j++, l++)
     Ret += fabs(ax[l] - bx[l]);

 delete [] ax; delete [] bx;

 return Ret;
}

long MODIFICATOR Thurstone27 (double z[],long n,double p[])
//
// The function builds paired comparisons by Thurstone method.
// Designations:
//   z - matrix survey,
//   n - number of objects,
//   p - vector of weight coefficients.
// Return code:
//   0 - when normal end of computation,
//  -1 - when less of memory for work of array,
//  -2 - when computation error,
//  -3 - when reaching the limit number of iterations.
//
{
 long i,      // Cycle counter
      j;      // Counter of iterations

 double *p1,  // Work array
        l,    // Normalizing coefficient
        save; // Work variable

 if ((p1 = new double[n]) == 0)
   return -1;

 /////////////////
 // Initial values
 for (i = 0; i < n; i++)
   p[i] = 1;
 j = 0;

 ///////////////////////////////////////////////
 // Computation of vector of weight coefficients
 do
   {
    MatrixMultiplyMatrix (z,p,p1,n,n,1);
    for (i = 0, l = 0; i < n; i++)
      l += p1[i] * p1[i];
    l = sqrt(l);
    if (l == 0)
      {
       delete [] p1;
       return -2;
      }
    for (i = 0, save = 0; i < n; i++)
      {
       p1[i] /= l;
       save += (p[i] - p1[i]) * (p[i] - p1[i]);
       p[i] = p1[i];
      }
    if (++j > 300)
      {
       delete [] p1;
       return -3;
      }
   } while (sqrt(save) > 0.000001);

 delete [] p1;

 return 0;
}

long MODIFICATOR Thurstone (double z[],long n,long m,double p[],double v[])
//
// The function builds groups expert estimation.
// Designations:
//   z - matrix of survey,
//   n - number of objects,
//   m - number of experts,
//   p - vector of estimation,
//   v - vector of competence.
// Return code:
//   0 when normal result of computation,
//  -1 when lack of memory for work of array,
//  -2 when computation error.
//
{
 long i,j,     // Counters
      m1,      // Work variable
      Code;    // Return code

 double *b,*r, // Work array
        save;  // Work variable

 if (n > m)
   i = n;
 else
   i = m;
 if ((b = new double[i * i]) == 0)
   return -1;
 if ((r = new double[i * i]) == 0)
   {
    delete [] b;
    return -1;
   }

 //////////////////////////////////////////
 // Computation of vector groups estimation
 MatrixMultiplyMatrixT (z,z,b,n,m,n);
 if ((Code = Jacobi (b,n,0.000001,&m1,r)) != 0)
   {
    delete [] b; delete [] r;
    return Code;
   }
 for (i = 1; i < n; i++)
   b[i] = b[i * (n + 1)];
 for (i = 0; i < n; i++)
   {
    Code = i;
    save = b[i];
    for (j = i + 1; j < n; j++)
      if (save < b[j])
        {
         Code = j;
         save = b[j];
        }

    ColInterchange (b,1,i,Code);
    ColInterchange (r,n,i,Code);
   }
 for (i = 0; i < n; i++)
   p[i] = r[i];

 ///////////////////////////////////
 // Computation of competence vector
 MatrixTMultiplyMatrix (z,z,b,m,n,m);
 if ((Code = Jacobi (b,m,0.000001,&m1,r)) != 0)
   {
    delete [] b; delete [] r;
    return Code;
   }
 for (i = 1; i < m; i++)
   b[i] = b[i * (m + 1)];
 for (i = 0; i < m; i++)
   {
    Code = i;
    save = b[i];
    for (j = i + 1; j < m; j++)
      if (save < b[j])
        {
         Code = j;
         save = b[j];
        }

    ColInterchange (b,1,i,Code);
    ColInterchange (r,m,i,Code);
   }
 for (i = 0; i < m; i++)
   v[i] = r[i];


 delete [] b; delete [] r;

 return 0;
}

long MODIFICATOR Torgerson (double z[],long k,double r[],double s[],long *m)
//
// The function of mulidimensional scaling by metric Torgerson method.
// Designations:
//   z - matrix of scalar of compositions,
//   k - order of matrix of scalar of compositions,
//   r - matrix coordinates size nevertheless k * k,
//   s - vector own values,
//  *m - number non-zero own values.
// Return code:
//   0 when normal result of computation,
//  -1 when lack of memory for work of array,
//  -2 when computation error.
// References:
//  Torgerson W.S. Multidimensional scaling. Theory and method // Psychometrica, 1952, vol. 17. no 4, pp. 401-419.
//  Torgerson W.S. Theory and methods of scaling. - New York, NY: John Wiley & Sons, 1958.
//  Davison M.L. Multidimensional scaling. - New York, NY: John Wiley & Sons, 1983.
//  Cox, T.F., Cox, M.A.A. Multidimensional scaling. - NewYork, NY: Chapman and Hall / CRC, 2001.
//
{
 long i,j,    // Counters
      m1,     // Work variable
      Code;   // Error code

 double *z1,  // Working copy of matrix of scalar of compositions
        save; // Buffer

 *m = 0;

 if ((z1 = new double[k * k]) == 0)
   return -1;

 ArrayToArray (z,z1,k * k);

 /////////////////////////////////////
 // Solving of the eigenvalues problem
 if ((Code = Jacobi (z1,k,0.000001,&m1,r)) != 0)
   {
    delete [] z1;
    return Code;
   }

 //////////////////////////////////
 // Ordering own values and vectors
 for (i = 1; i < k; i++)
   {
    z1[i] = z1[i * (k + 1)];  // 1st column - own values

    ///////////////////////////////
    // Correction of negative zeros
    if (fabs(z1[i]) < 0.000001)
      z1[i] = 0;
   }

 for (i = 0; i < k; i++)
   {
    Code = i;
    save = z1[i];
    for (j = i + 1; j < k; j++)
      if (save < z1[j])
        {
         Code = j;
         save = z1[j];
        }

    ColInterchange (z1,1,i,Code);
    ColInterchange (r,k,i,Code);
   }

 /////////////////////////////////////
 // Conservation of ordered own values
 ArrayToArray (z1,s,k);

 ///////////////////////
 // Selection own values
 for (i = 0, m1 = 0; i < k; i++)
   {
    if (z1[i] < 0)
      break;
    else
      {
       (*m)++;

       ////////////////////////////////////
       // Computation of matrix coordinates
       save = sqrt (z1[i]);
       for (j = 0; j < k; j++, m1++)
         r[m1] *= save;
       }
   }

 delete [] z1;

 return 0;
}

void MODIFICATOR ScalarMatrix (double a[],double b[],long n)
//
// The function for computation of matrix of the scalar compositions
// for multidemensional scaling.
// Designations:
//  a - matrix of differences (delta),
//  b - matrix of scalar of compositions (delta*),
//  n - the order of the matrices a and b.
// Return code:
//  none.
//
{
 long i,j,k,         // Counters
      m = 0,m1;      // Index of current element

 double dit,dtj,dtt; // Work variables

 ///////////////////
 // Work computation
 for (k = 0, dtt = 0; k < n * n; k++)
   dtt +=  a[k] * a[k];
 dtt = dtt / n / n;

 ///////////////////////////////////////////////
 // Building of matrix of scalar of compositions
 for (i = 0; i < n; i++)
   for (j = 0; j < n; j++, m++)
     {
      for (k = 0,dit = 0,m1 = i; k < n; k++, m1 += n)
        dit += a[m1] * a[m1];

      for (k = 0,dtj = 0,m1 = j * n; k < n; k++, m1++)
        dtj += a[m1] * a[m1];

      b[m] = - (a[m] * a[m] - dit / n - dtj / n + dtt) / 2;
     }
}

long MODIFICATOR Kruskal(double X[],long N,long K,long M1,double R[], double S[], long Link,long MaxI, long *I,double Str[],double Step,double Sharp)
//
// The function of mulidimensional scaling by Kruskal.
// Designations:
//      X - matrix of given data, on exit - its estimation
//      N - size of one sample
//      K - order of matrix of differences
//     M1 - given number of scales
//      R - matrix coordinates
//      S - vector own values
//   Link - type of metrica
//   MaxI - given number of iterations
//     *I - on exit - (spent number of iterations - 1)
//    Str - on exit - array of stress
//   Step - step of algorithm
//  Sharp - sharpness of algorithm
// Return code:
//   0 when normal end of computation
//  -1 when lack of memory for computation
//  -3 when the maximum allowed number of iterations is reached
//  -4 when he process diverges
// References:
//  Davison M.L. Multidimensional scaling. - New York, NY: John Wiley & Sons, 1983.
//  Cox, T.F., Cox, M.A.A. Multidimensional scaling. - New York, NY: Chapman and Hall / CRC, 2001.
//
{
 long J,L,I1,J1,L1, // Counter
      M,            // Number own values
      Result;       // Error code

 double *A,         // Matrix of differences
        *D,         // Estimation of matrix of differences
        *Z,         // Matrix of scalar of compositions
        *XX,        // Estimation of matrix of given data
        B,C;        // Work variable

 if ((A = new double[K * K]) == 0)
   return -1;
 if ((D = new double[K * K]) == 0)
   {
    delete [] A;
    return -1;
   }
 if ((Z = new double[K * K]) == 0)
   {
    delete [] A; delete [] D;
    return -1;
   }
 if ((XX = new double[N * K]) == 0)
   {
    delete [] A; delete [] D; delete [] Z;
    return -1;
   }

 for ((*I) = 0; (*I) < MaxI; (*I)++)
   {
    ///////////////
    // Metric stage
    DifferenceMatrix(X,A,N,K,Link);
    ScalarMatrix(A,Z,K);
    if ((Result = Torgerson(Z,K,R,S,&M)) != 0)
      {
       delete [] A; delete [] D; delete [] Z; delete [] XX;
       return Result;
      }

    /////////////////////////////////////
    // Estimation of matrix of given data
    for (J = 0, L1 = 0; J < K; J++)
      for (J1 = 0, L = J; J1 < M1; J1++, L += K)
        XX[L1++] = R[L];

    ///////////////////
    // Non-metric stage
    DifferenceMatrix(XX,D,M1,K,Link);
    for (J = 0, L = 0, B = 0; J < K; J++)
      for (J1 = 0; J1 < K; J1++, L++)
        B += A[L] * A[L];

    ///////////////////////////////////
    // Checking the accuracy of the fit
    Str[(*I)] = 0;
    L = 0;
    for (J = 0; J < K; J++)
      for (J1 = 0; J1 < K; J1++, L++)
        Str[(*I)] += (D[L] - A[L]) * (D[L] - A[L]);
    Str[(*I)] = sqrt(Str[(*I)] / B);
    if ((*I) > 0)
      {
       if (Str[(*I)] > Str[(*I) - 1])
         {
          delete [] A; delete [] D; delete [] Z; delete [] XX;
          return -4;
         }
       if (fabs(1.0 - Str[(*I)] / Str[(*I) - 1]) < Sharp)
         {
          delete [] A; delete [] D; delete [] Z; delete [] XX;
          return 0;
         }
      }

    /////////////////////////////////////////
    // New estimation of matrix of given data
    L = 0;
    for (J = 0; J < K; J++)
      {
       for (J1 = 0; J1 < N; J1++, L++)
         {
          for (I1 = 0, C = 0; I1 < K; I1++)
            {
             L1 = J * K + I1;
             if (D[L1] != 0)
             {
               if (Link == 1)
                 C += (1.0 - A[L1] / D[L1]) * (X[L] - X[J * N + I1]);
               else
                 C += (A[L1] - D[L1]) * SIGN(X[L] - X[J * N + I1]);
             }
            }
          XX[L] = X[L] + 2.0 * Step * C / B;
         }
      }
    for (J = 0, L = 0; J < K; J++)
      for (J1 = 0; J1 < N; J1++, L++)
        X[L] = XX[L];
   }

 delete [] A; delete [] D; delete [] Z; delete [] XX;

 return -3;
}

void MODIFICATOR DifferenceMatrix(double X[],double A[],long N,long K,long Link)
//
// The function for computation of matrix of differences.
// Designations:
//     X - matrix of given data
//     A - matrix of differences
//     N - size one of sample
//     K - order of matrix of differences
//  Link - type of metrica
// Return code:
//  none.
//
{
 long I,I1,I2,J, // Counter
      L = 0,     // Counter
      L1,        // Counter
      L2 = 0;    // Counter

 double *Y1,*Y2; // Vector of given data

 Y1 = new double[N];
 Y2 = new double[N];

 for (I = 0; I < K; I++)
   {
    for (J = 0; J < N; J++, L++)
      Y1[J] = X[L];
    for (I1 = 0, L1 = 0; I1 < K; I1++)
      {
       for (I2 = 0; I2 < N; I2++)
         Y2[I2] = X[L1++];
       if (Link == 1)
         A[L2++] = Euclid(Y1,Y2,N);
       else
         A[L2++] = CityBlock(Y1,Y2,N);
      }
   }

 delete [] Y1; delete [] Y2;
}

double MODIFICATOR CityBlock(double x[],double y[],long n)
//
// The function computes the Manhattan distance.
// Designations:
//  x - 1st compared sample,
//  y - 2nd compared sample,
//  n - each sample size.
// Return code:
//  value of Manhattan distance.
//
{
 long i;         // Counter

 double ret = 0; // Return code

 for (i = 0; i < n; i++)
  ret += fabs(x[i] - y[i]);

 return ret;
}

long MODIFICATOR InverseMatrix (double a[],double t[],long k)
//
// The function for computation of inverse of matrix.
// Designations:
//  a - given matrix,
//  t - inverse Matrix,
//  k - order of matrix.
// Return code:
//   0 - when normal end of computation,
//  -2 - when computation error.
//
{
 long Code; // Error code

 double *a1; // Copy of given matrix

 a1 = new double[k * k];

 ArrayToArray (a,a1,k * k);

 MatrixUnit (t,k);

 Code = Gauss (a1,t,k,k,0.000001);

 delete [] a1;

 return Code;
}

long MODIFICATOR Gauss(double a[],double b[],long n,long m,double eps)
//
// The function for solution of a matrix system of linear algebraic
// by method of Gauss equations with selection of the leading element.
// Designations:
//    a - matrix of system, break down,
//    b - matrix or vector of free members, break down, on
//        exit - accordingly matrix or vector of solution,
//    n - order of system,
//    m - number columns of matrix of free members,
//  eps - a small number for significance check.
// Return code:
//   0 when normal end of computation,
//  -2 when loss of significance.
// References:
//  Brandt S. Data Analysis. Statistical and computational methods for scientists and engineers. - New York: Springer-Verlag, 2003.
//
{
 long i,j,k,l; // Counters

 double save;  // Work variable

 ///////////////////////////////
 // Transformation of the system
 for (k = 0; k < n - 1; k++)
   {
    ///////////////////////////////
    // Selection of leading element
    save = a[k * n + k];
    l = k;
    for (i = k + 1; i < n; i++)
      if (fabs(save) < fabs(a[k * n + i]))
        {
         save = a[k * n + i];
         l = i;
        }

    //////////////////////
    // Permutation of rows
    if (l != k)
      {
       for (i = k; i < n; i++)
         {
          save = a[i * n + k];
          a[i * n + k] = a[i * n + l];
          a[i * n + l] = save;
         }
       for (i = 0; i < m; i++)
         {
          save = b[i * n + k];
          b[i * n + k] = b[i * n + l];
          b[i * n + l] = save;
         }
      }

    //////////////////////////////////////////
    // Converting the remaining part of arrays
    for (i = k + 1; i < n; i++)
      {
       save = a[k * n + i] / a[k * n + k];
       for (j = k + 1; j < n; j++)
         a[j * n + i] -= a[j * n + k] * save;
       for (j = 0; j < m; j++)
         b[j * n + i] -= b[j * n + k] * save;
      }
   }

 ////////////////////////////
 // Checking for significance
 if (fabs (a[n * n - 1]) < eps)
   return -2;

 ///////////////////////////
 // Disclosure of the system
 for (j = 0; j < m; j++)
   {
    b[j * n + n - 1] /= a[n * n - 1];
    if (n > 1)
      for (k = 1; k < n; k++)
        {
         i = n - 1 - k;
         for (l = i + 1; l < n; l++)
           b[j * n + i] -= a[l * n + i] * b[j * n + l];
         b[j * n + i] /= a[i * n + i];
        }
   }

 return 0;
}

long MODIFICATOR GaussL(long double a[],long double b[],long n,long m,long double eps)
//
// The function for solution of a matrix system of linear algebraic
// by method of Gauss equations with selection of the leading element.
// Designations:
//    a - matrix of system, break down,
//    b - matrix or vector of free members, break down, on
//        exit - accordingly matrix or vector of solution,
//    n - order of system,
//    m - number columns of matrix of free members,
//  eps - a small number for significance check.
// Return code:
//   0 when normal end of computation,
//  -2 when loss of significance.
// References:
//  Brandt S. Data Analysis. Statistical and computational methods for scie//
{
 long i,j,k,l;     // Counters

 long double save; // Work variable

 ///////////////////////////////
 // Transformation of the system
 for (k = 0; k < n - 1; k++)
   {
    ///////////////////////////////
    // Selection of leading element
    save = a[k * n + k];
    l = k;
    for (i = k + 1; i < n; i++)
      if (fabsl(save) < fabsl(a[k * n + i]))
        {
         save = a[k * n + i];
         l = i;
        }

    //////////////////////
    // Permutation of rows
    if (l != k)
      {
       for (i = k; i < n; i++)
         {
          save = a[i * n + k];
          a[i * n + k] = a[i * n + l];
          a[i * n + l] = save;
         }
       for (i = 0; i < m; i++)
         {
          save = b[i * n + k];
          b[i * n + k] = b[i * n + l];
          b[i * n + l] = save;
         }
      }

    //////////////////////////////////////////
    // Converting the remaining part of arrays
    for (i = k + 1; i < n; i++)
      {
       save = a[k * n + i] / a[k * n + k];
       for (j = k + 1; j < n; j++)
         a[j * n + i] -= a[j * n + k] * save;
       for (j = 0; j < m; j++)
         b[j * n + i] -= b[j * n + k] * save;
      }
   }

 ////////////////////////////
 // Checking for significance
 if (fabs (a[n * n - 1]) < eps)
   return -2;

 ///////////////////////////
 // Disclosure of the system
 for (j = 0; j < m; j++)
   {
    b[j * n + n - 1] /= a[n * n - 1];
    if (n > 1)
      for (k = 1; k < n; k++)
        {
         i = n - 1 - k;
         for (l = i + 1; l < n; l++)
           b[j * n + i] -= a[l * n + i] * b[j * n + l];
         b[j * n + i] /= a[i * n + i];
        }
   }

 return 0;
}

void MODIFICATOR MatrixUnit (double *e,long n)
//
// The function for computation of single matrix.
// Designations:
//  e - on exit - single matrix,
//  n - size of matrix.
// Return code:
//  none.
//
{
 long i,j; // Counters

 for (i = 0; i < n * n; i++)
   e[i] = 0;
 for (i = 0, j = 0; i < n; i++, j += n + 1)
   e[j] = 1;
}

double MODIFICATOR Krauth (double a[],long n,double eps)
//
// The function of decomposition of a non-degenerate real of matrix by the
// Krauth algorithm with the computation of the value of the determinant.
// Designations:
//    a - given matrix, break down, on exit - a combination of the lower
//        triangular of matrix (in the given L) and the upper triangular
//        of matrix (in the given U) with the implied unit diagonal,
//    n - order of matrix a,
//  eps - a small number for checking of significance.
// Return code:
//  value of determinant of given matrix.
// References:
//  Wilkinson J.H., Reinsch C. Handbook for automatic computation, Volume II, Linear Algebra. - New York, NY: Springer Verlag, 1971.
//
{
 long i,k,r,     // Counters
      rr = 0;    // Index of diagonal element

 double Det = 1, // Initial value of determinant
        buf;     // Work variable

 for (r = 0; r < n; r++, rr += n + 1)
   {
    /////////////////////////////////////
    // Computation of members of matrix L
    for (i = r; i < n; i++)
      {
       for (k = 0, buf = 0; k < r; k++)
         buf += a[i + k * n] * a[k + r * n];

       a[i + r * n] -= buf;
      }

    ////////////////////////////
    // Checking for significance
    if (fabs(a[rr]) < eps)
        return 0;

    /////////////////////////////////////
    // Computation of members of matrix U
    for (i = r + 1; i < n; i++)
      {
       for (k = 0, buf = 0; k < r; k++)
         buf += a[r + k * n] * a[k + i * n];
       a[r + i * n] = (a[r + i * n] - buf) / a[rr];
      }

    ///////////////////////////////////
    // Computation of determinant value
    Det *= a[rr];
   }

 return Det;
}

long MODIFICATOR Jacobi (double a[],long n,double eps,long *it,double b[])
//
// The function for computation own values and corresponding own vectors of a real symmetric matrix by Jacobi method.
// Designations:
//  a    - symmetric real matrix, break down, on exit
//         on main diagonal - eigen values,
//  n    - order of matrix a,
//  eps  - a small value for checking of significance,
//  *it  - counter of iterations,
//  b    - array of own vectors, corresponding to the calculated
//         own values, located by columns.
// Notice:
//  Checking the degeneracy of the matrix of the system is not possible - for the degenerate of the matrix, zero values will be obtained within
//  the accuracy of computations of eigen values. For checking the degeneracy use the Krauth function.
// Return code:
//   0 - when normal end of computation,
//  -1 - when lack of memory for work of array,
//  -2 - when computation error.
// References:
//  Wilkinson J.H., Reinsch C. Handbook for automatic computation, Volume II, Linear Algebra. - New York, NY: Springer Verlag, 1971.
//
{
 long i,j;         // Cycle counters

 long p = 0,q = 0, // Coordinates the largest non-diagonal element
      m1;          // Index one-dimensional arrays

 double max = 0,   // The module of the largest non-diagonal element
        t,         // Parameter of rotation angle
        s,s2,c,c2, // The sine and cosine of the rotation angle
                   // and their squares
        *r1,*r2;   // Work array

 /////////////
 // For size 1
 if (n == 1)
   {
    if (a[0] == 0)
      return -2;
    else
      a[0] = 1.0 / a[0];
    b[0] = 1;
    *it = 1;
    return 0;
   }

 /////////////////
 // Initial values
 MatrixUnit (b,n);
 *it = 0;

 if ((r1 = new double[n * n]) == 0)
   return -1;
 if ((r2 = new double[n * n]) == 0)
   {
    delete [] r1;
    return -1;
   }

 do
   {
    //////////////////////////////////////////////
    // Search for the largest non-diagonal element
    for (i = 0; i < n; i++)
      for (j = i + 1, m1 = i * n + i + 1; j < n; j++, m1++)
        {
         t = fabs (a[m1]);
         if (m1 == 1 || t > max)
           {
            p = i;
            q = j;
            max = t;
           }
        }

    /////////////////////////////////////
    // Computation of rotation parameters
    t = a[p * (n + 1)] == a[q * (n + 1)] ? M_PI / 4 * SIGN(a[n * q + p]) :
        atan (a[q * n + p] / (a[p * n + p] - a[q * n + q]) * 2) / 2;
    s = sin (t);
    c = cos (t);
    s2 = s * s;
    c2 = c * c;

    ///////////////////////////
    // Rotation of given matrix
    ArrayToArray (a,r1,n * n);

    for (i = 0; i < n; i++)
      if (i != p && i != q)
        {
         r1[p * n + i] =   a[p * n + i] * c + a[q * n + i] * s;
         r1[i * n + p] =  r1[p * n + i];
         r1[q * n + i] = - a[p * n + i] * s + a[q * n + i] * c;
         r1[i * n + q] =  r1[q * n + i];
        }

    r1[p * n + p] =  a[p * n + p] * c2 + 2 * a[q * n + p] * c * s + a[q * n + q] * s2;
    r1[q * n + q] =  a[p * n + p] * s2 - 2 * a[q * n + p] * c * s + a[q * n + q] * c2;
    r1[q * n + p] = (a[q * n + q] - a[p * n + p]) * c * s + a[q * n + p] * (c2 - s2);
    r1[p * n + q] = r1[q * n + p];

    (*it)++;

    ArrayToArray (r1,a,n * n);

    ///////////////////////////
    // Conservation own vectors
    MatrixUnit (r1,n);

    r1[p * n + p] = c;
    r1[q * n + q] = c;
    r1[q * n + p] = s;
    r1[p * n + q] = - s;

    MatrixMultiplyMatrixT (b,r1,r2,n,n,n);
    ArrayToArray (r2,b,n * n);

    /////////////////////////////////////////////////
    // Condition for the end of the iterative process
   } while (EuklidNorma0 (a,n) > eps);

 delete [] r1; delete [] r2;

 return 0;
}

double MODIFICATOR EuklidNorma0 (double a[],long n)
//
// The function for computation Euclid norma of matrix without diagonal elements.
// Designations:
//  a - given matrix,
//  n - order of matrix.
// Return code:
//  value of norma.
//
{
 long i,j; // Counter

 long m;

 double s = 0;

 for (i = 0, m = 0; i < n; i++)
   for (j = 0; j < n; j++, m++)
     if (i != j)
       s += a[m] * a[m];

 return sqrt (s);
}

double MODIFICATOR EuclidNormaVector (double* a,long n)
//
// The function for computation Euclid norma of vector.
// Designations:
//  a - given vector by length n.
// Return code:
//  value of norma.
//
{
 return sqrt (VectorSumSQ (a,n));
}


void MODIFICATOR MatrixMultiplyMatrixT (double a[],double b[],double r[],long n,long m,long l)
//
// The function for multiplication of matrix on transpose matrix.
// Designations:
//  a - 1st multiplier, n x m,
//  b - 2nd multiplier, l x m,
//  r - result, n x l,
//  n - number rows in matrix a,
//  m - number columns in matrix a and columns in matrix b,
//  l - number rows in matrix b.
// Return code:
//  none.
//
{
 long i,           // Cycle counter by columns b
      j,           // Cycle counter by columns a
      k,           // Cycle counter b
      ia,          // Counter of members of matrix a
      ib,          // Counter of members of matrix b
      ir = -1,     // Counter of members of matrix r
      ik = -1 - l; // 1st element current of rows b

 ////////////////////
 // Cycle of matrix b
 for (k = 0; k < l; k++)

   ////////////////////
   // Cycle of matrix a
   for (j = 0, ik++; j < n; j++)

     ///////////////////////////////
     // Cycle by columns of matrix a
     for (i = 0, ia = j - n, ib = ik, r[++ir] = 0; i < m; i++)
       {
        ia += n;
        ib += l;
        r[ir] += a[ia] * b[ib];
       }
}

long MODIFICATOR Choleski (double a[],double b[],long n)
//
// The function a real triangular decomposition of a positively
// defined symmetric of matrix according to the Choleski scheme.
// Designations:
//  a - given matrix, in which only the lower triangle can be correct,
//  b - obtained low triangular matrix,
//  n - order of each matrix.
// Return code:
//   0 when normal result of computation,
//  -2 when cannot obtain real triangular decomposition
//     (given matrix is degenerated).
// References:
//  Wilkinson J.H., Reinsch C. Handbook for automatic computation, Volume II, Linear Algebra. - New York, NY: Springer Verlag, 1971.
//
{
 long i,j,k,                             // Cycle counters
      ii,jj,ij,is,js,ik,jk,kk = - n - 1; // Counters

 double s;                               // Work variable

 /////////////////////////////////////////
 // Zeroing the upper triangle of matrix b
 for (i = 1, jj = 0; i < n; i++)
   {
    jj += n;
    for (j = 1, ij = jj; j <= i; j++)
      b[ij++] = 0;
   }

 for (i = 0, ii = kk, is = kk; i < n; i++)
   {
    /////////////////////////////////////
    // Obtaining of non-diagonal elements
    for (j = 0, js = kk, ij = ++is, jj = kk; j <= i - 1; j++)
      {
       ij += n;
       s = a[ij];
       for (k = 0, ik = is, jk = ++js; k <= j - 1; k++)
     {
      ik += n;
      jk += n;
      s -= b[ik] * b[jk];
     }
       jj += n + 1;
       if (!b[jj])
     return -2;
       else
     b[ij] = s / b[jj];
      }
    //////////////////////////////////
    // Obtaining non-diagonal elements
    ii -= kk;
    s = a[ii];
    for (k = 0, ik = is; k <= i - 1; k++)
      {
       ik += n;
       s -= b[ik] * b[ik];
      }
    if (s < 0)
      return -2;
    else
      b[ii] = sqrt(s);
   }

 return 0;
}

long MODIFICATOR QRDecomposition (double a[],double q[],double r[],long n,long m)
//
// The function for QR-decomposition of matrix of general form.
// Designations:
//   a - given matrix, non break down,
//   q - orthogonal matrix,
//   r - upper triangular matrix,
//   n - number rows of given matrix,
//   m - number columns of given matrix.
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory for computation,
//  -2 when computation error.
// References:
//  Wilkinson J.H., Reinsch C. Handbook for automatic computation, Volume II, Linear Algebra. - New York, NY: Springer Verlag, 1971.
//
{
 long i,j,k,    // Counters
      Code = 0, // Return code
      l,        // Counter of members of matrix a
      l1,l2,    // Counter of members of system
      l3,       // Counter of members of matrix q
      l4;       // Counter of members of matrix r

 double *v,     // Work array
        s;      // Work variable

 if ((s = EuclidNormaVector (a,n)) == 0)
   return -2;
 if ((v = new double[n]) == 0)
   return -1;

 /////////////////////////
 // 1st of column matrix q
 for (l3 = 0; l3 < n; l3++)
   q[l3] = a[l3] / s;

 /////////////////////////
 // 1st of column matrix r
 r[0] = s;
 for (l4 = 1; l4 < m; l4++)
   r[l4] = 0;

 ////////////////////
 // Recurrent process
 for (i = 1; i < m; i++)
   {
    for (j = 0,l = i * n; j < n; j++)
      v[j] = a[l++];

    for (k = 0,l = 0,l1 = 0; k < i; k++)
      {
       for (j = 0,s = 0,l2 = i * n; j < n; j++)
     s += q[l++] * a[l2++];
       for (j = 0; j < n; j++)
     v[j] -= s * q[l1++];
       r[l4++] = s;
      }

    if ((s = EuclidNormaVector (v,n)) == 0)
      {
       Code = -2;
       break;
      }
    for (j = 0; j < n; j++)
      q[l3++] = v[j] / s;
    r[l4++] = s;
    for (j = i + 1; j < m; j++)
      r[l4++] = 0;
   }

 delete [] v;

 return Code;
}

double MODIFICATOR VectorSumSQ (double* x,long n)
//
// The function for computation sums of squares of members arrays.
// Designations:
//  - - array,
//  n - size arrays.
// Return code:
//  Sum of members arrays.
//
{
 long i; // Counter

 double s = 0;

 for (i = 0; i < n; i++)
   s += x[i] * x[i];

 return s;
}

long MODIFICATOR SingularValueDecomposition (double a[],long m,long n,double u[],double s[],double v[])
//
// The function for decomposition of matrix by singular values (SVG).
// Designations:
//   a - given matrix m x n, no break down,
//   m - number rows of matrix a,
//   n - number columns of matrix a,
//   u - left matrix orthogonal transformations m x n,
//   s - matrix of singular values n x n,
//   v - right matrix orthogonal transformations n x n.
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory for computation,
//  -2 when loss of significance or error in computation.
// References:
//  Wilkinson J.H., Reinsch C. Handbook for automatic computation, Volume II, Linear Algebra. - New York, NY: Springer Verlag, 1971.
//
{
 long i,j,     // Counter
      k,       // Work variable
      Code;    // Return code

 double save,  // Work variable
        *c,*t; // Work array

 if ((c = new double[n * n]) == 0)
   return -1;
 if ((t = new double[n * n]) == 0)
   {
    delete [] c;
    return -1;
   }

 /////////////////////////////////////
 // Computation own values and vectors
 MatrixTMultiplyMatrix (a,a,s,n,m,n);
 if ((Code = Jacobi (s,n,0.000001,&k,v)) != 0)
   {
    delete [] c; delete [] t;
    return Code;
   }

 //////////////////////////////////
 // Ordering own values and vectors
 for (i = 1, k = n + 1; i < n; i++, k += n + 1)
   s[i] = s[k];
 for (i = 0; i < n; i++)
   {
    k = i;
    save = s[i];
    for (j = i + 1; j < n; j++)
    if (save < s[j])
      {
       k = j;
       save = s[j];
      }
    ColInterchange (s,1,i,k);
    ColInterchange (v,n,i,k);
   }

 ////////////////////////////
 // Matrix of singular values
 for (i = 1, k = n + 1; i < n; i++, k += n + 1)
   {
    s[k] = s[i];
    s[i] = 0;
   }
 for (i = 0; i < n * n; i += n + 1)
   s[i] = sqrt (s[i]);
 MatrixMultiplyMatrixT (s,v,c,n,n,n);
 if ((Code = InverseMatrix (c,t,n)) != 0)
   {
    delete [] c; delete [] t;
    return Code;
   }
 MatrixMultiplyMatrix (a,t,u,m,n,n);

 delete [] c; delete [] t;

 return Code;
}

void MODIFICATOR ColInterchange (double a[],long n,long k,long l)
//
// The function for column permutations of matrix.
// Designations:
//     a - given matrix, break down,
//     n - number rows of matrix,
//  k, l - numbers of the columns to be rearranged (numeration from zero).
// Notice:
//  result writing on place of given matrix.
// Return code:
//  none.
//
{
 long i,     // Cycle counter
      m1,m2; // Counters one-dimensional arrays

 double save; // Work variable

 for (i = 0, m1 = k * n, m2 = l * n; i < n; i++)
   {
    save = a[m1];
    a[m1++] = a[m2];
    a[m2++] = save;
   }
}

void MODIFICATOR MatrixTMultiplyMatrix (double a[],double b[],double r[],long n,long m,long l)
//
// The function for multiplication transpose matrix on matrix.
// Designations:
//  a - 1st multiplier, m x n,
//  b - 2nd multiplier, m x l,
//  r - result, n x l,
//  n - number columns in matrix a,
//  m - number rows in matrix a and rows in matrix b,
//  l - number columns in matrix b.
// Return code:
//  none.
//
{
 long i,           // Cycle counter by columns b
      j,           // Cycle counter by columns a
      k,           // Cycle counter b
      ia,          // Counter of members of matrix a
      ib,          // Counter of members of matrix b
      ir = -1,     // Counter of members of matrix r
      bs = -m - 1; // 1st element of current column b

 ///////////////////////////////
 // Cycle by columns of matrix b
 for (k = 0; k < l; k++)

   ///////////////////////////////
   // Cycle by columns of matrix a
   for (j = 0, ia = -1, bs += m; j < n; j++)

     ////////////////////
     // Cycle of matrix a
     for (i = 0, r[++ir] = 0, ib = bs; i < m; i++)
        r[ir] += a[++ia] * b[++ib];
}

void MODIFICATOR MatrixMultiplyMatrix (double a[],double b[],double r[],long n,long m,long l)
//
// The function for multiplication of matrix.
// Designations:
//  a - 1st multiplier, n x m,
//  b - 2nd multiplier, m x l,
//  r - composition, n x l,
//  n - number rows in matrix a,
//  m - number columns in matrix a and rows in matrix b,
//  l - number columns in matrix b.
// Return code:
//  none.
//
{
 long i,            // Cycle counter by columns b
      j,            // Cycle counter by columns a
       k,           // Cycle counter b
       ia,          // Counter of members of matrix a
       ib,          // Counter of members of matrix b
       ir = -1,     // Counter of members of matrix r
       ik = -m - 1; // Index before 1st element of column b

 //////////////////////////////
 // Cycle by columns of matrix b
 for (k = 0; k < l; k++)

   ////////////////////
   // Cycle of matrix a
   for (j = 0, ik += m; j < n; j++)

       ///////////////////////////////
       // Cycle by columns of matrix a
     for (i = 0, ia = j, ib = ik, r[++ir] = 0; i < m; i++, ia += n)
       r[ir] += a[ia] * b[++ib];
}

long MODIFICATOR PseudoInverseMatrix (double a[],double ap[],long m,long n)
//
// The function for computation pseudoinverse of matrix.
// Designations:
//   a - given matrix m x n, no break down,
//  ap - pseudoinverse Matrix n x m,
//   m - number rows of matrix a,
//   n - number columns of matrix a,
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory for computation,
//  -2 when loss of significance or error of computation.
//
{
 long i,          // Cycle counter
      Code;       // Return code

 double *u,*v,*s, // SVG matrices
        *r;       // Work array, you can not select if
                  // destroy the given matrix

 if ((u = new double[m * m]) == 0)
   return -1;
 if ((v = new double[n * n]) == 0)
   {
    delete [] u;
    return -1;
   }
 if ((s = new double[n * n]) == 0)
   {
    delete [] u; delete [] v;
    return -1;
   }
 if ((r = new double[n * n]) == 0)
   {
    delete [] u; delete [] v; delete [] s;
    return -1;
   }

 if ((Code = SingularValueDecomposition (a,m,n,u,s,v)) != 0)
   {
    delete [] u; delete [] v; delete [] s; delete [] r;
    return Code;
   }

 for (i = 0; i < n * n; i += n + 1)
   if (s[i] != 0)
     s[i] = 1.0 / s[i];

 MatrixMultiplyMatrix (v,s,r,n,n,n);
 MatrixMultiplyMatrixT (r,u,ap,n,n,m);

 delete [] u; delete [] v; delete [] s; delete [] r;

 return Code;
}

void MODIFICATOR CorrelationMatrix (double x[],double y[],long r,long c)
//
// The function computes correlation matrix.
// Designations:
//  x - given r x c array,
//  y - calculated correlation matrix,
//  r - number of rows,
//  c - number of columns and order of correlation matrix.
// Return code:
//   None.
//
{
 long i,j,k,     // Cycle counters
      m;         // Array counter

 double *c1,*c2; // Work arrays

 c1 = new double[r];
 c2 = new double[r];

 //////////////////////////////////////////////////////
 // Calculation of upper triangle of correlation matrix
 for (i = 0; i < c; i++)
   {
    for (k = 0; k < r; k++)
      c1[k] = x[i * r + k];
    m = i * c + i;
    for (j = i; j < c; j++)
      {
       for (k = 0; k < r; k++)
         c2[k] = x[j * r + k];
       y[m++] = Pearson (c1,c2,r);
      }
   }

 //////////////////////////////////////////////////////
 // Calculation of lower triangle of correlation matrix
 for (i = 0; i < c; i++)
    for (j = i; j < c; j++)
       y[j * c + i] = y[i * c + j];

 delete [] c1; delete [] c2;
}

double MODIFICATOR Pearson (double x[],double y[],long n)
//
// The function computes value of correlation coefficient by Pearson' formula.
// Designations:
//  x - 1st array,
//  y - 2nd array,
//  n - size of each array.
// Return code:
//  value of coefficient correlation.
//
{
 double s; // Work variable

 if ((s = Cov (x,x,n) * Cov (y,y,n)) == 0)
   return 0;
 else
   return Cov (x,y,n) / sqrt (s);
}

double MODIFICATOR Cov (double x[],double y[],long n)
//
// The function computes covariation (by x = y - variations).
// Designations:
//  x - 1st array,
//  y - 2nd array,
//  n - size of each array.
// Return code:
//  value of covariation.
//
{
 long i;                          // Cycle counter

 double xm = VectorSum (x,n) / n, // Medium value x
        ym = VectorSum (y,n) / n, // Medium value y
        s = 0;                    // Work variable

 for (i = 0; i < n; i++)
   s += (x[i] - xm) * (y[i] - ym);

 return s / (n - 1);
}

double MODIFICATOR VectorSum (double x[],long n)
//
// The function for computation sum of array.
// Designations:
//  - - array,
//  n - size arrays.
// Return code:
//  sum of array.
//
{
 long i;       // Cycle counter

 double s = 0; // Return value

 for (i = 0; i < n; i++)
   s += x[i];

 return s;
}

double MODIFICATOR FarrarGlauber1 (double dat[],long n,long k,double *d,double *xi)
//
// The function checks the set of vectors for the presence of multi-collinearity.
// Designations:
//   x - array of given data,
//   n - number rows (sample size),
//   k - number columns (number of parameters),
//   d - determinant,
//  xi - value chi-square.
// Return code:
//  Value of multi-collinearity.
//
{
 long i,j,i1,  // Cycle counters
      m = 0,   // Counter of members arrays
      m1 = 0,  // Counter of members arrays
      l;       // Counter of members arrays

 double *x,*y, // Copy of column of given matrix
        *r;    // Correlaton Matrix

 x = new double[n];
 y = new double[n];
 r = new double[k * k];

 ///////////////////////////////////
 // Forming of correlation matrix
 for (i = 0; i < k; i++)
   {
    for (j = 0; j < n; j++)
      x[j] = dat[m++];

    for (j = 0, l = 0; j < k; j++)
      {
       for (i1 = 0; i1 < n; i1++)
     y[i1] = dat[l++];

       r[m1++] = Pearson (x,y,n);
      }
   }

 delete [] x; delete [] y;

 /////////////////////////////
 // Compensation for stability
 if ((*d = Krauth (r,k,0.000000001)) <= 0)
   *d = 0.000001;

 delete [] r;

 *xi = -(n - 1 - (2 * k + 5) / 6) * log (*d);

 return ChiSquareDistribution (0.5 * k * (k - 1),*xi);
}

long MODIFICATOR FarrarGlauber2 (double dat[],long n,long k,double r[],double f[],double p[])
//
// The function checks the set of vectors for the presence of multi-collinearity.
// Designations:
//  x - array of given data,
//  n - number rows (sample size),
//  k - number columns (number of parameters),
//  r - coefficients of determination (k),
//  f - values of F-test (k),
//  p - significance (k).
// Return code:
//   0 when normal end of computation,
//  -2 when loss of significance.
//
{
 long i,j,i1,  // Cycle counters
      m = 0,   // Counter of members arrays
      m1 = 0,  // Counter of members arrays
      l,       // Counter of members arrays
      Code;    // Error code

 double *x,*y, // Copy of column of given matrix
        *a,    // Correlaton Matrix
        *c,    // Matrix, inverse of correlation
        z;     // Work variable

 x = new double[n];
 y = new double[n];
 a = new double[k * k];
 c = new double[k * k];

 ////////////////////////////////
 // Forming of correlation matrix
 for (i = 0; i < k; i++)
   {
    for (j = 0; j < n; j++)
      x[j] = dat[m++];

    for (j = 0, l = 0; j < k; j++)
      {
       for (i1 = 0; i1 < n; i1++)
     y[i1] = dat[l++];

       a[m1++] = Pearson (x,y,n);
      }
   }
 delete [] x; delete [] y;

 //////////////////////////////////
 // Inversion of correlation matrix
 if ((Code = InverseMatrix (a,c,k)) != 0)
   {
    delete [] a; delete [] c;
    return Code;
   }

 delete [] a;

 ///////////////////////////////////
 // Parameters of multi-collinearity
 for (i = 0; i < k; i++)
   {
    if ((z = c[i * (k + 1)]) == 0)
      {
       delete [] c;
       return -2;
      }
    r[i] = 1.0 - 1.0 / z;
    f[i] = (z - 1) * (n - k) / (k - 1);
    p[i] = FDistribution(n - k,k - 1,f[i]);
    p[i] = MIN(p[i],1 - p[i]);
   }

 delete [] c;

 return 0;
}

long MODIFICATOR FarrarGlauber3 (double dat[],long n,long k,double r[],double t[],double f[],long ii[],long jj[])
//
// The function checks the set of vectors for the presence of multi-collinearity.
// Designations:
//   x - array of given data,
//   n - number rows (sample size),
//   k - number columns (number of parameters),
//   r - partial coefficients correlation by ((k * k - k) / 2),
//   t - values of t-test by ((k * k - k) / 2),
//   f - significances by ((k * k - k) / 2),
//  ii - numbers of compared columns (numeration with 1),
//  jj - numbers of compared columns (numeration with 1).
// Return code:
//   0 when normal end of computation,
//  -2 loss of significance.
//
{
 long i,j,i1,  // Cycle counters
      m = 0,   // Counter of members arrays
      m1 = 0,  // Counter of members arrays
      l,       // Counter of members arrays
      Code;    // Error code

 double *x,*y, // Copy of column of given matrix
        *a,    // Correlaton Matrix
        *c,    // Matrix, inverse of correlation
        z;     // Work variable

 x = new double[n];
 y = new double[n];
 a = new double[k * k];
 c = new double[k * k];

 ////////////////////////////////
 // Forming of correlation matrix
 for (i = 0; i < k; i++)
   {
    for (j = 0; j < n; j++)
      x[j] = dat[m++];

    for (j = 0, l = 0; j < k; j++)
      {
       for (i1 = 0; i1 < n; i1++)
     y[i1] = dat[l++];

       a[m1++] = Pearson (x,y,n);
      }
   }

 delete [] x; delete [] y;

 //////////////////////////////////
 // Inversion of correlation matrix
 if ((Code = InverseMatrix (a,c,k)) != 0)
   {
    delete [] a; delete [] c;
    return Code;
   }

 delete [] a;

 ///////////////////////////////////
 // Parameters of multi-collinearity
 for (i = 0, m = 0; i < k - 1; i++)
   for (j = i + 1; j < k; j++, m++)
     {
      if ((z  = c[i * (k + 1)] * c[j * (k + 1)]) == 0)
        {
         delete [] c;
         return -2;
        }
      r[m] = -c[i * k + j] / sqrt (z);
      t[m] = r[m] * sqrt(n - k) / sqrt(1 - r[m] * r[m]);
      f[m] = TDistribution (n - k,t[m]);
      f[m] = MIN(f[m],1 - f[m]);
      ii[m] = i + 1;
      jj[m] = j + 1;
     }

 delete [] c;

 return 0;
}

void MODIFICATOR MatrixMultiplyNumber (double x[],long n,double number)
//
// The function for multiplication of matrix by element on number.
// Designations:
//       x - 1st multiplier - matrix,
//  number - 2nd multiplier - number,
//       n - number of members of matrix.
// Return code:
//  none.
//
{
 long i; // Cycle counter

 for (i = 0; i < n; i++)
   x[i] *= number;
}

void MODIFICATOR MatrixPlusMatrix (double a[],double b[], long n)
//
// The function sums up by element two arrays.
// Designations:
//  a - 1st term,
//  b - 2nd term,
//  n - size of each arrays.
// Notice:
//  result saved in the 1st arrays.
// Return code:
//  none.
//
{
 long i; // Cycle counter

 for (i = 0; i < n; i++)
   a[i] += b[i];
}

void MODIFICATOR MatrixMinusMatrix (double a[],double b[], long n)
//
// The function subtracts by element two arrays.
// Designations:
//  a - 1st term,
//  b - 2nd term,
//  n - size of each arrays.
// Notice:
//  result saved in 1st arrays.
// Return code:
//  none.
//
{
 long i; // Cycle counter

 for (i = 0; i < n; i++)
   a[i] -= b[i];
}

void MODIFICATOR MatrixMinusMatrix2 (double a[],double b[],double c[],long n)
//
// The function subtracts by element two arrays.
// Designations:
//  a - reduced,
//  b - deductible,
//  c - difference,
//  n - size of each arrays.
// Return code:
//  none.
//
{
 long i; // Cycle counter

 for (i = 0; i < n; i++)
   c[i] = a[i] - b[i];
}

void MODIFICATOR ArrayToArray (double x[],double y[],long n)
//
// The function copy array in array.
// Designations:
//  - - array - source,
//  y - array - receiver,
//  n - number of copying elements.
// Return code:
//  none.
//
{
 long i; // Cycle counter

 for (i = 0; i < n; i++)
   y[i] = x[i];
}

void MODIFICATOR SortArrayCommon (double x[],double y[],long n)
//
// The function joint sorting of array of numbers.
// Designations:
//  x - sorting array,
//  y - joint array,
//  n - size of each arrays,
// Return code:
//  none.
// References:
//  Skiena S.S. The algorithm design manual. - New York, NY: Springer, 2008.
//
{
 long i,j,k,ia,is; // Counters

 double a;         // Work variable

 ////////////////////////////
 // Sort in ascending order X
 for (i = 0; i < n; i++)
   {
    ia = i;
    a = x[ia];
    for (j = i + 1; j < n; j++)
      if (a >= x[j])
       {
        ia = j;
        a = x[j];
       }
    a = x[i];
    x[i] = x[ia];
    x[ia] = a;
    a = y[i];
    y[i] = y[ia];
    y[ia] = a;
   }

 //////////////////////
 // Sorting in series Y
 i = 0;
 do
   {
    //////////////////////////////////////////////
    // Counting numbers of equal adjacent elements
    is = 1;
    for (j = i; j < n; j++)
      {
       if (x[j + 1] == x[j])
     is++;
       else
         break;
      }

    ///////////////////////////////////////////////
    // Next are the nevertheless two equal elements
    if (is > 1)
      {
       for (j = i; j < i + is; j++)
         {
          ia = j;
          a = y[ia];
          for (k = j + 1; k < i + is; k++)
            if (a > y[k])
              {
               ia = k;
               a = y[k];
              }
          a = y[j];
          y[j] = y[ia];
          y[ia] = a;
         }
       i += is;
      }
    else
      i++;
   } while (i < n);
}

void MODIFICATOR MatrixT (double x[],double y[],long n,long k)
//
// The function for transpose of matrix.
// Designations:
//  - - given matrix from k vectors by n of members one by one,
//  y - transposed matrix from n vectors by k elements.
// Return code:
//  none.
//
{
 long i,j,     // Counter of cycles
      mm1,mm2; // Counters of members of array

 for (i = 0, mm2 = 0; i < k; i++)
   for (j = 0, mm1 = i; j < n; j++, mm1 += k)
     y[mm1] = x[mm2++];
}

void FillUp (double a[],long n,double p)
//
// The function fills the array with the specified value.
//  a - array by length n,
//  p - placeholder.
// Return code:
//  none.
//
{
 long i; // Cycle counter

 for (i = 0; i < n; i++)
   a[i] = p;
}

long MODIFICATOR Eigen (double a[],double b[],long n,double eps,long *it)
//
// The function solutions to the generalized own values
// for problem of a special type of system.
// Designations:
//    a - symmetric real matrix, break down, on
//        exit on main diagonal - eigen values,
//    b - symmetric real positively defined Matrix,
//        break down, on exit - array own vectors, corresponding to
//        calculated own values, located by columns,
//    n - the order of the matrices a and b,
//  eps - small value for checking significance,
//  *it - counter of iterations.
// Return code:
//   0 when normal result of computation,
//  -1 when lack of memory for work of array,
//  -2 when computation error.
// References:
//  Gantmacher F.R. The theory of matrices. Volume one. - New York, NY: AMS Chelsea Publishing, 1990.
//
{
 long Code,Code1; // Work variable

 double *l,*li;   // Work array

 if ((l = new double[n * n]) == 0)
   return -1;

 ////////////////////////////////////////
 // Decomposition of matrix b by Choleski
 if ((Code = Choleski (b,l,n)) != 0)
   {
    delete [] l;
    return Code;
   }

 ////////////////////////////////////////////
 // Save a copy of l for recovery own vectors
 ArrayToArray (l,b,n * n);

 ////////////////////////////////
 // Computation of inverse matrix
 if ((li = new double[n * n]) == 0)
   {
    delete [] l;
    return -1;
   }
 if ((Code = InverseMatrix (l,li,n)) != 0)
   {
    delete [] l; delete [] li;
    return Code;
   }

 ///////////////////////////////////////////
 // Bringing the system to the standard form
 MatrixMultiplyMatrix (li,a,l,n,n,n);
 MatrixMultiplyMatrixT (l,li,a,n,n,n);

 delete [] l;

 //////////////////////////////////////////
 // Matrix systems for recovery own vectors
 MatrixT (b,li,n,n);

 ///////////////////////////////
 // Solution of standard problem
 if ((Code = Jacobi (a,n,eps,&Code1,b)) != 0)
   {
    delete [] li;
    return Code;
   }

 *it = Code1;

 ///////////////////////
 // Recovery own vectors
 Code = Gauss (li,b,n,n,eps);

 delete [] li;

 return Code;
}

long MODIFICATOR Gram (double a[], long n)
//
// The function of checking whether a given matrix is the matrix of
// Gram (symmetric and positive semidefinite).
// Designations:
//  a - given matrix,
//  n - order of matrix.
// Return code:
//   0 - when matrix is the matrix of Gram,
//  -1 - when less memory for computation,
//  -2 - matrix is not the matrix of Gram.
// References:
//  Gantmacher F.R. The theory of matrices. Volume one. - New York, NY: AMS Chelsea Publishing, 1990.
//
{
 long i,j,k,   // Cycle counters
      m,m1,m2; // Array counter

 double *a1;   // Major minor

 ////////////////////////
 // Checking the symmetry
 for (i = 0; i < n; i++)
   for (j = 0; j < n; j++)
     if (a[i + j * n] != a[j + i * n])
       return -2;

 if ((a1 = new double[n * n]) == 0)
   return -1;

 //////////////////////////////////////
 // Checking positive semi-definiteness
 for (i = 0; i < n; i++)
   {
    /////////////////////////////
    // Computation of major minor
    for (j = i, m = 0, m1 = i * (n + 1); j < n; j++,m1 += n)
      for (k = i, m2 = m1; k < n; k++)
        a1[m++] = a[m2++];
    if (Krauth (a1,n - i,0.000000001) < 0)
      {
       delete [] a1;
       return -2;
      }
   }

 delete [] a1;

 return 0;
}

void MODIFICATOR SortArrayUp (double x[],long n,long ix[])
//
// The function for sorting arrays of numbers in ascending order.
// Designations:
//   x - given array,
//   n - its size,
//  ix - saved information about given arrays.
// Return code:
//  none.
// References:
//  Skiena S.S. The algorithm design manual. - New York, NY: Springer, 2008.
//
{
 long i,j,ia,ii;

 double a;

 /////////////////
 // Initialization
for (i = 0; i < n; i++)
  ix[i] = i;

 ////////////////////////////////////
 // Sorting arrays in ascending order
 for (i = 0; i < n; i++)
   {
    ia = i;
    a = x[ia];
    for (j = i + 1; j < n; j++)
      if (a > x[j])
       {
        ia = j;
        a = x[j];
       }
    a = x[i];
    x[i] = x[ia];
    x[ia] = a;

    /////////////////////////////
    // conservation of given info
    ii = ix[i];
    ix[i] = ix[ia];
    ix[ia] = ii;
   }
}

void MODIFICATOR SortArrayUp0 (double x[],long n)
//
// The function for sorting arrays of numbers in ascending order.
// Designations:
//   x - given array,
//   n - its size.
// Return code:
//  none.
// References:
//  Skiena S.S. The algorithm design manual. - New York, NY: Springer, 2008.
//
{
 long i,j,ia;

 double a;

  ///////////////////////////////////
 // Sorting arrays in ascending order
 for (i = 0; i < n; i++)
   {
    ia = i;
    a = x[ia];
    for (j = i + 1; j < n; j++)
      if (a > x[j])
       {
        ia = j;
        a = x[j];
       }
    a = x[i];
    x[i] = x[ia];
    x[ia] = a;
  }
}

void MODIFICATOR SortArrayDown (double x[],long n,long ix[])
//
// The function of sorting arrays of numbers in descending order.
// Designations:
//   x - given array,
//   n - its size,
//  ix - saved information about given arrays.
// Return code:
//  none.
// References:
//  Skiena S.S. The algorithm design manual. - New York, NY: Springer, 2008.
//
{
 long i,j,ia,ii;

 double a;

 /////////////////
 // Initialization
for (i = 0; i < n; i++)
  ix[i] = i;

 /////////////////////////////////////
 // Sorting arrays in descending order
 for (i = 0; i < n; i++)
   {
    ia = i;
    a = x[ia];
    for (j = i + 1; j < n; j++)
      if (a <= x[j])
       {
        ia = j;
        a = x[j];
       }
    a = x[i];
    x[i] = x[ia];
    x[ia] = a;

    ////////////////////////////////////
    // conservation of given information
    ii = ix[i];
    ix[i] = ix[ia];
    ix[ia] = ii;
   }
}

void MODIFICATOR SortArrayDown0 (double x[],long n)
//
// The function of sorting arrays of numbers in descending order.
// Designations:
//   x - given array,
//   n - its size.
// Return code:
//  none.
// References:
//  Skiena S.S. The algorithm design manual. - New York, NY: Springer, 2008.
//
{
 long i,j,ia;

 double a;

 /////////////////////////////////////
 // Sorting arrays in descending order
 for (i = 0; i < n; i++)
   {
    ia = i;
    a = x[ia];
    for (j = i + 1; j < n; j++)
      if (a <= x[j])
       {
        ia = j;
        a = x[j];
       }
    a = x[i];
    x[i] = x[ia];
    x[ia] = a;
   }
}

void MODIFICATOR SortArrayCommonI (double x[],long y[],long n)
//
// The function of joint sorting of an array of numbers.
// Designations:
//  x - a sortable Array,
//  y - a conjugate Array,
//  n - size,
// Return code:
//  none.
// References:
//  Skiena S.S. The algorithm design manual. - New York, NY: Springer, 2008.
//
{
 long i,j,ia;

 double a;

 ////////////////////////////////////
 // Sorting arrays in ascending order
 for (i = 0; i < n; i++)
   {
    ia = i;
    a = x[ia];
    for (j = i + 1; j < n; j++)
      if (a >= x[j])
       {
    ia = j;
    a = x[j];
       }
    a = x[i];
    x[i] = x[ia];
    x[ia] = a;
    j = y[i];
    y[i] = y[ia];
    y[ia] = j;
   }
}

void MODIFICATOR UnSortArray (double x[],double y[],long n,long ix[])
//
// The function of restoring the order of members sorted
// arrays of numbers in accordance with the stored information about
// the original arrays.
// Designations:
//  x - sorted Array,
//  y - restored Array,
//  n - size of each arrays,
//  ix - array of ordinal numbers of members of given arrays.
// Return code:
//  none.
//
{
 for (long i = 0 ; i < n; i++)
   y[ix[i]] = x[i];
}

double MODIFICATOR LogFactorial (double n)
//
// The function computes logarithm of factorial.
// Designations:
//   n - number.
// Return code:
//  value of logarithm of factorial.
//
{
 double i = 1,   // Counter
        ret = 0; // Return code

 while (i <= n)
   {
    ret += log (i);
    i++;
   }

 return ret;
}

double Pow(long x,long y)
//
// The function computes integer degree of integer value.
// Designations:
//  x - basis,
//  y - indicator.
// Return code:
//  degree of value.
//
{
 long i;         // Counter

 double ret = 1; // Return code

 for (i = 0; i < y; i++)
   ret *= x;

 return ret;
}

double MODIFICATOR MaximumOf (double data[],long n)
//
// The function for search of maximal element of array.
// Designations:
//  data - given array,
//     n - size of array.
// Return code:
//  value of maximal element of array.
//
{
 long i;                    // Counter

 double RetValue = data[0]; // Return code

 for (i = 1; i < n; i++)
   if (data[i] > RetValue)
     RetValue = data[i];

 return RetValue;
}

double MODIFICATOR VectorNorma (double a[],long n)
//
// The function for computation of square of vector norma.
// Designations:
//  a - given vector,
//  n - size of vector.
// Return code:
//  value of square of vector norma.
//
{
 long i;

 double s = 0;

 for (i = 0; i < n; i++)
   s += a[i] * a[i];

 return s;
}

long MODIFICATOR TwoDegree (long n)
//
// The function computes value two in degree n.
// Designations:
//   n - degree.
// Return code:
//  degree of value two.
//
{
 long i,       // Counter
      Ret = 1; // Return code

 for (i = 1; i <= n; i++)
   Ret *= 2;

 return Ret;
}

void MODIFICATOR SortArrayUpI (double x[],long n)
//
// The function for sorting arrays of numbers in ascending order.
// Designations:
//  x - given array,
//  n - size of array.
// Return code:
//  none.
// References:
//  Skiena S.S. The algorithm design manual. - New York, NY: Springer, 2008.
//
{
 long i,j, // Counter
      ia;  // Index

 double a; // Work variable

 ////////////////////////////////////
 // Sorting arrays in ascending order
 for (i = 0; i < n; i++)
   {
    ia = i;
    a = x[ia];
    for (j = i + 1; j < n; j++)
      if (a > x[j])
       {
        ia = j;
        a = x[j];
       }
    a = x[i];
    x[i] = x[ia];
    x[ia] = a;
   }
 }

double MaximumOf (double data[],long n,long *k)
//
// The function for computation of value of maximal element of array.
// Designations:
//  data - given array,
//     n - size of array,
//     k - number of maximal element of array.
// Return code:
//  value of maximal element of array.
//
{
 long i;                    // Cycle counter

 double RetValue = data[0];

 *k = 0;

 for (i = 1; i < n; i++)
   if (data[i] > RetValue)
     {
      RetValue = data[i];
      *k = i;
     }

 return RetValue;
}

void FillUp (long a[],long n,long p)
//
// The function for fill array by given value.
//  a - array,
//  n - size of array,
//  p - filler.
// Return code:
//  none.
//
{
 long i; // Cycle counter

 for (i = 0; i < n; i++)
   a[i] = p;
}

void FillUpCol (double a[],long n,long c,double p)
//
// The function for fill of column of matrix by given value.
// Designations:
//  a - matrix,
//  n - number rows of matrix a,
//  c - number of filling column (numeration from zero),
//  p - filler.
// Return code:
//  none.
//
{
 long l = c * n;

 for (long i = 0; i < n; i++)
   a[l++] = p;
}

double MODIFICATOR Rank (double x[],double xx[],long n)
//
// The function for array ranking.
// Designations:
//   x - array of given data,
//  xx - array of ranks,
//   n - size of each arrays.
// Notice:
//  to get the correct result Array, given array must be
//  sorted in ascending order, for example, by the function SortArrayUp.
// Return code:
//  correction to the merger of ranks.
//
{
 long j,       // Counter
       is,     // Number equals elements
       ii,
       i = 1;  // Current rank

 double xi,    // Average rank equals elements
        xj,    // Sum of ranks equals elements
        r = 0, // Bundle adjustment of ranks
        rs;

 do
   {
    ////////////////////////////////////////////
    // Counting numbers equals adjacent elements
    is = 1;
    for (j = i - 1 ; j < n - 1; j++)
      {
       if (x[j + 1] == x[j])
     is++;
       else
     break;
      }

    ////////////////////////////////////////////////
    // Next are the nevertheless two equals elements
    if (is > 1)
      {
       rs = is * is * is - is;
       r += rs / 12;
       ii = i - 1;

       for (j = 1,xj = 0; j <= is; j++)
         xj += (++ii);
       xi = xj / is;
       for (j = 1; j <= is ; j++)
         xx[i++ - 1] = xi;
      }

    ///////////////////////////////////////////////////////////////////
    // There are no equals standing next to members on the current step
    else
    {
      xx[i - 1] = i;
      i++;
    }

   } while (i <= n);

 return r;
}

double MODIFICATOR RankFriedman (double *x,double *xx,long n)
//
// The function of array ranking of computation Friedman test.
// Designations:
//   x - sorted up Array of given data,
//  xx - array of ranks,
//   n - size of each arrays.
// Return code:
//  correction to the merger of ranks.
//
{
 long j,ii,    // Counter
      is,      // Number equals elements
      i = 1;   // Current rank

 double xi,    // Average rank equals elements
        xj,    // Sum of ranks equals elements
        r = 0; // Bundle adjustment of ranks

 do
   {
    ////////////////////////////////////////////
    // Counting numbers equals adjacent elements
    is = 1;
    for (j = i - 1 ; j < n - 1; j++)
      {
       if (x[j + 1] == x[j])
         is++;
       else
         break;
      }

    ////////////////////////////////////////////////
    // Next are the nevertheless two equals elements
    if (is > 1)
      {
       ii = i - 1;

       for (j = 1,xj = 0; j <= is; j++)
         xj += (++ii);

       xi = xj / is;

       for (j = 1; j <= is ; j++)
         xx[i++ - 1] = xi;
      }

    ///////////////////////////////////////////////////////////////////
    // There are no equals standing next to members on the current step
    else
    {
      xx[i - 1] = i;
      i++;
    }
    r += is * is * is;

   } while (i <= n);

 return r;
}

double MODIFICATOR RankKruskal (double x[],double xx[],long n)
//
// The function of array ranking of computation Kruskal-Wallis test.
// Designations:
//   x - array of given data,
//  xx - array of ranks,
//   n - size of each arrays.
// Notice:
//  to get the correct result Array, given array must be
//  sorted in ascending order, for example, by the function SortArrayUp.
// Return code:
//  correction to the merger of ranks.
//
{
 long j,      // Counter
       is,     // Number equals elements
       ii,
       i = 1;  // Current rank

 double xi,    // Average rank equals elements
        xj,    // Sum of ranks equals elements
        r = 0, // Bundle adjustment of ranks
        rs;

 do
   {
    ////////////////////////////////////////////////////
    // Counting numbers equals adjacent elements
    is = 1;
    for (j = i - 1 ; j < n - 1; j++)
      {
       if (x[j + 1] == x[j])
     is++;
       else
     break;
      }

    /////////////////////////////////////////////
    // Next are the nevertheless two equals elements
    if (is > 1)
      {
       rs = is * is * is - is;
       r += rs;
       ii = i - 1;

       for (j = 1,xj = 0; j <= is; j++)
         xj += (++ii);
       xi = xj / is;
       for (j = 1; j <= is ; j++)
         xx[i++ - 1] = xi;
      }

    /////////////////////////////////////////////////////
    // There are no equals standing next to members on the current step
    else
    {
      xx[i - 1] = i;
      i++;
    }

   } while (i <= n);

 return 1.0 - r / n / (n * n - 1);
}

double MODIFICATOR RankKendall (double x[],double xx[],long n)
//
// The function of array ranking of computation of Kendall coefficient.
// Designations:
//   x - array of given data,
//  xx - array of ranks,
//   n - size of each arrays.
// Notice:
//  to get the correct result Array, given array must be
//  sorted in ascending order, for example, by the function SortArrayUp.
// Return code:
//  correction to the merger of ranks.
//
{
 long j,      // Counter
       is,     // Number equals elements
       ii,
       i = 1;  // Current rank

 double xi,    // Average rank equals elements
        xj,    // Sum of ranks equals elements
        r = 0, // Bundle adjustment of ranks
        rs;

 do
   {
    ////////////////////////////////////////////////////
    // Counting numbers equals adjacent elements
    is = 1;
    for (j = i - 1 ; j < n - 1; j++)
      {
       if (x[j + 1] == x[j])
         is++;
       else
         break;
      }

    /////////////////////////////////////////////
    // Next are the nevertheless two equals elements
    if (is > 1)
      {
       rs = is * (is - 1);
       r += rs / 2;
       ii = i - 1;
       xj = 0;
       for (j = 1; j <= is; j++)
         xj = xj + ++ii;
       xi = xj / is;
       for (j = 1; j <= is ; j++)
         xx[i++ - 1] = xi;
      }

    /////////////////////////////////////////////////////
    // There are no equals standing next to members on the current step
    else
    {
      xx[i - 1] = i;
      i++;
    }
   } while (i <= n);

 return r;
}

void MODIFICATOR RankSavage (double xx[],long n)
//
// The function of array ranking for Savage test.
// Designations:
//   xx - array of ranks,
//    n - size arrays.
// Return code:
//  none.
//
{
 long i,j; // Counter

 for (i = 0; i < n; i++)
   for (j = 0, xx[i] = 0; j <= i; j++)
     xx[i] += 1.0 / (n - j);
}

long MODIFICATOR CovMatrix (double data[],double z[],long n,long k)
//
// The function build covariation matrix.
// Designations:
//  data - array of given data (k vectors one by one).
//     z - covariation matrix size k x k.
//     n - size of each from k vectors.
// Return code:
//   0 - when normal end of computation,
//  -1 - when lack of memory.
//
{
 long i,j,l,m,m1,m2;  // Cycle counters

 double *dat1,*dat2;  // Copies vectors of data

 if ((dat1 = new double[n]) == 0)
   return -1;
 if ((dat2 = new double[n]) == 0)
   {
    delete [] dat1;
    return -1;
   }

 for (i = 0, m = 0, m2 = 0; i < k; i++)
   {
    for (l = 0; l < n; l++)
      dat1[l] = data[m++];

    for (j = 0, m1 = 0; j < k; j++)
      {
       for (l = 0; l < n; l++)
         dat2[l] = data[m1++];
       z[m2++] = Cov (dat1,dat2,n);
      }
   }
 delete [] dat1; delete [] dat2;

 return 0;
}

long MODIFICATOR Varimax (double r[],long k,long m)
//
// The function builds varimax rotation of matrix of the factor-s representation.
// Designations:
//   r - matrix of factor representation size k * m,
//   k - size of correlation matrix, k > 2,
//   m - number of factors.
// Return code:
//   0 when normal result of computation,
//  -1 when lack of memory for work of array,
//  -2 when error of computation - cannot varimax.
//
{
 long i,j,j1,       // Cycle counters
      m1,m2,m3,m4;  // Array counter

 double *x,         // Rotated pair of vectors
        *y,         // Turned pair of vectors
        *h,         // Sum of loads
        u,v,        // Work variables
        A,B,C,D,
        vc = 0,vc1, // The value for checking the end of the procedure
        num,        // Numerator and denominator
        cs[4];      // Matrix of rotation

 if ((h = new double[k]) == 0)
   return -1;

 ////////////////////////////////
 // Normalization of factor loads
 for (i = 0; i < k; i++)
   {
    for (j = 0, h[i] = 0, m1 = i; j < m; j++, m1 += k)
      h[i] += r[m1] * r[m1];
    h[i] = sqrt (h[i]);
   }

 for (i = 0,m1 = 0; i < m; i++)
   for (j = 0; j < k; j++)
     {
      if (h[j] == 0)
        {
         delete [] h;
         return -2;
        }
      r[m1++] /= h[j];
     }

 if ((x = new double[k * 2]) == 0)
   {
    delete [] h;
    return -1;
   }
 if ((y = new double[k * 2]) == 0)
   {
    delete [] x;
    return -1;
   }

 do
   {
    vc1 = vc;

    /////////////////
    // Rotation cycle
    for (i = 0; i < m - 1; i++)
      {
       for (j = i + 1,m3 = j * k,m4 = m3; j < m; j++)
         {
          ///////////
          // Vector x
          for (j1 = 0,m1 = i * k,m2 = m1; j1 < k; j1++)
            x[j1] = r[m1++];

          ///////////
          // Vector y
          for (j1 = k; j1 < k + k; j1++)
            x[j1] = r[m3++];

          /////////////////////////////
          // Computation of work values
          for (j1 = 0,A = 0,B = 0,C = 0,D = 0; j1 < k; j1++)
            {
             u = x[j1] * x[j1] - x[j1 + k] * x[j1 + k];
             v = 2 * x[j1] * x[j1 + k];
             A += u;
             B += v;
             C += u * u - v * v;
             D += u * v;
            }

          //////////////////////////////////////////////
          // The tangent of the quadruple rotation angle
          num = 2.0 * (D - A * B / k);
          u = num / (C - (A * A - B * B) / k);

          //////////////////////////////////////////////////////////////
          // Selection of the angle from the criterion maximum condition
          if (u > 0 && num < 0)
            u = atan (u) - M_PI;
          else if (u < 0 && num > 0)
            u = atan (u) + M_PI;
          else
            u = atan (u);

          u /= 4;

          //////////////////////////
          // If rotation builds sense
          if (fabs (u) >= 0.000001)
            {
             /////////////////////////////////
             // Computation of rotation matrix
             cs[0] = cos (u);
             cs[1] = sin (u);
             cs[2] = -cs[1];
             cs[3] = cs[0];

             ///////////////////////////
             // Rotation of pair vectors
             MatrixMultiplyMatrix (x,cs,y,k,2,2);

             //////////////////////
             // Vector x - on place
             for (j1 = 0; j1 < k; j1++)
               r[m2++] = y[j1];

             //////////////////////
             // Vector y - on place
             for (j1 = k; j1 < k + k; j1++)
               r[m4++] = y[j1];
            }
         }
      }

    ////////////////////////////////////////////////
    // Checking convergence by the varimax criterion
    for (i = 0,vc = 0,m1 = 0; i < m; i++)
      {
       for (j = 0,u = 0,v = 0; j < k; j++,m1++)
         {
          A = r[m1] * r[m1];
          u += A * A;
          v += A;
         }
       vc += k * u - v * v;
      }

   } while (fabs (vc1 - vc) > 0.000001);

 delete [] x; delete [] y;

 //////////////////////////////////
 // Denormalization of factor loads
 for (i = 0,m1 = 0; i < m; i++)
   for (j = 0; j < k; j++)
     r[m1++] *= h[j];

 delete [] h;

 return 0;
}

long MODIFICATOR MinimumOf(double *x, long n)
//
// The function for computation of number of minimal sample member.
// Designations:
//  x - empirical sample,
//  n - sample size.
// Return code:
//  number of minimal sample member.
//
{
 long i,     // Counter
      k = 0; // Return code

 double t = x[0]; // Work variable

 for (i = 1; i < n; i++)
  if (x[i] < t)
    {
     t = x[i];
     k = i;
    }

 return k;
}

long MaximumOf (long dat[],long n)
//
// The function for computation of number of maximal sample member.
// Designations:
//  dat - empirical sample,
//    n - sample size.
// Return code:
//  number of maximal sample member.
//
{
 long i,            // Counter
      Ret = dat[0]; // Return code

 for (i = 1; i < n; i++)
   if (dat[i] > Ret)
     Ret = dat[i];

 return Ret;
}

long MODIFICATOR Solves (double a[],double b[],long n,long m)
//
// The function for solutions of an indefinite system of linear equations.
// Designations:
//  a - given matrix of the system size m x n, break down,
//  b - column of free members by length n, on exit - solution,
//      only m <= n of cells is used at the input,
//  n - order of the system (number of unknowns),
//  m - rank of the matrix of the system <= n.
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory for computation,
//  -2 when loss of significance or error of computation.
//
{
 long Code; // Return code

 double *t; // Work array

 if ((t = new double[m * n]) == 0)
   return -1;

 MatrixT (a,t,m,n);

 if ((Code = PseudoInverseMatrix (t,a,n,m)) == 0)
   {
    MatrixTMultiplyMatrix (b,a,t,1,m,n);
    ArrayToArray (t,b,n);
   }

 delete [] t;

 return Code;
}

long VectorSum (long *x,long n)
//
// The function for computation sums of array members.
// Designations:
//  - - array,
//  n - size of array.
// Return code:
//  Sum of array members.
//
{
 long i,     // Counter
      s = 0; // Return code

 for (i = 0; i < n; i++)
   s += x[i];

 return s;
}

double MODIFICATOR VasicekTest(double *z,long n,long m,double *p)
//
// The function for computation of Vasicek test.
// Designations:
//  z - empirical sample,
//  n - sample size,
//  m - parameter, from 1 to 9,
//  p - critical test value of significance level 0.05.
// Return code:
//  value of test.
// References:
//  Tiku M.L., Akkaya A.D. Robust estimation and hypothesis testing. - New Delhi: New Age International, 2004.
//
{
 long i,j,k;     // Cycle counter

 double r = 0,   // Work variable, later Test value
        s = 0,   // Standard
        *x,      // Copy of sample
        t05[] =  // Table for significance level 0,05
                {0.99,0.00,0.00,0.00,0.00,0.00,0.00,0.00,
                 1.05,0.00,0.00,0.00,0.00,0.00,0.00,0.00,
                 1.18,1.70,0.00,0.00,0.00,0.00,0.00,0.00,
                 1.34,1.77,0.00,0.00,0.00,0.00,0.00,0.00,
                 1.43,1.88,1.96,0.00,0.00,0.00,0.00,0.00,
                 1.62,1.97,2.07,0.00,0.00,0.00,0.00,0.00,
                 1.63,2.06,2.14,2.11,0.00,0.00,0.00,0.00,
                 1.75,2.15,2.20,2.20,0.00,0.00,0.00,0.00,
                 1.82,2.21,2.27,2.27,2.21,0.00,0.00,0.00,
                 1.89,2.28,2.34,2.34,2.29,0.00,0.00,0.00,
                 1.94,2.38,2.41,2.38,2.34,2.27,0.00,0.00,
                 2.03,2.43,2.49,2.45,2.40,2.33,0.00,0.00,
                 2.07,2.51,2.55,2.53,2.48,2.40,2.32,0.00,
                 2.11,2.53,2.58,2.57,2.53,2.45,2.39,0.00,
                 2.15,2.57,2.66,2.62,2.58,2.52,2.45,2.37,
                 2.16,2.61,2.71,2.67,2.61,2.56,2.49,2.42};

 x = new double[n];
 for (i = 0; i < n; i++)
   x[i] = z[i];

 ////////////////////////////////
 // Computation of critical value
 if (n <= 18)
   *p = t05[m - 1 + (n - 3) * 8];
 else
   {
    if (m == 1)
      *p = -0.000326 * n * n + 0.035874 * n + 1.663319;
    else if (m == 2)
      *p = -0.000399 * n * n + 0.042853 * n + 1.992706;
    else if (m == 3)
      *p = -0.000435 * n * n + 0.047175 * n + 2.010823;
    else if (m == 4)
      *p = -0.000505 * n * n + 0.054087 * n + 1.889386;
    else if (m == 5)
      *p = -0.000561 * n * n + 0.060148 * n + 1.742410;
    else if (m == 6)
      *p = -0.000564 * n * n + 0.061995 * n + 1.651499;
    else if (m == 7)
      *p = -0.000573 * n * n + 0.064336 * n + 1.540101;
    else if (m == 8)
      *p = -0.000580 * n * n + 0.066468 * n + 1.429087;
    else
      *p = -0.000595 * n * n + 0.068882 * n + 1.316973;
   }

 //////////
 // Sort up
 SortArrayUpI (x,n);

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n; i++)
   {
    r += x[i];
    s += x[i] * x[i];
   }
 r /= n;
 s = sqrt((s - n * r * r) / n);
 if (s == 0)
   {
    delete [] x;
    return 0;
   }

 //////////////////////////////
 // Computing of test statistic
 for (i = 0, r = 0; i < n; i++)
   {
    if ((j = i + m) > n - 1)
      j = n - 1;
    if ((k = i - m) < 0)
      k = 0;
    if ((x[j] - x[k]) != 0)
      r = r + log (x[j] - x[k]);
   }
 r = exp (r / n) * n / 2 / m / s;

 delete [] x;

 return r;
}

double MODIFICATOR AndersonDarlingTest(double *z,long n,double *p)
//
// The function for computation of Anderson-Darling omega-square test.
// Designations:
//  z - empirical sample,
//  n - sample size,
//  p - P-value of test.
// Return code:
//  value of test.
//
{
 long i;             // Cycle counter

 double r = 0,       // Value of test
        m = 0,s = 0, // Parameters distribution, later work variables
        *x;          // Copy of sample

 x = new double[n];
 for (i = 0; i < n; i++)
   x[i] = z[i];

 //////////
 // Sort up
 SortArrayUpI (x,n);

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n; i++)
   {
    m += x[i];
    s += x[i] * x[i];
   }
 m /= n;
 s = sqrt((s - n * m * m) / (n - 1));
 if (s == 0)
   return 0;

 ////////////////
 // Normalization
 for (i = 0; i < n; i++)
   x[i] = (x[i] - m) / s;

 //////////////////////////////
 // Computing of test statistic
 for (i = 0; i < n; i++)
   {
    m = (2.0 * i + 1) / 2 / n;
    s = NormalDistribution(x[i]);
    if (s > 0 && s < 1)
      r += m * log (s) + (1.0 - m) * log (1.0 - s);
   }
 r = - r * 2 - n;

 delete [] x;

 *p = SuJohnsonDistribution(r,-2.7057,1.7154,0.0925,0.1043);

 return r;
}

double MODIFICATOR CramerVonMisesTest(double *z,long n,double *p)
//
// The function for computation Cramer-Von Mises omega-square test.
// Designations:
//  z - empirical sample,
//  n - sample size,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  Lemeshko B., Lemeshko S. Statistical distribution convergence and homogeneity test power for Smirnov and Lehmann-Rosenblatt tests // Measurement Techniques, December 2005, vol. 48, no. 12, pp. 1159-1166.
//
{
 long i;                  // Cycle counter

 double r = 1.0 / 12 / n, // Value of test
        m = 0,s = 0,      // Parameters distribution, later work variables
        *x;               // Copy of sample

 x = new double[n];
 for (i = 0; i < n; i++)
   x[i] = z[i];

 //////////
 // Sort up
 SortArrayUpI (x,n);

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n; i++)
   {
    m += x[i];
    s += x[i] * x[i];
   }
 m /= n;
 s = sqrt((s - n * m * m) / (n - 1));
 if (s == 0)
   return 0;

 ////////////////
 // Normalization
 for (i = 0; i < n; i++)
   x[i] = (x[i] - m) / s;

 //////////////////////////////
 // Computing of test statistic
 for (i = 0; i < n; i++)
   {
    m = NormalDistribution(x[i]) - (2.0 * i + 1) / 2 / n;
    r += m * m;
   }

 delete [] x;

 *p = LogNormalDistribution(r,-2.9794,0.5330);

 return r;
}

double MODIFICATOR EppsPulleyTest(double *x,long n,double *p)
//
// The function for computation of Epps-Pulley test.
// Designations:
//  x - empirical sample,
//  n - sample size,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  Epps T.W. Tests for location-scale families based on the empirical characteristic function // Metrika, 2005, vol. 62, no. 1, pp. 99-114.
//  Epps T.W., Pulley L.B. A test for normality based on the empirical characteristic function // Biometrika, 1983, vol. 70, pp. 723-726.
//
{
 long k,j;                         // Cycle counter

 double m = 0,                     // Sample medium
        d = 0,                     // Standard
        b = 0,                     // Work variable
        z,                         // Parameter of normal approximation
        t = 1.0 * n / sqrt(3) + 1; // Value of test

 ///////////////////////////////////
 // Estimation parameters distribution
 for (j = 0; j < n; j++)
   {
    m += x[j];
    d += x[j] * x[j];
   }
 m /= n;
 d = (d - n * m * m) / (n - 1);
 if (d == 0)
   return 0;

 //////////////////////////////
 // Computing of test statistic
 for (k = 1; k < n; k++)
   for (j = 0; j < k; j++)
     b += exp(-(x[j] - x[k]) * (x[j] - x[k]) / 2 / d);
 t += b / n * 2;

 for (j = 0, b = 0; j < n; j++)
   b += exp(-(x[j] - m) * (x[j] - m) / 4 / d);
 t -= b * sqrt(2);

 ////////////////////////
 // Approximation P-value
 if (n >= 4 && n <= 9)
   {
    if (n == 4)
      *p = 0.000004220828 / PowerOf(t,7.369420999768);
    if (n == 5)
      *p = 0.000636023436 / PowerOf(t,3.690587754895);
    if (n == 6)
      *p = 0.000907082768 / PowerOf(t,3.560934199857);
    if (n == 7)
      *p = 0.001049523019 / PowerOf(t,3.503440054279);
    if (n == 8)
      *p = 0.001196973326 / PowerOf(t,3.450785535968);
    if (n == 9)
      *p = 0.001264701758 / PowerOf(t,3.437060244217);
    if (*p > 0.9999)
      *p = 0.9999;
    if (*p < 0.0001)
      *p = 0.0001;
   }
 else
   {
    z = (t - 0.365 / n + 1.34 / n / n) * (1.0 + 1.3 / n);
    if (z >= 2.245958)
      *p = NormalDistribution(z);
    else
      *p = NormalDistribution(3.55295 + 1.23062 * log ((z + 0.020682) / (2.245958 - z)));
   }

 return t;
}

double MODIFICATOR HenzeZirklerTest(double *x,long n,long k,double *p)
//
// The function for computation of Henze-Zirkler test.
// Designations:
//  x - multidimensional  empirical sample,
//  n - sample size,
//  k - sample size,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  Svantesson T., Wallace J. Tests for assessing multivariate normality and the covariance structure of MIMO data // IEEE International Conference on Acoustics, Speech and Signal Processing, Hong Kong, April 6-10, 2003 (ICASSP'03).
//
{
 long i,j,l,         // Cycle counter
       m1,m2;        // Index

 double *m,          // Vector of sample mean
        *s,          // Covariation matrix
        *t,          // Inverse covariation matrix
        *mi,*mj,*t1, // Work array
        g = 0,       // Value of test
        g1 = 0,b;    // Work variable

 s = new double[k * k];
 t = new double[k * k];

 /////////////////////////////////////
 // Estimation parameters distribution
 CovMatrix (x,s,n,k);
 if(InverseMatrix (s,t,k) == -2)
   {
    delete [] s; delete [] t;
    return 0;
   }

 m = new double[k];
 mi = new double[k];
 mj = new double[k];
 t1 = new double[1];

 /////////////////////////////////////
 // Estimation parameters distribution
 MeanVector (x,m,n,k);

 /////////////////////////////////////////
 // Parameter, dependent on size of sample
 b = PowerOf(0.25 * (2 * k + 1),1.0 / (k + 4)) *
     PowerOf(n,1.0 / (k + 4)) / sqrt(2);
 b *= b;

 //////////////////////////////
 // Computing of test statistic
 for (i = 0; i < n; i++)
   {
    for (l = 0, m2 = i; l < k; l++)
      {
       mi[l] = x[m2] - m[l];
       m2 += n;
      }
    MatrixTMultiplyMatrix(mi,t,s,1,k,k);
    for (j = 0; j < n; j++)
      {
       for (l = 0, m1 = j; l < k; l++)
         {
          mj[l] = x[m1] - m[l];
          m1 += n;
         }
       MatrixMultiplyMatrix(s,mj,t1,1,k,1);
       g += exp(- b * t1[0] / 2);
      }
    MatrixMultiplyMatrix(s,mi,t1,1,k,1);
    g1 += exp(-b * t1[0] / 2 / (b + 1));
   }
 b = PowerOf(b + 1,-0.5 * k);

 delete [] s; delete [] t; delete [] m; delete [] mi; delete [] mj; delete [] t1;

 *p = NormalDistribution(g / n / n - g1 / n * 2 * b + b);

 return g;
}

double MODIFICATOR KolmogorovTest(double *z,long n,double *p)
//
// The function for computation of modifed Kolmogorov test.
// Designations:
//   z - empirical sample,
//   n - sample size,
//   p - P-value of test.
// Return code:
//  value of statistic Sk of test.
// References:
//  Conover W.J. Practical nonparametric statistics. - New York, NY: John Wiley & Sons, 1999.
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//
{
 long i;       // Cycle counter

 double r = 0, // Value of test
        m = 0, // Parameters distribution,
        s = 0, // later work variables
        d = 0, // Work variable, later modifed statistic
        *x;    // Copy of sample

 x = new double[n];
 for (i = 0; i < n; i++)
   x[i] = z[i];

 //////////
 // Sort up
 SortArrayUpI (x,n);

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n; i++)
   {
    m += x[i];
    s += x[i] * x[i];
   }
 m /= n;
 s = sqrt((s - n * m * m) / (n - 1));
 if (s == 0)
   return 0;

 ////////////////
 // Normalization
 for (i = 0; i < n; i++)
   x[i] = (x[i] - m) / s;

 //////////////////////////////
 // Computing of test statistic
 for (i = 0; i < n; i++)
   {
    m = (1.0 + i) / n - NormalDistribution(x[i]);
    if (m > r)
      r = m;
    s = NormalDistribution(x[i]) - 1.0 * i / n;
    if (s > d)
      d = s;
   }
 if (r < d)
   r = d;

 delete [] x;

 d = (r * 6 * n + 1) / 6 / sqrt(n);
 *p = GammaDistribution(d, 4.9014, 0.0691, 0.2951);

 return r;
}

double MODIFICATOR KolmogorovSimpleTest(double *z,double m,double s,long n,double *p)
//
// The function for computation of Kolmogorov test for normal sample.
// Designations:
//   z - empirical sample,
//   m - given medium value,
//   s - given standard deviation,
//   n - sample size,
//   p - P-value of test.
// Return code:
//  value of statistic Sk of test.
// References:
//  Conover W.J. Practical nonparametric statistics. - New York, NY: John Wiley & Sons, 1999.
//
{
 long i;       // Cycle counter

 double r = 0, // Value of test
        d = 0, // Work variable, later modifed statistic
        *x;    // Copy of sample

 x = new double[n];

 for (i = 0; i < n; i++)
   x[i] = z[i];

 //////////
 // Sort up
 SortArrayUpI (x,n);

 ////////////////
 // Normalization
 for (i = 0; i < n; i++)
   x[i] = (x[i] - m) / s;

 //////////////////////////////
 // Computing of test statistic
 for (i = 0; i < n; i++)
   {
    m = (1.0 + i) / n - NormalDistribution(x[i]);
    if (m > r)
      r = m;
    s = NormalDistribution(x[i]) - 1.0 * i / n;
    if (s > d)
      d = s;
   }
 if (r < d)
   r = d;

 delete [] x;

 *p = LambdaDistribution(r * sqrt(n));

 return r;
}

double MODIFICATOR SmirnovTest(double *z,long n,double *p)
//
// The function for computation of Smirnov test.
// Designations:
//  z - empirical sample,
//  n - sample size,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  Bolshev L.N., Smirnov N.V. Tables of mathematical statistics (in Russian). - Moscow: Nauka, 1983.
//  Lemeshko B., Lemeshko S. Statistical distribution convergence and homogenity test power for Smirnov and Lehmann-Rosenblatt tests // Measurement Techniques, December 2005, vol. 48, no. 12, pp. 1159-1166.
//
{
 long i;       // Cycle counter

 double r = 0, // Value of test
        m = 0, // Parameter distribution, later work variable
        s = 0, // Parameter distribution, later modifed statistic
        *x;    // Copy of sample

 x = new double[n];

 for (i = 0; i < n; i++)
   x[i] = z[i];

 //////////
 // Sort up
 SortArrayUpI (x,n);

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n; i++)
   {
    m += x[i];
    s += x[i] * x[i];
   }
 m /= n;
 s = sqrt((s - n * m * m) / (n - 1));
 if (s == 0)
   return 0;

 ////////////////
 // Normalization
 for (i = 0; i < n; i++)
   x[i] = (x[i] - m) / s;

 //////////////////////////////
 // Computing of test statistic
 for (i = 0; i < n; i++)
   {
    m = (1.0 + i) / n - NormalDistribution(x[i]);
    if (m > r)
      r = m;
   }

 delete [] x;

 s = r * 6 * n + 1;
 s = s * s / 9 / n;
 *p = LogNormalDistribution(s,0.1164,0.5436);

 return r;
}

double MODIFICATOR Kolmogorov (double *z,long n,double *p)
//
// The function for computation of Kolmogorov test for simple hypotheses.
// in the case of uniform distribution.
// Designations:
//  z - empirical sample,
//  n - sample size,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  Conover W.J. Practical nonparametric statistics. - New York, NY: John Wiley & Sons, 1999.
//
{
 long i;             // Cycle counter

 double r = 0,d = 0, // Work variable
        m,s,
        *x;          // Copy of sample

 x = new double[n];
 for (i = 0; i < n; i++)
   x[i] = z[i];

 //////////
 // Sort up
 SortArrayUpI (x,n);

 //////////////////////////////
 // Computing of test statistic
 for (i = 0; i < n; i++)
   {
    m = (1.0 + i) / n - x[i];
    if (m > r)
      r = m;
    s = x[i] - 1.0 * i / n;
    if (s > d)
      d = s;
   }
 if (r < d)
   r = d;

 delete [] x;

 *p = LambdaDistribution(r * sqrt(n));

 return r;
}

double MODIFICATOR DAgostinoTest(double *z,long n,double *p)
//
// The function for computation of DAgostino test.
// Designations:
//  z - empirical sample,
//  n - sample size,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  D-Agostino R.B. An omnibus test of normality for moderate and large size samples // Biometrika, 1971, vol. 58, no. 2, pp. 341-348.
//
{
 long i;                    // Cycle counter

 double d = 0,              // Value of test
        m = 0,              // Medium
        s = 0,              // Variance
        n1 = 0.5 - 0.5 * n, // Work Parameter
        *x;                 // Copy of sample

 x = new double[n];

 for (i = 0; i < n; i++)
   x[i] = z[i];

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n; i++)
   {
    m += x[i];
    s += x[i] * x[i];
   }
 m /= n;
 s = (s - n * m * m) / n;

 SortArrayUpI (x,n);

 for (i = 0; i < n; i++)
   d = d + x[i] * (n1 + i);

 delete [] x;

 if (s == 0)
   {
    *p = 0;
    return 0;
   }
 d = d / n / n / sqrt (s);

 ////////////////////////
 // Approximation P-value
 *p = NormalDistribution ((d - 0.28209479) / 0.02998598 * sqrt (n));

 return d;
}

double MODIFICATOR KurtosisTest(double *x,long n,double *p)
//
// The function for computation of kurtosis test.
// Designations:
//  x - empirical sample,
//  n - sample size,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  Cramer H. Mathematical methods of statistics. - Princeton: Princeton University Press, 1946.
//
{
 long i;          // Cycle counter

 double m = 0,    // Sample medium
        s = 0,    // Standard
        b = 0,    // Value of test
        d = 0, t; // Work variable

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n; i++)
   {
    m += x[i];
    s += x[i] * x[i];
   }
 m /= n;
 s = (s - n * m * m) / (n - 1);
 if (s == 0)
   return 0;

 //////////////////////////////
 // Computing of test statistic
 for (i = 0; i < n; i++)
   {
    t = (x[i] - m) * (x[i] - m);
    b += t * t;
    d += t;
   }
 b = (b * n * (n + 1) - d * d * 3 * (n - 1)) / (n - 1) / (n - 2) / (n - 3) / s / s;

 ////////////////////////
 // Approximation P-value
 *p = NormalDistribution (b / sqrt (24.0 * n * (n - 1) * (n - 1) / (n - 3) / (n - 2) / (n + 3) / (n + 5)));

 return b;
}

double MODIFICATOR GearyTest(double *x,long n,double *p)
//
// The function for computation of Geary test.
// Designations:
//  x - empirical sample,
//  n - sample size,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  D-Agostino R.B., Rosman B. The power of Geary-s test of normality // Biometrika, 1974, vol. 61, no. 1, pp. 181-184.
//  Bolshev L.N., Smirnov N.V. Tables of mathematical statistics (in Russian). - Moscow: Nauka, 1983.
//
{
 long i;       // Cycle counter

 double m = 0, // Sample medium
        s = 0, // Standard
        d = 0; // Value of test

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n; i++)
   {
    m += x[i];
    s += x[i] * x[i];
   }
 m /= n;
 s = (s - n * m * m) / n;
 if (s == 0)
   return 0;

 //////////////////////////////
 // Computing of test statistic
 for (i = 0; i < n; i++)
   d += fabs (x[i] - m);
 d /= sqrt(s) * n;

 ////////////////////////
 // Approximation P-value
 *p = NormalDistribution ((d - 0.797884560803) * sqrt(n) / 0.212297766000);

 return d;
}

double MODIFICATOR JarqueBeraTest(double *x,long n,double *p)
//
// The function for computation of Jarque-Bera test.
// Designations:
//  x - empirical sample,
//  n - sample size,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  Romao X., Delgado R., Costa A. An empirical power comparison of univariate goodness-of-fit tests for normality // Journal of Statistical Computation and Simulation, May 2010, vol. 80, no. 5, pp. 545-591.
//
{
 long i;        // Cycle counter

 double m = 0,  // Sample medium
        s = 0,  // Standard
        a3 = 0, // Kurtosis
        a4 = 0, // Skewness
        t;      // Work variable

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n; i++)
   {
    m += x[i];
    s += x[i] * x[i];
   }
 m /= n;
 s = (s - n * m * m) / n;
 if (s == 0)
   return 0;

 //////////////////////////////
 // Computing of test statistic
 for (i = 0; i < n; i++)
   {
    t = x[i] - m;
    a3 += t * t * t;
    a4 += t * t * t * t;
   }
 a3 = a3 / n / s / sqrt (s);
 a4 = a4 / n / s / s;
 t = (a3 * a3 / 6 + (a4 - 3) * (a4 - 3) / 24) * n;

 ////////////////////////
 // Approximation P-value
 *p = ChiSquareDistribution (2, t);

 return t;
}

double MODIFICATOR MardiaKurtosisTest(double *x,long n,long k,double *p)
//
// The function for computation kurtosis test by Mardia.
// Designations:
//  x - multidimensional  empirical sample,
//  n - sample size,
//  k - number of samples,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  Mardia K.V. Measures of multivariate skewness and kurtosis with applications // Biometrika, 1970, vol. 57, no. 3, pp. 519-530.
//  Kankainen A., Oja H., Taskinen S. On Mardia-s tests of multinormality // Theory and applications of recent robust methods / Ed. by M. Hubert, G. Pison, A. Stryuf et al. - Basel: Birkhauser, 2003.
//
{
 long i,j,       // Cycle counter
       m1;       // Index

 double *m,      // Vector of sample mean
        *s,      // Covariation matrix
        *t,      // Inverse covariation matrix
        *mi,*t1, // Work array
        g = 0;   // Value of test

 s = new double[k * k];
 t = new double[k * k];

 CovMatrix (x,s,n,k);
 if(InverseMatrix (s,t,k) == -2)
   {
    delete [] s; delete [] t;
    return 0;
   }

 m = new double[k];
 mi = new double[k];
 t1 = new double[1];

 MeanVector (x,m,n,k);

 for (i = 0; i < n; i++)
   {
    for (j = 0, m1 = i; j < k; j++)
      {
       mi[j] = x[m1] - m[j];
       m1 += n;
      }
    MatrixTMultiplyMatrix (mi,t,s,1,k,k);
    MatrixMultiplyMatrix (s,mi,t1,1,k,1);
    g += t1[0] * t1[0];
   }

 delete [] s; delete [] t;  delete [] m;  delete [] mi;  delete [] t1;

 ////////////////////////
 // Approximation P-value
 *p = NormalDistribution ((g / n - 1.0 * k * (k + 2)) / sqrt (8.0 * k / n * (k + 2)));

 return g;
}

double MODIFICATOR MardiaSkewnessTest(double *x,long n,long k,double *p)
//
// The function for computation skewness test by Mardia.
//  x - multidimensional empirical sample,
//  n - sample size,
//  k - number of samples,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  Mardia K.V. Measures of multivariate skewness and kurtosis with applications // Biometrika, 1970, vol. 57, no. 3, pp. 519-530.
//  Kankainen A., Oja H., Taskinen S. On Mardia-s tests of multinormality // Theory and applications of recent robust methods / Ed. by M. Hubert, G. Pison, A. Stryuf et al. - Basel: Birkhauser, 2003.
//
{
 long i,j,l,         // Cycle counter
      m1,m2;         // Index

 double *m,          // Vector of sample mean
        *s,          // Covariation matrix
        *t,          // Inverse covariation matrix
        *mi,*mj,*t1, // Work array
        g = 0;       // Value of test

 s = new double[k * k];
 t = new double[k * k];

 CovMatrix (x,s,n,k);
 if (InverseMatrix (s,t,k) == -2)
   {
    delete [] s; delete [] t;
    return 0;
   }

 m = new double[k];
 mi = new double[k];
 mj = new double[k];
 t1 = new double[1];

 MeanVector (x,m,n,k);

 for (i = 0; i < n; i++)
   {
    for (l = 0, m2 = i; l < k; l++)
      {
       mi[l] = x[m2] - m[l];
       m2 += n;
      }
    MatrixTMultiplyMatrix (mi,t,s,1,k,k);
    for (j = 0; j < n; j++)
      {
       for (l = 0, m1 = j; l < k; l++)
         {
          mj[l] = x[m1] - m[l];
          m1 += n;
         }
       MatrixMultiplyMatrix (s,mj,t1,1,k,1);
       g += t1[0] * t1[0] * t1[0];
      }
   }

 delete [] s; delete [] t; delete [] m; delete [] mi; delete [] mj; delete [] t1;

 ////////////////////////
 // Approximation P-value
 *p = ChiSquareDistribution (g / 6 / n, 0.166666666667 * k * (k + 1) * (k + 2));

 return g;
}

double MODIFICATOR ShapiroWilkTest(double *z,long n,double *p)
//
// The function for computation of Shapiro-Wilk W-test.
// Designations:
//  z - empirical sample,
//  n - sample size,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  Hahn G.J., Shapiro S.S. Statistical models in engineerting. - New York, NY: John Wiley & Sons, 1967.
//
{
 long i,j,l = 0,     // Cycle counters
      s = 0;         // The beginning of the back reference arrays of coefficients

 double d = 0,b = 0, // Work variables
        w,           // Value of test
        *x,          // Copy of sample
        a[] = {      // Array of coefficients
 0.7071,
 0.6872,0.1677,
 0.6646,0.2413,
 0.6431,0.2806,0.0875,
 0.6233,0.3031,0.1401,
 0.6052,0.3164,0.1743,0.0561,
 0.5888,0.3244,0.1976,0.0947,
 0.5739,0.3291,0.2141,0.1224,0.0399,
 0.5601,0.3315,0.2260,0.1429,0.0695,
 0.5475,0.3325,0.2347,0.1586,0.0922,0.0303,
 0.5359,0.3325,0.2412,0.1707,0.1099,0.0539,
 0.5251,0.3318,0.2460,0.1802,0.1240,0.0727,0.0240,
 0.5150,0.3306,0.2495,0.1878,0.1353,0.0880,0.0433,
 0.5056,0.3290,0.2521,0.1939,0.1447,0.1005,0.0593,0.0196,
 0.4968,0.3273,0.2540,0.1988,0.1524,0.1109,0.0725,0.0359,
 0.4886,0.3253,0.2553,0.2027,0.1587,0.1197,0.0837,0.0496,0.0163,
 0.4808,0.3232,0.2561,0.2059,0.1641,0.1271,0.0932,0.0612,0.0303,
 0.4734,0.3211,0.2565,0.2085,0.1686,0.1334,0.1013,0.0711,0.0422,0.0140,
 0.4643,0.3185,0.2578,0.2119,0.1736,0.1399,0.1092,0.0804,0.0530,0.0263,
 0.4590,0.3156,0.2571,0.2131,0.1764,0.1443,0.1150,0.0878,0.0618,0.0368,
 0.0122,
 0.4542,0.3126,0.2563,0.2139,0.1787,0.1480,0.1201,0.0941,0.0696,0.0459,
 0.0228,
 0.4493,0.3098,0.2554,0.2145,0.1807,0.1512,0.1245,0.0997,0.0764,0.0539,
 0.0321,0.0107,
 0.4450,0.3069,0.2543,0.2148,0.1822,0.1539,0.1283,0.1046,0.0823,0.0610,
 0.0403,0.02,
 0.4407,0.3043,0.2533,0.2151,0.1836,0.1563,0.1316,0.1089,0.0876,0.0672,
 0.0476,0.0284,0.0094,
 0.4366,0.3018,0.2522,0.2152,0.1848,0.1584,0.1346,0.1128,0.0923,0.0728,
 0.0540,0.0358,0.0178,
 0.4328,0.2992,0.2510,0.2151,0.1857,0.1601,0.1372,0.1162,0.0965,0.0778,
 0.0598,0.0424,0.0253,0.0084,
 0.4291,0.2968,0.2499,0.2150,0.1864,0.1616,0.1395,0.1192,0.1002,0.0822,
 0.0650,0.0483,0.0320,0.0159,
 0.4254,0.2944,0.2487,0.2148,0.1870,0.1630,0.1415,0.1219,0.1036,0.0862,
 0.0697,0.0537,0.0381,0.0227,0.0076,
 0.4220,0.2921,0.2475,0.2145,0.1874,0.1641,0.1433,0.1243,0.1066,0.0899,
 0.0739,0.0585,0.0435,0.0289,0.0144,
 0.4188,0.2898,0.2463,0.2141,0.1878,0.1651,0.1449,0.1265,0.1093,0.0931,
 0.0777,0.0629,0.0485,0.0344,0.0206,0.0068,
 0.4156,0.2876,0.2451,0.2137,0.1880,0.1660,0.1463,0.1284,0.1118,0.0961,
 0.0812,0.0669,0.0530,0.0395,0.0262,0.0131,
 0.4127,0.2854,0.2439,0.2132,0.1882,0.1667,0.1475,0.1301,0.1140,0.0988,
 0.0844,0.0706,0.0572,0.0441,0.0314,0.0187,0.0062,
 0.4096,0.2834,0.2427,0.2127,0.1883,0.1673,0.1487,0.1317,0.1160,0.1013,
 0.0873,0.0739,0.0610,0.0484,0.0361,0.0239,0.0119,
 0.4068,0.2813,0.2415,0.2121,0.1883,0.1678,0.1496,0.1331,0.1179,0.1036,
 0.0900,0.0770,0.0645,0.0523,0.0404,0.0287,0.0172,0.0057,
 0.4040,0.2794,0.2403,0.2116,0.1883,0.1683,0.1505,0.1344,0.1196,0.1056,
 0.0924,0.0798,0.0677,0.0559,0.0444,0.0331,0.0220,0.0110,
 0.4015,0.2774,0.2391,0.2110,0.1881,0.1686,0.1513,0.1356,0.1211,0.1075,
 0.0947,0.0824,0.0706,0.0592,0.0481,0.0372,0.0264,0.0158,0.0053,
 0.3989,0.2755,0.2380,0.2104,0.1880,0.1689,0.1520,0.1366,0.1225,0.1092,
 0.0967,0.0848,0.0733,0.0622,0.0515,0.0409,0.0305,0.0203,0.0101,
 0.3964,0.2737,0.2368,0.2098,0.1878,0.1691,0.1526,0.1376,0.1237,0.1108,
 0.0986,0.0870,0.0759,0.0651,0.0546,0.0444,0.0343,0.0244,0.0146,0.0049,
 0.3940,0.2719,0.2357,0.2091,0.1876,0.1693,0.1531,0.1384,0.1249,0.1123,
 0.1004,0.0891,0.0782,0.0677,0.0575,0.0476,0.0379,0.0283,0.0188,0.0094,
 0.3917,0.2701,0.2345,0.2085,0.1874,0.1694,0.1535,0.1392,0.1259,0.1136,
 0.1020,0.0909,0.0804,0.0701,0.0602,0.0506,0.0411,0.0318,0.0227,0.0136,
 0.0045,
 0.3894,0.2684,0.2334,0.2078,0.1871,0.1695,0.1539,0.1398,0.1269,0.1149,
 0.1035,0.0927,0.0824,0.0724,0.0628,0.0534,0.0442,0.0352,0.0263,0.0175,
 0.0087,
 0.3872,0.2667,0.2323,0.2072,0.1868,0.1695,0.1542,0.1405,0.1278,0.1160,
 0.1049,0.0943,0.0842,0.0745,0.0651,0.0560,0.0471,0.0383,0.0296,0.0211,
 0.0126,0.0042,
 0.3850,0.2651,0.2313,0.2065,0.1865,0.1695,0.1545,0.1410,0.1286,0.1170,
 0.1062,0.0959,0.0860,0.0765,0.0673,0.0584,0.0497,0.0412,0.0328,0.0245,
 0.0163,0.0081,
 0.3830,0.2635,0.2302,0.2058,0.1862,0.1695,0.1548,0.1415,0.1293,0.1180,
 0.1073,0.0972,0.0876,0.0783,0.0694,0.0607,0.0522,0.0439,0.0357,0.0277,
 0.0197,0.0118,0.0039,
 0.3808,0.2620,0.2291,0.2052,0.1859,0.1695,0.1550,0.1420,0.1300,0.1189,
 0.1085,0.0986,0.0892,0.0801,0.0713,0.0628,0.0546,0.0465,0.0385,0.0307,
 0.0229,0.0153,0.0076,
 0.3789,0.2604,0.2281,0.2045,0.1855,0.1693,0.1551,0.1423,0.1306,0.1197,
 0.1095,0.0998,0.0906,0.0817,0.0731,0.0648,0.0568,0.0489,0.0411,0.0335,
 0.0259,0.0185,0.0111,0.0037,
 0.3770,0.2589,0.2271,0.2038,0.1851,0.1692,0.1553,0.1427,0.1312,0.1205,
 0.1105,0.1010,0.0919,0.0832,0.0748,0.0667,0.0588,0.0511,0.0436,0.0361,
 0.0288,0.0215,0.0143,0.0071,
 0.3751,0.2574,0.2260,0.2032,0.1847,0.1691,0.1554,0.1430,0.1317,0.1212,
 0.1113,0.1020,0.0932,0.0846,0.0764,0.0685,0.0608,0.0532,0.0459,0.0386,
 0.0314,0.0244,0.0174,0.0104,0.0035},
        Gamma[] = {
 0.6250,1.1070,1.5300,2.0100,2.3560,2.6960,2.9680,3.2620,3.4850,3.7310,
 3.9360,4.1550,4.3730,4.5670,4.7130,4.8850,5.0180,5.1530,5.2910,5.4130,
 5.5080,5.6050,5.7040,5.8030,5.9050,5.9880,6.0740,6.1600,6.2480,6.3240,
 6.4020,6.4800,6.5590,6.6400,6.7210,6.8030,6.8870,6.9610,7.0350,7.1110,
 7.1880,7.2660,7.3450,7.4140,7.4840,7.5550,7.6150,7.6770},
        Eta[] = {
 0.3860,0.7140,0.9350,1.1380,1.2450,1.3330,1.4000,1.4710,1.5150,1.5710,
 1.6130,1.6550,1.6950,1.7240,1.7390,1.7700,1.7860,1.8020,1.8180,1.8350,
 1.8480,1.8620,1.8760,1.8900,1.9050,1.9150,1.9340,1.9490,1.9650,1.9760,
 1.9880,2.0000,2.0120,2.0240,2.0370,2.0490,2.0620,2.0750,2.0880,2.1010,
 2.1140,2.1280,2.1410,2.1550,2.1690,2.1830,2.1980,2.2120},
        Epsylon[] = {
 0.7500,0.6297,0.5521,0.4963,0.4533,0.4186,0.3900,0.3660,0.3451,0.3270,
 0.3111,0.2969,0.2842,0.2727,0.2622,0.2528,0.2440,0.2359,0.2264,0.2207,
 0.2157,0.2106,0.2063,0.2020,0.1980,0.1943,0.1907,0.1872,0.1840,0.1811,
 0.1781,0.1755,0.1727,0.1702,0.1677,0.1656,0.1633,0.1612,0.1591,0.1572,
 0.1552,0.1534,0.1516,0.1499,0.1482,0.1466,0.1451,0.1436};

 x = new double[n];
 for (i = 0; i < n; i++)
   x[i] = z[i];

 SortArrayUpI (x,n);

 for (i = 0; i < n; i++)
   {
    b += x[i];
    d += x[i] * x[i];
   }
 d -= b * b / n;
 if (d == 0)
   return 0;
 for (i = 1; i < n - 2; i++)
   for (j = 0; j <= i / 2; j++)
     s++;
 for (i = 0, j = n - 1, b = 0; i < n / 2; i++)
   b += a[s++] * (x[j--] - x[l++]);
 w = b * b / d;

 delete [] x;

 //////////////////////////////////////////////
 // Correction if the test cannot be calculated
 if (n >= 3 && n <= 50)
   *p = NormalDistribution(- Gamma[n - 3] + Eta[n - 3] * log ((w - Epsylon[n - 3]) / (1.0 - w)));
 else
   *p = 0.5;

 return w;
}

double MODIFICATOR ShapiroFranciaTest(double *z,long n,double *p)
//
// The function for computation of Shapiro-Francia W'-test.
// Designations:
//  z - empirical sample,
//  n - sample size,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  Royston J.P. A simple method for evaluating the Shapiro-Francia W- test for non-normality // The Statistician, 1983, vol. 32, pp. 297-230.
//
{
 long i;          // Counter

 double w = 0,    // Value of test
        d = 0,    // Work variable
        my = 0,   // Work variable, medium value
        xe,mw,    // Parameters of approximation
        *y,       // Work array, copy of sample
        *b;       // Vector of coefficients

 y = new double[n];
 b = new double[n];

 /////////////////
 // Sort of sample
 for (i = 0; i < n; i++)
   {
    b[i] = InverseNormalDistribution ((0.625 + i) / (0.25 + n));
    my += b[i] * b[i];
   }
 my = sqrt(my);
 for (i = 0; i < n; i++)
   b[i] /= my;

 //////////////////////////////
 // Computing of test statistic
 for (i = 0; i < n; i++)
   y[i] = z[i];
 SortArrayUpI (y,n);
 my = VectorSum (z,n) / n;
 for (i = 0; i < n; i++)
   {
    w += b[i] * y[i];
    d += (y[i] - my) * (y[i] - my);
   }

 delete [] y;  delete [] b;

 if (d == 0)
   return 0;
 w *= w / d;

 ///////////////////////////
 // Approximation of P-value
 xe = log(n) - 5;
 d = -0.048157 + 0.0197196 * xe - 0.0119065 * xe * xe * xe;
 mw = exp (d * log (1.0 - w));
 mw = (mw - 1) / d;
 my = -exp(1.693067 + 0.144167 * xe - 0.0184928 * xe * xe +
           0.031074485 * xe * xe * xe + 0.0055717663 * xe * xe * xe * xe);
 d = exp(-0.510725 - 0.1160364 * xe - 0.0067021 * xe * xe +
           0.054465944 * xe * xe * xe + 0.0087397329 * xe * xe * xe * xe);
 *p = NormalDistribution((mw - my) / d);

 return w;
}

double MODIFICATOR SkewnessTest(double *x,long n,double *p)
//
// The function for computation of skewness test.
// Designations:
//  x - empirical sample,
//  n - sample size,
//  p - P-value of test.
// Return code:
//  value of test.
// References:
//  Cramer H. Mathematical methods of statistics. - Princeton: Princeton University Press, 1946.
//
{
 long i;       // Cycle counter

 double m = 0, // Sample medium
        s = 0, // Standard
        g = 0; // Value of test

 ////////////////////////////////////////
 // Estimation of parameters distribution
 for (i = 0; i < n; i++)
   {
    m += x[i];
    s += x[i] * x[i];
   }
 m /= n;
 s = (s - n * m * m) / (n - 1);
 if (s == 0)
   return 0;

 //////////////////////////////
 // Computing of test statistic
 for (i = 0; i < n; i++)
   g += (x[i] - m) * (x[i] - m) * (x[i] - m);
 g = g * n / (n - 1) / (n - 2) / s / sqrt (s);

 ////////////////////////
 // Approximation P-value
 *p = NormalDistribution (g / sqrt (6.0 / (n - 2) * n / (n + 3) * (n - 1) / (n + 1)));

  return g;
}

void MODIFICATOR MeanVector (double x[],double m[],long n,long k)
//
// The function for computation of vector medium values.
// Designations:
//  x - array of given data (k vectors one by one).
//  m - vector medium values by length k.
//  n - size of each from k vectors.
// Return code:
//  none.
//
{
 long i,j,    // Cycle counters
      m1 = 0; // Index

 for (i = 0; i < k; i++)
   {
    for (j = 0,m[i] = 0; j < n; j++)
      m[i] += x[m1++];
    m[i] /= n;
   }
}

void MODIFICATOR MedianDifference(double x[],long n,double y[],long m,double a,double *med,double *low,double *upp)
//
// The function for computation of difference of medians estimation.
// Designations:
//    x - sample,
//    n - sample size,
//    y - sample,
//    m - sample size,
//    a - confidence probability,
//  med - median difference,
//  low - low bound 100*a% of interval,
//  upp - upper bound 100*a% of interval.
// Return code:
//  none.
//
{
 long i,j,k = 0, // Counter
      c;         // Work Parameter

 double *w;      // Array of differences

 w = new double[n * m];

 //////////////////////
 // Work computation
 for (i = 0; i < n; i++)
   for (j = 0; j < m; j++)
     w[k++] = x[i] - y[j];
 SortArrayUp0 (w,n * m);
 c = int(n * m / 2 - InverseNormalDistribution ((1 + a) / 2) *
     sqrt(0.083333333333 * n * m * (n + m + 1)));
 if (c < 1)
   c = 1;

 /////////////////////////////////////////////
 // Computation confidence of median intervals
 *low = w[c - 1];
 *upp = w[n * m - c];

 ///////////////////////////////////
 // Computation of median difference
 if (!Parity (n * m))
   *med = w[n * m / 2];
 else
   *med = (w[n * m / 2 - 1] + w[n * m / 2]) / 2;

 delete [] w;
}

long MODIFICATOR ROC(double x[],long n,double y[],long m,double se[],double sp[],double t[],double *auc,double *seauc)
//
// The function for computation parameters of ROC-curve.
// Designations:
//     x - sample,
//     n - sample size x,
//     y - sample,
//     m - sample size y,
//    se - array sensitivity by length n + m,
//    sp - array specificity by length n + m,
//     t - array clipping thresholds by length n + m,
//   auc - AUC,
// Seauc - SE(AUC).
// Return code:
//  0 - ROC was no turned over,
//  1 - ROC was turned over.
//
{
 long i,j,   // Cycle counter
      l = 0, // Counter of members arrays
      f = 0; // Return code

 double q1,q2; // Work variables

 for (i = 0; i < n; i++)
   t[l++] = x[i];
 for (i = 0; i < m; i++)
   t[l++] = y[i];
 SortArrayDown0 (t,n + m);

 ///////////////
 // Building ROC
 for (i = 0; i < n + m; i++)
   {
    for (j = 0, se[i] = 0; j < n; j++)
      if (x[j] >= t[i])
        se[i] += 1;
    for (j = 0, sp[i] = 0; j < m; j++)
      if (y[j] >= t[i])
        sp[i] += 1;
   }
 for (i = 0; i < n + m; i++)
   {
    se[i] = se[i] / n;
    sp[i] = sp[i] / m;
   }

 ///////////////////////////////
 // Computation square under ROC
 *auc = 0;
 for (i = 0; i < n + m - 1; i++)
   *auc += (se[i + 1] + se[i]) * (sp[i + 1] - sp[i]) / 2;

 ///////////////////////////////////
 // Inverting the curve if necessary
 if ((*auc) < 0.5)
   {
    f = 1;                 // ROC-curve turned over

    ///////////////
    // Building ROC
    for (i = 0; i < n + m; i++)
      {
       for (j = 0, se[i] = 0; j < n; j++)
         if (x[j] <= t[i]) // Change the decisive rule
           se[i] += 1;
       for (j = 0, sp[i] = 0; j < m; j++)
         if (y[j] <= t[i]) // Change the decisive rule
           sp[i] += 1;
      }
    for (i = 0; i < n + m; i++)
      {
       se[i] = se[i] / n;
       sp[i] = sp[i] / m;
      }

    ///////////////////////////////
    // Computation square under ROC
    *auc = 0;
    for (i = 0; i < n + m - 1; i++)
      *auc += (se[i + 1] + se[i]) * (sp[i] - sp[i + 1]) / 2;
   }

 /////////////////////////////////
 // Computation of standard errors
 q1 = *auc / (2.0 - *auc);
 q2 = 2.0 * (*auc) * (*auc) / (1.0 + *auc);
 *seauc = (*auc * (1.0 - *auc) + (q1 - (*auc) * (*auc)) * (n - 1) + (q2 - (*auc) * (*auc)) * (m - 1)) / n / m;
 *seauc = sqrt(*seauc);

 return f;
}

double MODIFICATOR McNemarTest(double b,double c,long s,double *p)
//
// The function for computation of McNemar test for nominal signs with Yates correction.
// Designations:
//   b - cell of table 2 - 2,
//   c - cell of table 2 - 2,
//   s - correction for continuity,
//  *p - P-value of the two-sided hypothesis.
// Return code:
//  value of test.
// References:
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//  Bennett B.M., Underwood R.E. On McNemar-s test for the 2 x 2 table and its power // Biometrics, June 1970, vol. 26, no. 2, pp. 339-343.
//  McNemar Q. Psychological statistics. - New York, NY: John Wiley & Sons, 1966.
//  Siegel S., Castellan N.J. Nonparametric statistics for the behavioral sciences. - New York, NY: McGraw Hill, 1988.

//
{
 double f; // Test statistic

 f = fabs(b - c) - s;

 /////////////////////////////////////////
 // Test statistic computation and P-value
 if (b + c == 0)
   {
    f = 0;
    *p = 0.5;
   }
 else
   {
    f = f * f / (b + c);
    *p = ChiSquareDistribution (1,f) / 2;
   }

 return f;
}

double MODIFICATOR ChiSquareTest(double a,double b,double c,double d,double *p,long s)
//
// The function for computation of Chi-square test for nominal signs.
//   a - cell of table 2 - 2,
//   b - cell of table 2 - 2,
//   c - cell of table 2 - 2,
//   d - cell of table 2 - 2,
//  *p - P-value of the two-sided hypothesis,
//   s - flag of correction for continuity.
// Return code:
//  value of test.
// References:
//  Lwanga S.K., Lemeshow S. Sample size determination in health studies. A practical manual. - Geneva: World Health Organization, 1991.
//  Hahn G.J., Shapiro S.S. Statistical models in engineerting. - New York, NY: John Wiley & Sons, 1967.
//
{
 double a1,b1,c1,d1,             // Expected frequencies
        a2,b2,c2,d2,             // Work variables
        n = a + b + c + d,       // Size of table
        r = s == 0 ? 0 : 0.5,    // Correction for continuity
        f;                       // Test statistic

 ///////////////////
 // Work computation
 a1 = (a + b) * (a + c) / n;
 b1 = (a + b) * (b + d) / n;
 c1 = (c + d) * (a + c) / n;
 d1 = (c + d) * (b + d) / n;
 a2 = fabs (a1 - a) - r;
 b2 = fabs (b1 - b) - r;
 c2 = fabs (c1 - c) - r;
 d2 = fabs (d1 - d) - r;

 /////////////////////////////////////////
 // Test statistic computation and P-value
 if (a1 == 0 || b1 == 0 || c1 == 0 || d1 == 0)
   {
    f = 0;
    *p = 0.5;
   }
 else
   {
    f = a2 * a2 / a1 + b2 * b2 / b1 + c2 * c2 / c1 + d2 * d2 / d1;
    *p = ChiSquareDistribution (1,f);
   }

 return f;
}

double MODIFICATOR ProportionDifference (double n11,double n12,double n21,double n22,double *si,double *l,double *h,double p,long s)
//
// The function for computation of proportion difference.
//  n11 - cell of table 2 - 2,
//  n12 - cell of table 2 - 2,
//  n21 - cell of table 2 - 2,
//  n22 - cell of table 2 - 2,
//  *si - standard error,
//   *l - low bound of interval,
//   *h - upper bound of interval,
//    p - confidence level,
//    s - flag of correction for continuity.
// Return code:
//  the difference of proportion.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//
{
 double n1t,n2t, // Work variables
        p1,p2,   // Proportions
        y;       // Correction for continuity

 ///////////////////
 // Work computation
 n1t = n11 + n12;
 n2t = n21 + n22;
 p1 = n11 / n1t;
 p2 = n21 / n2t;
 *si = sqrt(p1 * (1.0 - p1) / n1t + p2 * (1.0 - p2) / n2t);
 y = s == 0 ? 0 : 0.5 * (1.0 / n1t + 1.0 / n2t);

 //////////////////////////////////////
 // Computation of confidence intervals
 *l = InverseNormalDistribution ((1.0 + p) / 2) * (*si) + y;
 *h = fabs (p2 - p1) + *l;
 *l = fabs (p2 - p1) - *l;
 if ((*h) > 1)
   *h = 0;
 if ((*l) < 0)
   *l = 0;

 return fabs(p2 - p1);
}

double MODIFICATOR ProportionDifferencePair (double a,double b,double c,double d,double *si,double *l,double *h,double p,long s)
//
// The function for computation of proportion defference for related samples (Wald formula).
//   a - cell of table 2 - 2,
//   b - cell of table 2 - 2,
//   c - cell of table 2 - 2,
//   d - cell of table 2 - 2,
// *si - standard error,
//  *l - low bound of interval,
//  *h - upper bound of interval,
//   p - confidence level,
//   s - flag of correction for continuity.
// Return code:
//  proportion defference.
//
{
 double n = a + b + c + d; // sample size.

 double p1 = (a + b) / n,  // Proportions
        p2 = (a + c) / n,
        y;                 // Correction for continuity

 //////////////////////
 // Work computation
 *si = sqrt(b + c - (b - c) * (b - c) / n) / n;
 y = s == 0 ? 0 : 1.0 / n;

 //////////////////////////////////////
 // Computation of confidence intervals
 *l = InverseNormalDistribution ((1.0 + p) / 2) * (*si);
 *h = fabs (p2 - p1) + *l + y;
 *l = fabs (p2 - p1) - *l - y;
 if ((*h) > 1)
   *h = 0;
 if ((*l) < 0)
   *l = 0;

 return fabs(p2 - p1);
}

double MODIFICATOR ProportionDifferencePairA (double a,double b,double c,double d,double *si,double *l,double *h,double p,long s)
//
// The function for computation of proportions difference for related samples (refined Wald formula).
//   a - cell of table 2 - 2,
//   b - cell of table 2 - 2,
//   c - cell of table 2 - 2,
//   d - cell of table 2 - 2,
// *si - standard error,
//  *l - low bound of interval,
//  *h - upper bound of interval,
//   p - confidence level,
//   s - flag of correction for continuity.
// Return code:
//  proportions difference.
//
{
 double n = a + b + c + d; // sample size.

 double p1 = (a + b) / n,  // Proportions
        p2 = (a + c) / n,
        y;                 // Correction for continuity

 ///////////////////
 // Work computation
 *si = sqrt(b + c + 1 - (b - c) * (b - c) / (n + 2)) / (n + 2);
 y = s == 0 ? 0 : 1.0 / n;

 //////////////////////////////////////
 // Computation of confidence intervals
 *l = InverseNormalDistribution ((1.0 + p) / 2) * (*si);
 *h = fabs (b - c) / (n + 2) + *l + y;
 *l = fabs (b - c) / (n + 2) - *l - y;
 if ((*h) > 1)
   *h = 0;
 if ((*l) < 0)
   *l = 0;

 return fabs(p2 - p1);
}

double MODIFICATOR Kappa (double a,double b,double c,double d,double *l,double *h,double p)
//
// The function for computation of kappa by Cohen.
//   a - cell of table 2 - 2,
//   b - cell of table 2 - 2,
//   c - cell of table 2 - 2,
//   d - cell of table 2 - 2,
//  *l - low bound of interval,
//  *h - upper bound of interval,
//   p - confidence level.
// Return code:
//  kappa by Cohen.
//
{
 double r1 = a + b,        // Sum 1 of rows
        r2 = c + d,        // Sum 2 of rows
        c1 = a + c,        // Sum 1 of column
        c2 = b + d,        // Sum 2 of column
        n = a + b + c + d, // Sum of table
        p0,pe,se,          // Parameters
        k;                 // Return code

 ///////////////////////
 // Computation of kappa
 p0 = (a + d) / n;
 pe = r1 / n * c1 / n + r2 / n * c2 / n;
 k = (p0 - pe) / (1 - pe);

 //////////////////////////////////////
 // Computation of confidence intervals
 se = sqrt(p0 * (1.0 - p0) / n / (1.0 - pe) / (1.0 - pe)) * InverseNormalDistribution ((1.0 + p) / 2);
 *h = k + se;
 *l = k - se;

 return k;
}

double MODIFICATOR ZTest(double n11,double n21,double n12,double n22,double *p,long s)
//
// The function for computation z-test for independent signs.
//  n11 - cell of table 2 - 2,
//  n12 - cell of table 2 - 2,
//  n21 - cell of table 2 - 2,
//  n22 - cell of table 2 - 2,
//   *p - P-value of the two-sided hypothesis,
//    s - flag of correction for continuity.
// Return code:
//  value of test.
//
{
 double m1 = 1.0 / (n11 + n12),                    // Work variable
        m2 = 1.0 / (n21 + n22);                    // Work variable

 double y = s == 0 ? 0 : 0.5 * (m1 + m2),          // Correction for continuity
        f = (n11 + n21) / (n11 + n12 + n21 + n22); // Return code

 /////////////////////////////////////////
 // Test statistic computation and P-value
 f = (fabs(n11 * m1 - n21 * m2) - y) / sqrt(f * (1.0 - f) * (m1 + m2));
 *p = ChiSquareDistribution (1,f * f);

 return f;
}

double MODIFICATOR ZTestPair (double a,double b,double c,double d,double *p,long s)
//
// The function for computation z-test for pair signs.
//   a - cell of table 2 - 2,
//   b - cell of table 2 - 2,
//   c - cell of table 2 - 2,
//   d - cell of table 2 - 2,
//  *p - P-value of the two-sided hypothesis,
//   s - flag of correction for continuity.
// Return code:
//  value of test.
//
{
 double n = a + b + c + d;                                   // Size of table

 double y = s == 0 ? 0 : 1.0 / n;                            // Correction for continuity

 double f = (fabs(b - c) - y) * (fabs(b - c) - y) / (b + c); // Return code

 *p = ChiSquareDistribution (1,f);

 return f;
}

double MODIFICATOR OddsRatio (double n11,double n12,double n21,double n22,double *l,double *h,double p)
//
// The function for computation of the odds ratio with confidence intervals.
//  n11 - cell of table 2 - 2,
//  n12 - cell of table 2 - 2,
//  n21 - cell of table 2 - 2,
//  n22 - cell of table 2 - 2,
//   *l - low bound of interval,
//   *h - upper bound of interval,
//    p - confidence level.
// Return code:
//  value of odds ratio.
//
{
 double s, // Standard error, later work variable
        r; // Return code

 ///////////////////////////////////////////
 // Computation of corrections, if necessary
 if (n11 == 0 || n12 == 0 || n21 == 0 || n22 == 0)
   {
    n11 += 0.5;
    n12 += 0.5;
    n21 += 0.5;
    n22 += 0.5;
   }

 ///////////////////
 // Work computation
 r = n11 * n22 / n12 / n21;
 s = sqrt(1 / n12 + 1 / n21 + 1/ n11 + 1 / n22);

 //////////////////////////////////////
 // Computation of confidence intervals
 s = InverseNormalDistribution ((1 + p) / 2) * s;
 *l = exp(log(r) - s);
 *h = exp(log(r) + s);

 return r;
}

double MODIFICATOR Sensitivity (double n11,double n21,double *l,double *h,double p)
//
// The function for computation of sensitivity with confidence intervals.
//  n11 - cell of table 2 - 2,
//  n21 - cell of table 2 - 2,
//   *l - low bound of interval,
//   *h - upper bound of interval,
//    p - confidence level.
// Return code:
//  value of sensitivity.
//
{
 double se, // Sensitivity
        s;  // Variance, later work variable

 /////////////////////////////
 // Work computation
 se = n11 / (n11 + n21);
 s = se * (1 - se) / (n11 + n21);

 //////////////////////////////////////
 // Computation of confidence intervals
 s = InverseNormalDistribution ((1 + p) / 2) * sqrt (s);
 *l = se - s;
 *h = se + s;

 return se;
}

double MODIFICATOR Specificity (double n12,double n22,double *l,double *h,double p)
//
// The function for computation of specificity with confidence intervals.
//  n12 - cell of table 2 - 2,
//  n22 - cell of table 2 - 2,
//   *l - low bound of interval,
//   *h - upper bound of interval,
//    p - confidence level.
// Return code:
//  value of specificity.
//
{
 double sp, // Specificity
        s;  // Variance, later work variable

 ///////////////////
 // Work computation
 sp = n22 / (n12 + n22);
 s = sp * (1 - sp) / (n12 + n22);

 //////////////////////////////////////
 // Computation of confidence intervals
 s = InverseNormalDistribution ((1 + p) / 2) * sqrt (s);
 *l = sp - s;
 *h = sp + s;

 return sp;
}

double MODIFICATOR PPV (double n11,double n12,double n21,double n22,double *l,double *h,double p,double prev)
//
// The function for computation of confidence intervals for prognosticality of the positive result.
//   n11 - cell of table 2 - 2,
//   n12 - cell of table 2 - 2,
//   n21 - cell of table 2 - 2,
//   n22 - cell of table 2 - 2,
//    *l - low bound of interval,
//    *h - upper bound of interval,
//     p - confidence level,
//  prev - prevalence.
// Return code:
//  prognosticality of the positive result.
//
{
 double se, // Sensitivity
        sp, // Specificity
        r,  // Return code
        s,  // Variance
        s1; // Work variable

 ///////////////////
 // Work computation
 se = n11 / (n11 + n21);
 sp = n22 / (n12 + n22);
 if (se * prev + (1 - sp) * (1 - prev) == 0)
   r = 0;
 else
   r = se * prev / (se * prev + (1 - sp) * (1 - prev));
 s1 = prev * (1 - sp) * (1 - prev);
 if (n11 + n21 == 0)
   s = 0;
 else
   s = s1 * s1 * se * (1 - se) / (n11 + n21);
 s1 = prev * se * (1 - prev);
 if (n12 + n22 == 0)
   s = 0;
 else
   s = s + s1 * s1 * sp * (1- sp) / (n12 + n22);
 s1 = se * prev + (1 - sp) * (1 - prev);
 if (s1 == 0)
   s = 0;
 else
   s = s / s1 / s1 / s1 / s1;

 //////////////////////////////////////
 // Computation of confidence intervals
 if (s < 0)
   s1 = 0;
 else
   s1 = InverseNormalDistribution ((1 + p) / 2) * sqrt (s);
 *l = r - s1;
 *h = r + s1;

 return r;
}

double MODIFICATOR NPV (double n11,double n12,double n21,double n22,double *l,double *h,double p,double prev)
//
// The function for computation of confidence intervals for prognosticality of the negative result.
//   n11 - cell of table 2 - 2,
//   n12 - cell of table 2 - 2,
//   n21 - cell of table 2 - 2,
//   n22 - cell of table 2 - 2,
//    *l - low bound of interval,
//    *h - upper bound of interval,
//     p - confidence level,
//  prev - prevalence.
// Return code:
//  prognosticality of the negative result.
//
{
 double se, // Sensitivity
        sp, // Specificity
        r,  // Return code
        s,  // Variance
        s1; // Work variable

 ///////////////////
 // Work computation
 se = n11 / (n11 + n21);
 sp = n22 / (n12 + n22);
 if ((1.0 - se) * prev + sp * (1.0 - prev) == 0)
   r = 0;
 else
   r = sp * (1.0 - prev) / ((1.0 - se) * prev + sp * (1.0 - prev));
 s1 = sp * (1.0 - prev) * prev;
 if (n11 + n21 == 0)
   s = 0;
 else
   s = s1 * s1 * se * (1.0 - se) / (n11 + n21);
 s1 = (1.0 - se) * (1.0 - prev) * prev;
 if (n12 + n22 == 0)
   s = 0;
 else
   s = s + s1 * s1 * sp * (1.0 - sp) / (n12 + n22);
 s1 = (1.0 - se) * prev + sp * (1 - prev);
 if (s1 == 0)
   s = 0;
 else
   s = s / s1 / s1 / s1 / s1;

 //////////////////////////////////////
 // Computation of confidence intervals
 if (s < 0)
   s1 = 0;
 else
   s1 = InverseNormalDistribution ((1.0 + p) / 2) * sqrt (s);
 *l = r - s1;
 *h = r + s1;

 return r;
}

double MODIFICATOR Prevalence (double n11,double n12,double n21,double n22,double *l,double *h,double p)
//
// The function for computation of prevalence with confidence intervals.
//   n11 - cell of table 2 - 2,
//   n12 - cell of table 2 - 2,
//   n21 - cell of table 2 - 2,
//   n22 - cell of table 2 - 2,
//    *l - low bound of interval,
//    *h - upper bound of interval,
//     p - confidence level.
// Return code:
//  value of prevalence.
//
{
 double r,                         // Return code
        s,                         // Variance, later work variable
        n = n11 + n21 + n12 + n22; // Common size

 ///////////////////
 // Work computation
 r = (n11 + n21) / n;
 s = r * (1.0 - r) / n;

 //////////////////////////////////////
 // Computation of confidence intervals
 s = InverseNormalDistribution ((1 + p) / 2) * sqrt (s);
 if (n11 + n21 == 0)
   *l = 0;
 else
   *l = r - s;
 if (n11 + n21 == n)
   *h = 1;
 else
   *h = r + s;

 return r;
}

double MODIFICATOR RelativeRisk (double a,double b,double c,double d,double *l,double *h,double p)
//
// The function for computation of relative risk with confidence intervals.
//   a - cell of table 2 - 2,
//   b - cell of table 2 - 2,
//   c - cell of table 2 - 2,
//   d - cell of table 2 - 2,
//  *l - low bound of interval,
//  *h - upper bound of interval,
//   p - confidence level.
// Return code:
//  relative risk.
//
{
 double s, // Standard error, later work variable
        r; // Relative risk

 ///////////////////////////////////////////
 // Computation of corrections, if necessary
 if (a == 0 || b == 0 || c == 0 || d == 0)
   {
    a += 0.5;
    b += 0.5;
    c += 0.5;
    d += 0.5;
   }

 ///////////////////
 // Work computation
 r = a * (c + d) / (a + b) / c;
 s = sqrt(b / a / (a + b) + d / c / (c + d));

 //////////////////////////////////////
 // Computation of confidence intervals
 s = InverseNormalDistribution ((1 + p) / 2) * s;
 *l = exp(log(r) - s);
 *h = exp(log(r) + s);

 return r;
}

double MODIFICATOR SmirnovExactTest(double *x,long nx,double *y,long ny,double *p)
//
// The function of exact computation of Smirnov test.
// Designations:
//    x - 1st sample,
//   nx - size of 1st sample,
//    y - 2nd sample,
//   ny - size of 2nd sample,
//   *p - P-value of the hypothesis.
// Return code:
//  value of test.
//
{
 long i,j;       // Counters

 double s = 0,   // Work variable
        t = 0,   // Value of test
        *xx,*yy; // Samples copy

 xx = new double[nx];
 yy = new double[ny];

 ///////////////
 // Samples copy
 for (i = 0; i < nx; i++)
   xx[i] = x[i];
 for (i = 0; i < ny; i++)
   yy[i] = y[i];

 ///////////////
 // Sort samples
 SortArrayUp0 (xx,nx);
 SortArrayUp0 (yy,ny);

 /////////////////////////////////////////////////////
 // Iterate over all members of 2nd sample relative to
 // all members of 1st sample
 for (i = 0, j = 0; i < ny; i++)

   ///////////////////////////////////////////////////////////////////
   // Current member of 2nd sample is less than or equal to 1st member
   // of 1st sample
   if (yy[i] <= xx[0])
     {
      s = 1.0 * i / ny;
      if (s > t)
        t = s;
     }

   ////////////////////////////////////////////////////////
   // Current member of 2nd sample is greater than or equal
   // to the last member of 1st sample
   else if (yy[i] >= xx[nx - 1])
     {
      s = 1.0 - 1.0 * i / ny;
      if (s > t)
        t = s;

      //////////////////////////////////////////////////////////////
      // Further of computation can not be carried out - the maximum
      // has already been achieved in the computation process
      break;
     }

   ////////////////////////////////////////////////////
   // Current member of 2nd sample within of 1st sample
   else
     {
      //////////////////////////////////////////////
      // Let's clarify between which members exactly
      for (; j < nx - 1; j++)
        if (yy[i] >= xx[j] && yy[i] < xx[j + 1])
          {
           s = fabs (1.0 * i / ny - 1.0 * (j + 1) / nx);

           //////////////////////////////////////////////////////////
           // To interrupt the cycle with conservation of the reached
           // numbers current member of 2nd sample - so reduce the volume
           // of computation
           break;
          }
      if (s > t)
        t = s;
     }

 /////////////////////////////////////////////////////
 // Iterate over all members of 1st sample relative to
 // all members of 2nd sample
 for (i = 0, j = 0; i < nx; i++)

   ////////////////////////////////////////////////////////////
   // Current member of 1st sample is less than or equal to 1st
   // member of 2nd sample
   if (xx[i] <= yy[0])
     {
      s = 1.0 * i / nx;
      if (s > t)
        t = s;
     }

   ////////////////////////////////////////////////////////////////////
   // Current member of 1st sample is greater than or equal to the last
   // member of 2nd sample
   else if (xx[i] >= yy[ny - 1])
     {
      s = 1.0 - 1.0 * i / nx;
      if (s > t)
        t = s;

      //////////////////////////////////////////////////////////////
      // Further of computation can not be carried out - the maximum
      // has already been reached in the computation process
      break;
     }

   //////////////////////////////////////////////////////////
   // Current member of 1st sample is in limits of 2nd sample
   else
     {
      //////////////////////////////////////////////
      // Let's clarify between which members exactly
      for (; j < ny - 1; j++)
        if (xx[i] >= yy[j] && xx[i] < yy[j + 1])
          {
           s = fabs (1.0 * i / nx - 1.0 * (j + 1) / ny);

           //////////////////////////////////////////////////////////
           // Break the loop with conservation of the reached numbers
           // of current members of 1st sample - so reduce the volume of
           // computation
           break;
          }
      if (s > t)
        t = s;
     }

 delete [] xx; delete [] yy;

 ////////////////////////
 // P-value computation
 *p = LambdaDistribution (t * sqrt(1.0 * nx / (nx + ny) * ny));

 return t;
}

double MODIFICATOR KuiperExactTest(double *x,long nx,double *y,long ny,double *p)
//
// The function for exact computation of Kuiper test.
// Designations:
//    x - 1st sample,
//   nx - size of 1st sample,
//    y - 2nd sample,
//   ny - size of 2nd sample,
//   *p - P-value of the hypothesis.
// Return code:
//  value of test.
//
{
 long i;                         // Counter

 double v,                       // Value of test
        *xx,*yy,                 // Samples copy
        n = nx * ny / (nx + ny); // Parameter of size

 xx = new double[nx];
 yy = new double[ny];

 ///////////////
 // Samples copy
 for (i = 0; i < nx; i++)
   xx[i] = x[i];
 for (i = 0; i < ny; i++)
   yy[i] = y[i];

 ///////////////
 // Sort samples
 SortArrayUp0 (xx,nx);
 SortArrayUp0 (yy,ny);

 v = KuiperExactTestAdd (xx,nx,yy,ny) + KuiperExactTestAdd (yy,ny,xx,nx);

 delete [] xx; delete [] yy;

 //////////////////////
 // P-value computation
 *p = KuiperDistribution (v * sqrt(n));

 return v;
}

double MODIFICATOR KuiperExactTestAdd (double *x,long nx,double *y,long ny)
//
// Component of algorithm for exact computation of function D+ of Kuiper test.
// Designations:
//   xx - ordered 1st sample,
//   nx - size of 1st sample,
//   yy - ordered 2nd sample.
//   ny - size of 2nd sample.
// Return code:
//  maximal difference D+ between frequency.
//
{
 long i,j = 0; // Counters

 double s = 0, // Work variable
        t = 0; // Return code

 /////////////////////////////////////////////////////////////
 // Sort through all variants of the of 2nd sample relative to
 // all variants of the of 1st sample
 for (i = 0; i < ny; i++)

   ///////////////////////////////////////////////////////////
   // The current version of the of 2nd sample is less than or
   // equal to the 1st version of 1st sample
   if (y[i] <= x[0])
     {
      s = 1.0 * i / ny;
      if (s > t)
        t = s;
     }

   ///////////////////////////////////////////////////////////
   // The current version of the of 2nd sample is greater than
   // or equal to the last version of 1st sample
   else if (y[i] >= x[nx - 1])
     {
      s = 1.0 - 1.0 * i / ny;
      if (s > t)
        t = s;

      //////////////////////////////////////////////////////////////
      // Further of computation can not be carried out - the maximum
      // has already been achieved in the computation process
      break;
     }

   ////////////////////////////////////////////////////////////////
   // Current version of the of 2nd sample within the of 1st sample
   else
     {
      //////////////////////////////////////////////
      // Let's clarify between which options exactly
      for (; j < nx - 1; j++)
        if (y[i] >= x[j] && y[i] < x[j + 1])
          {
           s = 1.0 * i / ny - 1.0 * (j + 1) / nx;

           ///////////////////////////////////////////////////////////
           // Interrupt the cycle with the Conservation of the reached
           // number of the current variants of the of 2nd sample - so
           // reduce the amount of computations
           break;
          }
      if (s > t)
        t = s;
     }

 return t;
}

double MODIFICATOR WilcoxonTest(double *x,long nx,double *y,long ny,double *p,long a1,long a2)
//
// The function for computation of the Wilcoxon rank sum test.
//    x - 1st sample,
//   nx - size of 1st sample,
//    y - 2nd sample,
//   ny - size of 2nd sample,
//   *p - P-value of the two-sided hypothesis,
//   a1 - the flag of accounting for bundles,
//   a2 - the flag of correction for continuity.
// Return code:
//  value of test.
// References:
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//  Hettmansperger T.P. Statistical inference based on ranks. - New York, NY: John Wiley & Sons, 1984.
//  Van der Waerden B.L. Mathematische Statistik (in Deutsch). - Berlin: Springer, 1971.
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//  Brownlee K.A. Statistical theory and metodology in sciences and engineering. - New York, NY: John Wiley & Sons, 1965.
//
{
 long i,               // Counter
      *ix,             // Permutation info
      n = nx + ny;     // Common size

 double *xx,*xy,       // Work array
        tt,            // Bundle adjustment of ranks and work variable
        criterion = 0, // Value of test
        t;             // Parameter of normal approximation

 ix = new long[n];
 xy = new double[n];
 xx = new double[n];

 ////////////////////////
 // Given arrays junction
 for (i = 0; i < nx; i++)
   xy[i] = x[i];
 for (i = 0; i < ny; i++)
   xy[i + nx] = y[i];

 //////////
 // Sort up
 SortArrayUp (xy,n,ix);

 //////////
 // Ranking
 tt = Rank (xy,xx,n);
 if (a1 == 0)
   tt = 0;

 //////////////////////
 // Restore ranks order
 UnSortArray (xx,xy,n,ix);

 delete [] ix; delete [] xx;

 ////////////////////////////
 // Sums computation of ranks
 for (i = 0; i < nx; i++)
   criterion += xy[i];

 delete [] xy;

 ////////////////////////////////////
 // Parameter of normal approximation
 t = (criterion - 0.5 * nx * (n + 1)) /
     sqrt (0.0833333333 * nx * ny * (n + 1 - tt / n / (n - 1)));

 tt = 0.0833333333 * nx * ny * (n + 1 - tt / n / (n - 1));
 if (tt <= 0)
   return 0.5;

 //////////////////////////////////////
 // P-value computation with correction
 if (NormalDistribution (t) > 0.02 && a2 == 1)
   t = ((criterion + 0.5) - 0.5 * nx * (n + 1)) / sqrt (tt);
 *p = NormalDistribution (t);

 return criterion;
}

double MODIFICATOR RosenblattTest(double *x1,long nx,double *y1,long ny,double *p)
//
// The function for computation of Lehman-Rosenblatt test.
//   x1 - 1st sample,
//   nx - size of 1st sample,
//   y1 - 2nd sample,
//   ny - size of 2nd sample,
//   *p - P-value of the hypothesis.
// Return code:
//  value of test.
//
{
 long i,               // Counter
      *ix,             // Permutation info
      n = nx + ny;     // Common size

 double *xx,*xy,       // Work array
        *x,*y,         // Copy of given samples
        xn = 0,yn = 0, // Sums
        criterion;     // Value of test

 ix = new long[n];
 xy = new double[n];
 xx = new double[n];
 x = new double[nx];
 y = new double[ny];

 ///////////////
 // Samples copy
 for (i = 0; i < nx; i++)
   x[i] = x1[i];
 for (i = 0; i < ny; i++)
   y[i] = y1[i];

 ///////////////
 // Sort samples
 SortArrayUp0 (x,nx);
 SortArrayUp0 (y,ny);

 ////////////////////////
 // Given arrays junction
 for (i = 0; i < nx; i++)
   xy[i] = x[i];
 for (i = 0; i < ny; i++)
   xy[i + nx] = y[i];

 //////////
 // Sort up
 SortArrayUp (xy,n,ix);

 ///////////////
 // Ranking
 Rank (xy,xx,n);

 //////////////////////
 // Restore ranks order
 UnSortArray (xx,xy,n,ix);

 delete [] ix; delete [] xx; delete [] x; delete [] y;

 ///////////////////
 // Sums computation
 for (i = 0; i < nx; i++)
   xn = xn + (xy[i] - i - 1) * (xy[i] - i - 1);
 for (i = 0; i < ny; i++)
   yn = yn + (xy[i + nx] - i - 1) * (xy[i + nx] - i - 1);

 delete [] xy;

 ////////////////////
 // Value of the test
 criterion = (xn / ny + yn / nx - (4.0 * nx * ny - 1) / 6) / n;

 //////////////////////
 // P-value computation
 *p = A1 (criterion);

 return criterion;
}

double MODIFICATOR MannWhitneyTest(double *x,long nx,double *y,long ny,double *p)
//
// The function for computation of Mann-Whitney U-test.
// Designations:
//    x - 1st sample,
//   nx - size of 1st sample,
//    y - 2nd sample,
//   ny - size of 2nd sample,
//   *p - P-value of the one-sided hypothesis.
// Return code:
//  value of test.
// References:
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//  Johnson N.L., Leone F.C. Statistics and experimental design in engineering and the physical sciences. Vol. I. - New York, NY: John Wiley & Sons, 1977.
//  Di Bucchianico A.A. Combinatorics, computer algebra and Wilcoxon-Mann-Whitney test / A.A. Di Bucchianico // Journal of Statistical Planning and Inference. - 1999. - Vol. 79. - No. 2. - P. 349-364.
//
{
 long i,           // Counter
      *ix,         // Permutation info
      n = nx + ny; // Common size

 double *xx,*xy,   // Work array
        t = 0;     // Value of test

 ix = new long[n];
 xy = new double[n];
 xx = new double[n];

 ////////////////////////
 // Given arrays junction
 for (i = 0; i < nx; i++)
   xy[i] = x[i];
 for (i = 0; i < ny; i++)
   xy[i + nx] = y[i];

 //////////
 // Sort up
 SortArrayUp (xy,n,ix);

 //////////
 // Ranking
 Rank (xy,xx,n);

 //////////////////////
 // Restore ranks order
 UnSortArray (xx,xy,n,ix);

 delete [] ix; delete [] xx;

 ////////////////////////////
 // Sums computation of ranks
 for (i = 0; i < ny; i++)
   t += xy[i + nx];
 t = 0.5 * ny * (ny + 1) - t + nx * ny;

 delete [] xy;

 /////////////////////////////////////////
 // Test statistic computation and P-value
 *p = NormalDistribution ((t - 0.5 * nx * ny) / sqrt (0.0833333333 * nx * ny * (n + 1)));

 return t;
}

double MODIFICATOR WilcoxonSignedTest(double *x,long n,double *y,double *p,long a1)
//
// The function for computation of Wilcoxon signed T-test.
// Designations:
//   x - 1st sample,
//   n - each sample size,
//   y - 2nd sample,
//  *p - P-value of the one-sided hypothesis,
//  a1 - the flag of accounting for bundles.
// Return code:
//  value of test.
// References:
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//  Hettmansperger T.P. Statistical inference based on ranks. - New York, NY: John Wiley & Sons, 1984.
//  Brownlee K.A. Statistical theory and metodology in sciences and engineering. - New York, NY: John Wiley & Sons, 1965.
//
{
 long i,j = 0,     // Counter
      *ix;         // Sort info

 double *spx,*spy, // Copies of given of array
        *xx,       // Array of ranks
        save,      // Buffer, later correction to the merger of ranks
        n1 = 0,    // Corrected size
        t = 0;     // Value of test

 spx = new double[n];
 spy = new double[n];
 xx = new double[n];
 ix = new long[n];

 ///////////////
 // Samples copy
 for (i = 0; i < n; i++)
   {
    spx[i] = x[i];
    spy[i] = y[i];
   }

 ////////////////////////
 // Arrays of differences
 for (i = 0; i < n; i++)
   {
    save = spx[i];
//    if (spx[i] != spy[i])
    if (fabs (spx[i] - spy[i]) > 0.000001)
      {
       spx[j] = fabs (spx[i] - spy[i]); // of difference without signs
       n1++;
       spy[j++] = save - spy[i];        // of difference with signs
      }
   }

 //////////
 // Sort up
 SortArrayUp (spx,n1,ix);

 //////////
 // Ranking
 save = Rank (spx,xx,n1);
 if (a1 == 0)
   save = 0;

 ////////////////////////////////////////
 // Restore ranks order arrays with signs
 UnSortArray (xx,spx,n1,ix);

 delete [] ix; delete [] xx;

 ///////////////////////////////////////
 // Sum of ranks of positive differences
 for (i = 0; i < n1; i++)
   if (spy[i] > 0)
     t += spx[i];

 delete [] spx; delete [] spy;

 //////////////////////
 // P-value computation
 *p = NormalDistribution ((t - 0.25 * n1 * (n1 + 1)) /
      sqrt (0.0416666667 * n1 * (n1 + 1) * (2 * n1 + 1) - 0.0208333333 * save));

 return t;
}

double MODIFICATOR VanDerWaerdenTest(double *x,long nx,double *y,long ny,double *p)
//
// The function for computation Van Der Waerden X-test.
// Designations:
//   x - 1st sample,
//  nx - size of 1st sample,
//   y - 2nd sample,
//  ny - size of 2nd sample,
//  *p - P-value of the one-sided hypothesis.
// Return code:
//  value of test.
// References:
//  Van der Waerden B.L. Mathematische Statistik (in Deutsch). - Berlin: Springer, 1971.
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag, 1979.
//
{
 long i,               // Counter
       *ix,            // Information array
       n = nx + ny;    // Common size

 double *xx,*xy,       // Work array
        criterion = 0, // Cumulative parameter value
        q = 0,         // Parameter of normal approximation
        s;             // Work variable

 ix = new long[n];
 xx = new double[n];
 xy = new double[n];

 ////////////////////////
 // Given arrays junction
 for (i = 0; i < nx; i++)
   xy[i] = x[i];
 for (i = 0; i < ny; i++)
   xy[i + nx] = y[i];

 //////////
 // Sort up
 SortArrayUp (xy,n,ix);

 //////////
 // Ranking
 Rank (xy,xx,n);

 //////////////////////
 // Restore ranks order
 UnSortArray (xx,xy,n,ix);

 delete [] ix; delete [] xx;

 ////////////////////////////
 // Value of test computation
 for (i = 0; i < nx; i++)
   criterion += InverseNormalDistribution (xy[i] / (n + 1));

 delete [] xy;

 for (i = 1; i <= n; i++)
   {
    s = InverseNormalDistribution (1.0 * i / (n + 1));
    q += s * s;
   }

 //////////////////////
 // P-value computation
 *p = NormalDistribution (criterion / sqrt(q * nx / n * ny / (n - 1)));

 return criterion;
}

double MODIFICATOR AnsariBradleyTest(double *y,long ny,double *x,long nx,double *p,long a1)
//
// The function for computation of Ansari-Bradley test.
// Designations:
//   y - 1st sample,
//  ny - size of 1st sample,
//   x - 2nd sample,
//  nx - size of 2nd sample,
//  *p - P-value of the two-sided hypothesis,
//  a1 - the flag of accounting for bundles.
// Return code:
//  value of test.
// References:
//  Hollander M., Wolfe D.A., Chiken E. Nonparametric statistical methods. - New York, NY: Wley, 2014.
//  Johnson N.L., Leone F.C. Statistics and experimental design in engineering and the physical sciences. Vol. I. - New York, NY: John Wiley & Sons, 1977.
//
{
 long i,            // Counter
       *ix,         // Information array
       n = nx + ny; // Common size

 double *xx,*xy,    // Work array
        t = 0,      // Cumulative parameter value
        b,          // Bundle adjustment of ranks
        e,          // Parameter of normal approximation
        var;        // Parameter of normal approximation

 //////////////////////////////////////////////
 // Control of the admissibility of estimations
 if (Variance(x,nx) < 0.000001 && Variance(y,ny) < 0.000001)
   {
    *p = 0.5;
    return 0;
   }

 ix = new long[n];
 xx = new double[n];
 xy = new double[n];

 ////////////////////////
 // Given arrays junction
 for (i = 0; i < nx; i++)
   xy[i] = x[i];
 for (i = 0; i < ny; i++)
   xy[i + nx] = y[i];

 //////////
 // Sort up
 SortArrayUp (xy,n,ix);

 //////////
 // Ranking
 b = RankAB (xy,xx,n);
 if (a1 == 0)
   b = 0;

 //////////////////////
 // Restore ranks order
 UnSortArray (xx,xy,n,ix);

 delete [] ix; delete [] xx;

 ////////////////////////////
 // Value of test computation
 for (i = 0; i < nx; i++)
   t += xy[i];

 delete [] xy;

 /////////////////////////////////////
 // Parameters of normal approximation
 if (Parity(n))
   {
    e = 1.0 * nx / 4 * (n + 2);
    if (b == 0)
      var = 1.0 * nx / 48 * ny * (n + 2) / (n - 1) * (n - 2);
    else
      var = 1.0 * nx / 16 * ny * (b / (n - 1) * 16 / n - 1.0 * (n + 2) / (n - 1) * (n + 2));
   }
 else
   {
    e = 1.0 * nx / 4 * (n + 1) / n * (n + 1);
    if (b == 0)
      var = 1.0 * nx / 48 * ny / n * (n + 1) / n * (n * n + 3);
    else
      var = 1.0 * nx / 16 * ny * (b / (n - 1) * 16 / n - 1.0 * (n + 1) / n * (n + 1) / n * (n + 1) / (n - 1) * (n + 1));
   }

 //////////////////////
 // P-value computation
 *p = NormalDistribution ((t - e) / sqrt (var));

 return t;
}

double MODIFICATOR SiegelTukeyTest(double *y,long ny,double *x,long nx,double *p)
//
// The function for computation of Siegel-Tukey test.
// Designations:
//   y - 1st sample,
//  ny - size of 1st sample,
//   x - 2nd sample,
//  nx - size of 2nd sample,
//  *p - P-value of the two-sided hypothesis.
// Return code:
//  value of test.
// References:
//  Klotz J.H. Nonparametric tests for scale // Annals of Mathematical Statistics, 1962, vol. 33, no. 2, pp. 498-512.
//
{
 long i,           // Counter
      *ix,         // Information array
      n = nx + ny; // Common size

 double *xx,*xy,   // Work array
        t = 0;     // Cumulative parameter value

 //////////////////////////////////////////////
 // Control of the admissibility of estimations
 if (Variance(x,nx) < 0.000001 && Variance(y,ny) < 0.000001)
   {
    *p = 0.5;
    return 0;
   }

 ix = new long[n];
 xx = new double[n];
 xy = new double[n];

 ////////////////////////
 // Given arrays junction
 for (i = 0; i < nx; i++)
   xy[i] = x[i];
 for (i = 0; i < ny; i++)
   xy[i + nx] = y[i];

 //////////
 // Sort up
 SortArrayUp (xy,n,ix);

 //////////
 // Ranking
 RankST (xy,xx,n);

 //////////////////////
 // Restore ranks order
 UnSortArray (xx,xy,n,ix);

 delete [] ix; delete [] xx;

 /////////////////////////
 // Value of test computation
 for (i = 0; i < nx; i++)
   t += xy[i];

 delete [] xy;

 //////////////////////
 // P-value computation
 *p = NormalDistribution ((t - 0.5 * nx * (n + 1)) /
      sqrt(0.083333333333 * nx * ny * (n + 1)));

 return t;
}

double MODIFICATOR MedianTest(double *xx,long n1,double *yy,long n2,double *p)
//
// The function for computation of exact median test.
// Designations:
//  xx - 1st sample,
//  n1 - size of 1st sample,
//  yy - 2nd sample,
//  n2 - size of 2nd sample,
//  *p - P-value of the one-sided hypothesis.
// Return code:
//  value of test.
// References:
//  Brownlee K.A. Statistical theory and metodology in sciences and engineering. - New York, NY: John Wiley & Sons, 1965.
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//
{
 long i,                         // Counter
      n = n1 + n2;               // Common size

 double a = 0,b = 0,c = 0,d = 0, // Cells A, B, C and D of table 2 - 2
        med,                     // Median, test statistic
        p1,p2,                   // Work variables
        *x;                      // Merged array

 x = new double[n];

 ////////////////////////
 // Merge of given arrays
 for (i = 0; i < n1; i++)
   x[i] = xx[i];
 for (i = 0; i < n2; i++)
   x[i + n1] = yy[i];

 ////////////////////////
 // Computation of median
 med = Median(x,n);

 delete [] x;

 for (i = 0; i < n1; i++)
   if (xx[i] > med)
     a++;
   else
     b++;
 for (i = 0; i < n2; i++)
   if (yy[i] > med)
     c++;
   else
     d++;

 //////////////////////////////
 // Computing of test statistic
 med = FisherTest (a,b,c,d,&p1,&p2);

 if (Median(xx,n1) > Median (yy,n2))
   *p = p2;
 else
   *p = p1;

 return med;
}

double MODIFICATOR Wald (double a,double b,double c,double d)
//
// The function for computation of Wald statistic.
// Designations:
//  a - number of observations with effect A in 1st sample,
//  b - number of observations without effect A in 1st sample,
//  c - number of observations with effect A in 2nd sample,
//  d - number of observations without effect A in 2nd sample.
// Return code:
//  Wald statistic.
//
{
 double ac = a + c,
        bd = b + d;

 if (ac != 0 && bd != 0 && a + b != 0 && c + d != 0)
   return fabs(a / ac - b / bd) / sqrt((a + b) * (c + d) * (1 / ac + 1 / bd)) * (ac + bd);
 else
   return 0;
}

double MODIFICATOR SavageTest(double *x,long nx,double *y,long ny,double *p)
//
// The function for computation of Savage test.
// Designations:
//   x - 1st sample,
//  nx - size of 1st sample,
//   y - 2nd sample,
//  ny - size of 2nd sample,
//  *p - P-value of the two-sided hypothesis.
// Return code:
//  value of test.
//
{
 long i,           // Counter
      *ix,         // Information array
      n = nx + ny; // Common size

 double *xx,*xy,   // Work array
        t = 0,     // Value of test
        dt = 0;    // Variance

 ix = new long[n];
 xx = new double[n];
 xy = new double[n];

 ////////////////////////
 // Given arrays junction
 if (nx >= ny)
   {
    for (i = 0; i < nx; i++)
      xy[i] = x[i];
    for (i = 0; i < ny; i++)
      xy[i + nx] = y[i];
   }
 else
   {
    for (i = 0; i < ny; i++)
      xy[i] = y[i];
    for (i = 0; i < nx; i++)
      xy[i + ny] = x[i];
    nx = ny;
   }

 //////////
 // Sort up
 SortArrayUp (xy,n,ix);

 //////////
 // Ranking
 RankSavage (xx,n);

 //////////////////////
 // Restore ranks order
 UnSortArray (xx,xy,n,ix);

 delete [] ix; delete [] xx;

 ////////////////////////////
 // Value of test computation
 for (i = 0; i < nx; i++)
   t += xy[i];

 delete [] xy;

 for (i = 1; i <= n; i++)
   dt += 1.0 / i;
 dt = (1.0 - dt / n) * nx * ny / (nx + ny - 1);

 //////////////////////
 // P-value computation
 *p = NormalDistribution ((t - nx) / sqrt (dt));

 return t;
}

double MODIFICATOR WaldWolfowitzTest (double *x,long n1,double *y,long n2,double *p,long a2)
//
// The function for computation of Wald-Wolfowitz test (asymptotic version).
//   x - 1st sample,
//  n1 - size of 1st sample,
//   y - 2nd sample,
//  n2 - size of 2nd sample,
//  *p - P-value of the hypothesis,
//  a2 - flag of correction for continuity.
// Return code:
//  value of test.
// References:
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//  Brownlee K.A. Statistical theory and metodology in sciences and engineering. - New York, NY: John Wiley & Sons, 1965.
//
{
 long i,           // Counter
      *b,          // Array of conditions
      *ix,         // Information about given arrays
      n = n1 + n2; // Common size

 double *xx,       // Merged array
        t = 1;     // Cumulative parameter value

 b = new long[n];
 ix = new long[n];
 xx = new double[n];

 //////////////////
 // Merger of array
 for(i = 0; i < n1; i++)
   xx[i] = x[i];
 for(i = 0; i < n2; i++)
   xx[i + n1] = y[i];

 //////////
 // Sort up
 SortArrayUp (xx,n,ix);

 /////////////////////////////////////
 // Value 1 - 1st array, 2 - 2nd array
 for(i = 0; i < n; i++)
   b[i] = ix[i] < n1 ? 1 : 2;

 ///////////////////////////////
 // Computation of series number
 for(i = 1; i < n; i++)
   if (b[i - 1] == b[i])
     t += 1.0;

 delete [] xx; delete [] ix; delete [] b;

 if (a2 == 1)
   *p = NormalDistribution (((t - 2.0 * n1 / n * n2 + 1) - 0.5) /
        sqrt(2.0 * n1 / n * n2 / n * (2.0 * n1 * n2 - n) / (n - 1)));
 else
   *p = NormalDistribution ((t - 2.0 * n1 / n * n2 + 1) /
        sqrt(2.0 * n1 / n * n2 / n * (2.0 * n1 * n2 - n) / (n - 1)));

 return t;
}

double MODIFICATOR RankAB (double x[],double xx[],long n)
//
// The function for array ranking of Ansari-Bredley test.
// Designations:
//   x - array of given data,
//  xx - array of ranks,
//   n - size of each arrays.
// Notice:
//  to get the correct result, the Array of given data must be sorted in ascending order.
// Return code:
//  correction to the merger of ranks.
// References:
//  Hollander M., Wolfe D.A., Chiken E. Nonparametric statistical methods. - New York, NY: Wley, 2014.
//  Johnson N.L., Leone F.C. Statistics and experimental design in engineering and the physical sciences. Vol. I. - New York, NY: John Wiley & Sons, 1977.
//
{
 long j,         // Counter
      is,        // Number equals elements
      ii,        // Work variable
      m = n / 2, // Variable for back reference
      flag = 0,  // Flag of bundles
      i = 0;     // Current rank

 double xi,      // Average rank equals elements
        xj,      // Sum of ranks equals elements
        *s,      // Array of ranks numbers
        r = 0;   // Bundle adjustment of ranks

 ////////////////////////////////////////////////
 // Computation arrays of ranks (without bundles)
 s = new double[n];
 for (j = 1; j <= n; j++)
   {
    if (n / 2 * 2 == n)
      {
       if (j <= n / 2)
         i++;
       else
         i = m--;
      }
    else
      {
       if (j <= n / 2 + 1)
         i++;
       else
         i = m--;
      }
     s[j - 1] = i;
   }

 /////////////////////////////////////////////
 // Computation arrays of ranks (with bundles)
 i = 1;
 do
   {
    ////////////////////////////////////////////
    // Counting numbers equals adjacent elements
    is = 1;
    for (j = i - 1 ; j < n - 1; j++)
      {
       if (x[j + 1] == x[j])
        is++;
       else
         break;
      }

    ////////////////////////////////////////////////
    // Next are the nevertheless two equals elements
    if (is > 1)
      {
       ii = i - 1;

       for (j = 1,xj = 0; j <= is; j++)
         xj += s[ii++];

       xi = xj / is;

       for (j = 1; j <= is ; j++)
         xx[i++ - 1] = xi;

       r += is * xi * xi;

       flag = 1;
      }

    ////////////////////////////////////
    // No equals members standing nearby
    else
      {
       r += s[i - 1] * s[i - 1];
       xx[i - 1] = s[i - 1];
       i++;
      }

   } while (i <= n);

 delete [] s;

 if (flag == 0)
   return 0;
 else
   return r;
}

void MODIFICATOR RankST (double x[],double xx[],long n)
//
// The function for ranking of Siegel-Tukey test.
// Designations:
//   x - array of given data,
//  xx - array of ranks,
//   n - size of each arrays.
// Notice:
//  to get the correct result, the array of given data must be
//  sorted in ascending order.
// Return code:
//  none.
//
{
 long j = 1,     // Counter
      m = n - 1, // Variable for back reference
      i = 0,     // Variable for direct reference
      is,        // Number of members in  bundles
      ii,        // Work variable
      *s;        // Array of ranks numbers

 double xi,      // Average rank equals elements
        xj;      // Sum of ranks equals elements

 ////////////////////////////////////////////////
 // Computation arrays of ranks (without bundles)
 s = new long[n];
 s[i++] = j++;
 do
   {
    s[m--] = j++;
    if (j > n)
      break;
    s[m--] = j++;
    if (j > n)
      break;
    s[i++] = j++;
    if (j > n)
      break;
    s[i++] = j++;
   } while (j <= n);

 /////////////////////////////////////////////
 // Computation arrays of ranks (with bundles)
 i = 1;
 do
   {
    ////////////////////////////////////////////
    // Counting numbers equals adjacent elements
    is = 1;
    for (j = i - 1 ; j < n - 1; j++)
      {
       if (x[j + 1] == x[j])
        is++;
       else
         break;
      }

    ////////////////////////////////////////////////
    // Next are the nevertheless two equals elements
    if (is > 1)
      {
       ii = i - 1;

       for (j = 1,xj = 0; j <= is; j++)
         xj += s[ii++];

       xi = xj / is;

       for (j = 1; j <= is ; j++)
         xx[i++ - 1] = xi;
      }

    ///////////////////////////////////////////////////////////////////
    // There are no equals standing next to members on the current step
    else
    {
      xx[i - 1] = s[i - 1];
      i++;
    }

   } while (i <= n);

 delete [] s;
}

double MODIFICATOR QuartileLow(double x[],long n)
//
// The function for computation of low quartile.
// Designations:
//    x - sample,
//    n - sample size,
// Return code:
//  value of low quartile.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//
{
 long i,          // Counter
      n1 = n / 2;

 double *w,       // Work array
        ret;      // Return code

 w = new double[n];

 ///////////////////
 // Work computation
 for (i = 0; i < n; i++)
   w[i] = x[i];
 SortArrayUp0 (w,n);

 ////////////////////////
 // Computation of median
 ret = Median(w,n1);

 delete [] w;

 return ret;
}

double MODIFICATOR QuartileUpp(double x[],long n)
//
// The function for computation of upper quartile.
// Designations:
//    x - sample,
//    n - sample size,
// Return code:
//  value of upper quartile.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//
{
 long i,          // Counter
      n1 = n / 2;

 double *w,       // Work array
        ret;      // Return code

 w = new double[n];

 ///////////////////
 // Work computation
 for (i = 0; i < n; i++)
   w[i] = x[i];
 SortArrayDown0 (w,n);

 ////////////////////////
 // Computation of median
 ret = Median(w,n1);

 delete [] w;

 return ret;
}

double Median(double x[],long n)
//
// The function for computation of median.
// Designations:
//    x - sample,
//    n - sample size,
// Return code:
//  value of median.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//
{
 long i;     // Counter

 double *w,  // Work array
        ret; // Return code

 w = new double[n];

 ///////////////////
 // Work computation
 for (i = 0; i < n; i++)
   w[i] = x[i];
 SortArrayUp0 (w,n);

 ////////////////////////
 // Computation of median
 if (!Parity (n))
   ret = w[n / 2];
 else
   ret = (w[n / 2 - 1] + w[n / 2]) / 2;

 delete [] w;

 return ret;
}

double MODIFICATOR Bernoulli (long n,double x)
//
// The function for computation of Bernoulli polynom.
// Designations:
//  n - degree of polynom,
//  x - value of variable.
// Return code:
//  value of Bernoulli polynom.
//
{
 long s; // Counter

 double sum = 0; // Work variable

 if (!n)
   return 1;
 else
   {
    for (s = 0; s < n; s++)
      sum += Combination (n + 1,s) * Bernoulli (s,x);

    return pow (x,n) - sum / (n + 1);
   }
}

double MODIFICATOR Laguerre (long a,long n,double x)
//
// The function for computation of generalized Laguerre polynom.
// Designations:
//  a - parameter,
//  n - degree of polynom,
//  x - value of variable.
// Return code:
//  value of Laguerre polynom.
//
{
 if (!n)
   return 1;
 else if (n == 1)
   {
    return x - (a + 1) / pow (x,a);
   }
 else
   return (x - a - 2 * (n - 1) - 1) * Laguerre (a,n - 1,x) -
      (n - 1) * (a + n - 1) * Laguerre (a,n - 2,x);
}

double MODIFICATOR Hermite (long n,double x)
//
// The function for computation of Hermite polynom.
// Designations:
//  n - degree of polynom,
//  x - value of variable.
// Return code:
//  value of Hermite polynom.
//
{
 if (!n)
   return 1;
 else if (n == 1)
   return 2 * x;
 else
   return 2 * x * Hermite (n - 1,x) - 2 * (n - 1) * Hermite (n - 2,x);
}

double MODIFICATOR Legendre (long n,double x)
//
// The function for computation of Legendre polymon.
// Designations:
//  n - degree of polymon,
//  x - value of variable.
// Return code:
//  value of Legendre polymon.
//
{
 if (!n)
   return 1;
 else if (n == 1)
   return x;
 else
   return ((2 * n - 1) * x * Legendre (n - 1,x) -
       (n - 1) * Legendre (n - 2, x)) / n;
}

double MODIFICATOR ChebyshevHermite (long n,double x)
//
// The function for computation of Chebyshev-Hermite polynom.
// Designations:
//  n - degree of polynom,
//  x - value of variable.
// Return code:
//  value of Chebyshev-Hermite polynom.
//
{
 if (!n)
   return 1;
 else if (n == 1)
   return - x;
 else
   return - x * ChebyshevHermite (n - 1,x) - (n - 1) * ChebyshevHermite (n - 2,x);
}

double Combination (long m,long n)
//
// The function return the number of combinations from m by n
//
{
 return exp (LogFactorial (m) - LogFactorial (n) - LogFactorial (m - n));
}

double MODIFICATOR LogFactorial (long n)
//
// The function return the logarithm of factorial n.
// Notice:
//  The function does not track the error n <= 0.
//
{
 int i;                  // Counter

 double Ret = log (1.0); // Return code

 for (i = 1; i <= n; i++)
   Ret += log (1.0 * i);

 return Ret;
}

long MODIFICATOR PoissonRegression (double y[],double x[],double w[],long m,long n,long *it,double d[])
//
// The function for building of Poisson regression.
// Designations:
//    y - vector of outcomes by length n,
//    x - matrix of regressors n x m (n m-dimensional vectors one by one),
//    w - vector of computed of coefficients by length m,
//    m - number of regressors,
//    n - number of experiments,
//  *it - acceptable number of iterations, when normal ending - spent,
//    d - variations of coefficients.
// Return code:
//   0 when normal result of computation,
//  -1 when lack of memory,
//  -2 when computation error,
//  -3 if the allowed number is exceeded of iterations.
// References:
//  Cameron A.C., Trivedi P.K. Regression analysis of count data. - Cambridge, NY: Cambridge University Press, 1998.
//  Davidson R., MacKinnon J.G. Estimation and inference in econometrics. - Oxford, NY: Oxford University Press, 1993.
//  Long J.S. Regression models for categorical and limited dependent variables. - Thousand Oaks, CA: SAGE Publications, 1997.
//
{
 long i,j,k,            // Cycle counters
      l,                // Counter of members arrays
      s,                // Counter of iterations
      Code = -3;        // Error code

 double *g,             // Gradient
        *h,             // Hessian, vector of increments
        *t,             // Work array, inverse Matrix
        p,              // Current outcome
        eps = 0.000001; // error rate

 if ((g = new double[m]) == 0)
   return -1;
 if ((h = new double[m * m]) == 0)
   {
    delete [] g;
    return -1;
   }
 if ((t = new double[m * m]) == 0)
   {
    delete [] g; delete [] h;
    return -1;
   }

 /////////////////
 // Initial values
 for (i = 0; i < m; i++)
   w[i] = 0;

 /////////////
 // Main cycle
 for (s = 1; s <= *it; s++)
   {
    for (i = 0; i < m; i++)
      g[i] = 0;
    for (i = 0; i < m * m; i++)
      h[i] = 0;
    for (i = 0; i < n; i++)
      {
       for (j = 0, l = i, p = 0; j < m; j++, l += n)
         {
          t[j] = x[l];
          p += w[j] * t[j];
         }
       p = exp(p);

       for (j = 0, l = 0; j < m; j++)
         {
          g[j] += (y[i] - p) * t[j];
          for (k = 0; k < m; k++)
            h[l++] -= t[j] * t[k] * p;
         }
      }

    /////////////////////////////
    //  Computation of increments
    if ((Code = InverseMatrix (h,t,m)) != 0)
      break;

    MatrixMultiplyMatrix (t,g,d,m,m,1);
    if (s > 1 && EuclidNormaVector(d,m) / EuclidNormaVector(w,m) <= eps)
      {
       Code = 0;
       *it = s;

       ////////////////////////////
       // Estimations of variations
       for (i = 0; i < m * m; i++)
         t[i] = -h[i];
       InverseMatrix (t,h,m);
       for (i = 0,l = 0; i < m; i++, l += m + 1)
         d[i] = h[l];

       break;
      }

    ////////////////////////
    // The new weight vector
    for (j = 0; j < m; j++)
     w[j] -= d[j];
   }

 delete [] g; delete [] h; delete [] t;

 return Code;
}

long MODIFICATOR LogisticRegression (double y[],double x[],double w[],long m,long n,long *it,double *d)
//
// The function for building of logistic regression.
// Designations:
//    y - vector of outcomes by length n,
//    x - matrix of regressors n x m (n m-dimensional vectors one by one),
//    w - vector of computed of coefficients by length m,
//    m - number of regressors,
//    n - number of experiments,
//  *it - acceptable number of iterations, when normal ending - spent,
//    d - variations of coefficients.
// Return code:
//   0 when normal result of computation,
//  -1 when lack of memory,
//  -2 when computation error,
//  -3 if the allowed number is exceeded of iterations.
// References:
//  Hosmer D.W., Lemeshow S. Applied logistic regression. - New York, NY: John Wiley & Sons, 2000.
//
{
 long i,j,k,            // Cycle counters
      l,                // Counter of members arrays
       s,               // Counter of iterations
       Code = -3;       // Error code

 double *g,             // Gradient
        *h,             // Hessian, vector of increments
        *t,             // Work array
        p,              // Current outcome
        eps = 0.000001; // error rate

 if ((g = new double[m]) == 0)
   return -1;
 if ((h = new double[m * m]) == 0)
   {
    delete [] g;
    return -1;
   }
 if ((t = new double[m * m]) == 0)
   {
    delete [] g; delete [] h;
    return -1;
   }

 /////////////////
 // Initial values
 for (i = 0; i < m; i++)
   w[i] = 0;

 /////////////
 // Main cycle
 for (s = 1; s <= *it; s++)
   {
    for (i = 0; i < m; i++)
      g[i] = 0;
    for (i = 0; i < m * m; i++)
      h[i] = 0;
    for (i = 0; i < n; i++)
      {
       for (j = 0, l = i, p = 0; j < m; j++, l += n)
         {
          t[j] = x[l];
          p += w[j] * t[j];
         }
       p =  Logit(p);

       for (j = 0, l = 0; j < m; j++)
         {
          g[j] += (y[i] - p) * t[j];
          for (k = 0; k < m; k++)
            h[l++] -= t[j] * t[k] * p * (1 - p);
         }
      }

    /////////////////////////////
    //  Computation of increments
    if ((Code = InverseMatrix (h,t,m)) != 0)
      break;

    MatrixMultiplyMatrix (t,g,d,m,m,1);
    if (s > 1 && EuclidNormaVector(d,m) / EuclidNormaVector(w,m) <= eps)
      {
       Code = 0;
       *it = s;

       ////////////////////////////
       // Estimations of variations
       for (i = 0; i < m * m; i++)
         t[i] = -h[i];
       InverseMatrix (t,h,m);
       for (i = 0,l = 0; i < m; i++, l += m + 1)
         d[i] = h[l];

       break;
      }

    ////////////////////////
    // The new weight vector
    for (j = 0; j < m; j++)
     w[j] -= d[j];
   }

 delete [] g; delete [] h; delete [] t;

 return Code;
}

double MODIFICATOR Logit (double y)
//
// The function for computation of logit activation.
// Designations:
//    y - logit.
// Return code:
//  The function of activation logit.
//
{
 return 1.0 / (1.0 + exp(-y));
}

long MODIFICATOR ProbitRegression (double y[],double x[],double w[],long m,long n,long *it,double *d)
//
// The function for building of probit regression.
// Designations:
//    y - vector of outcomes by length n,
//    x - matrix of regressors n x m (n m-dimensional vectors one by one),
//    w - vector of computed of coefficients by length m,
//    m - number of regressors,
//    n - number of experiments,
//  *it - acceptable number of iterations, when normal ending - spent,
//    d - variations of coefficients.
// Return code:
//   0 when normal result of computation,
//  -1 when lack of memory,
//  -2 when computation error,
//  -3 if the allowed number is exceeded of iterations.
// References:
//  Cameron A.C., Trivedi P.K. Regression analysis of count data. - Cambridge, NY: Cambridge University Press, 1998.
//
{
 long i,j,k,            // Cycle counters
      l,                // Counter of members arrays
      s,                // Counter of iterations
      Code = -3;        // Error code

 double *g,             // Gradient
        *h,             // Hessian, vector of increments
        *t,             // Work array, inverse Matrix
        v,p1,           // Work variable
        p,              // Current outcome
        eps = 0.000001; // Error rate

 if ((g = new double[m]) == 0)
   return -1;
 if ((h = new double[m * m]) == 0)
   {
    delete [] g;
    return -1;
   }
 if ((t = new double[m * m]) == 0)
   {
    delete [] g; delete [] h;
    return -1;
   }

 /////////////////
 // Initial values
 for (i = 0; i < m; i++)
   w[i] = 0;

 /////////////
 // Main cycle
 for (s = 1; s <= *it; s++)
   {
    for (i = 0; i < m; i++)
      g[i] = 0;
    for (i = 0; i < m * m; i++)
      h[i] = 0;
    for (i = 0; i < n; i++)
      {
       for (j = 0, l = i, p1 = 0; j < m; j++, l += n)
         {
          t[j] = x[l];
          p1 += w[j] * t[j];
         }
       p = Probit(p1);

       v = 0.398942280401432678 / exp (p * p / 2) * (y[i] - p) / p / (1.0 - p);

       for (j = 0, l = 0; j < m; j++)
         {
          g[j] += v * t[j];
          for (k = 0; k < m; k++)
            h[l++] -= v * (v + p1) * t[j] * t[k];
         }
      }

    /////////////////////////////
    //  Computation of increments
    if ((Code = InverseMatrix (h,t,m)) != 0)
      break;

    MatrixMultiplyMatrix (t,g,d,m,m,1);
    if (s > 1 && EuclidNormaVector(d,m) / EuclidNormaVector(w,m) <= eps)
      {
       Code = 0;
       *it = s;

       ////////////////////////////
       // Estimations of variations
       for (i = 0; i < m * m; i++)
         t[i] = -h[i];
       InverseMatrix (t,h,m);
       for (i = 0,l = 0; i < m; i++, l += m + 1)
         d[i] = h[l];

       break;
      }

    ////////////////////////
    // The new weight vector
    for (j = 0; j < m; j++)
     w[j] -= d[j];
   }

 delete [] g; delete [] h; delete [] t;

 return Code;
}

double MODIFICATOR Probit (double y)
//
// The function for computation of probit activation.
// Designations:
//    y - probit.
// Return code:
//  The function of activation probit.
//
{
 return NormalDistribution(y);
}

long MODIFICATOR FDA (double z[],long n,long k,long l[],long g,long h[],double p[])
//
// The function builds linear discriminant analysis by Fisher.
// Designations:
//   z - matrix of given data size n * k (k n-dimensional vectors one by one),
//   n - common number of objects, include training sample,
//   k - number of parameters,
//   l - array by length g size of k-dimensional samples (number of objects)
//       in each training classes,
//   g - number of training groups, aka number of discriminant functions,
//   h - vector containing the numbers of the objects that make up the
//       training classes, the numbers are arranged according to l one by one,
//   p - array by length g * (k + 1) of coefficients g of discriminant
//       (simple classifying) functions arranged one by one,
//       starting with free member.
// Notice:
//  Suppose, each element of arrays h no less 1 and does not exceed n.
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory,
//  -2 when loss of significance.
// References:
//  Afifi F.F., Azen S.P. Statistical analysis. A computer oriented approach. - New York, NY: Academic Press, 1979.
//
{
 long i,j,i1,          // Cycle counters
      m1,m2,m3,m4,m5,  // Index
      Code;            // Error code

 double *x,            // Matrix group medium
        *t,*d,         // Work array
        *s;            // Current groups variance matrix

 if ((d = new double[k * k]) == 0)
   return -1;
 if ((s = new double[k * k]) == 0)
   {
    delete [] d;
    return -1;
   }
 if ((x = new double[k * g]) == 0)
   {
    delete [] s; delete [] d;
    return -1;
   }

 ////////////////////////////////////////
 // Initial values united variance matrix
 FillUp (d,k * k,0);

 //////////////////////////////////
 // Building united variance matrix
 // and of matrix group medium
 for (i = 0, m3 = 0, m5 = 0; i < g; i++)
   {
    //////////////////////////////
    // Groups matrix of given data
    if ((t = new double[l[i] * k]) == 0)
      {
       delete [] s; delete [] d; delete [] x;
       return -1;
      }
    for (i1 = 0, m1 = 0, m4 = 0; i1 < k; i1++, m4 += n)
      for (j = 0, m2 = m3; j < l[i]; j++)
        t[m1++] = z[h[m2++] + m4 - 1];
    m3 += l[i];

    //////////////////////
    // Matrix group medium
    for (i1 = 0, m2 = 0; i1 < k; i1++)
      {
       for (j = 0, x[m5] = 0; j < l[i]; j++)
         x[m5] += t[m2++];
       x[m5++] /= l[i];
      }

    /////////////////////////////////////
    // Groups variance-covariation matrix
    if (Likeness (t,s,l[i],k,1) != 0)
      {
       delete [] s; delete [] d; delete [] x; delete [] t;
       return -1;
      }

    ///////////////////////////////////////
    // Groups matrix of mutual compositions
    // deviations of medium
    MatrixMultiplyNumber (s,k * k,l[i] - 1);

    ///////////////////////////////////////
    // United matrix of mutual compositions
    // deviations of medium
    MatrixPlusMatrix (d,s,k * k);

    delete [] t;
   }

 delete [] s;

 /////////////////////////////////////
 // United variance-covariation matrix
 MatrixMultiplyNumber (d,k * k,1.0 / (m3 - g));

 ///////////////////////////////////////
 // Matrix, inverse variance-covariation
 if ((t = new double[k * k]) == 0)
   {
    delete [] d; delete [] x;
    return -1;
   }

 if ((Code = InverseMatrix (d,t,k)) != 0)
   {
    delete [] d; delete [] x; delete [] t;
    return Code;
   }

 delete [] d;

 //////////////////////////
 // Vector of common medium
 if ((d = new double[k]) == 0)
   {
    delete [] x; delete [] t;
    return -1;
   }
 for (j = 0; j < k; j++)
   {
    for (i1 = 0, m2 = j, d[j] = 0; i1 < g; i1++, m2 += k)
      d[j] += l[i1] * x[m2];
    d[j] /= m3;
   }

 //////////////////////////////////////////////////////////////
 // Computation of coefficients of simple classifying functions
 for (i1 = 0, m1 = 0; i1 < g; i1++)
   {
    for (j = 0, p[m1] = 0; j < k; j++)
      for (i = 0; i < k; i++)
        p[m1] += t[j + i * k] * x[j + i1 * k] * x[i + i1 * k];
    p[m1++] /= (-2);

    for (i = 0; i < k; i++, m1++)
      for (j = 0, p[m1] = 0; j < k; j++)
        p[m1] += t[i + j * k] * x[j + i1 * k];
   }

 delete [] d; delete [] x; delete [] t;

 return 0;
}

long MODIFICATOR CDA (double z[],long n,long k,long l[],long g,long h[],double p[],double c[])
//
// The function builds the canonical discriminant analysis.
// Designations:
//   z - matrix of given data size n * k (k n-dimensional vectors one by one),
//   n - common number of objects, include the self and training sample,
//   k - number of parameters  (number of canonical function),
//   l - array by length g size of k-dimensional samples (number of objects) in each of training classes,
//   g - number  training of classes,
//   h - vector of objects numbers from training classes,
//       numbers located in accordance with l one by one,
//   p - array by length (k + 1) * k of coefficients of canonical
//       discrimanant functions, located one by one, beginning with free member,
//   c - array  training centroids of classes size k * g.
// Notice:
//   Assumed that each element of array h is at least 1 and does not exceed n.
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory,
//  -2 loss of significance.
// References:
//  Klecka W.R. Discriminant analysis. - Beverly Hills, CA: SAGE Publications, 1986.
//
{
 long i,j,i1,       // Cycle counters
       m1,m2,m4,m5, // Index
       m3,          // Common number of training samples (objects)
       Code;        // Work variable

 double *t,*d,      // Work array
        *x,         // Vector common medium
        buf,        // Work variable
        *u,         // Vector own values
        *r,         // Common correlation Matrix
        *w;         // insidegroups correlation Matrix

 if ((d = new double[k * k]) == 0)
   return -1;
 if ((r = new double[k * k]) == 0)
   {
    delete [] d;
    return -1;
   }
 if ((w = new double[k * k]) == 0)
   {
    delete [] d; delete [] r;
    return -1;
   }
 if ((u = new double[k]) == 0)
   {
    delete [] d; delete [] r; delete [] w;
    return -1;
   }

 /////////////////
 // Initial values
 FillUp (d,k * k,0);

 for (i = 0, m3 = 0, m5 = 0; i < g; i++)
   {
    //////////////////////////////
    // Groups matrix of given data
    if ((t = new double[l[i] * k]) == 0)
      {
       delete [] d; delete [] u; delete [] r; delete [] w;
       return -1;
      }
    for (i1 = 0, m1 = 0, m4 = 0; i1 < k; i1++, m4 += n)
      for (j = 0, m2 = m3; j < l[i]; j++)
        t[m1++] = z[h[m2++] + m4 - 1];
    m3 += l[i];

    ////////////////////////////////////
    // Array medium  training of classes
    for (j = 0, m1 = 0; j < k; j++, m5++)
      {
       for (i1 = 0, c[m5] = 0; i1 < l[i]; i1++)
         c[m5] += t[m1++];
       c[m5] /= l[i];
      }

    ////////////////////////////////////////////
    // Inside groups variance-covariation matrix
    if (Likeness (t,p,l[i],k,1) != 0)
      {
       delete [] d; delete [] t; delete [] u; delete [] r; delete [] w;
       return -1;
      }

    ///////////////////////////////////////
    // Groups Matrix of mutual compositions
    // of deviations from medium
    MatrixMultiplyNumber (p,k * k,l[i] - 1);

    ///////////////////////////////////////
    // Inside groups Matrix of mutual compositions
    // of deviations from medium
    MatrixPlusMatrix (d,p,k * k);

    delete [] t;
   }

 //////////////////////////////////////
 // Inside groups of correlation matrix
 for (i = 0, m1 = 0, m2 = 0; i < k; i++, m2 += k + 1)
   for (j = 0, m4 = 0; j < k; j++, m1++, m4 += k + 1)
     w[m1] = d[m1] / sqrt (d[m2] * d[m4]);

 ////////////////////////////////
 // Common matrix of observations
 if ((t = new double[m3 * k]) == 0)
   {
    delete [] d; delete [] u; delete [] r; delete [] w;
    return -1;
   }
 for (j = 0; j < m3; j++)
   for (i = 0, m1 = j, m2 = h[j] - 1; i < k; i++, m1 += m3, m2 += n)
     t[m1] = z[m2];

 ///////////////////////////
 // Vector of common mediums
 if ((x = new double[k]) == 0)
   {
    delete [] d; delete [] t; delete [] u; delete [] r; delete [] w;
    return -1;
   }
 for (j = 0; j < k; j++)
   {
    for (i = 0, m1 = j, x[j] = 0; i < g; i++, m1 += k)
      x[j] += c[m1];
    x[j] /= g;
   }

 /////////////////////////////////////
 // Common variance-covariation matrix
 if (Likeness (t,p,m3,k,1) == -1)
   {
    delete [] d; delete [] t; delete [] x; delete [] u; delete [] r; delete [] w;
    return -1;
   }

 ////////////////////////////
 // Common correlation matrix
 if (Likeness (t,r,m3,k,0) == -1)
   {
    delete [] d; delete [] t; delete [] x; delete [] u; delete [] r; delete [] w;
    return -1;
   }

 ///////////////////////////////////////////////////////////////
 // Common matrix of mutual compositions of deviations of medium
 MatrixMultiplyNumber (p,k * k,m3 - 1);

 ////////////////////////////////////////////////////
 // Inside groups of matrix sums of squares and pairs
 // of compositions
 MatrixMinusMatrix (p,d,k * k);

 ////////////////////////////////////
 // The solving of own values problem
 if ((Code = Eigen (p,d,k,0.000001,&m1)) != 0)
   {
    delete [] d; delete [] t; delete [] x; delete [] u; delete [] r; delete [] w;
    return Code;
   }

 /////////////////////////////////
 // Sort up own values and vectors
 for (i = 0; i < k; i++)
   u[i] = p[i * (k + 1)];

 for (i = 0; i < k; i++)
   {
    Code = i;
    buf = u[i];
    for (j = i + 1; j < k; j++)
      if (buf < u[j])
        {
         Code = j;
         buf = u[j];
        }
    ColInterchange (u,1,i,Code);
    ColInterchange (d,k,i,Code);
   }

 /////////////////////////////////////////////////////
 // Computation of coefficients of canonical functions
 ArrayToArray (d,t,k * k);
 MatrixMultiplyNumber (t,k * k,sqrt (m3 - g));
 for (i = 0, m2 = 0, m4 = 0; i < k; i++, m4 += k + 1)
   for (j = 0, m1 = i * (k + 1) + 1, p[m4] = 0; j < k; j++, m2++)
     {
      p[m1++] = t[m2];
      p[m4] -= t[m2] * x[j];
     }

 delete [] x; delete [] t; delete [] d;

 /////////////////////
 // Array of centroids
 if ((t = new double[k * g]) == 0)
   return -1;
 ArrayToArray (c,t,k * g);
 for (i = 0, m1 = 0; i < g; i++)
   for (j = 0, m4 = 0; j < k; j++, m1++)
     {
      c[m1] = p[m4++];
      for (i1 = 0, m2 = i * k; i1 < k; i1++)
        c[m1] += p[m4++] * t[m2++];
     }

 delete [] t; delete [] u; delete [] r; delete [] w;

 return 0;
}

long MODIFICATOR LDA (double z[],long n,long k,long l[],long g,long h[],double p[],double x[],double v[])
//
// The function builds the linead discriminant analysis.
// Designations:
//   z - matrix of given data size n * k (k n-dimensional vectors one by one),
//   n - common number of objects, include the self and training sample,
//   k - number of parameters,
//   l - array by length g size of k-dimensional samples (number of objects) in each of training classes,
//   g - number  training of classes,
//   h - vector of objects numbers from training classes,
//       numbers located in accordance with l one by one,
//   p - array by length g * k * k covariation matrix, located one by one,
//   x - array by length g * k medium,
//   v - array by length g of square roots from determinants of covariation matrices.
// Notice:
//   Assumed that each element of array h is at least 1 and does not exceed n.
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory,
//  -2 loss of significance.
// References:
//  Klecka W.R. Discriminant analysis. - Beverly Hills, CA: SAGE Publications, 1986.
//
{
 long i,j,i1,      // Cycle counters
      m1,m2,m3,m4, // Index
      m5 = 0,      // Counter arrays of covariation matrices
      m6 = 0;      // Counter arrays medium

 double *t,        // groups matrix of given data
        *s;        // groups variance Matrix

 if ((s = new double[k * k]) == 0)
   return -1;

 ///////////////////////////////
 // Building variance- of matrix
 for (i = 0, m3 = 0; i < g; i++)
   {
    if ((t = new double[l[i] * k]) == 0)
      {
       delete [] s;
       return -1;
      }
    for (i1 = 0, m1 = 0, m4 = 0; i1 < k; i1++, m4 += n)
      for (j = 0, m2 = m3; j < l[i]; j++)
        t[m1++] = z[h[m2++] + m4 - 1];
    m3 += l[i];

    //////////////////////
    // Matrix group medium
    for (i1 = 0, m2 = 0; i1 < k; i1++)
      {
       for (j = 0, x[m6] = 0; j < l[i]; j++)
         x[m6] += t[m2++];
       x[m6++] /= l[i];
      }

    /////////////////////////////////////
    // Groups variance-covariation matrix
    if (Likeness (t,s,l[i],k,1) != 0)
      {
       delete [] s; delete [] t;
       return -1;
      }
    for (i1 = 0, m1 = 0; i1 < k; i1++)
      for (j = 0; j < k; j++)
        p[m5++] = s[m1++];

    v[i] = Krauth (s,k,0.000000001);

    delete [] t;

    /////////////////////
    // Significance check
    if (v[i] <= 0.000001)
      {
       delete [] s;
       return -2;
      }

    v[i] = sqrt (v[i]);
   }

 delete [] s;

 return 0;
}

double MODIFICATOR EuclidNormaWithoutDiag (double a[],long n)
//
// The function for computation Euclid norma of matrix without accounting
// for diagonal elements.
// Designations:
//  a - given matrix,
//  n - order of matrix.
// Return code:
//  value of norma.
//
{
 long i,j;

 long m = 0;

 double s = 0;

 for (i = 0; i < n; i++)
   for (j = 0; j < n; j++, m++)
     if (i != j)
       s += a[m] * a[m];

 return sqrt (s);
}

double MODIFICATOR EuclidNorma (double a[],long n)
//
// The function for computation Euclid norma of matrix .
// Designations:
//  a - given matrix,
//  n - order of matrix.
// Return code:
//  value of norma.
//
{
 long i,j,
       m = 0;

 double s = 0;

 for (i = 0; i < n; i++)
   for (j = 0; j < n; j++, m++)
     s += a[m] * a[m];

 return sqrt (s);
}

double MODIFICATOR Euclid (double x[],double y[],long n)
//
// The function computes the Euclid distance.
// Designations:
//  x - 1st compared sample,
//  y - 2nd compared sample,
//  n - each sample size.
// Return code:
//  value of metric.
//
{
 long i;

 double s = 0;

 for (i = 0; i < n; i++)
   s += (x[i] - y[i]) * (x[i] - y[i]);

 return sqrt (s);
}

long Likeness (double data[],double z[],long n,long k,long Mode)
//
// The function builds likeness matrix by type of correlation.
// Designations:
//  data - array of given data (k vectors one by one).
//     z - correlation or covariation matrix size k * k.
//     n - size of each from k vectors.
//  Mode - mode of likeness:
//       = 0 - Pearson coefficient,
//       = 1 - variance-covariance.
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory.
//
{
 long i,j,l,         // Cycle counters
      m,m1,m2;       // Index

 double criterion,   // Work variables
        *dat1,*dat2; // Copies vectors of data

 if ((dat1 = new double[n]) == 0)
   return -1;
 if ((dat2 = new double[n]) == 0)
   {
    delete [] dat1;
    return -1;
   }

 //////////////////////////////
 // Building of likeness matrix
 for (i = 0, m = 0, m2 = 0; i < k; i++)
   {
    for (l = 0; l < n; l++)
      dat1[l] = data[m++];

    for (j = 0, m1 = 0; j < k; j++)
      {
       for (l = 0; l < n; l++)
         dat2[l] = data[m1++];
       switch (Mode)
         {
          case 0:
            criterion = Pearson (dat1,dat2,n);
            break;
          default:
            criterion = Cov (dat1,dat2,n);
         }
       z[m2++] = criterion;
      }
   }

 delete [] dat1; delete [] dat2;

 return 0;
}

long MODIFICATOR FDAClass (double x[],double w[],long k,long g,long *c,double *p)
//
// The function makes object-s classification by results of FDA.
// Designations:
//   x - vector of given data,
//   w - array of simple classification functions,
//   k - number of parameters,
//   g - number of classes,
//  *c - number of class,
//  *p - corresponding probability.
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory.
//
{
 long i,j,         // Counter
      s = 0;       // Index of element arrays

 double Value = 0, // Value of discriminant function
        *d;        // Vector of discriminant functions of object

 if ((d = new double[g]) == 0)
   return -1;

 ////////////////////////////
 // Computation class numbers
 for (i = 0, *c = 0; i < g; i++)
   {
    d[i] = w[s++];
    for (j = 0; j < k; j++)
     d[i] += w[s++] * x[j];
    if (d[i] > Value)
      {
       Value = d[i];
       *c = i + 1;
      }
   }

 ////////////////////////////////////////////////////////////////
 // Calculating the probability of an object belonging to a class
 for (i = 0, *p = 0; i < g; i++)
   *p = (*p) + exp(d[i] - Value);

 *p = 1.0 / (*p);

 delete [] d;

 return 0;
}

long MODIFICATOR CDAClass (double x[],double w[],double u[],long k,long g,long *c)
//
// The function makes object-s classification by results of CDA.
// Designations:
//   x - vector of given data,
//   w - array of centroids by length g * k,
//   u - array by length k * (k + 1) of coefficients k of discriminant functions,
//   k - number of parameters,
//   g - number of classes,
//  *c - number of class.
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory.
//
{
 long i,j,    // Cycle counters
      m1,m2,  // Array counter
      y = k;  // Number of discriminant functions, <= k

 double r,r1, // The distance between object and cluster
        *s1,  // Coordinates of object
        *s2;  // Coordinates of cluster center

 if ((s1 = new double[y]) == 0)
   return -1;
 if ((s2 = new double[y]) == 0)
   {
    delete [] s1;
    return -1;
   }

 ////////////////////////
 // Coordinates of object
 for (j = 0, m1 = 0; j < y; j++)
   {
    s1[j] = u[m1++];
    for (i = 0; i < k; i++)
      s1[j] += u[m1++] * x[i];
   }

 ////////////////////////////////
 // Coordinates of cluster center
 for (j = 0, m2 = 0; j < y; j++)
   s2[j] = w[m2++];

 ////////////////////////////////
 // Skip discriminant of function,
 // corresponding to the discarded own values
 for (j = y; j < k; j++)
   m2++;

 /////////////////
 // Initial values
 r = Euclid (s1,s2,y);
 *c = 1;

 ///////////////////////////////
 // Checking all cluster centers
 for (j = 1; j < g; j++)
   {
    for (i = 0; i < y; i++)
      s2[i] = w[m2++];
    for (i = y; i < k; i++)
      m2++;
    r1 = Euclid (s1,s2,y);
    if (r1 < r)
      {
       r = r1;
       *c = j + 1;
      }
   }

 delete [] s1; delete [] s2;

 return 0;
}

long MODIFICATOR LDAClass (double x[],double u[],double d[],double v[],long k,long g,long *c,double *p)
//
// The function makes object-s classification by results of LDA.
// Designations:
//   x - vector of given data,
//   u - array by length g * k * k of covariation matrices (g metrices by size k * k),
//   d - array by length k * g of mediums,
//   v - array by length g of square roots from determinants of covariation matrices.
//   k - number of parameters,
//   g - number of classes,
//  *c - number of class,
//  *p - corresponding probability.
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory,
//  -2 when computation error.
//
{
 long i,j,        // Cycle counters
       Code,      // Error code
       buf,       // Number of maximal element
       m1,        // Counter medium
       m2;        // Counter of covariation matrix

 double buf1,     // Work variable
        *s,       // Work array
        *u1,      // Groups covariation matrix
        *u2,      // Matrix, inverse to covariation matrix
        *z;       // Work array

 if ((s = new double[g]) == 0)
   return -1;
 if ((u1 = new double[k * k]) == 0)
   {
    delete [] s;
    return -1;
   }
 if ((u2 = new double[k * k]) == 0)
   {
    delete [] s; delete [] u1;
    return -1;
   }
 if ((z = new double[k]) == 0)
   {
    delete [] s; delete [] u1; delete [] u2;
    return -1;
   }

 ///////////////////////////////////////////
 // computation of arrays dencity for object
 for (i = 0, m1 = 0, m2 = 0; i < g; i++)
   {
    /////////////////////////
    // Covariation matrix
    for (j = 0; j < k * k; j++)
      u1[j] = u[m1++];

    ////////////////////////////////////////
    // Matrix, inverse to covariation matrix
    if ((Code = InverseMatrix (u1,u2,k)) != 0)
      {
       delete [] s; delete [] u1; delete [] u2; delete [] z;
       return Code;
      }

    ////////////////
    // Vector medium
    for (j = 0; j < k; j++)
      z[j] = x[j] - d[m2++];

    MatrixMultiplyMatrix (u2,z,u1,k,k,1);
    MatrixTMultiplyMatrix (z,u1,u2,1,k,1);

    s[i] = exp (- 0.5 * u2[0]) / v[i];
   }

 buf1 = MaximumOf (s,g,&buf);
 *c = buf + 1;
 *p = buf1 / VectorSum (s,g);

 delete [] s; delete [] u1; delete [] u2; delete [] z;

 return 0;
}

long MODIFICATOR LinearRegression (double y[],double a[],double b[],long k,long n,double c[],double h[],double v[])
//
// The function for building of multiple linear regression model.
// Designations:
//  y - vector of free members by length n,
//  a - matrix of system by size n x k,
//  b - vector of computed of coefficients by length k,
//  k - number of parameters,
//  n - number of experiments,
//  c - variance-covariation matrix by size k x k,
//  h - vector of conditional correlations by size n,
//  v - error vector of coefficients by length k.
// Return code:
//   0 when normal result of computation,
//  -1 when lack of memory,
//  -2 when loss of significance.
// Notice:
//  k <= n.
// References:
//  Brandt S. Data analysis. Statistical and computational methods for scientists and engineers. - New York, NY: Springer-Verlag, 2003.
//  Montgomery D.C. Design and analysis of experiments. - New York: John Wiley & Sons, 2017.
//
{
 long i,     // Cycle counter
      s,     // Counter
      Code;  // Error code

 double *zz, // Work array
        *bb, // Work array
        *hh; // Work array

 if ((zz = new double[n * n]) == 0)
   return -1;
 if ((bb = new double[k * n]) == 0)
   {
    delete [] zz;
    return -1;
   }
 if ((hh = new double[n * n]) == 0)
   {
    delete [] zz; delete [] bb;
    return -1;
   }

 MatrixTMultiplyMatrix (a,a,zz,k,n,k);
 if ((Code = InverseMatrix (zz,c,k)) == 0)
   {
    for (i = 0, s = 0; i < k; i++, s += k + 1)
      v[i] = sqrt(c[s]);
    MatrixMultiplyMatrixT (c,a,bb,k,k,n);
    MatrixMultiplyMatrix (bb,y,b,k,n,1);
    MatrixMultiplyMatrix (a,bb,hh,n,k,n);
    for (i = 0, s = 0; i < n; i++, s += n + 1)
      h[i] = hh[s];
   }

 delete [] zz; delete [] bb; delete [] hh;

 return Code;
}

long MODIFICATOR DurbinWatsonBetaPars (double x[],long k,long n,double *ed,double *dd)
//
// The function for computation of Durbin-Watson statistics.
// Designations:
//    x - matrix of system by size n x k,
//    k - number of parameters,
//    n - number of experiments,
//  *ed - mean of Durbin-Watson statistics,
//  *dd - variance of Durbin-Watson statistics.
// Return code:
//   0 when normal result of computation,
//  -1 when lack of memory,
//  -2 loss of significance.
//
{
 long i,     // Cycle counter
      s,     // Counter
      Code;  // Error code

 double *a,  // Array A
        *a2, // Work array
        *b,  // Work array
        *c,  // Work array
        *d,  // Work array
        *e,  // Work array
        *f,  // Work array
        *g,  // Work array
        *h,  // Work array
        *r,  // Work array
        p,q; // Parameters

 if ((a = new double[n * n]) == 0)
   return -1;
 if ((b = new double[n * k]) == 0)
   {
    delete [] a;
    return -1;
   }
 if ((c = new double[n * k]) == 0)
   {
    delete [] a; delete [] b;
    return -1;
   }
 if ((d = new double[k * k]) == 0)
   {
    delete [] a; delete [] b; delete [] c;
    return -1;
   }
 if ((e = new double[k * k]) == 0)
   {
    delete [] a; delete [] b; delete [] c; delete [] d;
    return -1;
   }
 if ((f = new double[n * k]) == 0)
   {
    delete [] a; delete [] b; delete [] c; delete [] d; delete [] e;
    return -1;
   }
 if ((g = new double[k * k]) == 0)
   {
    delete [] a; delete [] b; delete [] c; delete [] d; delete [] e;
    delete [] f;
    return -1;
   }
 if ((h = new double[k * k]) == 0)
   {
    delete [] a; delete [] b; delete [] c; delete [] d; delete [] e;
    delete [] f; delete [] g;
    return -1;
   }
 if ((r = new double[k * k]) == 0)
   {
    delete [] a; delete [] b; delete [] c; delete [] d; delete [] e;
    delete [] f; delete [] g; delete [] h;
    return -1;
   }
 if ((a2 = new double[n * n]) == 0)
   {
    delete [] a; delete [] b; delete [] c; delete [] d; delete [] e;
    delete [] f; delete [] g; delete [] h; delete [] r;
    return -1;
   }

 //////////////////////
 // Forming of matrix A
 for (i = 0; i < n * n; i++)
   a[i] = 0;
 a[0] = 1;
 if (n > 1)
   {
    a[1] = -1;
    if (n == 2)
      {
       a[2] = -1;
       a[3] = 1;
      }
    else
      {
       s = n;
       for (i = 1; i < n - 1; i++, s += n + 1)
         {
          a[s] = -1;
          a[s + 1] = 2;
          a[s + 2] = -1;
         }
       a[n * n - 2] = -1;
       a[n * n - 1] = 1;
      }
   }

 MatrixTMultiplyMatrix (x,x,h,k,n,k);
 if ((Code = InverseMatrix (h,r,k)) != 0)
   {
    delete [] a; delete [] b; delete [] c; delete [] d; delete [] e;
    delete [] f; delete [] g; delete [] h; delete [] r; delete [] a2;
    return Code;
   }
 MatrixMultiplyMatrix (x,r,b,n,k,k);
 MatrixMultiplyMatrix (a,b,c,n,n,k);
 MatrixTMultiplyMatrix (x,c,d,k,n,k);
 MatrixMultiplyMatrix (d,d,e,k,k,k);
 MatrixMultiplyMatrix (a,c,f,n,n,k);
 MatrixTMultiplyMatrix (x,f,g,k,n,k);
 MatrixMultiplyMatrix (a,a,a2,n,n,n);

 p = Trace (a,n) - Trace (d,k);
 q = Trace (a2,n) - 2 * Trace (g,k) + Trace (e,k);

 *ed = p / (n - k - 1);
 *dd = 2 * (q - p * (*ed)) / (n - k - 1) / (n - k + 1);

 delete [] a; delete [] b; delete [] c; delete [] d; delete [] e;
 delete [] f; delete [] g; delete [] h; delete [] r; delete [] a2;

 return 0;
}

double MODIFICATOR Trace (double* a,long n)
//
// The function for computation of trace of square matrix.
// Designations:
//  a - square matrix,
//  n - order of matrix.
// Return code:
//  value of trace of matrix.
//
{
 long i,j = 0; // Counters

 double s = 0; // Return code

 for (i = 0; i < n; i++, j += n + 1)
   s += a[j];

 return s;
}

double MODIFICATOR CochranCoxTest(double *x1,long n1,double *x2,long n2,double *p)
//
// The function for computation of Cochran-Cox test.
// Designations:
//  x1 - 1st sample,
//  n1 - size of 1st sample,
//  x2 - 2nd sample,
//  n2 - size of 2nd sample,
//  *p - P-value of test.
// Return code:
//  value of test.
// References:
//  Cochran W.G., --- G.M. Experimental designs. - New York, NY: Willey, 1957.
//
{
 long i;               // Cycle counter

 double s1 = 0,s2 = 0, // Variations, assigned to (size - 1)
        m1 = 0,m2 = 0, // Mediums
        w;             // Test statistic

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n1; i++)
   {
    m1 += x1[i];
    s1 += x1[i] * x1[i];
   }
 m1 /= n1;
 s1 = (s1 - n1 * m1 * m1) / (n1 - 1) / (n1 - 1);
 for (i = 0; i < n2; i++)
   {
    m2 += x2[i];
    s2 += x2[i] * x2[i];
   }
 m2 /= n2;
 s2 = (s2 - n2 * m2 * m2) / (n2 - 1) / (n2 - 1);

 //////////////////////////////////////////////
 // Control of the admissibility of estimations
 if (s1 + s2 == 0)
   {
    *p = 0.5;
    return 0;
   }
 w = (m1 - m2) / sqrt (s1 + s2);

 //////////////////////
 // P-value computation
 *p = TDistribution((s1 + s2) * (s1 + s2) / (s1 * s1 / (n1 + 1) + s2 * s2 / (n2 + 1)) - 2,w);

 return w;
}

double MODIFICATOR WelchTest(double *x1,long n1,double *x2,long n2,double *p)
//
// The function for computation of Welch test.
// Designations:
//  x1 - 1st sample,
//  n1 - size of 1st sample,
//  x2 - 2nd sample,
//  n2 - size of 2nd sample,
//  *p - P-value of test.
// Return code:
//  value of test.
// References:
//  Sachs L. Statistische Auswertungsmethoden (in Deutsch). - Berlin: Springer-Verlag, 1972.
//
{
 long i;               // Cycle counter

 double s1 = 0,s2 = 0, // Variations, assigned to size
        m1 = 0,m2 = 0, // Mediums
        w;             // Test statistic

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n1; i++)
   {
    m1 += x1[i];
    s1 += x1[i] * x1[i];
   }
 m1 /= n1;
 s1 = (s1 - n1 * m1 * m1) / (n1 - 1) / n1;
 for (i = 0; i < n2; i++)
   {
    m2 += x2[i];
    s2 += x2[i] * x2[i];
   }
 m2 /= n2;
 s2 = (s2 - n2 * m2 * m2) / (n2 - 1) / n2;

 //////////////////////////////////////////////
 // Control of the admissibility of estimations
 if (s1 < 0.000001 && s2 < 0.000001)
   {
    *p = 0.5;
    return 0;
   }
 w = (m1 - m2) / sqrt (s1 + s2);

 ////////////////////////
 // P-value computation
 *p = TDistribution((s1 + s2) * (s1 + s2) / (s1 * s1 / (n1 - 1) + s2 * s2 / (n2 - 1)),w);

 return w;
}

double MODIFICATOR StudentUnpairedTest(double *x1,long n1,double *x2,long n2,double *p)
//
// The function for computation of Student test for independent samples.
// Designations:
//  x1 - 1st sample,
//  n1 - size of 1st sample,
//  x2 - 2nd sample,
//  n2 - size of 2nd sample,
//  *p - P-value of test.
// Return code:
//  value of test.
//
{
 long i;               // Cycle counter

 double s1 = 0,s2 = 0, // Variations
        m1 = 0,m2 = 0, // Mediums
        t;             // Test statistic

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n1; i++)
   {
    m1 += x1[i];
    s1 += x1[i] * x1[i];
   }
 m1 /= n1;
 s1 = (s1 - n1 * m1 * m1) / (n1 - 1);
 for (i = 0; i < n2; i++)
   {
    m2 += x2[i];
    s2 += x2[i] * x2[i];
   }
 m2 /= n2;
 s2 = (s2 - n2 * m2 * m2) / (n2 - 1);

 //////////////////////////////////////////////
 // Control of the admissibility of estimations
 if ((s1 = (s1 * (n1 - 1) + s2 * (n2 - 1)) / (n1 + n2 - 2)) == 0)
   {
    *p = 0.5;
    return 0;
   }
 t = (m1 - m2) / sqrt (s1 / n1 + s1 / n2);

 //////////////////////
 // P-value computation
 *p = TDistribution(n1 + n2 - 2,t);

 return t;
}

double MODIFICATOR CramerTest(double *x1,long n1,double *x2,long n2,double *p)
//
// The function for computation of Cramer test.
// Designations:
//  x1 - 1st sample,
//  n1 - size of 1st sample,
//  x2 - 2nd sample,
//  n2 - size of 2nd sample,
//  *p - P-value of test.
// Return code:
//  value of test.
// References:
//  Cramer H. Mathematical methods of statistics. - Princeton: Princeton University Press, 1946.
//  Sachs L. Statistische Auswertungsmethoden (in Deutsch). - Berlin: Springer-Verlag, 1972.
//
{
 long i;               // Cycle counter

 double s1 = 0,s2 = 0, // Variations
        m1 = 0,m2 = 0, // Mediums
        t;             // Test statistic

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n1; i++)
   {
    m1 += x1[i];
    s1 += x1[i] * x1[i];
   }
 m1 /= n1;
 s1 = (s1 - n1 * m1 * m1) / (n1 - 1);
 for (i = 0; i < n2; i++)
   {
    m2 += x2[i];
    s2 += x2[i] * x2[i];
   }
 m2 /= n2;
 s2 = (s2 - n2 * m2 * m2) / (n2 - 1);

 //////////////////////////////////////////////
 // Control of the admissibility of estimations
 if ((s1 = s1 * n2 + s2 * n1) == 0)
   {
    *p = 0.5;
    return 0;
   }
 t = (m1 - m2) * sqrt(n1 * n2) / sqrt (s1);

 //////////////////////
 // P-value computation
 *p = NormalDistribution(t);

 return t;
}

double MODIFICATOR StudentPairedTest(double *x1,long n,double *x2,double *p)
//
// The function for computation of Student test for related sapmles.
//  x1 - 1st sample,
//   n - each sample size,
//  x2 - 2nd sample,
//  *p - P-value of test.
// Return code:
//  value of test.
//
{
 long i;       // Cycle counter

 double d,     // Difference of paired members, later work variable
        s = 0, // Sum of squares of differences
        t = 0; // Sum of differences, later test statistic

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n; i++)
   {
    d = x1[i] - x2[i];
    t += d;
    s += d * d;
   }

 //////////////////////////////////////////////
 // Control of the admissibility of estimations
 if ((d = n * s - t * t) == 0)
   {
    *p = 0.5;
    return 0;
   }
 t = t / sqrt (d / (n - 1));

 //////////////////////
 // P-value computation
 *p = TDistribution(n - 1,t);

 return t;
}

double MODIFICATOR FTest(double *x1,long n1,double *x2,long n2,double *p)
//
// The function for computation of F-test.
// Designations:
//  x1 - 1st sample,
//  n1 - size of 1st sample,
//  x2 - 2nd sample,
//  n2 - size of 2nd sample,
//  *p - P-value of the one-sided hypothesis.
// Return code:
//  value of test.
// References:
//  Brandt S. Data analysis. Statistical and computational methods for scientists and engineers. - New York, NY: Springer-Verlag, 2003.
//  Van der Waerden B.L. Mathematische Statistik (in Deutsch). - Berlin: Springer, 1971.
//
{
 long i;               // Cycle counter

 double s1 = 0,s2 = 0, // Variations
        m1 = 0,m2 = 0, // Mediums
        f;             // Test statistic

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n1; i++)
   {
    m1 += x1[i];
    s1 += x1[i] * x1[i];
   }
 m1 /= n1;
 s1 = (s1 - n1 * m1 * m1) / (n1 - 1);
 for (i = 0; i < n2; i++)
   {
    m2 += x2[i];
    s2 += x2[i] * x2[i];
   }
 m2 /= n2;
 s2 = (s2 - n2 * m2 * m2) / (n2 - 1);

 //////////////////////////////////////////////
 // Control of the admissibility of estimations
 if (s1 == 0 || s2 == 0 || s1 == s2)
   {
    *p = 0.5;
    return 0;
   }

 //////////////////////
 // P-value computation
// if (s2 >= s1)
   {
    f = s2 / s1;
    *p = FDistribution (n2 - 1, n1 - 1,f);
   }
// else
//   {
//    f = s1 / s2;
//    *p = FDistribution (n1 - 1, n2 - 1,f);
//   }

 return f;
}

double MODIFICATOR StudentTest(double *x1,long n,double *x2,double *p)
//
// The function for computation of Student test.
// Designations:
//     x1 - sample,
//      n - sample size,
//  x2[0] - mean,
//      p - P-value of test.
// Return code:
//  value of test.
// References:
//  Brandt S. Data analysis. Statistical and computational methods for scientists and engineers. - New York, NY: Springer-Verlag, 2003.
//  Van der Waerden B.L. Mathematische Statistik (in Deutsch). - Berlin: Springer, 1971.
//
{
 long i;       // Cycle counter

 double s = 0, // Variance
        t,     // Return code
        m = 0; // Medium

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n; i++)
   {
    m += x1[i];
    s += x1[i] * x1[i];
   }
 m /= n;
 s = (s - n * m * m) / (n - 1);

 //////////////////////////////////////////////
 // Control of the admissibility of estimations
 if (s == 0)
   {
    *p = 0.5;
    return 0;
   }
 t = fabs(m - x2[0]) / sqrt (s / n);

 //////////////////////
 // P-value computation
 *p = TDistribution(n - 1,t);

 return t;
}

double MODIFICATOR ChenTest(double *x1,long n,double *x2,double *p)
//
// The function for computation of Chen test.
// Designations:
//     x1 - sample,
//      n - sample size,
//  x2[0] - mean,
//      p - P-value of test.
// Return code:
//  value of test.
//
{
 long i;       // Cycle counter

 double s = 0, // Variance
        m = 0, // Medium, Work variable
        b = 0; // Skewness, test statistic

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n; i++)
   {
    m += x1[i];
    s += x1[i] * x1[i];
   }
 m /= n;
 s = (s - n * m * m) / (n - 1);

 //////////////////////////////////////////////
 // Control of the admissibility of estimations
 if (s == 0)
   {
    *p = 0.5;
    return 0;
   }

 //////////////////////////
 // Computation of skewness
 for (i = 0; i < n; i++)
   b += (x1[i] - m) * (x1[i] - m) * (x1[i] - m);
 b = b * n / (n - 1) / (n - 2) / s / sqrt (s);

 //////////////////////////////
 // Computing of test statistic
 b = b / 6 / sqrt (n);
 m = (m - x2[0]) / sqrt (s / n);
 b = m + b * (m * m * 2 + 1) + b * b * 4 * (m + m * m * m * 2);

 //////////////////////
 // P-value computation
 *p = NormalDistribution(b);

 return b;
}

double MODIFICATOR Transgression (double *x1,long n1,double *x2,long n2)
//
// The function computes the transgression.
// Designations:
//  x1 - 1st sample,
//  n1 - size of 1st sample,
//  x2 - 2nd sample,
//  n2 - size of 2nd sample.
// Return code:
//  value of test.
//
{
 long i;                 // Cycle counter

 double s1 = 0,s2 = 0,   // Standards
        m1 = 0,m2 = 0,   // Mediums
        t1,t2,min2,max1; // Parameters

 /////////////////////////////////////
 // Estimation parameters distribution
 for (i = 0; i < n1; i++)
   {
    m1 += x1[i];
    s1 += x1[i] * x1[i];
   }
 m1 /= n1;
 s1 = (s1 - n1 * m1 * m1) / (n1 - 1);
 if (s1 == 0)
   return 0;
 for (i = 0; i < n2; i++)
   {
    m2 += x2[i];
    s2 += x2[i] * x2[i];
   }
 m2 /= n2;
 s2 = (s2 - n2 * m2 * m2) / (n2 - 1);
 if (s2 == 0)
   return 0;
 s1 = sqrt(s1);
 s2 = sqrt(s2);

 ///////////////////////////////
 // Computation of transgression
 min2 = m2 - s2 * 3;
 max1 = m1 + s1 * 3;
 t1 = (min2 - m1) / s1;
 t2 = (max1 - m2) / s2;
 if (min2 > m1 || max1 < m2)
   return ((0.5 - NormalIntegral(t1) / 2) * n1 + (0.5 - NormalIntegral(t2) / 2) * n2) / (n1 + n2);
 else
   return ((0.5 + NormalIntegral(t1) / 2) * n1 + (0.5 + NormalIntegral(t2) / 2) * n2) / (n1 + n2);
}

long MODIFICATOR CoxHazard (double t[],double d[],double x[],double b[],double si[],long n,long m,long *it)
//
// The function for building proportional hazard Cox model.
// Designations:
//    t - vector of durations by length n,
//    d - vector of censorship indicators by length n,
//    x - matrix of covariates size n x m,
//    b - vector of computed coefficients by length m,
//   si - matrix of variances-covariations by order m,
//    n - number durations,
//    m - number of covariates,
//  *it - number spent of iterations.
// Return code:
//   0 when normal result of computation,
//  -2 when computation error.
//
{
 long i,j,k,        // Cycle counters
      s,s1,         // Counter of members arrays
      Code,         // Error code
      m2 = m * m,   // Work variable
      nm = n * m;   // Work variable

 double *g,         // Vector of gradient
        *h,*h1,*h2, // Matrix of Hesse, work array
        *u,         // Inverse matrix of Hesse, work array
        *w,         // Work array
        *xm,        // Matrix medium
        xmi,xmj,    // Work variable
        *eb;        // Vector of exponents

 g = new double[m];
 w = new double[m];
 h = new double[m2];
 h1 = new double[m2];
 h2 = new double[m2];
 u = new double[m2];
 eb = new double[n];
 xm = new double[nm];

 /////////////////
 // Initial values
 for (i = 0; i < m; i++)
   b[i] = 0;
 *it = 0;

 do
   {
    ////////////////
    // Matrix medium
    for (j = 0; j < n; j++)
      {
       for (i = 0, s = j, eb[j] = 0; i < m; i++, s += n)
         eb[j] += b[i] * x[s];
       eb[j] = exp(eb[j]);
      }
    for (i = 0; i < nm; i++)
      xm[i] = 0;
    for (i = 0; i < n; i++)
      {
       for (j = 0,xmj = 0; j < n; j++)
         {
          xmi = eb[j] * (t[j] >= t[i] ? 1 : 0);
          s = i;
          s1 = j;
          for (k = 0; k < m; k++, s += n, s1 += n) // Numerator
            xm[s] += xmi * x[s1];
          xmj += xmi;                              // Denominator
         }
        for (k = 0,s = i; k < m; k++, s += n)
          xm[s] /= xmj;
      }

    /////////////////////
    // Vector of gradient
    for (k = 0; k < m; k++)
      for (i = 0, g[k] = 0, s = k * n; i < n; i++, s++)
        g[k] += d[i] * (x[s] - xm[s]);

    ///////////////
    // Matrix Hesse
    for (i = 0; i < m2; i++)
      h[i] = 0;
    for (i = 0; i < n; i++)
      {
       for (k = 0; k < m2; k++)
         h2[k] = 0;
       for (k = 0,s = i; k < m; k++,s += n)
         u[k] = xm[s];
       for (j = 0,xmj = 0; j < n; j++)
         {
          for (k = 0,s = j; k < m; k++,s += n)
            w[k] = x[s] - u[k];
          MatrixMultiplyMatrixT(w,w,h1,m,1,m);
          xmi = eb[j] * (t[j] >= t[i] ? 1 : 0);
          for (k = 0; k < m2; k++)              // Numerator
            h2[k] += xmi * h1[k];
          xmj += xmi;                           // Denominator
         }
       for (k = 0; k < m2; k++)
         h[k] -= h2[k] / xmj * d[i];
      }

    /////////////////////////////
    //  Computation of increments
    if ((Code = InverseMatrix (h,u,m)) != 0)
      break;
    MatrixMultiplyMatrix (u,g,w,m,m,1);

    ////////////////////////
    // New vector of weigths
    for (j = 0; j < m; j++)
      b[j] -= w[j];
    *it += 1;

  } while (VectorNorma(g,m) > 0.000001);

 for (i = 0,s = 0; i < m; i++, s += m + 1)
   si[i] = -h[s];

 delete [] g; delete [] h; delete [] h1; delete [] h2;
 delete [] u; delete [] w; delete [] eb; delete [] xm;

 return Code;
}

void MODIFICATOR GammaEstimate (double *x,double *d,long n,double *a,double *k)
//
// The function for estimate parameters of Gamma distribution.
// Designations:
//   x - given sample,
//   d - array censorship indicators,
//   n - sample size,
//  *a - parameter alpha,
//  *k - parameter kappa.
// Return code:
//  none.
// References:
//  Bolshev L.N., Smirnov N.V. Tables of mathematical statistics (in Russian). - Moscow: Nauka, 1983.
//  Kobzar A.I. Applied mathematical statistics. For engineers and researchers (in Russian). - M.: FIZMATLIT, 2006.
//
{
 long i;               // Counter

 double r = 0,         // Number of uncensored samples
        l,l1 = 1,
        l2 = MaximumOf(x,n),
        h1,
        g,g1 = 0.0001,
        g2 = 10,
        h2 = 1,
        s,s1 = -1000000;

 ///////////////////
 // Work computation
 for (i = 0; i < n; i++)
    r += d[i];

 h1 = l2 / 10;

 for (i = 0; i < 4; i++)
   {
    for (l = l1; l < l2; l += h1)
      for (g = g1; g < g2; g += h2)
        if ((s = GammaAdd(x,d,n,l,g,r)) > s1)
          {
           *a = l;
           *k = g;
           s1 = s;
          }
    l1 = *a - h1;
    l2 = *a + h1;
    g1 = *k - h2;
    g2 = *k + h2;
    h1 /= 10;
    h2 /= 10;
   }
}

double MODIFICATOR GammaAdd(double *x,double *d,long n,double a,double k,double r)
//
// The function for computation logarithm of maximum likelihood function of Gamma distribution.
// Designations:
//   x - given sample,
//   d - array censorship indicators,
//   n - sample size,
//   a - parameter alpha,
//   k - parameter kappa,
//   r - number uncensored samples.
// Return code:
//  value of logarithm of maximum likelihood function of Gamma distribution.
//
{
 long i;        // Counter

 double s,      // Work variable
        s1 = 0, // Work variable
        s2 = 0, // Work variable
        s3 = 0; // Work variable

 for (i = 0; i < n; i++)
   {
    s1 += d[i] * log(x[i]);
    s2 += d[i] * x[i];
    s = 1 - GammaIncomplete(k,x[i] / a);
    if (!s)
      s = 0.000000001;
    s3 += (1 - d[i]) * log(s);
   }

 return (k - 1) * s1 - s2 / a + s3 - r * k * log(a) - r * LogGamma(k);
}

void MODIFICATOR GammaFitting (double *z,double *dl,long n,double *p,double *y,long k,double *a,double *ka,double *aic)
//
// The function for fitting distribution Gamma.
// Designations:
//      z - given sample,
//     dl - array of censorship indicators,
//      n - sample size,
//      p - expected (theoretical) frequencies,
//      y - means of classes intervals,
//      k - size of variation series,
//     *a - parameter distribution (alpha),
//    *ka - parameter distribution (kappa),
//   *aic - test value Akaike.
// Return code:
//  none.
// References:
//  Bolshev L.N., Smirnov N.V. Tables of mathematical statistics (in Russian). - Moscow: Nauka, 1983.
//
{
 long i;     // Cycle counter

 double s,g, // Work variable
        d;   // Size of of classes interval

 /////////////////////////////////////
 // Estimation parameters distribution
 GammaEstimate (z,dl,n,&s,&g);
 *a = s;
 *ka = g;
 *aic = 0;

 //////////////////////////////////////
 // Computation of expected frequencies
 for (i = 0; i < k; i++)
   {
    if (i == 0)
      d = y[1] - y[0];
    else if (i == k - 1)
      d = y[k - 1] - y[k - 2];
    else
      d = (y[i + 1] - y[i - 1]) / 2;
    p[i] = exp((*ka - 1) * (log(y[i]) - log(*a)) - log(*a) - LogGamma(*ka) - y[i] / *a);
    *aic += log(p[i]);
    p[i] *= d * n;
   }
 *aic = -2.0 * (*aic) + 12.0 / (n - 3) + 4;
}

void MODIFICATOR LognormalEstimate (double *x,double *d,long n,double *m,double *s)
//
// The function for fitting parameters of lognormal distribution.
// Designations:
//   x - given sample,
//   d - array censorship indicators,
//   n - sample size,
//  *m - parameter mu,
//  *s - parameter sigma.
// Return code:
//  none.
// References:
//  Aitchison J., Brown J.A.C. The lognormal distribution. - Cambridge, UK: Cambridge University Press, 1963.
//  Lu J.Z. Fitting Weibull and lognormal distributions to medium-density fiberboard fiber and wood particle length / J.Z. Lu, C.J. Monlezun, Q. Wu et al. // Wood and Fiber Science, 2007, vol. 39, no. 1, pp. 82-94.
//
{
 long i,              // Counter
      k = 0;          // Counter of iterations

 double *lx,          // Array of logarithm of given of sample
        z,            // Standardized current variable
        r = 0,        // Number uncensored samples
        t,            // Work variable
        m1,s1,        // Parameters on current iteration
        g[2],         // Anti-gradient
        b[4],         // Matrix Hesse
        e[4],         // Inverse Matrix
        f0,           // Density distribution
        s0,           // The function distribution
        dlfdz,        // Derivative of density distribution
        dlsdz,        // Derivative of function distribution
        d2lfdz2 = -1, // 2nd Derivative of density distribution
        d2lsdz2,      // 2nd Derivative of function distribution
        l,l1 = 0,     // Logarithm of maximum likelihood function
        j,u;          // Work variable

 lx = new double[n];

 ///////////////////
 // Work computation
 for (i = 0; i < n; i++)
   {
    lx[i] = log(x[i]);
    r += d[i];
   }

 /////////////////
 // Initial values
 for(u = 0.001; u <= 6; u += 0.01)
   for(j = -20; j <= 20; j += 0.5)
     {
      for(i = 0, l = 0, t = 0; i < n; i++)
        {
         z = (lx[i] - j) / u;
         s0 = 1 - NormalDistribution (z);
         if (s0 <= 0)
           s0 = 0.000001;
         t += d[i] * z * z;
         l += (1 - d[i]) * log(s0);
        }
      l = -r * log(u) - t / 2 + l;
      if (u == 0.001 && j == -20)
        l1 = l;
      else
        if (l > l1)
          {
           l1 = l;
           *m = j;
           *s = u;
          }
    }

 /////////////////////////////
 // The Newton-Raphson process
 m1 = *m;
 s1 = *s;
 for(;;)
   {
    b[0] = 0;
    b[1] = 0;
    b[2] = 0;
    b[3] = 0;
    e[0] = 1;
    e[1] = 0;
    e[2] = 0;
    e[3] = 1;
    g[0] = 0;
    g[1] = 0;
    for(i = 0, t = 0; i < n; i++)
      {
       z = (lx[i] - *m) / (*s);
       f0 = 0.398942280401432678 * exp(-0.5 * z * z);
       s0 = 1 - NormalDistribution (z);
       dlfdz = -z;
       dlsdz = -f0 / s0;
       d2lsdz2 = -z * dlsdz - dlsdz * dlsdz;
       b[0] += d[i] * d2lfdz2 + (1 - d[i]) * d2lsdz2;
       b[1] += d[i] * dlfdz + (1 - d[i]) * dlsdz;
       b[2] += d[i] * z * d2lfdz2 + (1 - d[i]) * z * d2lsdz2;
          t += d[i] * z * dlfdz + (1 - d[i]) * z * dlsdz;
       b[3] += d[i] * z * z * d2lfdz2 + (1 - d[i]) * z * z * d2lsdz2;
       g[0] += d[i] * dlfdz + (1 - d[i]) * dlsdz;
       g[1] += d[i] * z * dlfdz + (1 - d[i]) * z * dlsdz;
      }
    b[0] = b[0] / (*s) / (*s);
    b[1] = (b[1] + b[2]) / (*s) / (*s);
    b[2] = b[1];
    b[3] = (r + 2 * t + b[3]) / (*s) / (*s);
    g[0] /= (*s);
    g[1] = r / (*s) + g[1] / (*s);
    Gauss(b,e,2,2,0.000001);
    *m += e[0] * g[0] + e[2] * g[1];
    *s += e[1] * g[0] + e[3] * g[1];
    if (fabs((*m - m1) / *m) <= 0.000001 && fabs((*s - s1) / *s) <= 0.000001)
      break;
    m1 = *m;
    s1 = *s;
    if (++k > 1000)
      break;
   }

 delete [] lx;
}

void MODIFICATOR LoglogisticEstimate (double *x,double *d,long n,double *m,double *s)
//
// The function for estimate parameters of loglogistic distribution.
// Designations:
//   x - given sample,
//   d - array censorship indicators,
//   n - sample size,
//  *m - parameter mu,
//  *s - parameter sigma.
// Return code:
//  none.
// References:
//  Collett D. Modelling survival data in medical research. - Boca Raton, FL: Chapman & Hall / CRC, 1993.
//  Bennett S. Log-logistic regression models for survival data // Journal of the Royal Statistical Society, Series C, vol. 32, no. 2, pp. 165-171.
//
{
 long i,          // Counter
      k = 0;      // Counter of iterations

 double *lx,      // Array of logarithm of given of sample
        z,        // Standardized current variable
        r = 0,    // Number uncensored samples
        t,        // Work variable
        m1,s1,    // Parameters on current iteration
        g[2],     // Anti-gradient
        b[4],     // Matrix Hesse
        e[4],     // Inverse Matrix
        dlfdz,    // Derivative of density distribution
        dlsdz,    // Derivative of function distribution
        d2lfdz2,  // 2nd Derivative of density distribution
        d2lsdz2,  // 2nd Derivative of function distribution
        l,l1 = 0, // Logarithm of maximum likelihood function
        j,u;      // Work variable

 lx = new double[n];

 ///////////////////
 // Work computation
 for (i = 0; i < n; i++)
   {
    lx[i] = log(x[i]);
    r += d[i];
   }

 /////////////////
 // Initial values
 for(u = 0.1; u <= 5; u += 0.01)
   for(j = -20; j <= 20; j += 0.5)
     {
      for(i = 0, l = 0, t = 0; i < n; i++)
        {
         z = (lx[i] - j) / u;
         t += d[i] * (z - 2 * log(1 + exp(z)));
         l += (1 - d[i]) * log(1 + exp(z));
        }
      l = -r * log(u) + t - l;
      if (u == 0.1 && j == -20)
        l1 = l;
      else
        if (l > l1)
          {
           l1 = l;
           *m = j;
           *s = u;
          }
    }

 /////////////////////////////
 // The Newton-Raphson process
 m1 = *m;
 s1 = *s;
 for(;;)
   {
    b[0] = 0;
    b[1] = 0;
    b[2] = 0;
    b[3] = 0;
    e[0] = 1;
    e[1] = 0;
    e[2] = 0;
    e[3] = 1;
    g[0] = 0;
    g[1] = 0;
    for (i = 0, t = 0; i < n; i++)
      {
       z = (lx[i] - *m) / (*s);
       dlsdz = -exp(z) / (1 + exp(z));
       dlfdz = 1 + 2 * dlsdz;
       d2lsdz2 = -exp(z) / (1 + exp(z)) / (1 + exp(z));
       d2lfdz2 = 2 * d2lsdz2;
       b[0] += d[i] * d2lfdz2 + (1 - d[i]) * d2lsdz2;
       b[1] += d[i] * dlfdz + (1 - d[i]) * dlsdz;
       b[2] += d[i] * z * d2lfdz2 + (1 - d[i]) * z * d2lsdz2;
          t += d[i] * z * dlfdz + (1 - d[i]) * z * dlsdz;
       b[3] += d[i] * z * z * d2lfdz2 + (1 - d[i]) * z * z * d2lsdz2;
       g[0] += d[i] * dlfdz + (1 - d[i]) * dlsdz;
       g[1] += d[i] * z * dlfdz + (1 - d[i]) * z * dlsdz;
      }
    b[0] = b[0] / (*s) / (*s);
    b[1] = (b[1] + b[2]) / (*s) / (*s);
    b[2] = b[1];
    b[3] = (r + 2 * t + b[3]) / (*s) / (*s);
    g[0] /= (*s);
    g[1] = r / (*s) + g[1] / (*s);
    Gauss(b,e,2,2,0.000001);
    *m += e[0] * g[0] + e[2] * g[1];
    *s += e[1] * g[0] + e[3] * g[1];
    if (fabs((*m - m1) / *m) <= 0.000001 && fabs((*s - s1) / *s) <= 0.000001)
      break;
    m1 = *m;
    s1 = *s;
    if (++k > 1000)
      break;
   }

 delete [] lx;
}

void MODIFICATOR LognormalFitting (double *z,double *dl,long n,double *p,double *y,long k,double *m,double *s,double *aic)
//
// The function for fitting of lognormal distribution.
// Designations:
//     z - given sample,
//    dl - array censorship indicators,
//     n - sample size,
//     p - expected (theoretical) frequencies,
//     y - means of classes intervals,
//     k - size of variation series,
//    *m - parameter distribution (mu),
//    *s - parameter distribution (sigma).
//  *aic - test value Akaike.
// Return code:
//  none.
//
{
 long i;     // Cycle counter

 double l,g, // Work variable
        d;   // Size of classes interval

 /////////////////////////////////////
 // Estimation parameters distribution
 LognormalEstimate (z,dl,n,&l,&g);
 *m = l;
 *s = g;
 *aic = 0;

 //////////////////////////////////////
 // Computation of expected frequencies
 for (i = 0; i < k; i++)
   {
    p[i] = (log(y[i]) - l) / g;
    if (i == 0)
      d = y[1] - y[0];
    else if (i == k - 1)
      d = y[k - 1] - y[k - 2];
    else
      d = (y[i + 1] - y[i - 1]) / 2;
    p[i] = 0.398942280401 / exp(p[i] * p[i] / 2) / y[i] / g;
    *aic += log(p[i]);
    p[i] *= d * n;
   }
 *aic = -2.0 * (*aic) + 12.0 / (n - 3) + 4;
}

void MODIFICATOR LoglogisticFitting (double *z,double *dl,long n,double *p,double *y,long k,double *m,double *s,double *aic)
//
// The function for fitting of loglogistic distribution.
// Designations:
//     z - given sample,
//    dl - array censorship indicators,
//     n - sample size,
//     p - expected (theoretical) frequencies,
//     y - means of classes intervals,
//     k - size of variation series,
//    *m - parameter distribution (mu),
//    *s - parameter distribution (sigma).
//  *aic - test value Akaike.
// Return code:
//  none.
//
{
 long i;     // Cycle counter

 double l,g, // Work variable
        d;   // Size of classe interval

 /////////////////////////////////////
 // Estimation parameters distribution
 LoglogisticEstimate (z,dl,n,&l,&g);
 *m = l;
 *s = g;
 *aic = 0;

 //////////////////////////////////////
 // Computation of expected frequencies
 for (i = 0; i < k; i++)
   {
    p[i] = (log(y[i]) - l) / g;
    if (i == 0)
      d = y[1] - y[0];
    else if (i == k - 1)
      d = y[k - 1] - y[k - 2];
    else
      d = (y[i + 1] - y[i - 1]) / 2;
    p[i] = exp(p[i]) / (1 + exp(p[i])) / (1 + exp(p[i])) / y[i] / g;
    *aic += log(p[i]);
    p[i] *= d * n;
   }
 *aic = -2.0 * (*aic) + 12.0 / (n - 3) + 4;
}

void MODIFICATOR GompertzEstimate (double *x,double *d,long n,double *l,double *g)
//
// The function for fitting parameters of Gompertz distribution.
// Designations:
//   x - given sample,
//   d - array censorship indicators,
//   n - sample size,
//  *l - parameter alpha,
//  *g - parameter beta.
// Return code:
//  none.
// References:
//  Yousef M.H. Estimation of parameters and truncation point for the truncated Gompertz distribution // Journal of King Saud University (Administrative Sciences), 1993, vol. 5, no. 2, pp. 35-48.
//
{
 long i;        // Counter

 double c = -1, // Left bound localization
        b = 0,  // Right bound localization
        y,      // The middle of interval localization
        cx,     // Value of function on left end of interval
        r = 0,  // Number uncensored samples
        s = 0,  // Work variable
        t = 0;  // Work variable

 ///////////////////
 // Work computation
 for (i = 0; i < n; i++)
   {
    r += d[i];
    s += d[i] * x[i];
   }

 /////////////////////////////////
 // Computation of parameter alpha
 do
   {
    cx = GompertzAdd (x,n,c,r,s);
    y = (c + b) / 2;
    cx *= GompertzAdd (x,n,y,r,s);
    if (!cx)
      break;
    else if (cx > 0)
      c = y;
    else
      b = y;
   } while (b - c > 1E-9);

 ////////////////////////////////////
 // Computation of optimal parameters
 for (i = 0; i < n; i++)
   t += exp(x[i] * y);
 *l = y;
 *g = y * r / (t - n);
}

double MODIFICATOR GompertzAdd (double *x,long n,double a,double r,double s)
//
// The nonlinear function for computation of parameter alpha distribution Gompertz.
// Designations:
//   x - given sample,
//   n - sample size,
//   a - parameter alpha,
//   r - work constant,
//   s - work constant.
// Return code:
//  value of nonlinear function.
//
{
 long i;       // Counter

 double t = 0, // Work variable
        c;     // Work variable

 for (i = 0; i < n; i++)
   {
    c = exp(x[i] * a);
    t += c;
   }

 return a * s - r / (t - n) * (n + (1 - a * a) * t);
}

void MODIFICATOR RayleighEstimate (double *x,double *d,long n,double *b)
//
// The function for estimate of parameters Rayleigh distribution.
// Designations:
//   x - given sample,
//   d - array censorship indicators,
//   n - sample size,
//  *b - parameter beta.
// Return code:
//  none.
// References:
//  Kobzar A.I. Applied mathematical statistics. For engineers and researchers (in Russian). - M.: FIZMATLIT, 2006.
//
{
 long i;       // Counter

 double r = 0, // Number uncensored samples
        s = 0; // Work variable

 ///////////////////
 // Work computation
 for (i = 0; i < n; i++)
   {
    r += d[i];
    s += x[i] * x[i];
   }

 ////////////////////////////////
 // Computation of parameter beta
 *b = sqrt (s / 2 / r);
}

void MODIFICATOR ExponentialEstimate (double *x,double *d,long n,double *l)
//
// The function for estimate parameters of exponential distribution.
// Designations:
//   x - given sample,
//   d - array censorship indicators,
//   n - sample size,
//  *l - parameter lambda.
// Return code:
//  none.
//
{
 long i;       // Counter

 double r = 0, // Number uncensored samples
        s = 0; // Work variable

 ///////////////////
 // Work computation
 for (i = 0; i < n; i++)
   {
    r += d[i];
    s += x[i];
   }

 //////////////////////////////////
 // Computation of parameter lambda
 *l = r / s;
}

void MODIFICATOR WeibullEstimate (double *x,double *d,long n,double *l,double *g)
//
// The function for estimate parameters of Weibull distribution.
// Designations:
//   x - given sample,
//   d - array censorship indicators,
//   n - sample size,
//  *l - parameter lambda,
//  *g - parameter Gamma.
// Return code:
//  none.
// References:
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//  Al-Fawzan M.A. Algorithms for estimating the parameters of the Weibull distribution // Statistics on the Internet (InterStat), October 2000, no. 1.
//  Angilletta M.J.Jr., Oufiero C.E., Leache A.D. Direct and indirect effects of environmental temperature on the evolution of reproductive strategies: An information-theoretic approach // The American Naturalist, October 2006, vol. 168, no. 4, pp. 123-135.
//
{
 long i;              // Counter

 double *lx,          // Array of logarithm of given of sample
        c = 0.000001, // Left bound localization Gamma
        b = 30,       // Right bound localization Gamma
        y,            // The middle of interval localization Gamma
        cx,           // Value of function on left end of interval
        r = 0,        // Number uncensored samples
        s = 0,        // Work variable
        t = 0;        // Work variable

 lx = new double[n];

 ///////////////////
 // Work computation
 for (i = 0; i < n; i++)
   {
    lx[i] = log(x[i]);
    r += d[i];
    s += d[i] * lx[i];
   }

 /////////////////////////////////
 // Computation of parameter Gamma
 do
   {
    cx = WeibullAdd (lx,n,c,r,s);
    y = (c + b) / 2;
    cx *= WeibullAdd (lx,n,y,r,s);
    if (!cx)
      break;
    else if (cx > 0)
      c = y;
    else
      b = y;
   } while (b - c > 1E-9);

 ////////////////////////////////////
 // Computation of optimal parameters
 for (i = 0; i < n; i++)
   t += exp(lx[i] * y);
 *l = r / t;
 *g = y;

 delete [] lx;
}

double MODIFICATOR WeibullAdd (double *lx,long n,double g,double r,double s)
//
// The nonlinear function of computation of parameter Gamma of Weibull distribution.
// Designations:
//  lx - logarithms of given of sample,
//   n - sample size,
//   g - parameter Gamma,
//   r - work constant,
//   s - work constant.
// Return code:
//  value of nonlinear function.
//
{
 long i;       // Counter

 double t = 0, // Work variable
        u = 0, // Work variable
        c;     // Work variable

 for (i = 0; i < n; i++)
   {
    c = exp(lx[i] * g);
    t += c;
    u += c * lx[i];
   }

 return r / g + s - r * u / t;
}

void MODIFICATOR RayleighFitting (double *z,double *dl,long n,double *p,double *y,long k,double *b,double *aic)
//
// The function for fitting of Rayleigh distribution.
// Designations:
//     z - given sample,
//    dl - array censorship indicators,
//     n - sample size,
//     p - expected (theoretical) frequencies,
//     y - means of classes intervals,
//     k - size of variation series,
//    *b - parameter distribution (beta),
//  *aic - test value Akaike.
// Return code:
//  none.
//
{
 long i;   // Cycle counter

 double d, // Size of classes interval
        l; // Work variable

 /////////////////////////////////////
 // Estimation parameters distribution
 RayleighEstimate (z,dl,n,&l);
 *b = l;
 *aic = 0;

 //////////////////////////////////////
 // Computation of expected frequencies
 for (i = 0; i < k; i++)
   {
    if (i == 0)
      d = y[1] - y[0];
    else if (i == k - 1)
      d = y[k - 1] - y[k - 2];
    else
      d = (y[i + 1] - y[i - 1]) / 2;
    p[i] = y[i] / (*b) / (*b) / exp(y[i] / (*b) * y[i] / (*b) / 2);
    *aic += log(p[i]);
    p[i] *= d * n;
   }
 *aic = -2.0 * (*aic) + 4.0 / (n - 2) + 2;
}

void MODIFICATOR ExpFitting (double *z,double *dl,long n,double *p,double *y,long k,double *b,double *aic)
//
// The function for fitting of distribution Gamma.
// Designations:
//     z - given sample,
//    dl - array censorship indicators,
//     n - sample size,
//     p - expected (theoretical) frequencies,
//     y - means of classes intervals,
//     k - size of variation series,
//    *b - parameter distribution (Gamma),
//  *aic - test value Akaike.
// Return code:
//  none.
//
{
 long i;   // Cycle counter

 double d, // Size of classes interval
        l; // Work variable

 ///////////////////////////////////////
 // Estimation of parameter distribution
 ExponentialEstimate (z,dl,n,&l);
 *b = l;
 *aic = 0;

 //////////////////////////////////////
 // Computation of expected frequencies
 for (i = 0; i < k; i++)
   {
    if (i == 0)
      d = y[1] - y[0];
    else if (i == k - 1)
      d = y[k - 1] - y[k - 2];
    else
      d = (y[i + 1] - y[i - 1]) / 2;
    p[i] = *b * exp(-y[i] * (*b));
    *aic += log(p[i]);
    p[i] *= d * n;
   }
 *aic = -2.0 * (*aic) + 4.0 / (n - 2) + 2;
}

void MODIFICATOR WeibullFitting (double *z,double *dl,long n,double *p,double *y,long k,double *l,double *h,double *aic)
//
// The function for fitting of Weibull distribution.
// Designations:
//     z - given sample,
//    dl - array censorship indicators,
//     n - sample size,
//     p - expected (theoretical) frequencies,
//     y - means of classes intervals,
//     k - size of variation series,
//    *l - parameter distribution (lambda),
//    *h - parameter distribution (gamma),
//  *aic - test value Akaike.
// Return code:
//  none.
// References:
//  Muller P.H., Neumann P., Storm R. Tafeln der mathematischen Statistik (in Deutsch). - Leipzig: VEB Fachbuchverlag,1979.
//  Al-Fawzan M.A. Algorithms for estimating the parameters of the Weibull distribution // Statistics on the Internet (InterStat), October 2000, no. 1.
//
{
 long i;     // Cycle counter

 double s,g, // Work variable
        d;   // Size of classes interval

 /////////////////////////////////////
 // Estimation parameters distribution
 WeibullEstimate (z,dl,n,&s,&g);
 *l = s;
 *h = g;
 *aic = 0;

 //////////////////////////////////////
 // Computation of expected frequencies
 for (i = 0; i < k; i++)
   {
    if (i == 0)
      d = y[1] - y[0];
    else if (i == k - 1)
      d = y[k - 1] - y[k - 2];
    else
      d = (y[i + 1] - y[i - 1]) / 2;
    p[i] = exp((*h - 1) * log(y[i]) - *l * exp(*h * log(y[i]))) * (*l) * (*h);
    *aic += log(p[i]);
    p[i] *= d * n;
   }
 *aic = -2.0 * (*aic) + 12.0 / (n - 3) + 4;
}

void MODIFICATOR GompertzFitting (double *z,double *dl,long n,double *p,double *y,long k,double *l,double *h,double *aic)
//
// The function for fitting of Gompertz distribution.
// Designations:
//     z - given sample,
//    dl - array censorship indicators,
//     n - sample size,
//     p - expected (theoretical) frequencies,
//     y - means of classes intervals,
//     k - size of variation series,
//    *l - parameter distribution (alpha),
//    *h - parameter distribution (beta),
//  *aic - test value Akaike.
// Return code:
//  none.
//
{
 long i;     // Cycle counter

 double s,g, // Work variable
        d;   // Size of classes interval

 /////////////////////////////////////
 // Estimation parameters distribution
 GompertzEstimate (z,dl,n,&s,&g);
 *l = s;
 *h = g;
 *aic = 0;

 //////////////////////////////////////
 // Computation of expected frequencies
 for (i = 0; i < k; i++)
   {
    if (i == 0)
      d = y[1] - y[0];
    else if (i == k - 1)
      d = y[k - 1] - y[k - 2];
    else
      d = (y[i + 1] - y[i - 1]) / 2;
    p[i] = exp (log(*h) + *l * y[i] + *h / (*l) - *h / (*l) * exp(*l * y[i]));
    *aic += log(p[i]);
    p[i] *= d * n;
   }
 *aic = -2.0 * (*aic) + 12.0 / (n - 3) + 4;
}

double MODIFICATOR CoxTest(double *x,double *jx,long nx,double *y,double *jy,long ny,double *p)
//
// The function for computation of Cox test.
// Designations:
//   x - 1st sample,
//  jx - indicators of 1st sample,
//  nx - size of 1st sample,
//   y - 2nd sample,
//  jy - indicators of 2nd sample,
//  ny - size of 2nd sample,
//  *p - P-value of the two-sided hypothesis.
// Return code:
//  value of test.
//
{
 long i,               // Counter
      *ix,             // Information array
      n = nx + ny;     // Common size

 double *xx,*xy,       // Work array
        *ij,           // Common array of indicators
        t = 0,         // Value of test
        rx = 0,ry = 0, // Numbers uncensored elements
        r,             // Correction coefficient
        dt = 0;        // Variance

 ix = new long[n];
 xx = new double[n];
 xy = new double[n];
 ij = new double[n];

 ////////////////////////////
 // Number uncensored samples
 for (i = 0; i < nx; i++)
   if (jx[i] == 1)
     rx++;
 for (i = 0; i < ny; i++)
   if (jy[i] == 1)
     ry++;
 r = rx * ry / (rx + ry - 1);

 ////////////////////////
 // Given arrays junction
 if (nx >= ny)
   {
    for (i = 0; i < nx; i++)
      {
       xy[i] = x[i];
       ij[i] = jx[i];
      }
    for (i = 0; i < ny; i++)
      xy[i + nx] = y[i];
   }
 else
   {
    for (i = 0; i < ny; i++)
      {
       xy[i] = y[i];
       ij[i] = jy[i];
      }
    for (i = 0; i < nx; i++)
      xy[i + ny] = x[i];
    nx = ny;
    rx = ry;
   }

 //////////
 // Sort up
 SortArrayUp (xy,n,ix);

 //////////
 // Ranking
 RankSavage (xx,n);

 //////////////////////
 // Restore ranks order
 UnSortArray (xx,xy,n,ix);

 delete [] ix; delete [] xx;

 ////////////////////////////
 // Value of test computation
 for (i = 0; i < nx; i++)
   t += ij[i] * xy[i];

 delete [] xy; delete [] ij;

 for (i = 1; i <= n; i++)
   dt += 1.0 / i;
 dt = (1.0 - dt / n) * r;

 //////////////////////
 // P-value computation
 *p = NormalDistribution ((t - rx) / sqrt (dt));

 return t;
}

double MODIFICATOR GehanTest(double *x,double *jx,long nx,double *y,double *jy,long ny,double *p)
//
// The function for computation of Gehan test.
// Designations:
//   x - 1st sample,
//  jx - indicators of 1st sample,
//  nx - size of 1st sample,
//   y - 2nd sample,
//  jy - indicators of 2nd sample,
//  ny - size of 2nd sample,
//  *p - P-value of the two-sided hypothesis.
// Return code:
//  value of test.
// References:
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//
{
 long i,j,         // Counter
      *ix,         // Common indicator array
      n = nx + ny; // Common size

 double *xy,       // Common array
        w = 0,     // Value of test
        dw = 0,    // Variance
        u;         // Work variable

 xy = new double[n];
 ix = new long[n];

 ////////////////////////
 // Given arrays junction
 for (i = 0; i < nx; i++)
   {
    xy[i] = x[i];
    ix[i] = jx[i];
   }
 for (i = 0; i < ny; i++)
   {
    xy[i + nx] = y[i];
    ix[i + nx] = jy[i];
   }

 ////////////////
 // Value of test
 for (i = 0; i < nx; i++)
   {
    for (j = 0, u = 0; j < ny; j++)
      {
       if (x[i] > y[j] && jx[i] == 1 && jy[j] == 1)
         u++;
       else if (x[i] >= y[j] && jx[i] == 0 && jy[j] == 1)
         u++;
       else if (x[i] < y[j] && jx[i] == 1 && jy[j] == 1)
         u--;
       else if (x[i] <= y[j] && jx[i] == 1 && jy[j] == 0)
         u--;
      }
    w += u;
   }

 ///////////
 // Variance
 for (i = 0; i < n; i++)
   {
    for (j = 0, u = 0; j < n; j++)
      {
       if (xy[i] > xy[j] && ix[i] == 1 && ix[j] == 1)
         u++;
       else if (xy[i] >= xy[j] && ix[i] == 0 && ix[j] == 1)
         u++;
       else if (xy[i] < xy[j] && ix[i] == 1 && ix[j] == 1)
         u--;
       else if (xy[i] <= xy[j] && ix[i] == 1 && ix[j] == 0)
         u--;
      }
    dw += u * u;
   }
 dw = dw * nx * ny / n / (n - 1);

 delete [] ix; delete [] xy;

 //////////////////////
 // P-value computation
 *p = NormalDistribution (w / sqrt(dw));

 return w;
}

double ChiSquareTest(double x[], double p[], long n)
//
// The function for computation of chi-square test.
// Designations:
//  X - empirical frequencies,
//  P - theoretical frequencies,
//  N - sample size.
// Return code:
//  value of test.
// References:
//  Runyon R.P. Nonparametric statistics. A contemporary approach. - Reading, MA: Addison-Wesley, 1977.
//  Glantz S.A. Primer of biostatistics. - New York, NY: McGraw-Hill, 2012.
//
{
 long i;         // Counter

 double ret = 0; //Returned value

 for (i = 0; i < n; i++)
   if (p[i] != 0)
     ret += (x[i] / p[i] - 1) * (x[i] - p[i]);

 return ret;
}

void MODIFICATOR Survive(double X[],double Y[],long K,double F[],double Z[], long *S,
                       double P,double D[],double Low[],double High[])
//
// The function for computation of Kaplan-Meyer estimates of the survival curve.
// Designations:
//     X - sorted array
//     Y - censoring indicators corresponding to X
//     K - size arrays
//     F - ordinates of the survival graph
//     Z - abscissae of the survival graph
//    *S - calculated number of graph points
//     P - confidence probability
//     D - standard error
//   Low - low CI
//  High - high CI
// Return code:
//  none.
// References:
//  Aalen O.O., Borgan O., Gjessing H.K. Survival and event history analysis: A process point of view. - New York, NY: Springer, 2008.
//  Cox D.R., Oakes D. Analysis of survival data. - New York, NY: Chapman and Hall, 1984.
//
{
 long I,J;     // Counter

 double *H,    // Number alive in moment of computation
        *M,    // Number deceased in moment of computation
        W = 0; // Work variable

 ////////////////////////////
 // computation of the number
 (*S) = K;
 for (J = 1; J < K; J++)
   if (X[J] == X[J - 1])
     (*S)--;

 /////////////////
 // Initialization
 H = new double[(*S)];
 M = new double[(*S)];

 ///////////////////////////
 // computation of abscissae
 Z[0] = X[0];
 for (J = 1, I = 1; J < K; J++)
   if (X[J] != X[J - 1])
     Z[I++] = X[J];

 ///////////////////////////
 // Number of alive patients
 for (J = 0; J < (*S); J++)
   for (I = 0, H[J] = 0; I < K; I++)
     if (X[I] >= Z[J])
       H[J] += 1;

 //////////////////////////////////////////////
 // Number of deceased in moment of computation
 for (J = 0; J < (*S); J++)
   for (I = 0, M[J] = 0; I < K; I++)
     if (X[I] == Z[J] && Y[I] == 1)
       M[J] += 1;

 ///////////////////////
 // Building of function
 F[0] = (1 - M[0] / H[0]);
 for (J = 1; J < (*S); J++)
   F[J] = F[J - 1] * (1.0 - M[J] / H[J]);

 /////////////////////////
 // Computation variations
 for (J = 0; J < (*S); J++)
   if (H[J] != M[J])
     {
      W += M[J] / H[J] / (H[J] - M[J]);
      D[J] = F[J] * sqrt(W);
     }
   else
     D[J] = F[J] * sqrt((1 - F[J]) / H[J]);

 //////////////
 // Building CI
 W = InverseNormalDistribution((1 + P) / 2);
 for (J = 0; J < (*S); J++)
   {
    if ((Low[J] = F[J] - W * D[J]) < 0)
      Low[J] = 0;
    if ((High[J] = F[J] + W * D[J]) > 1)
      High[J] = 1;
   }

 delete [] H; delete [] M;
}

void MODIFICATOR Hazard(double X[],double Y[],long K,double F[],double Z[],long *S,double P,double D[],double Low[],double High[])
//
// The function for computation of Kaplan-Meyer estimates of hazard function.
// Designations:
//     X - sorted array
//     Y - censoring indicators corresponding to X
//     K - size arrays
//     F - ordinates of the survival graph
//     Z - abscissae of the survival graph
//    *S - calculated number of graph points
//     P - confidence probability
//     D - standard error
//   Low - low CI
//  High - high CI
// Return code:
//  none.
// References:
//  Aalen O.O., Borgan O., Gjessing H.K. Survival and event history analysis: A process point of view. - New York, NY: Springer, 2008.
//  Cox D.R., Oakes D. Analysis of survival data. - New York, NY: Chapman and Hall, 1984.
//
{
 long I,J;     // Counter

 double *H,    // Number alive in moment of computation
        *M,    // Number deceased in moment of computation
        *O,    // The survival function
        W = 0; // Work variable

 ////////////////////////////
 // computation of the number
 (*S) = K;
 for (J = 1; J < K; J++)
   if (X[J] == X[J - 1])
     (*S)--;

 /////////////////
 // Initialization
 H = new double[(*S)];
 M = new double[(*S)];
 O = new double[(*S)];

 ///////////////////////////
 // computation of abscissae
 Z[0] = X[0];
 for (J = 1, I = 1; J < K; J++)
   if (X[J] != X[J - 1])
     Z[I++] = X[J];

 ///////////////////////////
 // Number of alive patients
 for (J = 0; J < (*S); J++)
   for (I = 0, H[J] = 0; I < K; I++)
     if (X[I] >= Z[J])
       H[J] += 1;

 //////////////////////////////////////////////
 // Number of deceased in moment of computation
 for (J = 0; J < (*S); J++)
   for (I = 0, M[J] = 0; I < K; I++)
     if (X[I] == Z[J] && Y[I] == 1)
       M[J] += 1;

 ///////////////////////
 // Building of function
 O[0] = (1 - M[0] / H[0]);
 for (J = 1; J < (*S); J++)
   O[J] = O[J - 1] * (1.0 - M[J] / H[J]);
 F[0] = M[0] / H[0];
 for (J = 1; J < (*S); J++)
   if ((F[J] = F[J - 1] + M[J] / H[J]) > 1)
     F[J] = 1;

 /////////////////////////
 // Computation variations
 for (J = 0; J < (*S); J++)
   if (H[J] != M[J])
     {
      W += M[J] / H[J] / (H[J] - M[J]);
      D[J] = sqrt(W);
     }
   else
     D[J] = sqrt((1.0 - O[J]) / H[J]);

 //////////////
 // Building CI
 W = InverseNormalDistribution((1 + P) / 2);
 for (J = 0; J < (*S); J++)
   {
    if ((Low[J] = F[J] - W * D[J]) < 0)
      Low[J] = 0;
    if ((High[J] = F[J] + W * D[J]) > 1)
      High[J] = 1;
   }

 delete [] H; delete [] M; delete [] O;
}

long MODIFICATOR Weights(long k,long l,double c[],double b[])
//
// The function for computation of weights coefficients for moving average.
// Designations:
//  k - width of window,
//  l - parameter of polynomial approximation,
//  c - matrix transformations size (l + 1)(l + 1),
//  b - matrix transformations size (l + 1)(2k + 1).
// Return code:
//   0 when normal end of computation,
//  -1 when lack of memory for computation,
//  -2 loss of significance or error of computation.
//
{
 long i,j,m = 0,       // Counter
       Code,           // Return code
       k2 = 2 * k + 1, // Parameter
       l1 = l + 1;     // Parameter

 double *a;            // Work array

 if ((a = new double[k2 * l1]) == 0)
   return -1;

 for (j = 0; j < l1; j++)
   for (i = 0; i < k2; i++)
     a[m++] = Pow (i - k,j);

 MatrixTMultiplyMatrix (a,a,b,l1,k2,l1);
 if ((Code = InverseMatrix (b,c,l1)) == 0)
   MatrixMultiplyMatrixT (c,a,b,l1,l1,k2);

 delete [] a;

 return Code;
}

void MODIFICATOR Fourier(double Y[],long N,long H,double A[],double B[])
//
// The function for computation of coefficients harmonic series.
// Designations:
//  Y - values of empirical function,
//  N - number values of function,
//  H - number harmonics,
//  A - vector of coefficients before COS by length H + 1,
//  B - vector of coefficients before SIN by length H + 1.
// Return code:
//  none.
//
{
 long J,K; // Counter

 double D; // Work variable

 A[0] = 0;
 B[0] = 0;
 for (K = 0; K < N; K++)
   A[0] += Y[K];
 A[0] /= N;

 for (J = 1; J < H + 1; J++)
   {
    A[J] = 0;
    B[J] = 0;
    for (K = 0; K < N; K++)
      {
       D = 6.28318530718 * J * (K + 1) / N;
       A[J] += Y[K] * cos(D);
       B[J] += Y[K] * sin(D);
      }
    A[J] = A[J] * 2 / N;
    B[J] = B[J] * 2 / N;
   }
}

void MODIFICATOR MovingAverage(double *X,double *Y,double *E1,double *E2,long N,long K,long L,double F)
//
// The function for computation of moving average.
// Designations:
//   X - given time series
//   Y - smoothed row
//  E1 - hight confidence interval
//  E2 - lof confidence interval
//   N - size of series
//   K - size of window
//   L - degree of interpolation polymon
//   F - confidence probability
// Return code:
//  none.
// References:
//  Brandt S. Data analysis. Statistical and computational methods for scientists and engineers. - New York, NY: Springer-Verlag, 2003.
//
{
 long I,J,    // Cycle counter
      M,      // Array counter
      P;      // Counter internal of intervals

 double *C,   // Matrix transformations
        *C1,  // Matrix transformations (vector form)
        *A,   // Matrix transformations
        *A1,  // Matrix transformations (vector form)
        *Z,   // Array of coefficients of polymon
        *S,   // Array mean-square deviations
        Save; // Work variable

 C = new double[(L + 1) * (L + 1)];
 C1 = new double[(L + 1) * (L + 1)];
 A = new double[(L + 1) * (2 * K + 1)];
 A1 = new double[(L + 1) * (2 * K + 1)];
 Z = new double[L + 1];
 S = new double[N];

 ///////////////////
 // Work computation
 Weights(K, L, C1, A1);
 for (I = 0; I < (L + 1) * (L + 1); I++)
   C[I] = C1[I];
 for (I = 0; I < (L + 1) * (2 * K + 1); I++)
   A[I] = A1[I];

 /////////////////////
 // Internal intervals
 for (P = K + 1; P <= N - K; P++)
   {
    for (I = P - K, M = 0; I <= P + K; I++)
      A1[M++] = X[I - 1];
    for (I = 0; I < L + 1; I++)
      for (J = 0, Z[I] = 0; J < 2 * K + 1; J++)
        Z[I] += A[I + J * (L + 1)] * A1[J];
    Y[P - 1] = Z[0];
    S[P - 1] = 0;
    for (I = -K; I <= K; I++)
      {
       for (J = 0, Save = 0; J <= L; J++)
         Save += Z[J] * Pow(I, J);
       S[P - 1] += (X[P - 1] - Save) * (X[P - 1] - Save);
      }
    S[P - 1] = sqrt(S[P - 1] / (2 * K - L));
   }

 /////////////////
 // Left intervals
 for (I = 0; I < 2 * K + 1; I++)
   A1[I] = X[I];
 for (I = 0; I < L + 1; I++)
   for (J = 0, Z[I] = 0; J < 2 * K + 1; J++)
     Z[I] += A[I + J * (L + 1)] * A1[J];
 for (I = 1; I <= K; I++)
   {
    for (J = 0, Y[I - 1] = 0; J <= L; J++)
      Y[I - 1] += Z[J] * Pow(I - K - 1, J);
    S[I - 1] = S[K];
   }

 //////////////////
 // Right intervals
 for (I = N - 2 * K, M = 0; I <= N; I++)
   A1[M++] = X[I - 1];
 for (I = 0; I < L + 1; I++)
   for (J = 0, Z[I] = 0; J < 2 * K + 1; J++)
     Z[I] += A[I + J * (L + 1)] * A1[J];
 for (I = N - K + 1; I <= N; I++)
   {
    for (J = 0, Y[I - 1] = 0; J <= L; J++)
      Y[I - 1] += Z[J] * Pow(I + K - N, J);
    S[I - 1] = S[N - K - 1];
   }

 ///////////////////////
 // Confidence intervals
 Save = C[0] * InverseTDistribution(2 * K - L, (1 + F) / 2);
 for (P = 0; P < N; P++)
   {
    E1[P] = Y[P] + S[P] * Save;
    E2[P] = Y[P] - S[P] * Save;
   }

 delete [] C; delete [] C1; delete [] A; delete [] A1; delete [] Z; delete [] S;
}

double MODIFICATOR BoxCox (double *x,long n)
//
// The function of Box-Cox transformation.
// Designations:
//  x - given sample,
//  n - sample size.
// Return code:
//  optimal value of parameter.
// References:
//  Sakia R.M. The Box-Cox transformation technique: A review // The Statistician, 1992, vol. 41, no. 2, pp. 169-178.
//  Box G.E.P., Cox D.R. An analysis of transformations // Journal of Royal Statistical Society: Series B (Statistical Methodology), 1964, vol. 26, no. 2, pp. 211-246.
//  Armitage P., Berry G., Matthews J.N.S. Statistical methods in medical research. - Oxford, UK: Blackwell Science, 2001.
//
{
 double a = -10,          // Left bound of root localization
        b = 10,           // Right bound of root localization
        m,                // Work variable
        eps = 0.00000001, // Parameter
        delta = 0.000001; // Parameter

 /////////////////////////////////////////////////
 // Maximum search by method of dichotomous search
 do
   {
    m = (a + b) / 2;

    if (BoxCoxML (x,n,m - eps) > BoxCoxML (x,n,m + eps))
      b = m;
    else
      a = m;

   } while (b - a > delta);

 return (a + b) / 2;
}

double MODIFICATOR BoxCoxML (double *x, long n, double l)
//
// The function for computation logarithm of maximum likelihood function of Box-Cox transformation.
// Designations:
//  x - given sample,
//  n - sample size,
//  l - parameter.
// Return code:
//  value of logarithm of maximum likelihood function.
//
{
 long i;        // Counter

 double y,      // Current value of transformation
        s = 0,  // Variance of converted data
        y2 = 0, // Work variable
        lx = 0; // Work variable

 for (i = 0; i < n; i++)
   {
    y = BoxCoxTransform(x[i],l);
    s += y * y;
    y2 += y;
    lx += log(x[i]);
   }

 s = (s - y2 * y2 / n) / (n - 1);

 if (s > 0)
   return (l - 1) * lx - log (s) / 2 * n;
 else
   return 0;
}

double MODIFICATOR BoxCoxTransform (double x, double l)
//
// The function for computation of Box-Cox transformation.
// Designations:
//  x - given member,
//  l - parameter.
// Return code:
//  value of transformed member.
//
{
 if (l == 0)
   return log(x);
 else
   return (pow(x,l) - 1) / l;
}

double MODIFICATOR Asinh (double *x,long n)
//
// The function of transformation of hyperbolic arcsinus.
// Designations:
//  x - given sample,
//  n - sample size.
// Return code:
//  optimal value of parameter.
// References:
//  Linton O.B. An analysis of transformations for additive nonparametric regression / O.B. Linton, R. Chen, N. Wang et al. // Journal of the American Statistical Association, 1997, vol. 92, no. 440, pp. 1512-1521.
//  Johnson R.A. Transformation of survival data // Survival Analysis: Proceedings of the Special Topics Meeting, 16-28 October, 1981 (Hayward, California, USA: Institute of Mathematical Statistics, 1982), pp. 118-136.
//
{
 double a = 0.0000001,    // Left bound of root localization
        b = 2,            // Right bound of root localization
        m,                // Work variable
        eps = 0.00000001, // Parameter
        delta = 0.000001; // Parameter

 /////////////////////////////////////////////////
 // Maximum search by method of dichotomous search
 do
   {
    m = (a + b) / 2;

    if (AsinhML (x,n,m - eps) > AsinhML (x,n,m + eps))
      b = m;
    else
      a = m;

   } while (b - a > delta);

 return (a + b) / 2;
}

double MODIFICATOR AsinhML (double *x, long n, double l)
//
// The function for computation of logarithm of maximum likelihood function of hyperbolic arcsinus transformation.
// Designations:
//  x - given sample,
//  n - sample size,
//  l - parameter.
// Return code:
//  value of logarithm of maximum likelihood function.
//
{
 long i;       // Counter

 double y,      // Current value of transformation
        s = 0,  // Variance of transformed data
        y2 = 0, // Work variable
        lx = 0; // Work variable

 for (i = 0; i < n; i++)
   {
    y = AsinhTransform(x[i],l);
    s += y * y;
    y2 += y;
    lx -= log(l * l * x[i] * x[i] + 1);
   }

 s = (s - y2 * y2 / n) / (n - 1);

 return lx / 2 - log (s) / 2 * n;
}

double MODIFICATOR AsinhTransform (double x, double l)
//
// The function for computation of hyperbolic arcsinus transformation.
// Designations:
//  x - given member,
//  l - parameter.
// Return code:
//  value of transformed member.
//
{
  return Arsh(l * x) / l;
}

double MODIFICATOR ZellnerRevankar (double *x,long n)
//
// The function for computation of Zellner-Revankar transformation.
// Designations:
//  x - given sample,
//  n - sample size.
// Return code:
//  optimal value of parameter.
// References:
//  Zellner A., Revankar N.S. Generalized production functions // Review of Economic Studies, 1969, vol. 37, no. 2, pp. 241-250.
//  Linton O.B. An analysis of transformations for additive nonparametric regression / O.B. Linton, R. Chen, N. Wang et al. // Journal of the American Statistical Association, 1997, vol. 92, no. 440, pp. 1512-1521.
//
{
 double a = 0.0000001,    // Left bound of root localization
        b = 1,            // Right bound of root localization
        m,                // Work variable
        eps = 0.00000001, // Parameter
        delta = 0.000001; // Parameter

 /////////////////////////////////////////////////
 // Maximum search by method of dichotomous search
 do
   {
    m = (a + b) / 2;

    if (ZellnerRevankarML (x,n,m - eps) > ZellnerRevankarML (x,n,m + eps))
      b = m;
    else
      a = m;

   } while (b - a > delta);

 return (a + b) / 2;
}

double MODIFICATOR ZellnerRevankarML (double *x, long n, double l)
//
// The function for computation of logarithm of maximum likelihood function of ZellnerRevankar transformation.
// Designations:
//  x - given sample,
//  n - sample size,
//  l - parameter.
// Return code:
//  value of logarithm of maximum likelihood function.
//
{
 long i;       // Counter

 double y,      // Current value of transformation
        s = 0,  // Variance of transformed data
        y2 = 0, // Work variable
        lx = 0; // Work variable

 for (i = 0; i < n; i++)
   {
    y = ZellnerRevankarTransform(x[i],l);
    s += y * y;
    y2 += y;
    lx += log(1.0 / x[i] + 2 * l * x[i]);
   }

 s = (s - y2 * y2 / n) / (n - 1);

 return lx - log (s) / 2 * n;
}

double MODIFICATOR ZellnerRevankarTransform (double x, double l)
//
// The function for computation of Zellner-Revankar transformation.
// Designations:
//  x - given member,
//  l - parameter.
// Return code:
//  value of transformed member.
//
{
  return log(x) + l * x * x;
}

double MODIFICATOR YeoJohnson (double *x,long n)
//
// The function for computation of Yeo-Johnson transformation.
// Designations:
//  x - given sample,
//  n - sample size.
// Return code:
//  optimal value of parameter.
// References:
//  Weisberg S. Applied linear regression. - New York, NY: John Wiley & Sons, 2005.
//
{
 double a = -2,           // Left bound of root localization
        b = 2,            // Right bound of root localization
        m,                // Work variable
        eps = 0.00000001, // Parameter
        delta = 0.000001; // Parameter

 /////////////////////////////////////////////////
 // Maximum search by method of dichotomous search
 do
   {
    m = (a + b) / 2;

    if (YeoJohnsonML (x,n,m - eps) > YeoJohnsonML (x,n,m + eps))
      b = m;
    else
      a = m;

   } while (b - a > delta);

 return (a + b) / 2;
}

double MODIFICATOR YeoJohnsonML (double *x, long n, double l)
//
// The function for computation of logarithm of maximum likelihood function of Yeo-Johnson transformation.
// Designations:
//  x - given sample,
//  n - sample size,
//  l - parameter.
// Return code:
//  value of logarithm of maximum likelihood function.
//
{
 long i;       // Counter

 double y,      // Current value of transformation
        s = 0,  // Variance of transformed data
        y2 = 0, // Work variable
        lx = 0; // Work variable

 for (i = 0; i < n; i++)
   {
    y = YeoJohnsonTransform(x[i],l);
    s += y * y;
    y2 += y;
    if (x[i] >= 0)
      lx += log(pow(x[i] + 1,l - 1));
    else
      lx += log(pow(1 - x[i],1 - l));
   }

 s = (s - y2 * y2 / n) / (n - 1);

 return lx - log (s) / 2 * n;
}

double MODIFICATOR YeoJohnsonTransform (double x, double l)
//
// The function for computation of Yeo-Johnson transformation.
// Designations:
//  x - given member,
//  l - parameter.
// Return code:
//  value of transformed member.
//
{
 if (x < 0)
   {
    if (l == 2)
      return -log(1 - x);
    else
      return -(pow(1 - x,2 - l) - 1) / (2 - l);
   }
 else
   {
    if (l == 0)
      return log(x + 1);
    else
      return (pow(x + 1,l) - 1) / l;
   }
}

double MODIFICATOR JohnDraper (double *x,long n)
//
// The function for computation of John-Draper transformation.
// Designations:
//  x - given sample,
//  n - sample size.
// Return code:
//  optimal value of parameter.
// References:
//  John J.A., Draper N.R. An alternative family of transformations // Applied Statistics, 1980, Vol. 29, No. 2, pp. 190-197.
//  Von Hippel P.T. Normalization // Encyclopedia of Social Science Research Methods. - Thousand Oaks, CA: SAGE Publications, 2003.
//  Chen C.-R., Wang L.-C. Likelihood inference under the general response transformation model with heteroscedastic errors // Taiwanese Journal of Mathematics, June 2003, vol. 7, no. 2, pp. 261-273.
//
{
 double a = -2,           // Left bound of root localization
        b = 2,            // Right bound of root localization
        m,                // Work variable
        eps = 0.00000001, // Parameter
        delta = 0.000001; // Parameter

 /////////////////////////////////////////////////
 // Maximum search by method of dichotomous search
 do
   {
    m = (a + b) / 2;

    if (JohnDraperML (x,n,m - eps) > JohnDraperML (x,n,m + eps))
      b = m;
    else
      a = m;

   } while (b - a > delta);

 return (a + b) / 2;
}

double MODIFICATOR JohnDraperML (double *x, long n, double l)
//
// The function for computation of logarithm of maximum likelihood function of John-Draper transformation.
// Designations:
//  x - given sample,
//  n - sample size,
//  l - parameter.
// Return code:
//  value of logarithm of maximum likelihood function.
//
{
 long i;        // Counter

 double y,      // Current value of transformation
        s = 0,  // Variance of transformed data
        y2 = 0, // Work variable
        lx = 0; // Work variable

 for (i = 0; i < n; i++)
   {
    if (l == 0)
      y = SIGN(x[i]) * log(fabs(x[i]) + 1);
    else
      y = SIGN(x[i]) * (pow(fabs(x[i]) + 1,l) - 1) / l;
    s += y * y;
    y2 += y;
    if (l == 0)
      lx += SIGN(x[i]) * log(1 / (fabs(x[i]) + 1));
    else
      lx += SIGN(x[i]) * log(pow(fabs(x[i]) + 1,l - 1));
   }

 s = (s - y2 * y2 / n) / (n - 1);

 return lx - log (s) / 2 * n;
}

double MODIFICATOR JohnDraperTransform (double x, double l)
//
// The function for computation of John-Draper transformation.
// Designations:
//  x - given member,
//  l - parameter.
// Return code:
//  value of transformed member.
//
{
 if (l == 0)
   return SIGN(x) * log(fabs(x) + 1);
 else
   return SIGN(x) * (pow(fabs(x) + 1,l) - 1) / l;
}

double MODIFICATOR Manly (double *x,long n)
//
// The function for computation of Manly transformation.
// Designations:
//  x - given sample,
//  n - sample size.
// Return code:
//  optimal value of parameter.
// References:
//  Manly B.F.J. Exponential data transformations // The Statistician, March 1976, vol. 25, no. 1, pp. 37-42.
//
{
 double a = -2,           // Left bound of root localization
        b = 2,            // Right bound of root localization
        m,                // Work variable
        eps = 0.00000001, // Parameter
        delta = 0.000001; // Parameter

 /////////////////////////////////////////////////
 // Maximum search by method of dichotomous search
 do
   {
    m = (a + b) / 2;

    if (ManlyML (x,n,m - eps) > ManlyML (x,n,m + eps))
      b = m;
    else
      a = m;

   } while (b - a > delta);

 return (a + b) / 2;
}

double MODIFICATOR ManlyML (double *x, long n, double l)
//
// The function for computation of  logarithm of maximum likelihood function of Manly transformation.
// Designations:
//  x - given sample,
//  n - sample size,
//  l - parameter.
// Return code:
//  value of logarithm of maximum likelihood function.
//
{
 long i;        // Counter

 double y,      // Current value of transformation
        s = 0,  // Variance of transformed data
        y2 = 0, // Work variable
        lx = 0; // Work variable

 for (i = 0; i < n; i++)
   {
    y = ManlyTransform(x[i],l);
    s += y * y;
    y2 += y;
    lx += log(exp(x[i] * l));
   }

 s = (s - y2 * y2 / n) / (n - 1);

 return lx - log (s) / 2 * n;
}

double MODIFICATOR ManlyTransform (double x, double l)
//
// The function for computation of Manly transformation.
// Designations:
//  x - given member,
//  l - parameter.
// Return code:
//  value of transformed member.
//
{
 if (l == 0)
   return x;
 else
   return (exp(x * l) - 1) / l;
}

long MODIFICATOR BoxCoxMulti (double *x,long n,double *l,long m)
//
// The function for computation of multivariate Box-Cox transformation.
// Designations:
//  x - given sample,
//  n - sample size,
//  m - sample size,
//  l - vector parameters.
// Return code:
//   0 - when normal end of compute,
//  -1 - when lack of memory for work array,
//  -2 - when the maximum allowed number of iterations is reached.
// References:
//  Andrews D.F., Gnanadesikan R., Warner J.L. Transformations of multivariate data // Biometrics, 1971, vol. 27, no. 4, pp. 825-840.
//  Williams J.T. Joint multipoint linkage analysis of multivariate qualitative and quantitative traits. I. Likelihood formulation and simulation results / J.T. Williams, P. Van Eerdewegh, L. Almasy et al. // The American Journal of Human Genetics, October 1999, vol. 65, no. 4, pp. 1134-1147.
//
{
 long i,                 // Counter
      Max = 100;         // Maximum allowed number of iterations

 double Eps = 0.00001,   // Stop parameter
        Step = 0.000001; // Rocking step

 /////////////////
 // Initial values
 for (i = 0; i < m; i++)
   l[i] = 0.5;

 return BFGS (x,n,l,m,Eps,&Max,Step,Gradient,BoxCoxMLMulti);
}

double BoxCoxMLMulti (double *x, long n, double *l, long m)
//
// The function for computation of logarithm of maximum likelihood function of multidimensional Box-Cox transformation.
// Designations:
//  x - given sample,
//  n - sample size,
//  m - sample size,
//  l - parameter.
// Return code:
//  value of the logarithm of maximum likelihood function.
//
{
 long i,j;       // Counter

 double *z,      // Current value of variable
        *y,      // Current value of transformation
        *y1,     // result of transformation
        *r,      // Covariation matrix
        s,       // Determinant
        buf = 0, // Work variable
        *lx;     // Work variable

 z = new double[m];
 y = new double[m];
 y1 = new double[n * m];
 lx = new double[m];
 r = new double[m * m];

 for (i = 0; i < m; i++)
   lx[i] = 0;

 for (i = 0; i < n; i++)
   {
    for (j = 0; j < m; j++)
      z[j] = x[j * n + i];
    BoxCoxTransformMulti(z,m,l,y);
    for (j = 0; j < m; j++)
      {
       y1[j * n + i] = y[j];
       lx[j] += log(z[j]);
      }
   }

 CovMatrix (y1,r,n,m);
 s = Krauth (r,m,0.000000001);

 for (i = 0; i < m; i++)
   buf += (l[i] - 1) * lx[i];

 delete [] z; delete [] y1; delete [] y; delete [] lx; delete [] r;

 return buf - log (s) / 2 * n;
}

void MODIFICATOR BoxCoxTransformMulti (double *x, long m, double *l,double *y)
//
// The function for computation multidimensional Box-Cox transformation.
// Designations:
//  x - given value,
//  m - sample size,
//  l - parameter,
//  y - converted value.
// Return code:
//  none.
//
{
 long i; // Counter

 for (i = 0; i < m; i++)
   if (l[i] == 0)
     y[i] = log(x[i]);
   else
     y[i] = (pow(x[i],l[i]) - 1) / l[i];
}

long BFGS (double *x,long n,double *y,long m,double Eps,long *Max,double Step,userFuncGrad Grad,userFuncRight Right)
//
// The function for minimization of function of many variables by the method
// of variable metrics with inversed hessian approximation by the
// Broyden-Fletcher-Goldfarb-Shanno algorithm.
// Designations:
//      x - given sample,
//      n - sample size,
//      y - on entry - initial approximations vector by length m, on exit - solution vector,
//    Eps - computation interruption constant (square of Euclid norma),
//   *Max - on entry maximal acceptable iteration number, on exit - spent,
//   Step - rocking step for gradient computation,
//   Grad - user defined function for computation of gradient,
//  Right - user defined function for computation optimazed function.
// Return code:
//   0 - when normal end of compute,
//  -1 - when lack of memory for work array,
//  -2 - when the maximum allowed number of iterations is reached.
// References:
//  Dennis J.E,Jr., Schnabel R.B. Numerical methods for unconstrained optimization and nonlinear equation. - Englewood Cliffs, NJ: Prentice-Hall, 1983.
//
{
 long i,j,      // Cycle counter
      l,        // Array counter
      Ret = -2, // Return code
      Iter = 0; // Iteration counter

 double *a,     // Approximaton of inversed hessian
        *d,     // Direction for minimum search
        *g,     // Vector of gradient
        *s,     // Vector of gradient on actual step, later - gradients difference,
        *z,     // Work array
        r1,r2;  // Work variable

 if ((g = new double[m]) == 0)
   return -1;
 if ((s = new double[m]) == 0)
   {
    delete [] g;
    return -1;
   }
 if ((a = new double[m * m]) == 0)
   {
    delete [] g; delete [] s;
    return -1;
   }
 if ((z = new double[m]) == 0)
   {
    delete [] g; delete [] s; delete [] a;
    return -1;
   }
 if ((d = new double[m]) == 0)
   {
    delete [] g; delete [] s; delete [] a; delete [] z;
    return -1;
   }

 ///////////////////////////////////////////
 // Computation of gradient on 1st iteration
 Grad (x,n,y,g,m,Step,Right);

 ////////////////////////////////////
 // Initial value of inversed hessian
 MatrixUnit (a,m);

 while (Iter++ < *Max)
   {
    ////////////////////////////////////////
    // Computation of optimal iteration step
    // by the Hartley quadratic interpolation method
    for (i = 0,l = 0; i < m; i++)
      {
       for (j = 0,d[i] = 0; j < m; j++)
     d[i] += a[l++] * g[j];
       z[i] = y[i] - 0.5 * d[i];
       s[i] = y[i] - d[i];
      }
    r1 = Right (x,n,y,m);
    r2 = Right (x,n,s,m);
    r1 = (r2 - r1) / (r1 - Right (x,n,z,m) * 2.0 + r2) / 4 - 0.5;

    /////////////////////
    // Next approximation
    for (i = 0; i < m; i++)
      {
       d[i] *= r1;
       y[i] += d[i];
      }

    //////////////////////////
    // Computation of gradient
    Grad (x,n,y,s,m,Step,Right);

    //////////////////////////////////////////////////
    // Difference of gradients and square of its norma
    for (i = 0,r1 = 0; i < m; i++)
      {
       s[i] -= g[i];
       r1 += s[i] * s[i];
      }

    ////////////////////////
    // Normal end of compute
    if (r1 <= Eps)
      {
       Ret = 0;
       break;
      }

    ///////////////////////////////////
    // Approximation of inverse hessian
    for (i = 0,l = 0,r1 = 0,r2 = 0; i < m; i++)
      {
       for (j = 0,z[i] = 0; j < m; j++)
     z[i] += a[l++] * s[j];
       r1 += d[i] * s[i];
       r2 += z[i] * s[i];
       g[i] += s[i];
      }
    for (i = 0,l = 0; i < m; i++)
      for (j = 0; j < m; j++)
    a[l++] += d[i] * d[j] / r1 - z[i] * z[j] / r2;
   }

 *Max = Iter;

 delete [] g; delete [] s; delete [] a; delete [] z; delete [] d;

 return Ret;
}

void Gradient (double *x,long n,double *y,double *g,long m,double s,userFuncRight Right)
//
// The function for computation of gradient vector by right deffirences.
// Designations:
//      x - given sample,
//      n - sample size,
//      y - vector variables by length m,
//      g - vector of gradient by length m,
//      s - rocking step,
//  Right - defined by user function for computation of optimized function.
// Return code:
//  none.
//
{
 for (long i = 0; i < m; i++)
   {
    y[i] += s;
    g[i] = Right (x,n,y,m);
    y[i] -= s;
    g[i] = (g[i] - Right (x,n,y,m)) / s;
   }
}

void MODIFICATOR Somers(double x[], long n, long k, double *dx, double *dy)
//
// The function for computation of Somers coefficient.
// Designations:
//    x - contingency table,
//    n - number of rows,
//    k - number of columns,
//  *dx - coefficient Dx,
//  *dy - coefficient Dy.
// Return code:
//  none.
//
{
 long i,j,m,l;  // Counter

 double p = 0,  // Parameter
        q = 0,  // Parameter
        t1 = 0, // Parameter
        t2 = 0, // Parameter
        s;      // Parameter, later sum of table

 for (j = 1; j <= k - 1; j++)
   for (i = 1; i <= n - 1; i++)
     {
      for (m = j + 1, s = 0; m <= k; m++)
        for (l = i + 1; l <= n; l++)
          s += x[(l - 1) + (m - 1) * n];
      p += x[(i - 1) + (j - 1) * n] * s;
     }
 for (j = 2; j <= k; j++)
   for (i = 1; i <= n - 1; i++)
     {
      for (m = 1, s = 0; m <= j - 1; m++)
        for (l = i + 1; l <= n; l++)
          s += x[(l - 1) + (m - 1) * n];
      q += x[(i - 1) + (j - 1) * n] * s;
     }
 for (i = 1; i <= n; i++)
   {
    for (j = 1, s = 0; j <= k; j++)
      s += x[(i - 1) + (j - 1) * n];
    t1 += s * (s - 1);
   }
 t1 = t1 / 2;
 for (i = 1; i <= k; i++)
   {
    for (j = 1, s = 0; j <= n; j++)
      s += x[(j - 1) + (i - 1) * n];
    t2 += s * (s - 1);
   }
 t2 = t2 / 2;

 for (i = 0, s = 0; i < n * k; i++)
   s += x[i];
 s = s * (s - 1) / 2;
 *dx = (p - q) / (s - t1);
 *dy = (p - q) / (s - t2);
}

double Kendall(double x[], long n, long k, double *d)
//
// The function for computation of Kendall tau-b coefficient.
// Designations:
//   x - contingency table,
//   n - number of rows,
//   k - number of columns,
//  *d - variance.
// Return code:
//  value of Kendall coefficient.
// References:
//  Van der Waerden B.L. Mathematische Statistik (in Deutsch). - Berlin: Springer, 1971.
//
{
 long i,j,m,l;  // Counter

 double p = 0,  // Parameter
        q = 0,  // Parameter
        t1 = 0, // Parameter
        t2 = 0, // Parameter
        s;      // Parameter, later Sum of table

 for (j = 1; j <= k - 1; j++)
   for (i = 1; i <= n - 1; i++)
     {
      for (m = j + 1, s = 0; m <= k; m++)
        for (l = i + 1; l <= n; l++)
          s += x[(l - 1) + (m - 1) * n];
      p += x[(i - 1) + (j - 1) * n] * s;
     }
 for (j = 2; j <= k; j++)
   for (i = 1; i <= n - 1; i++)
     {
      for (m = 1, s = 0; m <= j - 1; m++)
        for (l = i + 1; l <= n; l++)
          s += x[(l - 1) + (m - 1) * n];
      q += x[(i - 1) + (j - 1) * n] * s;
     }
 for (i = 1; i <= n; i++)
   {
    for (j = 1, s = 0; j <= k; j++)
      s += x[(i - 1) + (j - 1) * n];
    t1 += s * (s - 1);
   }
 t1 = t1 / 2;
 for (i = 1; i <= k; i++)
   {
    for (j = 1, s = 0; j <= n; j++)
      s += x[(j - 1) + (i - 1) * n];
    t2 += s * (s - 1);
   }
 t2 = t2 / 2;

 for (i = 0, s = 0; i < n * k; i++)
   s += x[i];
 *d = (4 * s + 10) / (9 * s * s - s);
 s = s * (s - 1) / 2;

 return (p - q) / sqrt((s - t1) * (s - t2));
}

double MODIFICATOR Cramer(double x[], long n, long m, double *d)
//
// The function for computation of Cramer contingency coefficient.
// Designations:
//   x - contingency table,
//   n - number of rows,
//   m - number of columns,
//  *d - variance.
// Return code:
//  value of Cramer coefficient.
//
{
 long i;              // Counter

 double q = MIN(n,m), // Parameter
        nn = 0;       // Sum of table

 for (i = 0; i < n * m; i++)
   nn += x[i];
 *d = 1.0 / (q - 1) / nn;

 return sqrt(ChiSquare(x, n, m) / (q - 1) / nn);
}

double MODIFICATOR PearsonX(double x[],long n,long m,double *c)
//
// The function for computation of Pearson contingency coefficient.
// Designations:
//   x - contingency table,
//   n - number of rows,
//   m - number of columns,
//  *c - chi-square value.
// Return code:
//  value of Pearson coefficient.
//
{
 long i;        // Counter

 double nn = 0; // Sum of table

 *c = ChiSquare(x, n, m);

 for (i = 0; i < n * m; i++)
   nn += x[i];

 return sqrt((*c) / ((*c) + nn));
}

double MODIFICATOR Hellinger(double x[], long n, long m, double z)
//
// The function for computation of blended weight Hellinger test.
// Designations:
//  x - contingency table,
//  n - number of rows,
//  m - number of columns.
// Return code:
//  value of test.
// References:
//  Bhattacharya B., Basu A. Disparity based goodness-of-fit tests for and against isotonic order restrictions for multinomial models // Journal of Nonparametric Statistics, 2003, vol. 15, no. 1, pp. 1-10.
//  Basu A. Improved power in multinomial goodness-of-fit tests / A. Basu, S. Ray, C. Park et al. // Journal of the Royal Statistical Society: Series D (The Statistician), 2002, vol. 51, pp. 381-393.
//
{
 long i,j,l;    // Counters

 double *s,     // Sums of rows
        *t,     // Sums of columns
        nn = 0, // Sum of table
        ed,d,   // Work variable
        c = 0;  // Return code

 s = new double[n];
 t = new double[m];

 for (i = 0; i < n; i++)
   {
    s[i] = 0;
    for (j = 0, l = i; j < m; j++, l += n)
      s[i] += x[l];
    nn += s[i];
   }
 for (i = 0, l = 0; i < m; i++)
   {
    t[i] = 0;
    for (j = 0; j < n; j++)
      t[i] += x[l++];
   }
 for (i = 0; i < n; i++)
   for (j = 0, l = i; j < m; j++, l += n)
     {
      ed = s[i] * t[j] / nn;
      d = (x[l] - ed) / (z * sqrt(x[l]) + (1 - z) * sqrt(ed));
      c += d * d;
     }

 delete [] s; delete [] t;

 return c;
}

double MODIFICATOR CressieRead(double x[], long n, long m, double z)
//
// The function for computation of power-divergence family Cressie-Read test.
// Designations:
//  x - contingency table,
//  n - number of rows,
//  m - number of columns.
// Return code:
//  value of test.
// References:
//  Von Davier M. Bootstrapping goodness-of-fit statistics for sparse categorical data - results of a Monte Carlo study // Methods of Psychological Research Online, 1997, vol.2, no. 2.
//  Bravo F. Bartlett-type adjustments for empirical discrepancy test statistics // Economics Working Paper Archive at York, 2004, vol. 14.
//  Basu A. Improved power in multinomial goodness-of-fit tests / A. Basu, S. Ray, C. Park et al. // Journal of the Royal Statistical Society: Series D (The Statistician), 2002, vol. 51, pp. 381-393.
//
{
 long i,j,l;    // Counters

 double *s,     // Sums of rows
        *t,     // Sums of columns
        nn = 0, // Sum of table
        ed,     // Work variable
        c = 0;  // Return code

 s = new double[n];
 t = new double[m];

 for (i = 0; i < n; i++)
   {
    s[i] = 0;
    for (j = 0, l = i; j < m; j++, l += n)
      s[i] += x[l];
    nn += s[i];
   }
 for (i = 0, l = 0; i < m; i++)
   {
    t[i] = 0;
    for (j = 0; j < n; j++)
      t[i] += x[l++];
   }
 for (i = 0; i < n; i++)
   for (j = 0, l = i; j < m; j++, l += n)
     {
      ed = s[i] * t[j] / nn;
      if (ed != 0 && x[l] != 0)
        c += x[l] * (exp(z * log(x[l] / ed)) - 1) / z / (z + 1);
     }

 delete [] s; delete [] t;

 return c * 2;
}

double MODIFICATOR LikelihoodRatio(double x[], long n, long m)
//
// The function for computation likelihood ratio test.
// Designations:
//  x - contingency table,
//  n - number of rows,
//  m - number of columns.
// Return code:
//  value of test.
// References:
//  Mehta C.R. , Patel N.R., Gray R. Computing an exact confidence interval for the common odds ratio in several 2 x 2 contingency tables // Journal of the American Statistical Association, 1985, vol. 80, no. 392, pp. 969-973.
//  Mehta C.R., Patel N.R. A network algorithm for performing Fisher-s exact test in r x c contingency tables // Journal of the American Statistical Association, 1983, vol. 78, no. 382, pp. 427-434.
//  Mehta C.R., Patel N.R. A network algorithm for the exact treatment of the 2 x K contingency table // Communications in Statistics: Simulation and Computation, 1980, vol. 9, pp. 649-664.
//  Mehta C.R., Patel N.R. Exact inference for categorical data // Biometrics, 1997, vol. 53, no. 1, 112-117.
//  Mehta C.R., Patel N.R., Gray R. Correction: Computing an exact confidence interval for the common odds ratio in several 2 x 2 contingency tables // Journal of the American Statistical Association, 1986, vol. 81, no. 396, p. 1132.
//
{
 long i,j,l;    // Counters

 double *s,     // Sums of rows
        *t,     // Sums of columns
        nn = 0, // Sum of table
        ed,     // Work variable
        c = 0;  // Return code

 s = new double[n];
 t = new double[m];

 for (i = 0; i < n; i++)
   {
    s[i] = 0;
    for (j = 0, l = i; j < m; j++, l += n)
      s[i] += x[l];
    nn += s[i];
   }
 for (i = 0, l = 0; i < m; i++)
   {
    t[i] = 0;
    for (j = 0; j < n; j++)
      t[i] += x[l++];
   }
 for (i = 0; i < n; i++)
   for (j = 0, l = i; j < m; j++, l += n)
     {
      ed = s[i] * t[j] / nn;
      if (ed != 0 && x[l] != 0)
        c += x[l] * log(x[l] / ed);
     }
 delete [] s; delete [] t;

 return c * 2;
}

double MODIFICATOR Haberman(double x[], long n, long m)
//
// The function for computation Haberman diagnostic.
// Designations:
//  x - contingency table,
//  n - number of rows,
//  m - number of columns.
// Return code:
//  value of diagnostic.
// References:
//  Khromov-Borisov N.N., Smolyanitsky A.G. Comprehensive catalog of statistical formulae, algorithms and software - step towards good statistics practice // Rechtsmedizin, 2003, No. 4, p. 278.
//  Lazzarotto G.B. SANCT - methodology and software for the structural analysis of forensic population data / G.B. Lazzarotto, N.N. Khoromov-Borisov, T.B.L. Kist et al. // Rechtsmedizin, 2003, No. 2, p. 279.
//
{
 long i,j,l;        // Counters

 double *s,         // Sums of rows
        *t,         // Sums of columns
        nn = 0,     // Sum of table
        nm = n * m, // Number of cells
        c = 0;      // Return code

 s = new double[n];
 t = new double[m];

 for (i = 0; i < n; i++)
   {
    s[i] = 0;
    for (j = 0, l = i; j < m; j++, l += n)
      s[i] += x[l];
    nn += s[i];
   }
 for (i = 0, l = 0; i < m; i++)
   {
    t[i] = 0;
    for (j = 0; j < n; j++)
      t[i] += x[l++];
   }
 for (i = 0; i < n; i++)
   for (j = 0, l = i; j < m; j++, l += n)
     c = c + nn / s[i] / t[j] - nm / nn;

 delete [] s; delete [] t;

 return c / sqrt((nm - 1) * 32);
}

double MODIFICATOR SimonoffTsai(double x[], long n, long m)
//
// The function for computation Simonoff-Tsai diagnostic.
// Designations:
//  x - contingency table,
//  n - number of rows,
//  m - number of columns.
// Return code:
//  value of diagnostic.
// References:
//  Simonoff J.S., Tsai C.-L. Higher order effects in log-linear and log-non-linear models for contingency tables with ordered categories // Journal of the Royal Statistical Society. Series C (Applied Statistics), 1991, vol. 40, no. 3, pp. 449-458.
//
{
 long i,j,l;    // Counters

 double *s,     // Sums of rows
        *t,     // Sums of columns
        nn = 0, // Sum of table
        ed,     // Work variable
        c = 0;  // Return code

 s = new double[n];
 t = new double[m];

 for (i = 0; i < n; i++)
   {
    s[i] = 0;
    for (j = 0, l = i; j < m; j++, l += n)
      s[i] += x[l];
    nn += s[i];
   }
 for (i = 0, l = 0; i < m; i++)
   {
    t[i] = 0;
    for (j = 0; j < n; j++)
      t[i] += x[l++];
   }
 for (i = 0; i < n; i++)
   for (j = 0, l = i; j < m; j++, l += n)
     {
      ed = s[i] * t[j] / nn;
      if (ed != 0)
       c += (x[l] - ed) * (x[l] - ed) / ed * fabs((x[l] - ed)) / ed;
     }
 delete [] s; delete [] t;

 ed = ChiSquare(x, n, m);

 return c * sqrt(InverseChiSquareDistribution((n - 1) * (m - 1), 0.05) / ed) / ed / 3;
}

double MODIFICATOR ChiSquare(double x[], long n, long m)
//
// The function for computation the Pearson chi-square test.
// Designations:
//  x - contingency table,
//  n - number of rows,
//  m - number of columns.
// Return code:
//  value of test.
// References:
//  Mehta C.R. , Patel N.R., Gray R. Computing an exact confidence interval for the common odds ratio in several 2 x 2 contingency tables // Journal of the American Statistical Association, 1985, vol. 80, no. 392, pp. 969-973.
//  Mehta C.R., Patel N.R. A network algorithm for performing Fisher-s exact test in r x c contingency tables // Journal of the American Statistical Association, 1983, vol. 78, no. 382, pp. 427-434.
//  Mehta C.R., Patel N.R. A network algorithm for the exact treatment of the 2 x K contingency table // Communications in Statistics: Simulation and Computation, 1980, vol. 9, pp. 649-664.
//  Mehta C.R., Patel N.R. Exact inference for categorical data // Biometrics, 1997, vol. 53, no. 1, 112-117.
//  Mehta C.R., Patel N.R., Gray R. Correction: Computing an exact confidence interval for the common odds ratio in several 2 x 2 contingency tables // Journal of the American Statistical Association, 1986, vol. 81, no. 396, p. 1132.
//  Stafford J.E. Exact cumulant computations for Pearson X? and Zelterman statistics for r-way contingency tables // Journal of Computational and Graphical Statistics, 1995, vol. 4, no. 3, pp. 199-212.
//
{
 long i,j,l;    // Counters

 double *s,     // Sums of rows
        *t,     // Sums of columns
        nn = 0, // Sum of table
        ed,     // Work variable
        c = 0;  // Return code

 s = new double[n];
 t = new double[m];

 for (i = 0; i < n; i++)
   {
    s[i] = 0;
    for (j = 0, l = i; j < m; j++, l += n)
      s[i] += x[l];
    nn += s[i];
   }
 for (i = 0, l = 0; i < m; i++)
   {
    t[i] = 0;
    for (j = 0; j < n; j++)
      t[i] += x[l++];
   }
 for (i = 0; i < n; i++)
   for (j = 0, l = i; j < m; j++, l += n)
     {
      ed = s[i] * t[j] / nn;
      if (ed != 0)
       c += (x[l] - ed) * (x[l] - ed) / ed;
     }

 delete [] s; delete [] t;

 return c;
}

double MODIFICATOR Zelterman(double x[], long n, long m)
//
// The function for computation Zelterman test.
// Designations:
//  x - contingency table,
//  n - number of rows,
//  m - number of columns.
// Return code:
//  value of test.
// References:
//  Lazzarotto G.B. SANCT - methodology and software for the structural analysis of forensic population data / G.B. Lazzarotto, N.N. Khoromov-Borisov, T.B.L. Kist et al. // Rechtsmedizin, 2003, No. 2, p. 279.
//
{
 long i,j,l;    // Counters

 double *s,     // Sums of rows
        *t,     // Sums of columns
        nn = 0, // Sum of table
        ed,     // Work variable
        z = 0;  // Return code

 s = new double[n];
 t = new double[m];

 for (i = 0; i < n; i++)
   {
    s[i] = 0;
    for (j = 0, l = i; j < m; j++, l += n)
      s[i] += x[l];
    nn += s[i];
   }
 for (i = 0, l = 0; i < m; i++)
   {
    t[i] = 0;
    for (j = 0; j < n; j++)
      t[i] += x[l++];
   }
 for (i = 0; i < n; i++)
   for (j = 0, l = i; j < m; j++, l += n)
     {
      ed = s[i] * t[j] / nn;
      if (ed != 0)
       z += ((x[l] - ed) * (x[l] - ed) - x[l]) / ed;
     }
 delete [] s; delete [] t;

 return z + m * n;
}

double MODIFICATOR StuartMaxwellTest(double *x,long k)
//
// The function for computation Stuart-Maxwell test.
// Designations:
//  x - contingency table,
//  k - size of table.
// Return code:
//  value of test.
// References:
//  Maxwell A.E. Comparing the classification of subjects by two independent judges // British Journal of Psychiatry, 1970, vol. 116, pp. 651-655.
//  Stuart A.A. A test for homogeneity of the marginal distributions in a two-way classification // Biometrika, 1955, vol. 42, pp. 412-416.
//
{
 long i,j,l,      // Cycle counter
      k1 = k - 1; // Work variable

 double *r,       // Sums of rows
        *c,       // Sums of columns
        *s,       // Covariation matrix, work array
        *t,       // Inverse covariation matrix, work array
        *d,       // Vector of differences
        g;        // Value of test

 r = new double[k];
 c = new double[k];
 s = new double[k1 * k1];
 t = new double[k1 * k1];
 d = new double[k1];

 for (i = 0; i < k; i++)
   for (j = 0, r[i] = 0, l = i; j < k; j++, l += k)
     r[i] += x[l];
 for (i = 0, l = 0; i < k; i++)
   for (j = 0, c[i] = 0; j < k; j++)
     c[i] += x[l++];
 for (i = 0; i < k1; i++)
   d[i] = c[i] - r[i];
 for (i = 0, l = 0; i < k1; i++)
   for (j = 0; j < k1; j++)
     if (i == j)
       s[l++] = c[i] + r[i] - x[i + i * k] * 2;
     else
       s[l++] = - x[i + j * k] - x[i * k + j];
 if (InverseMatrix (s,t,k1) == -2)
   {
    delete [] r; delete [] c; delete [] s; delete [] t; delete [] d;
    return 0;
   }

 MatrixTMultiplyMatrix (d,t,s,1,k1,k1);
 MatrixMultiplyMatrix (s,d,t,1,k1,1);
 g = t[0];

 delete [] r; delete [] c; delete [] s; delete [] t; delete [] d;

 return g;
}

double MODIFICATOR BowkerTest(double *x,long k)
//
// The function for computation Bowker test.
// Designations:
//  x - contingency table,
//  k - size of table.
// Return code:
//  value of test.
// References:
//  Bowker A.H. A test for symmetry in contingency tables // Journal of the American Statistical Association, 1948, vol. 43, pp. 572-574.
//
{
 long i,j;     // Cycle counter

 double w = 0, // Value of test
        s;     // Work variable

 for (i = 0; i < k - 1; i++)
   for (j = i + 1; j < k; j++)
     {
      s = x[i + j * k] - x[i * k + j];
      w += s * s / (x[i + j * k] + x[i * k + j]);
     }

 return w;
}

double MODIFICATOR BhapkarTest(double *x,long k)
//
// The function for computation Bhapkar test.
// Designations:
//  x - contingency table,
//  k - size of table.
// Return code:
//  value of test.
// References:
//  Bhapkar V.P. A note on the equivalence of two test criteria for hypotheses in categorical data // Journal of the American Statistical Association, 1966, vol. 61, pp. 228-235.
//  Bhapkar V.P., Gore A.P. A distribution-free test for symmetry in hierarchical data // Journal of Multivariate Analysis, 1973, vol. 3, pp. 483-489.
//
{
 long i,j,l,      // Cycle counter
      k1 = k - 1; // Work variable

 double *r,       // Sums of rows
        *c,       // Sums of columns
        *s,       // Covariation matrix, work array
        *t,       // Inverse covariation matrix, work array
        *d,       // Vector of differnces
        n = 0,    // Sum of contingency table
        g;        // Value of test

 r = new double[k];
 c = new double[k];
 s = new double[k1 * k1];
 t = new double[k * k];
 d = new double[k1];

 for (i = 0; i < k * k; i++)
   n += x[i];
 for (i = 0; i < k * k; i++)
   t[i] = x[i] / n;
 for (i = 0; i < k; i++)
   for (j = 0, r[i] = 0, l = i; j < k; j++, l += k)
     r[i] += t[l];
 for (i = 0, l = 0; i < k; i++)
   for (j = 0, c[i] = 0; j < k; j++)
     c[i] += t[l++];
 for (i = 0; i < k1; i++)
   d[i] = c[i] - r[i];
 for (i = 0, l = 0; i < k1; i++)
   for (j = 0; j < k1; j++, l++)
     {
      if (i == j)
        s[l] = c[i] + r[i] - t[i + i * k] * 2;
      else
        s[l] = - t[i + j * k] - t[i * k + j];
      s[l] -= (c[i] - r[i]) * (c[j] - r[j]);
     }
 if (InverseMatrix (s,t,k1) == -2)
   {
    delete [] r; delete [] c; delete [] s; delete [] t; delete [] d;
    return 0;
   }

 MatrixTMultiplyMatrix (d,t,s,1,k1,k1);
 MatrixMultiplyMatrix (s,d,t,1,k1,1);
 g = t[0] * n;

 delete [] r; delete [] c; delete [] s; delete [] t; delete [] d;

 return g;
}

double MODIFICATOR FreemanHaltonMC(double *xx,long r,long c,double Max)
//
// The function for computation Freeman-Halton test by Monte-Carlo method.
// Designations:
//    xx - contingency table,
//     r - number of table rows,
//     c - number of table columns,
//   Max - number of trials.
// Return code:
//  P-value.
// References:
//  Patefield W.M. Algorithm AS 159: An efficient method of generating random R X C tables with given row and column totals // Applied Statistics, 1981, vol. 30, no. 1, pp. 91-97.
//
{
 long i,j,l,            // Counters
      r1 = r - 1,       // Number rows without unit
      c1 = c - 1,       // Number columns without unit
      n = 0,            // Size of table
      jc,rl,ia,ic,id,   // Work variables
      ie,ib,ii,nlm,nll, // Work variables
      lp,lm,            // Work variables
      *Rows,            // Marginal sums of rows
      *Cols,            // Marginal sums of columns
      *Work;            // Work array

 int seed = 123456789;  // Seed for generator

 double k,              // Counter of trials number
        buf = 0,        // Work variable
        x = 0,y,sum,    // Work variables
        Limit = 0,      // Probability of given table
        *F,             // Array of factorials
        s = 0,          // Return code
        h;              // Probability of actual configuration

 Rows = new long[r];
 Cols = new long[c];
 Work = new long[c];

 for (i = 0; i < r; i++)
   {
    Rows[i] = 0;
    for (j = 0, ii = i; j < c; j++)
      {
       Rows[i] += (long)xx[ii];
       Limit += LogFactorial(xx[ii]);
       ii = ii + r;
      }
    x += LogFactorial(Rows[i]);
   }
 ii = 0;
 for (i = 0; i < c; i++)
   {
    Cols[i] = 0;
    for (j = 0; j < r; j++)
      Cols[i] += (long)xx[ii++];
    x += LogFactorial(Cols[i]);
    n += Cols[i];
   }
 F = new double[n + 1];
 F[0] = 0;
 for (i = 1; i <= n; i++)
   {
    buf += log(i);
    F[i] = buf;
   }
 Limit = exp(x - LogFactorial(n) - Limit);
 for (k = 1; k <= Max; k += 1)
   {
    for (i = 0; i < c1; i++)
      Work[i] = Cols[i];
    jc = n;
    h = 1;
    for (l = 1; l <= r1; l++)
      {
       rl = Rows[l - 1];
       ia = rl;
       ic = jc;
       jc -= rl;
       for (i = 1; i <= c1; i++)
         {
          id = Work[i - 1];
          ie = ic;
          ic -= id;
          ib = ie - ia;
          ii = ib - id;
          if (ie == 0)
            {
             delete [] Rows; delete [] Cols;  delete [] Work; delete [] F;
             return s / Max;
            }
          buf = r8_uniform_01(&seed);
          do
            {
             nlm = floor(1.0 * ia * id / ie + 0.5);
             x = exp(F[ia] + F[ib] + F[ic] + F[id] - F[ie] -
                     F[nlm] - F[id - nlm] - F[ia - nlm] - F[ii + nlm]);
             if (x >= buf)
               {
                h *= x;
                break;
               }
             sum = x;
             y = x;
             nll = nlm;
             lp = 0;
             lm = 0;
label140:
             j = (id - nlm) * (ia - nlm);
             if (j == 0)
               lp = 1;
             else
               {
                nlm++;
                x = x * j / nlm / (ii + nlm);
                sum += x;
                if (sum >= buf)
                  {
                   h *= x;
                   break;
                  }
               }
label150:
             if (lm != 1)
               {
                j = nll * (ii + nll);
                if (j == 0)
                  lm = 1;
                else
                  {
                   nll--;
                   y = y * j / (id - nll) / (ia - nll);
                   sum += y;
                   if (sum >= buf)
                     {
                      nlm = nll;
                      h *= y;
                      break;
                     }
                   if (lp == 0)
                     goto label140;
                   goto label150;
                  }
               }

             if (lp == 0)
               goto label140;
             buf = sum * r8_uniform_01(&seed);

            } while(1);

          ia -= nlm;
          Work[i - 1] -= nlm;
         }
      }

    if (h < Limit)
      s++;
   }

 delete [] Rows; delete [] Cols;  delete [] Work; delete [] F;

 return s / Max;
}

double MODIFICATOR GailMantel (double *x,long r,long c)
//
// The function for estimation of number of contingency tables by Gail-Mantel algorithm.
// Designations:
//  x - contingency table,
//  r - number of rows,
//  c - number of columns.
// Return code:
//  estimation of number of tables.
// References:
//  Gail M., Mantel N. Counting the number of r x c contingency tables with fixed margins // Journal of the American Statistical Association, December 1977, vol. 72, no. 360, pp. 859-862.
//
{
 long i,j,l;   // Cycle counter

 double p = 0, // Return code
        s = 0, // Size of table
        d = 0, // Variance
        q = 0, // Work variable
        *m,    // Sums of rows
        *t;    // Sums of columns

 m = new double[r];
 t = new double[c];

 for (i = 0; i < r; i++)
   for (j = 0, m[i] = 0, l = i; j < c; j++, l += r)
     m[i] += x[l];
 for (i = 0, l = 0; i < c; i++)
   for (j = 0, t[i] = 0; j < r; j++)
     t[i] += x[l++];

 if (r >= c)
   {
    for (i = 0; i < r; i++)
      {
       p += LogFactorial(m[i] + c - 1) - LogFactorial(c - 1) - LogFactorial(m[i]);
       s += m[i];
       d += m[i] * (m[i] + c);
      }
    p = exp(p);
    d = d * (c - 1) / (c + 1) / c / c;
    for (i = 0; i < c; i++)
      q += t[i] * t[i];
    q = (q - s * s / c) * (c - 1) / d / c;
    p *= exp (0.5 * (c - 1) * log(0.159154943092 * (c - 1) / d / c)) * sqrt (c) / exp (q / 2);
   }
 else
   {
    for (i = 0; i < c; i++)
      {
       p += LogFactorial(t[i] + r - 1) - LogFactorial(r - 1) - LogFactorial(t[i]);
       s += t[i];
       d += t[i] * (t[i] + r);
      }
    p = exp(p);
    d = d * (r - 1) / (r + 1) / r / r;
    for (i = 0; i < r; i++)
      q += m[i] * m[i];
    q = (q - s * s / r) * (r - 1) / d / r;
    p *= exp (0.5 * (r - 1) * log(0.159154943092 * (r - 1) / d / r)) * sqrt (r) / exp (q / 2);
   }

 delete [] m; delete [] t;

 return ceil(p);
}

double MODIFICATOR r8_uniform_01 (int *seed)
//
// The function computes random variable from interval [0;1].
// Designations:
//  seed - seed.
// Return code:
//  random variable.
//
{
 int k; // Work variable

 k = floor((*seed) / 127773);
 *seed = 16807 * ((*seed) - k * 127773) - k * 2836;
 if ((*seed) < 0 )
   *seed += 2147483647;

 return 0.0000000004656612875 * (*seed);
}

double MODIFICATOR KlotzTest (double *x,long nx,double *y,long ny,double *p)
//
// The function for computation Klotz test.
// Designations:
//   x - 1st sample,
//  nx - size of 1st sample,
//   y - 2nd sample,
//  ny - size of 2nd sample,
//  *p - one-sided P-value.
// Return code:
//  test value.
// References:
//  Hajek J., Sidak Z., Sen P.K. Theory of rank tests. - New York, NY: Academic Press, 1999.
//  Conover W.J. Practical nonparametric statistics. - New York, NY: John Wiley & Sons, 1999.
//
{
 long i,            // Counter
      *ix,          // Info array
      n = nx + ny;  // Common size

 double *xx,*xy,    // Work arrays
        t = 0,      // Return value
        m = 0,      // Parameter of normal approximation
        var = 0,    // Parameter of normal approximation
        s;          // Work variable

 //////////////////////////////////////////////
 // Control of the admissibility of estimations
 if (Variance(x,nx) < 0.000001 && Variance(y,ny) < 0.000001)
   {
    *p = 0.5;
    return 0;
   }

 ix = new long[n];
 xx = new double[n];
 xy = new double[n];

 ////////////////////////
 // Given arrays junction
 for (i = 0; i < nx; i++)
   xy[i] = x[i];
 for (i = 0; i < ny; i++)
   xy[i + nx] = y[i];

 ///////////////////////
 // Sort common array up
 SortArrayUp (xy,n,ix);

 ///////////////////////
 // Ranking common array
 Rank (xy,xx,n);

 /////////////////////
 // Restore rank order
 UnSortArray (xx,xy,n,ix);

 delete [] ix; delete [] xx;

 ////////////////////////////
 // Value of test computation
 for (i = 0; i < nx; i++)
   {
    s = InverseNormalDistribution (xy[i] / (n + 1));
    t += s * s;
   }

 ///////////////////////
 // Normal approximation
 for (i = 0; i < n; i++)
   {
//  s = InverseNormalDistribution (xy[i] / (n + 1));     // Conover's formula
    s = InverseNormalDistribution ((1.0 + i) / (n + 1)); // Hajek's formula
    m += s * s;
    var += s * s * s * s;
   }
 delete [] xy;

 m = m * nx / n;
 var = var * nx / n * ny / (n - 1) - m * m / nx * ny / (n - 1);

 //////////////////////
 // P-value computation
 *p = NormalDistribution ((t - m) / sqrt(var));

 return t;
}

double MODIFICATOR ConoverTest(double *x,long nx,double *y,long ny,double *p)
//
// The function for computation of Conover test.
//   x - 1st sample,
//  nx - size of 1st sample,
//   y - 2nd sample,
//  ny - size of 2nd sample,
//  *p - P-value of the one-sided hypothesis.
// Return code:
//  value of test.
// References:
//  Conover W.J. Practical nonparametric statistics. - New York, NY: John Wiley & Sons, 1999.
//
{
 long i,                 // Counter
       *ix,              // Information array
       n = nx + ny;      // Common size

 double *xx,*xy,         // Work array
        mx = Mean(x,nx), // Means
        my = Mean(y,ny),
        t = 0,           // Value of test
        m = 0,           // Parameter of normal approximationn
        var = 0,         // Parameter of normal approximationn
        s;               // Work variable

 //////////////////////////////////////////////
 // Control of the admissibility of estimations
 if (Variance(x,nx) < 0.000001 && Variance(y,ny) < 0.000001)
   {
    *p = 0.5;
    return 0;
   }

 ix = new long[n];
 xx = new double[n];
 xy = new double[n];

 ///////////////////////////////////////////
 // Transformation and given arrays junction
 for (i = 0; i < nx; i++)
   xy[i] = fabs(x[i] - mx);
 for (i = 0; i < ny; i++)
   xy[i + nx] = fabs(y[i] - my);

 //////////
 // Sort up
 SortArrayUp (xy,n,ix);

 //////////
 // Ranking
 Rank (xy,xx,n);

 //////////////////////
 // Restore ranks order
 UnSortArray (xx,xy,n,ix);

 delete [] ix; delete [] xx;

 ////////////////////////////
 // Value of test computation
 for (i = 0; i < nx; i++)
   {
    s = xy[i] * xy[i];
    t += s;
   }

 /////////////////////////////////////
 // Parameters of normal approximation
 for (i = 0; i < n; i++)
   {
    s = xy[i] * xy[i];
    m += s;
    var += s * s;
   }
 m /= n;
 var = var * nx / n * ny / (n - 1) - m * m * nx / (n - 1) * ny;

 //////////////////////
 // P-value computation
 *p = NormalDistribution ((t - m * nx) / sqrt(var));

 delete [] xy;

 return t;
}

double MODIFICATOR PagurovaDistribution (double n1,double n2, double a,double x)
//
// The function for computation of distribution function of Pagurova test.
// Designations:
//  n1 - number degree of freedom,
//  n2 - number degree of freedom,
//   a - parameter, dependent on the relationship of variances,
//   x - value inverse of function distribution.
// Return code:
//  confidence level of oneside hypothesis.
// References:
//  Pagurova V.I. On comparison of means of two normal samples // Teor. Veroyatnost. i Primenen., 1968, vol. 13, issue 3, pp. 561-568.
//
{
 double c = 0, // Left bound of root localization
        d = 1, // Right bound of root localization
        y,     // The middle of interval localization
        cx;    // Values of function on end of interval

 //////////////////////////////////////////////////////////////
 // Refinement of the root method of dividing a segment in half
 do
   {
    cx = x - InversePagurovaDistribution (n1,n2,a,c);
    y = (c + d) / 2;
    cx *= x - InversePagurovaDistribution (n1,n2,a,y);
    if (cx == 0)
      return y;
    else if (cx > 0)
      c = y;
    else
      d = y;
   } while (fabs (c - d) > 0.000001);

 return y / 2;
}

double MODIFICATOR InversePagurovaDistribution (double n1,double n2, double c,double p)
//
// The function for computation inverse function of distribution function of Pagurova test.
// Designations:
//  n1 - number degree of freedom,
//  n2 - number degree of freedom,
//   c - parameter, dependent on the relationship of variances,
//   p - confidence level.
// Return code:
//  value of inverse distribution function.
// References:
//  Pagurova V.I. On comparison of means of two normal samples // Teor. Veroyatnost. i Primenen., 1968, vol. 13, issue 3, pp. 561-568.
//
{
 double h,t,ht,t1,h1; // Work variables

 h = c - 2.0 * c * (1.0 - c) * ((1.0 - c) / n2 - c / n1);
 t = 1.0 * n1 / (n1 + n2);
 t1 = 1.0 - t;
 h1 = 1.0 - h;
 ht = h - t;

 return ht * ht * h1 / t / t * InverseTDistribution (n2,1.0 - p / 2) +
    (t * t1 + (2.0 * t - 1) * ht) * h * h1 / t / t / t1 / t1 *
    InverseTDistribution (n1 + n2,1.0 - p / 2) +
    ht * ht * h / t1 / t1 * InverseTDistribution (n1,1.0 - p / 2);
}

double MODIFICATOR PagurovaTest(double *x1,long n1,double *x2,long n2,double *p)
//
// The function for calculation of Pagurova test.
// Designations:
//  x1 - 1st sample,
//  n1 - size of 1st sample,
//  x2 - 2nd sample,
//  n2 - size of 2nd sample,
//  *p - one-sided P-value of test.
// Return code:
//  value of test.
// References:
//  Pagurova V.I. On comparison of means of two normal samples // Teor. Veroyatnost. i Primenen., 1968, vol. 13, issue 3, pp. 561-568.
//
{
 double s1,s2, // Variations
        m1,m2, // Mediums
        t;     // Test statistic

 /////////////////////////////////////
 // Estimation parameters distribution
  m1 = Mean(x1,n1);
  m2 = Mean(x2,n2);
  s1 = Variance(x1,n1);
  s2 = Variance(x2,n2);

 //////////////////////////////////////////////
 // Control of the admissibility of estimations
 if (s1 < 0.000001 && s2 < 0.000001)
   {
    *p = 0.5;
    return 0;
   }

 /////////////////////////////////////
 // Test vakue and P-value computation
 if (m1 > m2)
   {
     t = (m1 - m2) / sqrt (s1 / n1 + s2 / n2);
     *p = 1.0 - PagurovaDistribution(n1 - 1,n2 - 1,s1 / (n1 - 1) / (s1 / (n1 - 1) + s2 / (n2 - 1)),t);
   }
 else if (m1 < m2)
   {
    t = (m2 - m1) / sqrt (s1 / n1 + s2 / n2);
    *p = PagurovaDistribution(n1 - 1,n2 - 1,s1 / (n1 - 1) / (s1 / (n1 - 1) + s2 / (n2 - 1)),t);
    t = -t;
   }
 else
   {
    *p = 0.5;
    t = 0;
   }

 return t;
}

double MODIFICATOR MoodBrownTest (double *x,long nx,double *y,long ny,double *p)
//
// The function for calculation of the Mood-Brown median test (normal asymptotic).
// Designations:
//  x1 - 1st sample,
//  n1 - size of 1st sample,
//  x2 - 2nd sample,
//  n2 - size of 2nd sample,
//  *p - one-sided P-value of test.
// Return code:
//  value of test.
// References:
//  Hajek J., Sidak Z., Sen P.K. Theory of rank tests. - New York, NY: Academic Press, 1999.
//  Brown G.W., Mood A.M. On median tests for linear hypotheses // Berkeley Symp. on Math. Statist. and Prob., 1951, pp. 159-166.
//  Mood A.M. Introduction to the theory of statistics. - New Yortk, NY: McGraw-Hill, 1974.
//
{
 long i,           // Counter
      *ix,         // Information array
      n = nx + ny; // Common size

 double *xx,*xy,   // Work arrays
        t = 0;     // Returned value

 ix = new long[n];
 xx = new double[n];
 xy = new double[n];

 ////////////////////////
 // Given arrays junction
 for (i = 0; i < nx; i++)
   xy[i] = x[i];
 for (i = 0; i < ny; i++)
   xy[i + nx] = y[i];

 //////////
 // Sort up
 SortArrayUp (xy,n,ix);

 //////////
 // Ranking
 Rank (xy,xx,n);

 /////////////////////
 // Restore rank order
 UnSortArray (xx,xy,n,ix);

 delete [] ix; delete [] xx;

 /////////////////////////
 // Test value computation
 for (i = 0; i < nx; i++)
   t = t + SIGN(xy[i] - 0.5 * (n + 1));
 t = (t + nx) / 2;

 delete [] xy;

 //////////////////////
 // P-value computation
 if (!Parity(n))
   *p = NormalDistribution ((t - 0.5 * nx) / sqrt(0.25 * nx * ny / (n - 1)));
 else
   *p = NormalDistribution ((t - 0.5 * nx) / sqrt(0.25 * nx * ny / n));

 return t;
}
